@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -47,7 +47,6 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from types import ModuleType
-from typing import List
 
 # Bokeh imports
 from ...core.types import PathLike
@@ -80,14 +79,14 @@ class ScriptHandler(CodeHandler):
 
     _origin = "Script"
 
-    def __init__(self, *, filename: PathLike, argv: List[str] = [], package: ModuleType | None = None) -> None:
+    def __init__(self, *, filename: PathLike, argv: list[str] = [], package: ModuleType | None = None) -> None:
         '''
 
         Keywords:
             filename (str) : a path to a Python source (".py") file
 
         '''
-        with open(filename, 'r', encoding='utf-8') as f:
+        with open(filename, encoding='utf-8') as f:
             source = f.read()
 
         super().__init__(source=source, filename=filename, argv=argv, package=package)
