@@ -24,7 +24,6 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import os
 from types import ModuleType
-from typing import List
 
 # Bokeh imports
 from ...core.types import PathLike
@@ -57,7 +56,7 @@ class ServerRequestHandler(RequestHandler):
 
     _module: ModuleType
 
-    def __init__(self, *, filename: PathLike, argv: List[str] = [], package: ModuleType | None = None) -> None:
+    def __init__(self, *, filename: PathLike, argv: list[str] = [], package: ModuleType | None = None) -> None:
         '''
 
         Keyword Args:
@@ -69,7 +68,7 @@ class ServerRequestHandler(RequestHandler):
         '''
         super().__init__()
 
-        with open(filename, 'r', encoding='utf-8') as f:
+        with open(filename, encoding='utf-8') as f:
             source = f.read()
 
         self._runner = CodeRunner(source, filename, argv, package=package)
