@@ -34,12 +34,10 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from typing import TYPE_CHECKING, Any, Dict
+from typing import TYPE_CHECKING, Any, Literal
 from urllib.parse import quote_plus
 
-# External imports
-from typing_extensions import Literal
-
+## External imports
 if TYPE_CHECKING:
     from tornado.ioloop import IOLoop
 
@@ -54,7 +52,7 @@ from .util import server_url_for_websocket_url, websocket_url_for_server_url
 
 if TYPE_CHECKING:
     from ..document.events import DocumentPatchedEvent, SessionCallbackAdded, SessionCallbackRemoved
-    from ..models.layouts import LayoutDOM
+    from ..models.ui import UIElement
     from ..protocol.messages.patch_doc import patch_doc
     from ..protocol.messages.server_info_reply import ServerInfo
     from ..server.callbacks import DocumentCallbackGroup
@@ -80,7 +78,7 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 def pull_session(session_id: ID | None = None, url: str = "default", io_loop: IOLoop | None = None,
-        arguments: Dict[str, str] | None = None, max_message_size: int = 20*1024*1024) -> ClientSession:
+        arguments: dict[str, str] | None = None, max_message_size: int = 20*1024*1024) -> ClientSession:
     ''' Create a session by loading the current server-side document.
 
     ``session.document`` will be a fresh document loaded from
@@ -275,7 +273,7 @@ class ClientSession:
     _callbacks: DocumentCallbackGroup
 
     def __init__(self, session_id: ID | None = None, websocket_url: str = DEFAULT_SERVER_WEBSOCKET_URL,
-            io_loop: IOLoop | None = None, arguments: Dict[str, str] | None = None, max_message_size: int = 20*1024*1024):
+            io_loop: IOLoop | None = None, arguments: dict[str, str] | None = None, max_message_size: int = 20*1024*1024):
         ''' A connection which attaches to a particular named session on the
         server.
 
@@ -476,12 +474,12 @@ class ClientSession:
         '''
         return self._connection.request_server_info()
 
-    def show(self, obj: LayoutDOM | None = None, browser: str | None = None,
+    def show(self, obj: UIElement | None = None, browser: str | None = None,
             new: Literal["tab", "window"] = "tab") -> None:
         ''' Open a browser displaying this session.
 
         Args:
-            obj (LayoutDOM object, optional) : a Layout (Row/Column),
+            obj (UIElement object, optional) : a Layout (Row/Column),
                 Plot or Widget object to display. The object will be added
                 to the session's document.
 
