@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -35,7 +37,7 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-def server_url_for_websocket_url(url):
+def server_url_for_websocket_url(url: str) -> str:
     ''' Convert an ``ws(s)`` URL for a Bokeh server into the appropriate
     ``http(s)`` URL for the websocket endpoint.
 
@@ -62,7 +64,7 @@ def server_url_for_websocket_url(url):
         raise ValueError("websocket URL does not end in /ws")
     return reprotocoled[:-2]
 
-def websocket_url_for_server_url(url):
+def websocket_url_for_server_url(url: str) -> str:
     ''' Convert an ``http(s)`` URL for a Bokeh server websocket endpoint into
     the appropriate ``ws(s)`` URL
 
