@@ -24,8 +24,7 @@ import contextlib
 import errno
 import os
 import sys
-import warnings
-from typing import Dict, Iterator, List
+from typing import Iterator
 
 # Bokeh imports
 from bokeh.application import Application
@@ -35,6 +34,7 @@ from bokeh.application.handlers import (
     NotebookHandler,
     ScriptHandler,
 )
+from bokeh.util.warnings import warn
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -76,7 +76,7 @@ call "bokeh serve" on the directory instead. For example:
 If this is not the case, renaming main.py will suppress this warning.
 """
 
-def build_single_handler_application(path: str, argv: List[str] | None = None) -> Application:
+def build_single_handler_application(path: str, argv: list[str] | None = None) -> Application:
     ''' Return a Bokeh application built using a single handler for a script,
     notebook, or directory.
 
@@ -114,7 +114,7 @@ def build_single_handler_application(path: str, argv: List[str] | None = None) -
 
     '''
     argv = argv or []
-    path = os.path.abspath(path)
+    path = os.path.abspath(os.path.expanduser(path))
     handler: Handler
 
     # There are certainly race conditions here if the file/directory is deleted
@@ -127,7 +127,7 @@ def build_single_handler_application(path: str, argv: List[str] | None = None) -
             handler = NotebookHandler(filename=path, argv=argv)
         elif path.endswith(".py"):
             if path.endswith("main.py"):
-                warnings.warn(DIRSTYLE_MAIN_WARNING)
+                warn(DIRSTYLE_MAIN_WARNING)
             handler = ScriptHandler(filename=path, argv=argv)
         else:
             raise ValueError("Expected a '.py' script or '.ipynb' notebook, got: '%s'" % path)
@@ -141,7 +141,7 @@ def build_single_handler_application(path: str, argv: List[str] | None = None) -
 
     return application
 
-def build_single_handler_applications(paths: List[str], argvs: Dict[str, List[str]] | None = None) -> Dict[str, Application]:
+def build_single_handler_applications(paths: list[str], argvs: dict[str, list[str]] | None = None) -> dict[str, Application]:
     ''' Return a dictionary mapping routes to Bokeh applications built using
     single handlers, for specified files or directories.
 
@@ -163,7 +163,7 @@ def build_single_handler_applications(paths: List[str], argvs: Dict[str, List[st
         RuntimeError
 
     '''
-    applications: Dict[str, Application] = {}
+    applications: dict[str, Application] = {}
     argvs = argvs or {}
 
     for path in paths:
