@@ -75,17 +75,14 @@ log = logging.getLogger(__name__)
 # Standard library imports
 from typing import (
     Any,
-    Dict,
     Iterator,
-    List,
+    Literal,
+    get_args,
 )
 
-# External imports
-from typing_extensions import Literal, get_args
-
 # Bokeh imports
 from .. import colors, palettes
-from ..util.string import nice_join
+from ..util.strings import nice_join
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -98,6 +95,8 @@ __all__ = (
     'AutosizeMode',
     'ButtonType',
     'CalendarPosition',
+    'ContextWhich',
+    'CoordinateUnits',
     'DashPattern',
     'DateFormat',
     'DatetimeUnits',
@@ -106,11 +105,14 @@ __all__ = (
     'Direction',
     'Enumeration',
     'enumeration',
+    'FlowMode',
     'FontStyle',
+    'HAlign',
     'HatchPattern',
     'HatchPatternAbbreviation',
     'HoldPolicy',
     'HorizontalLocation',
+    'ImageOrigin',
     'JitterRandomDistribution',
     'LatLon',
     'LegendClickPolicy',
@@ -129,9 +131,9 @@ __all__ = (
     'Palette',
     'Place',
     'RenderLevel',
-    'RenderMode',
     'ResetPolicy',
     'RoundingFunction',
+    'ScrollbarPolicy',
     'SelectionMode',
     'SizingMode',
     'SizingPolicy',
@@ -143,9 +145,11 @@ __all__ = (
     'TextBaseline',
     'TextureRepetition',
     'TickLabelOrientation',
+    'ToolIcon',
     'TooltipAttachment',
     'TooltipFieldFormatter',
     'TrackPolicy',
+    'VAlign',
     'VerticalAlign',
     'VerticalLocation',
 )
@@ -168,7 +172,7 @@ class Enumeration:
     '''
     __slots__ = ()
 
-    _values: List[str]
+    _values: list[str]
     _default: str
     _case_sensitive: bool
     _quote: bool
@@ -232,7 +236,7 @@ def enumeration(*values: Any, case_sensitive: bool = True, quote: bool = False)
     if len(values) != len(set(values)):
         raise ValueError(f"enumeration items must be unique, got {nice_join(values)}")
 
-    attrs: Dict[str, Any] = {value: value for value in values}
+    attrs: dict[str, Any] = {value: value for value in values}
     attrs.update({
         "_values": list(values),
         "_default": values[0],
@@ -245,6 +249,12 @@ def enumeration(*values: Any, case_sensitive: bool = True, quote: bool = False)
 #: Alignment (vertical or horizontal) of a child item
 Align = enumeration("start", "center", "end")
 
+#: Horizontal alignment of a child item
+HAlign = enumeration("top", "center", "bottom")
+
+#: Vertical alignment of a child item
+VAlign = enumeration("left", "center", "right")
+
 #: Specify an anchor position on a box/frame
 Anchor = enumeration(
     "top_left",    "top_center",    "top_right",
@@ -265,6 +275,12 @@ ButtonType = enumeration("default", "primary", "success", "warning", "danger", "
 #: Specify a position for the DatePicker calendar to display
 CalendarPosition = enumeration("auto", "above", "below")
 
+#: Specify which tick to add additional context to
+ContextWhich = enumeration("start", "center", "end", "all")
+
+#: Specify units for mapping coordinates
+CoordinateUnits = enumeration("canvas", "screen", "data")
+
 #: Specify a named dashing patter for stroking lines
 DashPattern = enumeration("solid", "dashed", "dotted", "dotdash", "dashdot")
 
@@ -280,11 +296,15 @@ DatetimeUnits = enumeration("microseconds", "milliseconds", "seconds", "minsec",
 Dimension = enumeration("width", "height")
 
 #: Specify a vertical/horizontal dimensions
+DimensionsType = Literal["width", "height", "both"]
 Dimensions = enumeration("width", "height", "both")
 
 #: Specify a stroke direction for circles, wedges, etc.
 Direction = enumeration("clock", "anticlock")
 
+#: Specify the flow behavior in CSS layouts.
+FlowMode = enumeration("block", "inline")
+
 #: Specify the font style for rendering text
 FontStyle = enumeration("normal", "italic", "bold", "bold italic")
 
@@ -343,6 +363,9 @@ HoldPolicy = enumeration(HoldPolicyType)
 #: Specify a horizontal location in plot layouts
 HorizontalLocation = enumeration("left", "right")
 
+#: Defines the coordinate space within an image
+ImageOrigin = enumeration("bottom_left", "top_left", "bottom_right", "top_right")
+
 #: Specify a distribution to use for the Jitter class
 JitterRandomDistributionType = Literal["uniform", "normal"]
 JitterRandomDistribution = enumeration(JitterRandomDistributionType)
@@ -409,20 +432,20 @@ Place = enumeration(PlaceType)
 #: Specify a position in the render order for a Bokeh renderer
 RenderLevel = enumeration("image", "underlay", "glyph", "guide", "annotation", "overlay")
 
-#: Specify a render mode for renderers that support both Canvas or CSS rendering
-RenderMode = enumeration("canvas", "css")
-
 #: What reset actions should occur on a Plot reset
 ResetPolicy = enumeration("standard", "event_only")
 
 #: Specify a policy for  how numbers should be rounded
 RoundingFunction = enumeration("round", "nearest", "floor", "rounddown", "ceil", "roundup")
 
+#: Scrollbar policies
+ScrollbarPolicy = enumeration("auto", "visible", "hidden")
+
 #: Selection modes
 SelectionMode = enumeration("replace", "append", "intersect", "subtract")
 
 #: Sizing mode policies
-SizingModeType = Literal["stretch_width", "stretch_height", "stretch_both", "scale_width", "scale_height", "scale_both", "fixed"]
+SizingModeType = Literal["stretch_width", "stretch_height", "stretch_both", "scale_width", "scale_height", "scale_both", "fixed", "inherit"]
 SizingMode = enumeration(SizingModeType)
 
 #: Individual sizing mode policies
@@ -452,6 +475,42 @@ TextureRepetition = enumeration("repeat", "repeat_x", "repeat_y", "no_repeat")
 #: Specify how axis tick labels are oriented with respect to the axis
 TickLabelOrientation = enumeration("horizontal", "vertical", "parallel", "normal")
 
+#: Well known tool icon names
+ToolIcon = enumeration(
+  "append_mode",
+  "box_edit",
+  "box_select",
+  "box_zoom",
+  "clear_selection",
+  "copy",
+  "crosshair",
+  "freehand_draw",
+  "help",
+  "hover",
+  "intersect_mode",
+  "lasso_select",
+  "line_edit",
+  "pan",
+  "point_draw",
+  "poly_draw",
+  "poly_edit",
+  "polygon_select",
+  "range",
+  "redo",
+  "replace_mode",
+  "reset",
+  "save",
+  "subtract_mode",
+  "tap_select",
+  "undo",
+  "wheel_pan",
+  "wheel_zoom",
+  "xpan",
+  "ypan",
+  "zoom_in",
+  "zoom_out",
+)
+
 #: Specify an attachment for tooltips
 TooltipAttachment = enumeration("horizontal", "vertical", "left", "right", "above", "below")
 
