@@ -33,22 +33,16 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
+    ClassVar,
     Iterable,
-    List,
-    Mapping,
+    Literal,
     NoReturn,
-    Set,
-    Tuple,
     Type,
+    TypedDict,
     TypeVar,
     Union,
     overload,
 )
-from warnings import warn
-
-# External imports
-from typing_extensions import Literal, TypedDict
 
 if TYPE_CHECKING:
     F = TypeVar("F", bound=Callable[..., Any])
@@ -57,17 +51,25 @@ else:
     from functools import lru_cache
 
 # Bokeh imports
-from ..util.string import append_docstring, nice_join
+from ..util.strings import append_docstring, nice_join
+from ..util.warnings import warn
 from .property.descriptor_factory import PropertyDescriptorFactory
 from .property.descriptors import PropertyDescriptor, UnsetValueError
 from .property.override import Override
-from .property.singletons import Undefined
+from .property.singletons import Intrinsic, Undefined
 from .property.wrappers import PropertyValueContainer
-from .types import ID, JSON, Unknown
+from .serialization import (
+    ObjectRep,
+    Ref,
+    Serializable,
+    Serializer,
+)
+from .types import ID
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from ..client.session import ClientSession
-    from ..document.document import StaticSerializer
     from ..server.session import ServerSession
     from .property.bases import Property
     from .property.dataspec import DataSpec
@@ -80,6 +82,8 @@ __all__ = (
     'abstract',
     'HasProps',
     'MetaHasProps',
+    'NonQualified',
+    'Qualified',
 )
 
 #-----------------------------------------------------------------------------
@@ -91,7 +95,7 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 if TYPE_CHECKING:
-    Setter = Union[ClientSession, ServerSession]
+    Setter: TypeAlias = Union[ClientSession, ServerSession]
 
 C = TypeVar("C", bound=Type["HasProps"])
 
@@ -104,12 +108,12 @@ def abstract(cls: C) -> C:
     cls.__doc__ = append_docstring(cls.__doc__, _ABSTRACT_ADMONITION)
     return cls
 
-def is_DataModel(cls: Type[HasProps]) -> bool:
+def is_DataModel(cls: type[HasProps]) -> bool:
     from ..model import DataModel
     return issubclass(cls, HasProps) and getattr(cls, "__data_model__", False) and cls != DataModel
 
-def _overridden_defaults(class_dict: Dict[str, Any]) -> Dict[str, Unknown]:
-    overridden_defaults: Dict[str, Unknown] = {}
+def _overridden_defaults(class_dict: dict[str, Any]) -> dict[str, Any]:
+    overridden_defaults: dict[str, Any] = {}
     for name, prop in tuple(class_dict.items()):
         if isinstance(prop, Override):
             del class_dict[name]
@@ -117,30 +121,63 @@ def _overridden_defaults(class_dict: Dict[str, Any]) -> Dict[str, Unknown]:
                 overridden_defaults[name] = prop.default
     return overridden_defaults
 
-def _generators(class_dict: Dict[str, Any]):
-    generators: Dict[str, PropertyDescriptorFactory[Any]] = {}
+def _generators(class_dict: dict[str, Any]):
+    generators: dict[str, PropertyDescriptorFactory[Any]] = {}
     for name, generator in tuple(class_dict.items()):
         if isinstance(generator, PropertyDescriptorFactory):
             del class_dict[name]
             generators[name] = generator
     return generators
 
+class _ModelResolver:
+    """ """
+
+    _known_models: dict[str, type[HasProps]]
+
+    def __init__(self) -> None:
+        self._known_models = {}
+
+    def add(self, cls: type[HasProps]) -> None:
+        if not (issubclass(cls, Local) or cls.__name__.startswith("_")):
+            # update the mapping of view model names to classes, checking for any duplicates
+            previous = self._known_models.get(cls.__qualified_model__, None)
+            if previous is not None and not hasattr(cls, "__implementation__"):
+                raise Warning(f"Duplicate qualified model declaration of '{cls.__qualified_model__}'. Previous definition: {previous}")
+            self._known_models[cls.__qualified_model__] = cls
+
+    def remove(self, cls: type[HasProps]) -> None:
+        del self._known_models[cls.__qualified_model__]
+
+    @property
+    def known_models(self) -> dict[str, type[HasProps]]:
+        return dict(self._known_models)
+
+    def clear_extensions(self) -> None:
+        def is_extension(obj: type[HasProps]) -> bool:
+            return getattr(obj, "__implementation__", None) is not None or \
+                   getattr(obj, "__javascript__", None) is not None or \
+                   getattr(obj, "__css__", None) is not None
+
+        self._known_models = {key: val for key, val in self._known_models.items() if not is_extension(val)}
+
+_default_resolver = _ModelResolver()
+
 class MetaHasProps(type):
     ''' Specialize the construction of |HasProps| classes.
 
     This class is a `metaclass`_ for |HasProps| that is responsible for
-    creating and adding the |PropertyDescriptor| instances that delegate
+    creating and adding the ``PropertyDescriptor`` instances that delegate
     validation and serialization to |Property| attributes.
 
     .. _metaclass: https://docs.python.org/3/reference/datamodel.html#metaclasses
 
     '''
 
-    __properties__: Dict[str, Property[Any]]
-    __overridden_defaults__: Dict[str, Unknown]
-    __themed_values__: Dict[str, Unknown]
+    __properties__: dict[str, Property[Any]]
+    __overridden_defaults__: dict[str, Any]
+    __themed_values__: dict[str, Any]
 
-    def __new__(cls, class_name: str, bases: Tuple[type, ...], class_dict: Dict[str, Any]):
+    def __new__(cls, class_name: str, bases: tuple[type, ...], class_dict: dict[str, Any]):
         '''
 
         '''
@@ -163,9 +200,13 @@ class MetaHasProps(type):
 
         return super().__new__(cls, class_name, bases, class_dict)
 
-    def __init__(cls, class_name: str, bases: Tuple[type, ...], _) -> None:
+    def __init__(cls, class_name: str, bases: tuple[type, ...], _) -> None:
+        # HasProps itself may not have any properties defined
+        if class_name == "HasProps":
+            return
+
         # Check for improperly redeclared a Property attribute.
-        base_properties: Dict[str, Any] = {}
+        base_properties: dict[str, Any] = {}
         for base in (x for x in bases if issubclass(x, HasProps)):
             base_properties.update(base.properties(_with_props=True))
         own_properties = {k: v for k, v in cls.__dict__.items() if isinstance(v, PropertyDescriptor)}
@@ -174,14 +215,27 @@ class MetaHasProps(type):
             warn(f"Properties {redeclared!r} in class {cls.__name__} were previously declared on a parent "
                  "class. It never makes sense to do this. Redundant properties should be deleted here, or on"
                  "the parent class. Override() can be used to change a default value of a base class property.",
-                 RuntimeWarning, stacklevel=2)
+                 RuntimeWarning)
 
         # Check for no-op Overrides
         unused_overrides = cls.__overridden_defaults__.keys() - cls.properties(_with_props=True).keys()
         if unused_overrides:
-            warn(f"Overrides of {unused_overrides} in class {cls.__name__} does not override anything.", RuntimeWarning, stacklevel=2)
+            warn(f"Overrides of {unused_overrides} in class {cls.__name__} does not override anything.", RuntimeWarning)
 
-class HasProps(metaclass=MetaHasProps):
+    @property
+    def model_class_reverse_map(cls) -> dict[str, type[HasProps]]:
+        return _default_resolver.known_models
+
+class Local:
+    """Don't register this class in model registry. """
+
+class Qualified:
+    """Resolve this class by a fully qualified name. """
+
+class NonQualified:
+    """Resolve this class by a non-qualified name. """
+
+class HasProps(Serializable, metaclass=MetaHasProps):
     ''' Base class for all class types that have Bokeh properties.
 
     .. autoclasstoc::
@@ -189,9 +243,44 @@ class HasProps(metaclass=MetaHasProps):
     '''
     _initialized: bool = False
 
-    _property_values: Dict[str, Unknown]
-    _unstable_default_values: Dict[str, Unknown]
-    _unstable_themed_values: Dict[str, Unknown]
+    _property_values: dict[str, Any]
+    _unstable_default_values: dict[str, Any]
+    _unstable_themed_values: dict[str, Any]
+
+    __view_model__: ClassVar[str]
+    __view_module__: ClassVar[str]
+    __qualified_model__: ClassVar[str]
+    __implementation__: ClassVar[Any] # TODO: specific type
+    __data_model__: ClassVar[bool]
+
+    @classmethod
+    def __init_subclass__(cls):
+        super().__init_subclass__()
+
+        # use an explicitly provided view model name if there is one
+        if "__view_model__" not in cls.__dict__:
+            cls.__view_model__ = cls.__qualname__.replace("<locals>.", "")
+        if "__view_module__" not in cls.__dict__:
+            cls.__view_module__ = cls.__module__
+
+        if "__qualified_model__" not in cls.__dict__:
+            def qualified():
+                module = cls.__view_module__
+                model = cls.__view_model__
+
+                if issubclass(cls, NonQualified):
+                    return model
+
+                if not issubclass(cls, Qualified):
+                    head = module.split(".")[0]
+                    if head == "bokeh" or head == "__main__" or "__implementation__" in cls.__dict__:
+                        return model
+
+                return f"{module}.{model}"
+
+            cls.__qualified_model__ = qualified()
+
+        _default_resolver.add(cls)
 
     def __init__(self, **properties: Any) -> None:
         '''
@@ -203,11 +292,19 @@ class HasProps(metaclass=MetaHasProps):
         self._unstable_themed_values = {}
 
         for name, value in properties.items():
+            # TODO: this would be better to handle in descriptors
+            if value is Undefined or value is Intrinsic:
+                continue
             setattr(self, name, value)
 
+        for name in self.properties() - set(properties.keys()):
+            desc = self.lookup(name)
+            if desc.has_unstable_default(self):
+                desc._get(self) # this fills-in `_unstable_*_values`
+
         self._initialized = True
 
-    def __setattr__(self, name: str, value: Unknown) -> None:
+    def __setattr__(self, name: str, value: Any) -> None:
         ''' Intercept attribute setting on HasProps in order to special case
         a few situations:
 
@@ -235,7 +332,7 @@ class HasProps(metaclass=MetaHasProps):
 
         self._raise_attribute_error_with_matches(name, properties)
 
-    def __getattr__(self, name: str) -> Unknown:
+    def __getattr__(self, name: str) -> Any:
         ''' Intercept attribute setting on HasProps in order to special case
         a few situations:
 
@@ -244,22 +341,21 @@ class HasProps(metaclass=MetaHasProps):
 
         Args:
             name (str) : the name of the attribute to set on this object
-            value (obj) : the value to set
 
         Returns:
-            None
+            Any
 
         '''
         if name.startswith("_"):
-            return super().__getattr__(name)
+            return super().__getattribute__(name)
 
         properties = self.properties(_with_props=True)
         if name in properties:
-            return super().__getattr__(name)
+            return super().__getattribute__(name)
 
         descriptor = getattr(self.__class__, name, None)
         if isinstance(descriptor, property): # Python property
-            return super().__getattr__(name)
+            return super().__getattribute__(name)
 
         self._raise_attribute_error_with_matches(name, properties)
 
@@ -300,57 +396,22 @@ class HasProps(metaclass=MetaHasProps):
         else:
             return self.properties_with_values() == other.properties_with_values()
 
-    # TODO: this assumes that HasProps/Model are defined as in bokehjs, which
-    # isn't the case here. HasProps must be serializable through refs only.
-    __view_model__: str
-    __view_module__: str
+    def to_serializable(self, serializer: Serializer) -> ObjectRep:
+        rep = ObjectRep(
+            type="object",
+            name=self.__qualified_model__,
+        )
 
-    @classmethod
-    def static_to_serializable(cls, serializer: StaticSerializer) -> ModelRef:
-        # TODO: resolving already visited objects should be serializer's duty
-        modelref = serializer.get_ref(cls)
-        if modelref is not None:
-            return modelref
+        properties = self.properties_with_values(include_defaults=False)
+        attributes = {key: serializer.encode(val) for key, val in properties.items()}
 
-        bases: List[Type[HasProps]] = [ basecls for basecls in cls.__bases__ if is_DataModel(basecls) ]
-        if len(bases) == 0:
-            extends = None
-        elif len(bases) == 1:
-            extends = bases[0].static_to_serializable(serializer)
-        else:
-            raise RuntimeError("multiple bases are not supported")
+        if attributes:
+            rep["attributes"] = attributes
 
-        name = cls.__view_model__
-        module = cls.__view_module__
+        return rep
 
-        # TODO: remove this
-        if module == "__main__" or module.split(".")[0] == "bokeh":
-            module = None
-
-        properties: List[PropertyDef] = []
-        overrides: List[OverrideDef] = []
-
-        # TODO: don't use unordered sets
-        for prop_name in cls.__properties__:
-            descriptor = cls.lookup(prop_name)
-            kind = None # TODO: serialize kinds
-            default = descriptor.property._default # TODO: private member
-            properties.append(PropertyDef(name=prop_name, kind=kind, default=default))
-
-        for prop_name, default in getattr(cls, "__overridden_defaults__", {}).items():
-            overrides.append(OverrideDef(name=prop_name, default=default))
-
-        modeldef = ModelDef(name=name, module=module, extends=extends, properties=properties, overrides=overrides)
-        modelref = ModelRef(name=name, module=module)
-
-        serializer.add_ref(cls, modelref, modeldef)
-        return modelref
-
-    def to_serializable(self, serializer: Any) -> Any:
-        pass # TODO: new serializer, hopefully in near future
-
-    def set_from_json(self, name: str, json: JSON, *,
-            models: Dict[ID, HasProps] | None = None, setter: Setter | None = None) -> None:
+    # FQ type name required to suppress Sphinx error "more than one target found for cross-reference 'JSON'"
+    def set_from_json(self, name: str, value: Any, *, setter: Setter | None = None) -> None:
         ''' Set a property value on this object from JSON.
 
         Args:
@@ -379,9 +440,9 @@ class HasProps(metaclass=MetaHasProps):
 
         '''
         if name in self.properties(_with_props=True):
-            log.trace(f"Patching attribute {name!r} of {self!r} with {json!r}") # type: ignore # TODO: log.trace()
+            log.trace(f"Patching attribute {name!r} of {self!r} with {value!r}") # type: ignore # TODO: log.trace()
             descriptor = self.lookup(name)
-            descriptor.set_from_json(self, json, models=models, setter=setter)
+            descriptor.set_from_json(self, value, setter=setter)
         else:
             log.warning("JSON had attr %r on obj %r, which is a client-only or invalid attribute that shouldn't have been sent", name, self)
 
@@ -412,36 +473,6 @@ class HasProps(metaclass=MetaHasProps):
         for k, v in kwargs.items():
             setattr(self, k, v)
 
-    def update_from_json(self, json_attributes: Dict[str, JSON], *,
-            models: Mapping[ID, HasProps] | None = None, setter: Setter | None = None) -> None:
-        ''' Updates the object's properties from a JSON attributes dictionary.
-
-        Args:
-            json_attributes: (JSON-dict) : attributes and values to update
-
-            models (dict or None, optional) :
-                Mapping of model ids to models (default: None)
-
-                This is needed in cases where the attributes to update also
-                have values that have references.
-
-            setter(ClientSession or ServerSession or None, optional) :
-                This is used to prevent "boomerang" updates to Bokeh apps.
-
-                In the context of a Bokeh server application, incoming updates
-                to properties will be annotated with the session that is
-                doing the updating. This value is propagated through any
-                subsequent change notifications that the update triggers.
-                The session can compare the event setter to itself, and
-                suppress any updates that originate from itself.
-
-        Returns:
-            None
-
-        '''
-        for k, v in json_attributes.items():
-            self.set_from_json(k, v, models=models, setter=setter)
-
     @overload
     @classmethod
     def lookup(cls, name: str, *, raises: Literal[True] = True) -> PropertyDescriptor[Any]: ...
@@ -471,16 +502,16 @@ class HasProps(metaclass=MetaHasProps):
     @overload
     @classmethod
     @lru_cache(None)
-    def properties(cls, *, _with_props: Literal[False] = False) -> Set[str]: ...
+    def properties(cls, *, _with_props: Literal[False] = False) -> set[str]: ...
 
     @overload
     @classmethod
     @lru_cache(None)
-    def properties(cls, *, _with_props: Literal[True] = True) -> Dict[str, Property[Any]]: ...
+    def properties(cls, *, _with_props: Literal[True] = True) -> dict[str, Property[Any]]: ...
 
     @classmethod
     @lru_cache(None)
-    def properties(cls, *, _with_props: bool = False) -> Union[Set[str], Dict[str, Property[Any]]]:
+    def properties(cls, *, _with_props: bool = False) -> set[str] | dict[str, Property[Any]]:
         ''' Collect the names of properties on this class.
 
         .. warning::
@@ -492,8 +523,8 @@ class HasProps(metaclass=MetaHasProps):
             property names
 
         '''
-        props: Dict[str, Property[Any]] = {}
-        for c in cls.__mro__:
+        props: dict[str, Property[Any]] = {}
+        for c in reversed(cls.__mro__):
             props.update(getattr(c, "__properties__", {}))
 
         if not _with_props:
@@ -503,7 +534,13 @@ class HasProps(metaclass=MetaHasProps):
 
     @classmethod
     @lru_cache(None)
-    def properties_with_refs(cls) -> Dict[str, Property[Any]]:
+    def descriptors(cls) -> list[PropertyDescriptor[Any]]:
+        """ List of property descriptors in the order of definition. """
+        return [ cls.lookup(name) for name, _ in cls.properties(_with_props=True).items() ]
+
+    @classmethod
+    @lru_cache(None)
+    def properties_with_refs(cls) -> dict[str, Property[Any]]:
         ''' Collect the names of all properties on this class that also have
         references.
 
@@ -518,7 +555,7 @@ class HasProps(metaclass=MetaHasProps):
 
     @classmethod
     @lru_cache(None)
-    def dataspecs(cls) -> Dict[str, DataSpec]:
+    def dataspecs(cls) -> dict[str, DataSpec]:
         ''' Collect the names of all ``DataSpec`` properties on this class.
 
         This method *always* traverses the class hierarchy and includes
@@ -531,7 +568,7 @@ class HasProps(metaclass=MetaHasProps):
         from .property.dataspec import DataSpec  # avoid circular import
         return {k: v for k, v in cls.properties(_with_props=True).items() if isinstance(v, DataSpec)}
 
-    def properties_with_values(self, *, include_defaults: bool = True, include_undefined: bool = False) -> Dict[str, Unknown]:
+    def properties_with_values(self, *, include_defaults: bool = True, include_undefined: bool = False) -> dict[str, Any]:
         ''' Collect a dict mapping property names to their values.
 
         This method *always* traverses the class hierarchy and includes
@@ -556,20 +593,20 @@ class HasProps(metaclass=MetaHasProps):
             include_defaults=include_defaults, include_undefined=include_undefined)
 
     @classmethod
-    def _overridden_defaults(cls) -> Dict[str, Unknown]:
+    def _overridden_defaults(cls) -> dict[str, Any]:
         ''' Returns a dictionary of defaults that have been overridden.
 
         .. note::
             This is an implementation detail of ``Property``.
 
         '''
-        defaults: Dict[str, Unknown] = {}
+        defaults: dict[str, Any] = {}
         for c in reversed(cls.__mro__):
             defaults.update(getattr(c, "__overridden_defaults__", {}))
         return defaults
 
     def query_properties_with_values(self, query: Callable[[PropertyDescriptor[Any]], bool], *,
-            include_defaults: bool = True, include_undefined: bool = False) -> Dict[str, Unknown]:
+            include_defaults: bool = True, include_undefined: bool = False) -> dict[str, Any]:
         ''' Query the properties values of |HasProps| instances with a
         predicate.
 
@@ -586,29 +623,33 @@ class HasProps(metaclass=MetaHasProps):
             dict : mapping of property names and values for matching properties
 
         '''
-        themed_keys: Set[str] = set()
-        result: Dict[str, Unknown] = {}
+        themed_keys: set[str] = set()
+        result: dict[str, Any] = {}
 
+        keys = self.properties(_with_props=True)
         if include_defaults:
-            keys = self.properties(_with_props=True)
+            selected_keys = set(keys)
         else:
             # TODO (bev) For now, include unstable default values. Things rely on Instances
             # always getting serialized, even defaults, and adding unstable defaults here
             # accomplishes that. Unmodified defaults for property value containers will be
             # weeded out below.
-            keys = set(self._property_values.keys()) | set(self._unstable_default_values.keys())
+            selected_keys = set(self._property_values.keys()) | set(self._unstable_default_values.keys())
             themed_values = self.themed_values()
             if themed_values is not None:
                 themed_keys = set(themed_values.keys())
-                keys |= themed_keys
+                selected_keys |= themed_keys
 
         for key in keys:
+            if key not in selected_keys:
+                continue
+
             descriptor = self.lookup(key)
             if not query(descriptor):
                 continue
 
             try:
-                value = descriptor.serializable_value(self)
+                value = descriptor.get_value(self)
             except UnsetValueError:
                 if include_undefined:
                     value = Undefined
@@ -623,7 +664,7 @@ class HasProps(metaclass=MetaHasProps):
 
         return result
 
-    def themed_values(self) -> Dict[str, Unknown] | None:
+    def themed_values(self) -> dict[str, Any] | None:
         ''' Get any theme-provided overrides.
 
         Results are returned as a dict from property name to value, or
@@ -635,7 +676,7 @@ class HasProps(metaclass=MetaHasProps):
         '''
         return getattr(self, '__themed_values__', None)
 
-    def apply_theme(self, property_values: Dict[str, Unknown]) -> None:
+    def apply_theme(self, property_values: dict[str, Any]) -> None:
         ''' Apply a set of theme values which will be used rather than
         defaults, but will not override application-set values.
 
@@ -656,13 +697,13 @@ class HasProps(metaclass=MetaHasProps):
         if old_dict is property_values:  # lgtm [py/comparison-using-is]
             return
 
-        removed: Set[str] = set()
+        removed: set[str] = set()
         # we're doing a little song-and-dance to avoid storing __themed_values__ or
         # an empty dict, if there's no theme that applies to this HasProps instance.
         if old_dict is not None:
             removed.update(set(old_dict.keys()))
         added = set(property_values.keys())
-        old_values: Dict[str, Unknown] = {}
+        old_values: dict[str, Any] = {}
         for k in added.union(removed):
             old_values[k] = getattr(self, k)
 
@@ -698,27 +739,85 @@ class HasProps(metaclass=MetaHasProps):
         Values that are containers are shallow-copied.
 
         '''
-        return self.__class__(**self._property_values)
+        attrs = self.properties_with_values(include_defaults=False, include_undefined=True)
+        return self.__class__(**{key: val for key, val in attrs.items() if val is not Undefined})
 
 KindRef = Any # TODO
 
-class PropertyDef(TypedDict):
+class _PropertyDef(TypedDict):
     name: str
-    kind: KindRef | None
-    default: Unknown | None
+    kind: KindRef
+class PropertyDef(_PropertyDef, total=False):
+    default: Any
 
 class OverrideDef(TypedDict):
     name: str
-    default: Unknown
+    default: Any
 
-class ModelRef(TypedDict):
+class _ModelDef(TypedDict):
+    type: Literal["model"]
     name: str
-    module: str | None
+class ModelDef(_ModelDef, total=False):
+    extends: Ref | None
+    properties: list[PropertyDef]
+    overrides: list[OverrideDef]
 
-class ModelDef(ModelRef):
-    extends: ModelRef | None
-    properties: List[PropertyDef] | None
-    overrides: List[OverrideDef] | None
+def _HasProps_to_serializable(cls: type[HasProps], serializer: Serializer) -> Ref | ModelDef:
+    from ..model import DataModel, Model
+
+    ref = Ref(id=ID(cls.__qualified_model__))
+    serializer.add_ref(cls, ref)
+
+    if not is_DataModel(cls):
+        return ref
+
+    # TODO: consider supporting mixin models
+    bases: list[type[HasProps]] = [ base for base in cls.__bases__ if issubclass(base, Model) and base != DataModel ]
+    if len(bases) == 0:
+        extends = None
+    elif len(bases) == 1:
+        [base] = bases
+        extends = serializer.encode(base)
+    else:
+        serializer.error("multiple bases are not supported")
+
+    properties: list[PropertyDef] = []
+    overrides: list[OverrideDef] = []
+
+    # TODO: don't use unordered sets
+    for prop_name in cls.__properties__:
+        descriptor = cls.lookup(prop_name)
+        kind = "Any" # TODO: serialize kinds
+        default = descriptor.property._default
+
+        if default is Undefined:
+            prop_def = PropertyDef(name=prop_name, kind=kind)
+        else:
+            if descriptor.is_unstable(default):
+                default = default()
+
+            prop_def = PropertyDef(name=prop_name, kind=kind, default=serializer.encode(default))
+
+        properties.append(prop_def)
+
+    for prop_name, default in getattr(cls, "__overridden_defaults__", {}).items():
+        overrides.append(OverrideDef(name=prop_name, default=serializer.encode(default)))
+
+    modeldef = ModelDef(
+        type="model",
+        name=cls.__qualified_model__,
+    )
+
+    if extends is not None:
+        modeldef["extends"] = extends
+    if properties:
+        modeldef["properties"] = properties
+    if overrides:
+        modeldef["overrides"] = overrides
+
+    return modeldef
+
+Serializer.register(MetaHasProps, _HasProps_to_serializable)
 
 #-----------------------------------------------------------------------------
 # Private API
