@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -7,9 +7,6 @@
 ''' Provide a functions and classes to implement a custom JSON encoder for
 serializing objects for BokehJS.
 
-The primary interface is provided by the |serialize_json| function, which
-uses the custom |BokehJSONEncoder| to produce JSON output.
-
 In general, functions in this module convert values in the following way:
 
 * Datetime values (Python, Pandas, NumPy) are converted to floating point
@@ -36,16 +33,15 @@ In general, functions in this module convert values in the following way:
 * ``Color`` instances are converted to CSS color values.
 
 .. |serialize_json| replace:: :class:`~bokeh.core.json_encoder.serialize_json`
-.. |BokehJSONEncoder| replace:: :class:`~bokeh.core.json_encoder.BokehJSONEncoder`
 
 '''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import annotations
 
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -53,28 +49,18 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-import collections
-import decimal
-import json
-
-# External imports
-import numpy as np
+from json import JSONEncoder
+from typing import Any
 
 # Bokeh imports
 from ..settings import settings
-from ..util.dependencies import import_optional
-from ..util.serialization import convert_datetime_type, convert_timedelta_type, is_datetime_type, is_timedelta_type, transform_series, transform_array
+from .serialization import Buffer, Serialized
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
-rd = import_optional("dateutil.relativedelta")
-
-pd = import_optional('pandas')
-
 __all__ = (
-    'BokehJSONEncoder',
     'serialize_json',
 )
 
@@ -82,7 +68,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-def serialize_json(obj, pretty=None, indent=None, **kwargs):
+def serialize_json(obj: Any | Serialized[Any], *, pretty: bool | None = None, indent: int | None = None) -> str:
     ''' Return a serialized JSON representation of objects, suitable to
     send to BokehJS.
 
@@ -141,12 +127,6 @@ def serialize_json(obj, pretty=None, indent=None, **kwargs):
             }
 
     '''
-
-    # these args to json.dumps are computed internally and should not be passed along
-    for name in ['allow_nan', 'separators', 'sort_keys']:
-        if name in kwargs:
-            raise ValueError("The value of %r is computed internally, overriding is not permissible." % name)
-
     pretty = settings.pretty(pretty)
 
     if pretty:
@@ -157,98 +137,37 @@ def serialize_json(obj, pretty=None, indent=None, **kwargs):
     if pretty and indent is None:
         indent = 2
 
-    return json.dumps(obj, cls=BokehJSONEncoder, allow_nan=False, indent=indent, separators=separators, sort_keys=True, **kwargs)
+    content: Any
+    buffers: list[Buffer]
+    if isinstance(obj, Serialized):
+        content = obj.content
+        buffers = obj.buffers or []
+    else:
+        content = obj
+        buffers = []
 
+    encoder = PayloadEncoder(buffers=buffers, indent=indent, separators=separators)
+    return encoder.encode(content)
 
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
 
-class BokehJSONEncoder(json.JSONEncoder):
-    ''' A custom ``json.JSONEncoder`` subclass for encoding objects in
-    accordance with the BokehJS protocol.
-
-    '''
-    def transform_python_types(self, obj):
-        ''' Handle special scalars such as (Python, NumPy, or Pandas)
-        datetimes, or Decimal values.
-
-        Args:
-            obj (obj) :
-
-                The object to encode. Anything not specifically handled in
-                this method is passed on to the default system JSON encoder.
-
-        '''
-
-        # date/time values that get serialized as milliseconds
-        if is_datetime_type(obj):
-            return convert_datetime_type(obj)
-
-        if is_timedelta_type(obj):
-            return convert_timedelta_type(obj)
-
-        # slice objects
-        elif isinstance(obj, slice):
-            return dict(start=obj.start, stop=obj.stop, step=obj.step)
-
-        # NumPy scalars
-        elif np.issubdtype(type(obj), np.floating):
-            return float(obj)
-        elif np.issubdtype(type(obj), np.integer):
-            return int(obj)
-        elif np.issubdtype(type(obj), np.bool_):
-            return bool(obj)
-
-        # Decimal values
-        elif isinstance(obj, decimal.Decimal):
-            return float(obj)
-
-        # RelativeDelta gets serialized as a dict
-        elif rd and isinstance(obj, rd.relativedelta):
-            return dict(years=obj.years,
-                    months=obj.months,
-                    days=obj.days,
-                    hours=obj.hours,
-                    minutes=obj.minutes,
-                    seconds=obj.seconds,
-                    microseconds=obj.microseconds)
+class PayloadEncoder(JSONEncoder):
+    def __init__(self, *, buffers: list[Buffer] = [], threshold: int = 100,
+            indent: int | None = None, separators: tuple[str, str] | None = None):
+        super().__init__(sort_keys=False, allow_nan=False, indent=indent, separators=separators)
+        self._buffers = {buf.id: buf for buf in buffers}
+        self._threshold = threshold
 
+    def default(self, obj: Any) -> Any:
+        if isinstance(obj, Buffer):
+            if obj.id in self._buffers: # TODO: and len(obj.data) > self._threshold:
+                return obj.ref
+            else:
+                return obj.to_base64()
         else:
-            return super(BokehJSONEncoder, self).default(obj)
-
-    def default(self, obj):
-        ''' The required ``default`` method for ``JSONEncoder`` subclasses.
-
-        Args:
-            obj (obj) :
-
-                The object to encode. Anything not specifically handled in
-                this method is passed on to the default system JSON encoder.
-
-        '''
-
-        from ..model import Model
-        from ..colors import Color
-        from .has_props import HasProps
-
-        # array types -- use force_list here, only binary
-        # encoding CDS columns for now
-        if pd and isinstance(obj, (pd.Series, pd.Index)):
-            return transform_series(obj, force_list=True)
-        elif isinstance(obj, np.ndarray):
-            return transform_array(obj, force_list=True)
-        elif isinstance(obj, collections.deque):
-            return list(map(self.default, obj))
-        elif isinstance(obj, Model):
-            return obj.ref
-        elif isinstance(obj, HasProps):
-            return obj.properties_with_values(include_defaults=False)
-        elif isinstance(obj, Color):
-            return obj.to_css()
-
-        else:
-            return self.transform_python_types(obj)
+            return super().default(obj)
 
 #-----------------------------------------------------------------------------
 # Private API
