@@ -27,27 +27,19 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-import types
 from copy import copy
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
     ClassVar,
-    Dict,
-    List,
-    Tuple,
     Type,
     TypeVar,
     Union,
 )
 
-# External imports
-import numpy as np
-
 # Bokeh imports
-from ...util.dependencies import import_optional
-from ...util.string import nice_join
+from ...util.strings import nice_join
 from ..has_props import HasProps
 from ._sphinx import property_link, register_type_link, type_link
 from .descriptor_factory import PropertyDescriptorFactory
@@ -60,22 +52,20 @@ from .singletons import (
 )
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from ...document.events import DocumentPatchedEvent
-    from ..types import ID
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
-pd = import_optional('pandas')
-
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
 __all__ = (
     'ContainerProperty',
-    'DeserializationError',
     'PrimitiveProperty',
     'Property',
     'validation_on',
@@ -87,12 +77,9 @@ __all__ = (
 
 T = TypeVar("T")
 
-TypeOrInst = Union[Type[T], T]
+TypeOrInst: TypeAlias = Union[Type[T], T]
 
-Init = Union[T, UndefinedType, IntrinsicType]
-
-class DeserializationError(Exception):
-    pass
+Init: TypeAlias = Union[T, UndefinedType, IntrinsicType]
 
 class Property(PropertyDescriptorFactory[T]):
     """ Base class for Bokeh property instances, which can be added to Bokeh
@@ -107,34 +94,27 @@ class Property(PropertyDescriptorFactory[T]):
             used by the :ref:`bokeh.sphinxext.bokeh_prop` extension when
             generating Spinx documentation. (default: None)
 
-        serialized (bool, optional) :
-            Whether attributes created from this property should be included
-            in serialization (default: True)
-
-        readonly (bool, optional) :
-            Whether attributes created from this property are read-only.
-            (default: False)
-
     """
 
     # This class attribute is controlled by external helper API for validation
     _should_validate: ClassVar[bool] = True
 
-    _readonly: bool
+    _readonly: ClassVar[bool] = False
+    _serialized: ClassVar[bool] = True
 
-    alternatives: List[Tuple[Property[Any], Callable[[Property[Any]], T]]]
-    assertions: List[Tuple[Callable[[HasProps, T], bool], str | Callable[[HasProps, str, T], None]]]
+    _self_serialized: bool
 
-    def __init__(self, default: Init[T] = Intrinsic, help: str | None = None,
-            serialized: bool | None = None, readonly: bool = False):
+    alternatives: list[tuple[Property[Any], Callable[[Any], T]]]
+    assertions: list[tuple[Callable[[HasProps, T], bool], str | Callable[[HasProps, str, T], None]]]
+
+    def __init__(self, *, default: Init[T] = Intrinsic, help: str | None = None) -> None:
         default = default if default is not Intrinsic else Undefined
 
-        if serialized is None:
-            self._serialized = False if readonly and default is Undefined else True
+        if self._serialized:
+            self._self_serialized = not (self.readonly and default is Undefined)
         else:
-            self._serialized = serialized
+            self._self_serialized = False
 
-        self._readonly = readonly
         self._default = default
         self._help = help
         self.__doc__ = help
@@ -142,10 +122,41 @@ class Property(PropertyDescriptorFactory[T]):
         self.alternatives = []
         self.assertions = []
 
+    @property
+    def default(self) -> Init[T]:
+        return self._default
+
+    @property
+    def help(self) -> str | None:
+        return self._help
+
     def __str__(self) -> str:
         return self.__class__.__name__
 
-    def make_descriptors(self, name: str) -> List[PropertyDescriptor[T]]:
+    def __repr__(self) -> str:
+        return str(self)
+
+    def __call__(self, *, default: Init[T] = Intrinsic, help: str | None = None) -> Property[T]:
+        """ Clone this property and allow to override ``default`` and ``help``. """
+        default = self._default if default is Intrinsic else default
+        help = self._help if help is None else help
+        prop = self.__class__(default=default, help=help)
+        prop.alternatives = list(self.alternatives)
+        prop.assertions = list(self.assertions)
+        return prop
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, self.__class__):
+            return (
+                self._default == other._default and
+                self._help == other._help and
+                self.alternatives == other.alternatives and
+                self.assertions == other.assertions
+            )
+        else:
+            return False
+
+    def make_descriptors(self, name: str) -> list[PropertyDescriptor[T]]:
         """ Return a list of ``PropertyDescriptor`` instances to install
         on a class, in order to delegate attribute access to this property.
 
@@ -166,20 +177,22 @@ class Property(PropertyDescriptorFactory[T]):
         to be called).
 
         """
-        return isinstance(self._default, types.FunctionType)
+        return callable(self._default)
 
     @classmethod
-    def _copy_default(cls, default: Callable[[], T] | T) -> T:
+    def _copy_default(cls, default: Callable[[], T] | T, *, no_eval: bool = False) -> T:
         """ Return a copy of the default, or a new value if the default
         is specified by a function.
 
         """
-        if not isinstance(default, types.FunctionType):
+        if not callable(default):
             return copy(default)
         else:
+            if no_eval:
+                return default
             return default()
 
-    def _raw_default(self) -> T:
+    def _raw_default(self, *, no_eval: bool = False) -> T:
         """ Return the untransformed default value.
 
         The raw_default() needs to be validated and transformed by
@@ -187,9 +200,9 @@ class Property(PropertyDescriptorFactory[T]):
         subclass overrides or by themes.
 
         """
-        return self._copy_default(self._default)
+        return self._copy_default(self._default, no_eval=no_eval)
 
-    def themed_default(self, cls: Type[HasProps], name: str, theme_overrides: Dict[str, Any] | None) -> T:
+    def themed_default(self, cls: type[HasProps], name: str, theme_overrides: dict[str, Any] | None, *, no_eval: bool = False) -> T:
         """ The default, transformed by prepare_value() and the theme overrides.
 
         """
@@ -198,9 +211,9 @@ class Property(PropertyDescriptorFactory[T]):
             overrides = cls._overridden_defaults()
 
         if name in overrides:
-            default = self._copy_default(overrides[name])
+            default = self._copy_default(overrides[name], no_eval=no_eval)
         else:
-            default = self._raw_default()
+            default = self._raw_default(no_eval=no_eval)
         return self.prepare_value(cls, name, default)
 
     @property
@@ -210,7 +223,7 @@ class Property(PropertyDescriptorFactory[T]):
         This would be False for a "virtual" or "convenience" property that duplicates
         information already available in other properties, for example.
         """
-        return self._serialized
+        return self._self_serialized
 
     @property
     def readonly(self) -> bool:
@@ -234,15 +247,17 @@ class Property(PropertyDescriptorFactory[T]):
             True, if new and old match, False otherwise
 
         """
+        import numpy as np
+        import pandas as pd
+
         if isinstance(new, np.ndarray) or isinstance(old, np.ndarray):
             return np.array_equal(new, old)
 
-        if pd:
-            if isinstance(new, pd.Series) or isinstance(old, pd.Series):
-                return np.array_equal(new, old)
+        if isinstance(new, pd.Series) or isinstance(old, pd.Series):
+            return np.array_equal(new, old)
 
-            if isinstance(new, pd.Index) or isinstance(old, pd.Index):
-                return np.array_equal(new, old)
+        if isinstance(new, pd.Index) or isinstance(old, pd.Index):
+            return np.array_equal(new, old)
 
         try:
 
@@ -262,20 +277,6 @@ class Property(PropertyDescriptorFactory[T]):
         except ValueError:
             return False
 
-    def from_json(self, json: Any, *, models: Dict[ID, HasProps] | None = None) -> T:
-        """ Convert from JSON-compatible values into a value for this property.
-
-        JSON-compatible values are: list, dict, number, string, bool, None
-
-        """
-        return json
-
-    def serialize_value(self, value: T) -> Any:
-        """ Change the value into a JSON serializable format.
-
-        """
-        return value
-
     def transform(self, value: Any) -> T:
         """ Change the value into the canonical format for this property.
 
@@ -338,13 +339,12 @@ class Property(PropertyDescriptorFactory[T]):
     def _hinted_value(self, value: Any, hint: DocumentPatchedEvent | None) -> Any:
         return value
 
-    def prepare_value(self, owner: HasProps | Type[HasProps], name: str, value: Any, *, hint: DocumentPatchedEvent | None = None) -> T:
+    def prepare_value(self, owner: HasProps | type[HasProps], name: str, value: Any, *, hint: DocumentPatchedEvent | None = None) -> T:
         if value is Intrinsic:
             value = self._raw_default()
         if value is Undefined:
             return value
 
-
         error = None
         try:
             if validation_on():
@@ -387,7 +387,7 @@ class Property(PropertyDescriptorFactory[T]):
     def has_ref(self) -> bool:
         return False
 
-    def accepts(self, tp: TypeOrInst[Property[Any]], converter: Callable[[Property[Any]], T]) -> Property[T]:
+    def accepts(self, tp: TypeOrInst[Property[Any]], converter: Callable[[Any], T]) -> Property[T]:
         """ Declare that other types may be converted to this property type.
 
         Args:
@@ -433,13 +433,45 @@ class Property(PropertyDescriptorFactory[T]):
         self.assertions.append((fn, msg_or_fn))
         return self
 
-TItem = TypeVar("TItem", bound=Property[Any])
+    def replace(self, old: type[Property[Any]], new: Property[Any]) -> Property[Any]:
+        if self.__class__ == old:
+            return new
+        else:
+            return self
 
-class ParameterizedProperty(Property[TItem]):
+class ParameterizedProperty(Property[T]):
     """ A base class for Properties that have type parameters, e.g. ``List(String)``.
 
     """
 
+    _type_params: list[Property[Any]]
+
+    def __init__(self, *type_params: TypeOrInst[Property[T]], default: Init[T] = Intrinsic, help: str | None = None) -> None:
+        _type_params = [ self._validate_type_param(param) for param in type_params ]
+        default = default if default is not Intrinsic else _type_params[0]._raw_default()
+        self._type_params = _type_params
+        super().__init__(default=default, help=help)
+
+    def __str__(self) -> str:
+        class_name = self.__class__.__name__
+        item_types = ", ".join(str(x) for x in self.type_params)
+        return f"{class_name}({item_types})"
+
+    def __call__(self, *, default: Init[T] = Intrinsic, help: str | None = None) -> ParameterizedProperty[T]:
+        """ Clone this property and allow to override ``default`` and ``help``. """
+        default = self._default if default is Intrinsic else default
+        help = self._help if help is None else help
+        prop = self.__class__(*self.type_params, default=default, help=help)
+        prop.alternatives = list(self.alternatives)
+        prop.assertions = list(self.assertions)
+        return prop
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, self.__class__):
+            return super().__eq__(other) and self.type_params == other.type_params
+        else:
+            return False
+
     @staticmethod
     def _validate_type_param(type_param: TypeOrInst[Property[Any]], *, help_allowed: bool = False) -> Property[Any]:
         if isinstance(type_param, type):
@@ -456,45 +488,44 @@ class ParameterizedProperty(Property[TItem]):
         raise ValueError(f"expected a Property as type parameter, got {type_param}")
 
     @property
-    def type_params(self) -> List[Property[Any]]:
-        raise NotImplementedError("abstract method")
+    def type_params(self) -> list[Property[Any]]:
+        return self._type_params
 
     @property
     def has_ref(self) -> bool:
         return any(type_param.has_ref for type_param in self.type_params)
 
+    def _may_have_unstable_default(self) -> bool:
+        return super()._may_have_unstable_default() or \
+            any(type_param._may_have_unstable_default() for type_param in self.type_params)
+
+    def replace(self, old: type[Property[Any]], new: Property[Any]) -> Property[Any]:
+        if self.__class__ == old:
+            return new
+        else:
+            params = [ type_param.replace(old, new) for type_param in self.type_params ]
+            return self.__class__(*params)
+
 class SingleParameterizedProperty(ParameterizedProperty[T]):
     """ A parameterized property with a single type parameter. """
 
-    def __init__(self, type_param: TypeOrInst[Property[Any]], *, default: Init[T] = Intrinsic,
-            help: str | None = None, serialized: bool | None = None, readonly: bool = False):
-        self.type_param = self._validate_type_param(type_param)
-        default = default if default is not Intrinsic else self.type_param._raw_default()
-        super().__init__(default=default, help=help, serialized=serialized, readonly=readonly)
-
     @property
-    def type_params(self) -> List[Property[Any]]:
-        return [self.type_param]
+    def type_param(self) -> Property[T]:
+        return self._type_params[0]
 
-    def __str__(self) -> str:
-        return f"{self.__class__.__name__}({self.type_param})"
+    def __init__(self, type_param: TypeOrInst[Property[Any]], *, default: Init[T] = Intrinsic, help: str | None = None):
+        super().__init__(type_param, default=default, help=help)
 
     def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail=detail)
         self.type_param.validate(value, detail=detail)
 
-    def from_json(self, json: Any, *, models: Dict[str, HasProps] | None = None) -> T:
-        return self.type_param.from_json(json, models=models)
-
     def transform(self, value: T) -> T:
         return self.type_param.transform(value)
 
     def wrap(self, value: T) -> T:
         return self.type_param.wrap(value)
 
-    def _may_have_unstable_default(self) -> bool:
-        return self.type_param._may_have_unstable_default()
-
 class PrimitiveProperty(Property[T]):
     """ A base class for simple property types.
 
@@ -512,7 +543,7 @@ class PrimitiveProperty(Property[T]):
 
     """
 
-    _underlying_type: ClassVar[Tuple[Type[T]]]
+    _underlying_type: ClassVar[tuple[type[Any], ...]]
 
     def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
@@ -527,13 +558,6 @@ class PrimitiveProperty(Property[T]):
         msg = f"expected a value of type {expected_type}, got {value} of type {type(value).__name__}"
         raise ValueError(msg)
 
-    def from_json(self, json: Any, *, models: Dict[str, HasProps] | None = None) -> T:
-        if isinstance(json, self._underlying_type):
-            return json
-        expected_type = nice_join([ cls.__name__ for cls in self._underlying_type ])
-        msg = f"{self} expected {expected_type}, got {json} of type {type(json).__name__}"
-        raise DeserializationError(msg)
-
 class ContainerProperty(ParameterizedProperty[T]):
     """ A base class for Container-like type properties.
 
@@ -541,7 +565,7 @@ class ContainerProperty(ParameterizedProperty[T]):
 
     def _may_have_unstable_default(self) -> bool:
         # all containers are mutable, so the default can be modified
-        return True
+        return self._default is not Undefined
 
 def validation_on() -> bool:
     """ Check if property validation is currently active
@@ -561,5 +585,5 @@ def validation_on() -> bool:
 #-----------------------------------------------------------------------------
 
 @register_type_link(SingleParameterizedProperty)
-def _sphinx_type(obj):
+def _sphinx_type(obj: SingleParameterizedProperty[Any]):
     return f"{property_link(obj)}({type_link(obj.type_param)})"
