@@ -28,17 +28,27 @@ from collections.abc import (
     Sequence,
     Sized,
 )
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, TypeVar
 
 # Bokeh imports
-from ...util.serialization import decode_base64_dict, transform_column_source_data
 from ._sphinx import property_link, register_type_link, type_link
-from .bases import ContainerProperty, DeserializationError
+from .bases import (
+    ContainerProperty,
+    Init,
+    Property,
+    SingleParameterizedProperty,
+    TypeOrInst,
+)
 from .descriptors import ColumnDataPropertyDescriptor
 from .enum import Enum
 from .numeric import Int
-from .singletons import Undefined
-from .wrappers import PropertyValueColumnData, PropertyValueDict, PropertyValueList
+from .singletons import Intrinsic, Undefined
+from .wrappers import (
+    PropertyValueColumnData,
+    PropertyValueDict,
+    PropertyValueList,
+    PropertyValueSet,
+)
 
 if TYPE_CHECKING:
     from ...document.events import DocumentPatchedEvent
@@ -52,39 +62,33 @@ __all__ = (
     'ColumnData',
     'Dict',
     'List',
+    'NonEmpty',
     'RelativeDelta',
     'RestrictedDict',
     'Seq',
+    'Set',
     'Tuple',
 )
 
+T = TypeVar("T")
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-class Seq(ContainerProperty):
+class Seq(ContainerProperty[T]):
     """ Accept non-string ordered sequences of values, e.g. list, tuple, array.
 
     """
 
-    def __init__(self, item_type, default=Undefined, help=None) -> None:
-        self.item_type = self._validate_type_param(item_type)
-        super().__init__(default=default, help=help)
-
-    def __str__(self) -> str:
-        return f"{self.__class__.__name__}({self.item_type})"
+    def __init__(self, item_type: TypeOrInst[Property[T]], *, default: Init[T] = Undefined, help: str | None = None) -> None:
+        super().__init__(item_type, default=default, help=help)
 
     @property
-    def type_params(self):
-        return [self.item_type]
+    def item_type(self):
+        return self.type_params[0]
 
-    def from_json(self, json, *, models=None):
-        if isinstance(json, list):
-            return self._new_instance([ self.item_type.from_json(item, models=models) for item in json ])
-        else:
-            raise DeserializationError(f"{self} expected a list, got {json}")
-
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, True)
 
         if self._is_seq(value) and all(self.item_type.is_valid(item) for item in value):
@@ -102,31 +106,27 @@ class Seq(ContainerProperty):
         raise ValueError(msg)
 
     @classmethod
-    def _is_seq(cls, value):
-        return ((isinstance(value, Sequence) or cls._is_seq_like(value)) and
-                not isinstance(value, str))
+    def _is_seq(cls, value: Any) -> bool:
+        return ((isinstance(value, Sequence) or cls._is_seq_like(value)) and not isinstance(value, str))
 
     @classmethod
-    def _is_seq_like(cls, value):
+    def _is_seq_like(cls, value: Any) -> bool:
         return (isinstance(value, (Container, Sized, Iterable))
                 and hasattr(value, "__getitem__") # NOTE: this is what makes it disallow set type
                 and not isinstance(value, Mapping))
 
-    def _new_instance(self, value):
-        return value
-
-class List(Seq):
+class List(Seq[T]):
     """ Accept Python list values.
 
     """
 
-    def __init__(self, item_type, default=[], help=None) -> None:
-        # todo: refactor to not use mutable objects as default values.
+    def __init__(self, item_type: TypeOrInst[Property[T]], *, default: Init[T] = [], help: str | None = None) -> None:
+        # TODO: refactor to not use mutable objects as default values.
         # Left in place for now because we want to allow None to express
         # optional values. Also in Dict.
         super().__init__(item_type, default=default, help=help)
 
-    def wrap(self, value):
+    def wrap(self, value: list[T]) -> PropertyValueList[T]:
         """ Some property types need to wrap their values in special containers, etc.
 
         """
@@ -139,25 +139,45 @@ class List(Seq):
             return value
 
     @classmethod
-    def _is_seq(cls, value):
+    def _is_seq(cls, value: Any):
         return isinstance(value, list)
 
-class Array(Seq):
+class Set(Seq[T]):
+    """ Accept Python ``set()`` values.
+
+    """
+
+    def __init__(self, item_type: TypeOrInst[Property[T]], *, default: Init[T] = set(), help: str | None = None) -> None:
+        # TODO: refactor to not use mutable objects as default values.
+        # Left in place for now because we want to allow None to express
+        # optional values. Also in Dict.
+        super().__init__(item_type, default=default, help=help)
+
+    def wrap(self, value: set[T]) -> PropertyValueSet[T]:
+        """ Some property types need to wrap their values in special containers, etc. """
+        if isinstance(value, set):
+            if isinstance(value, PropertyValueSet):
+                return value
+            else:
+                return PropertyValueSet(value)
+        else:
+            return value
+
+    @classmethod
+    def _is_seq(cls, value: Any) -> bool:
+        return isinstance(value, set)
+
+class Array(Seq[T]):
     """ Accept NumPy array values.
 
     """
 
     @classmethod
-    def _is_seq(cls, value):
+    def _is_seq(cls, value: Any) -> bool:
         import numpy as np
         return isinstance(value, np.ndarray)
 
-    def _new_instance(self, value):
-        import numpy as np
-        return np.array(value)
-
-
-class Dict(ContainerProperty):
+class Dict(ContainerProperty[Any]):
     """ Accept Python dict values.
 
     If a default value is passed in, then a shallow copy of it will be
@@ -165,25 +185,19 @@ class Dict(ContainerProperty):
 
     """
 
-    def __init__(self, keys_type, values_type, default={}, help=None) -> None:
-        self.keys_type = self._validate_type_param(keys_type)
-        self.values_type = self._validate_type_param(values_type)
-        super().__init__(default=default, help=help)
-
-    def __str__(self) -> str:
-        return f"{self.__class__.__name__}({self.keys_type}, {self.values_type})"
+    def __init__(self, keys_type: TypeOrInst[Property[Any]], values_type: TypeOrInst[Property[Any]], *,
+            default: Init[T] = {}, help: str | None = None) -> None:
+        super().__init__(keys_type, values_type, default=default, help=help)
 
     @property
-    def type_params(self):
-        return [self.keys_type, self.values_type]
+    def keys_type(self):
+        return self.type_params[0]
 
-    def from_json(self, json, *, models=None):
-        if isinstance(json, dict):
-            return { self.keys_type.from_json(key, models=models): self.values_type.from_json(value, models=models) for key, value in json.items() }
-        else:
-            raise DeserializationError(f"{self} expected a dict, got {json}")
+    @property
+    def values_type(self):
+        return self.type_params[1]
 
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
         key_is_valid = self.keys_type.is_valid
@@ -231,41 +245,14 @@ class ColumnData(Dict):
         """
         return [ ColumnDataPropertyDescriptor(base_name, self) ]
 
-
-    def from_json(self, json, *, models=None):
-        """ Decodes column source data encoded as lists or base64 strings.
-        """
-        if not isinstance(json, dict):
-            raise DeserializationError(f"{self} expected a dict, got {json}")
-        new_data = {}
-        for key, value in json.items():
-            key = self.keys_type.from_json(key, models=models)
-            if isinstance(value, dict) and '__ndarray__' in value:
-                new_data[key] = decode_base64_dict(value)
-            elif isinstance(value, list) and any(isinstance(el, dict) and '__ndarray__' in el for el in value):
-                new_list = []
-                for el in value:
-                    if isinstance(el, dict) and '__ndarray__' in el:
-                        el = decode_base64_dict(el)
-                    elif isinstance(el, list):
-                        el = self.values_type.from_json(el)
-                    new_list.append(el)
-                new_data[key] = new_list
-            else:
-                new_data[key] = self.values_type.from_json(value, models=models)
-        return new_data
-
     def _hinted_value(self, value: Any, hint: DocumentPatchedEvent | None) -> Any:
         from ...document.events import ColumnDataChangedEvent, ColumnsStreamedEvent
         if isinstance(hint, ColumnDataChangedEvent):
-            return { col: hint.column_source.data[col] for col in hint.cols }
+            return { col: hint.model.data[col] for col in hint.cols }
         if isinstance(hint, ColumnsStreamedEvent):
             return hint.data
         return value
 
-    def serialize_value(self, value):
-        return transform_column_source_data(value)
-
     def wrap(self, value):
         """ Some property types need to wrap their values in special containers, etc.
 
@@ -282,25 +269,11 @@ class Tuple(ContainerProperty):
     """ Accept Python tuple values.
 
     """
-    def __init__(self, tp1, tp2, *type_params, **kwargs) -> None:
-        self._type_params = list(map(self._validate_type_param, (tp1, tp2) + type_params))
-        super().__init__(default=kwargs.get("default", Undefined), help=kwargs.get("help"))
 
-    def __str__(self) -> str:
-        item_types = ", ".join(str(x) for x in self.type_params)
-        return f"{self.__class__.__name__}({item_types})"
+    def __init__(self, *type_params: TypeOrInst[Property[Any]], default: Init[T] = Undefined, help: str | None = None) -> None:
+        super().__init__(*type_params, default=default, help=help)
 
-    @property
-    def type_params(self):
-        return self._type_params
-
-    def from_json(self, json, *, models=None):
-        if isinstance(json, list):
-            return tuple(type_param.from_json(item, models=models) for type_param, item in zip(self.type_params, json))
-        else:
-            raise DeserializationError(f"{self} expected a list, got {json}")
-
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
         if isinstance(value, (tuple, list)) and len(self.type_params) == len(value):
@@ -316,18 +289,12 @@ class Tuple(ContainerProperty):
         """
         return tuple(typ.transform(x) for (typ, x) in zip(self.type_params, value))
 
-    def serialize_value(self, value):
-        """ Change the value into a JSON serializable format.
-
-        """
-        return tuple(typ.serialize_value(x) for (typ, x) in zip(self.type_params, value))
-
 class RelativeDelta(Dict):
     """ Accept RelativeDelta dicts for time delta values.
 
     """
 
-    def __init__(self, default={}, help=None) -> None:
+    def __init__(self, default={}, *, help: str | None = None) -> None:
         keys = Enum("years", "months", "days", "hours", "minutes", "seconds", "microseconds")
         values = Int
         super().__init__(keys, values, default=default, help=help)
@@ -340,11 +307,11 @@ class RestrictedDict(Dict):
 
     """
 
-    def __init__(self, keys_type, values_type, disallow, default={}, help=None) -> None:
+    def __init__(self, keys_type, values_type, disallow, default={}, *, help: str | None = None) -> None:
         self._disallow = set(disallow)
         super().__init__(keys_type=keys_type, values_type=values_type, default=default, help=help)
 
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
         error_keys = self._disallow & value.keys()
@@ -353,6 +320,22 @@ class RestrictedDict(Dict):
             msg = "" if not detail else f"Disallowed keys: {error_keys!r}"
             raise ValueError(msg)
 
+TSeq = TypeVar("TSeq", bound=Seq[Any])
+
+class NonEmpty(SingleParameterizedProperty[TSeq]):
+    """ Allows only non-empty containers. """
+
+    def __init__(self, type_param: TypeOrInst[TSeq], *, default: Init[TSeq] = Intrinsic,
+            help: str | None = None) -> None:
+        super().__init__(type_param, default=default, help=help)
+
+    def validate(self, value: Any, detail: bool = True) -> None:
+        super().validate(value, detail)
+
+        if not value:
+            msg = "" if not detail else "Expected a non-empty container"
+            raise ValueError(msg)
+
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
@@ -366,14 +349,14 @@ class RestrictedDict(Dict):
 #-----------------------------------------------------------------------------
 
 @register_type_link(Dict)
-def _sphinx_type_dict(obj):
+def _sphinx_type_dict(obj: Dict):
     return f"{property_link(obj)}({type_link(obj.keys_type)}, {type_link(obj.values_type)})"
 
 @register_type_link(Seq)
-def _sphinx_type_seq(obj):
+def _sphinx_type_seq(obj: Seq[Any]):
     return f"{property_link(obj)}({type_link(obj.item_type)})"
 
 @register_type_link(Tuple)
-def _sphinx_type_tuple(obj):
+def _sphinx_type_tuple(obj: Tuple):
     item_types = ", ".join(type_link(x) for x in obj.type_params)
     return f"{property_link(obj)}({item_types})"
