@@ -21,24 +21,20 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from typing import TYPE_CHECKING
-
-# External imports
-from typing_extensions import TypedDict
+from typing import TYPE_CHECKING, Any
 
 # Bokeh imports
-from ... import colors
-from ...util.deprecation import deprecated
+from ...util.dataclasses import Unspecified
 from ...util.serialization import convert_datetime_type, convert_timedelta_type
-from ...util.string import nice_join
+from ...util.strings import nice_join
 from .. import enums
 from .color import Color
-from .container import Dict, List
 from .datetime import Datetime, TimeDelta
 from .descriptors import DataSpecPropertyDescriptor, UnitsSpecPropertyDescriptor
 from .either import Either
 from .enum import Enum
 from .instance import Instance
+from .nothing import Nothing
 from .nullable import Nullable
 from .primitive import (
     Float,
@@ -46,7 +42,15 @@ from .primitive import (
     Null,
     String,
 )
+from .serialized import NotSerialized
 from .singletons import Undefined
+from .struct import Optional, Struct
+from .vectorization import (
+    Expr,
+    Field,
+    Value,
+    Vectorized,
+)
 from .visual import (
     DashPattern,
     FontSize,
@@ -55,9 +59,7 @@ from .visual import (
 )
 
 if TYPE_CHECKING:
-    from ...core.types import Unknown
-    from ...models.expressions import Expression
-    from ...models.transforms import Transform
+    from ...core.has_props import HasProps
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -70,8 +72,6 @@ __all__ = (
     'DashPatternSpec',
     'DataSpec',
     'DistanceSpec',
-    'expr',
-    'field',
     'FontSizeSpec',
     'FontStyleSpec',
     'HatchPatternSpec',
@@ -84,17 +84,8 @@ __all__ = (
     'StringSpec',
     'TextAlignSpec',
     'TextBaselineSpec',
-    'value',
 )
 
-#-----------------------------------------------------------------------------
-# Private API
-#-----------------------------------------------------------------------------
-
-_ExprFieldValueTransform = Enum("expr", "field", "value", "transform")
-
-_ExprFieldValueTransformUnits = Enum("expr", "field", "value", "transform", "units")
-
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -183,26 +174,44 @@ class DataSpec(Either):
             color = ColorSpec(help="docs for color") # defaults to None
 
     """
-    def __init__(self, key_type, value_type, default, help=None) -> None:
+
+    def __init__(self, value_type, default, *, help: str | None = None) -> None:
         super().__init__(
             String,
-            Dict(
-                key_type,
-                Either(
-                    String,
-                    Instance('bokeh.models.transforms.Transform'),
-                    Instance('bokeh.models.expressions.Expression'),
-                    value_type)),
             value_type,
+            Instance(Value),
+            Instance(Field),
+            Instance(Expr),
+            Struct(
+                value=value_type,
+                transform=Optional(Instance("bokeh.models.transforms.Transform")),
+            ),
+            Struct(
+                field=String,
+                transform=Optional(Instance("bokeh.models.transforms.Transform")),
+            ),
+            Struct(
+                expr=Instance("bokeh.models.expressions.Expression"),
+                transform=Optional(Instance("bokeh.models.transforms.Transform")),
+            ),
             default=default,
             help=help
         )
-        self._type = self._validate_type_param(value_type)
-        self.accepts(Instance("bokeh.models.expressions.Expression"), lambda obj: expr(obj))
+        self.value_type = self._validate_type_param(value_type)
+        self.accepts(Instance("bokeh.models.expressions.Expression"), lambda obj: Expr(obj))
 
-    # TODO (bev) add stricter validation on keys
+    def transform(self, value: Any):
+        if isinstance(value, dict):
+            if "value" in value:
+                return Value(**value)
+            if "field" in value:
+                return Field(**value)
+            if "expr" in value:
+                return Expr(**value)
 
-    def make_descriptors(self, base_name):
+        return super().transform(value)
+
+    def make_descriptors(self, base_name: str):
         """ Return a list of ``DataSpecPropertyDescriptor`` instances to
         install on a class, in order to delegate attribute access to this
         property.
@@ -218,24 +227,23 @@ class DataSpec(Either):
         """
         return [ DataSpecPropertyDescriptor(base_name, self) ]
 
-    def to_serializable(self, obj, name, val):
+    def to_serializable(self, obj: HasProps, name: str, val: Any) -> Vectorized:
         # Check for spec type value
         try:
-            self._type.validate(val, False)
-            return dict(value=val)
+            self.value_type.replace(String, Nothing()).validate(val, False)
+            return Value(val)
         except ValueError:
             pass
 
         # Check for data source field name
         if isinstance(val, str):
-            return dict(field=val)
+            return Field(val)
 
-        # Must be dict, return a new dict
-        return dict(val)
+        return val
 
 class IntSpec(DataSpec):
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, Int, default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(Int, default=default, help=help)
 
 class NumberSpec(DataSpec):
     """ A |DataSpec| property that accepts numeric and datetime fixed values.
@@ -257,8 +265,9 @@ class NumberSpec(DataSpec):
         m.location = "foo" # field
 
     """
-    def __init__(self, default=Undefined, help=None, key_type=_ExprFieldValueTransform, accept_datetime=True, accept_timedelta=True) -> None:
-        super().__init__(key_type, Float, default=default, help=help)
+
+    def __init__(self, default=Undefined, *, help: str | None = None, accept_datetime=True, accept_timedelta=True) -> None:
+        super().__init__(Float, default=default, help=help)
         if accept_timedelta:
             self.accepts(TimeDelta, convert_timedelta_type)
         if accept_datetime:
@@ -270,13 +279,13 @@ class AlphaSpec(NumberSpec):
     Acceptable values are numbers in 0..1 range (transparent..opaque).
     """
 
-    def __init__(self, default=1.0, help=None) -> None:
+    def __init__(self, default=1.0, *, help: str | None = None) -> None:
         help = f"{help or ''}\n{self._default_help}"
-        super().__init__(default=default, help=help, key_type=_ExprFieldValueTransform, accept_datetime=False, accept_timedelta=False)
+        super().__init__(default=default, help=help, accept_datetime=False, accept_timedelta=False)
 
 class NullStringSpec(DataSpec):
-    def __init__(self, default=None, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, Nullable(List(String)), default=default, help=help)
+    def __init__(self, default=None, *, help: str | None = None) -> None:
+        super().__init__(Nullable(String), default=default, help=help)
 
 class StringSpec(DataSpec):
     """ A |DataSpec| property that accepts string fixed values.
@@ -293,15 +302,8 @@ class StringSpec(DataSpec):
         m.title = "foo"        # field
 
     """
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, List(String), default=default, help=help)
-
-    def prepare_value(self, cls, name, value):
-        if isinstance(value, list):
-            if len(value) != 1:
-                raise TypeError("StringSpec convenience list values must have length 1")
-            value = dict(value=value[0])
-        return super().prepare_value(cls, name, value)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(String, default=default, help=help)
 
 class FontSizeSpec(DataSpec):
     """ A |DataSpec| property that accepts font-size fixed values.
@@ -324,10 +326,10 @@ class FontSizeSpec(DataSpec):
 
     """
 
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, FontSize, default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(FontSize, default=default, help=help)
 
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         # We want to preserve existing semantics and be a little more restrictive. This
         # validations makes m.font_size = "" or m.font_size = "6" an error
         super().validate(value, detail)
@@ -338,28 +340,28 @@ class FontSizeSpec(DataSpec):
                 raise ValueError(msg)
 
 class FontStyleSpec(DataSpec):
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, Enum(enums.FontStyle), default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(Enum(enums.FontStyle), default=default, help=help)
 
 class TextAlignSpec(DataSpec):
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, Enum(enums.TextAlign), default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(Enum(enums.TextAlign), default=default, help=help)
 
 class TextBaselineSpec(DataSpec):
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, Enum(enums.TextBaseline), default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(Enum(enums.TextBaseline), default=default, help=help)
 
 class LineJoinSpec(DataSpec):
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, Enum(enums.LineJoin), default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(Enum(enums.LineJoin), default=default, help=help)
 
 class LineCapSpec(DataSpec):
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, Enum(enums.LineCap), default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(Enum(enums.LineCap), default=default, help=help)
 
 class DashPatternSpec(DataSpec):
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, DashPattern, default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(DashPattern, default=default, help=help)
 
 class HatchPatternSpec(DataSpec):
     """ A |DataSpec| property that accepts hatch pattern types as fixed values.
@@ -378,8 +380,8 @@ class HatchPatternSpec(DataSpec):
 
     """
 
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, Nullable(HatchPatternType), default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(Nullable(HatchPatternType), default=default, help=help)
 
 class MarkerSpec(DataSpec):
     """ A |DataSpec| property that accepts marker types as fixed values.
@@ -398,8 +400,8 @@ class MarkerSpec(DataSpec):
 
     """
 
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
-        super().__init__(key_type, MarkerType, default=default, help=help)
+    def __init__(self, default, *, help: str | None = None) -> None:
+        super().__init__(MarkerType, default=default, help=help)
 
 class UnitsSpec(NumberSpec):
     """ A |DataSpec| property that accepts numeric fixed values, and also
@@ -407,25 +409,44 @@ class UnitsSpec(NumberSpec):
 
     """
 
-    def __init__(self, default, units_enum, units_default, help=None) -> None:
-        super().__init__(default=default, help=help, key_type=_ExprFieldValueTransformUnits)
-        units_type = Enum(units_enum, default=units_default, serialized=False, help=f"""
+    def __init__(self, default, units_enum, units_default, *, help: str | None = None) -> None:
+        super().__init__(default=default, help=help)
+
+        units_type = NotSerialized(Enum(units_enum), default=units_default, help=f"""
         Units to use for the associated property: {nice_join(units_enum)}
         """)
         self._units_type = self._validate_type_param(units_type, help_allowed=True)
 
+        self._type_params += [
+            Struct(
+                value=self.value_type,
+                transform=Optional(Instance("bokeh.models.transforms.Transform")),
+                units=Optional(units_type),
+            ),
+            Struct(
+                field=String,
+                transform=Optional(Instance("bokeh.models.transforms.Transform")),
+                units=Optional(units_type),
+            ),
+            Struct(
+                expr=Instance("bokeh.models.expressions.Expression"),
+                transform=Optional(Instance("bokeh.models.transforms.Transform")),
+                units=Optional(units_type),
+            ),
+        ]
+
     def __str__(self) -> str:
         units_default = self._units_type._default
         return f"{self.__class__.__name__}(units_default={units_default!r})"
 
-    def get_units(self, obj, name):
-        return getattr(obj, name+"_units")
+    def get_units(self, obj: HasProps, name: str) -> str:
+        return getattr(obj, name + "_units")
 
-    def make_descriptors(self, base_name):
+    def make_descriptors(self, base_name: str):
         """ Return a list of ``PropertyDescriptor`` instances to install on a
         class, in order to delegate attribute access to this property.
 
-        Unlike simpler property types, ``PropertyUnitsSpec`` returns multiple
+        Unlike simpler property types, ``UnitsSpec`` returns multiple
         descriptors to install. In particular, descriptors for the base
         property as well as the associated units property are returned.
 
@@ -442,39 +463,32 @@ class UnitsSpec(NumberSpec):
         units_props = self._units_type.make_descriptors(units_name)
         return units_props + [ UnitsSpecPropertyDescriptor(base_name, self, units_props[0]) ]
 
-    def to_serializable(self, obj, name, val):
-        d = super().to_serializable(obj, name, val)
-        if d is not None and 'units' not in d:
-            # d is a PropertyValueDict at this point, we need to convert it to
-            # a plain dict if we are going to modify its value, otherwise a
-            # notify_change that should not happen will be triggered
+    def to_serializable(self, obj: HasProps, name: str, val: Any) -> Vectorized:
+        val = super().to_serializable(obj, name, val)
+        if val.units is Unspecified:
             units = self.get_units(obj, name)
             if units != self._units_type._default:
-                d = dict(**d, units=units)
-        return d
+                val.units = units # XXX: clone and update?
+        return val
 
-# Deprecated
-class PropertyUnitsSpec(UnitsSpec):
-    pass
-
-class AngleSpec(PropertyUnitsSpec):
+class AngleSpec(UnitsSpec):
     """ A |DataSpec| property that accepts numeric fixed values, and also
     provides an associated units property to store angle units.
 
     Acceptable values for units are ``"deg"``, ``"rad"``, ``"grad"`` and ``"turn"``.
 
     """
-    def __init__(self, default=Undefined, units_default="rad", help=None) -> None:
+    def __init__(self, default=Undefined, units_default="rad", *, help: str | None = None) -> None:
         super().__init__(default=default, units_enum=enums.AngleUnits, units_default=units_default, help=help)
 
-class DistanceSpec(PropertyUnitsSpec):
+class DistanceSpec(UnitsSpec):
     """ A |DataSpec| property that accepts numeric fixed values or strings
     that refer to columns in a :class:`~bokeh.models.sources.ColumnDataSource`,
     and also provides an associated units property to store units information.
     Acceptable values for units are ``"screen"`` and ``"data"``.
 
     """
-    def __init__(self, default=Undefined, units_default="data", help=None) -> None:
+    def __init__(self, default=Undefined, units_default="data", *, help: str | None = None) -> None:
         super().__init__(default=default, units_enum=enums.SpatialUnits, units_default=units_default, help=help)
 
     def prepare_value(self, cls, name, value):
@@ -487,9 +501,9 @@ class DistanceSpec(PropertyUnitsSpec):
 
 class NullDistanceSpec(DistanceSpec):
 
-    def __init__(self, default=None, units_default="data", help=None) -> None:
+    def __init__(self, default=None, units_default="data", *, help: str | None = None) -> None:
         super().__init__(default=default, units_default=units_default, help=help)
-        self._type = Nullable(self._type)
+        self.value_type = Nullable(Float)
         self._type_params = [Null()] + self._type_params
 
     def prepare_value(self, cls, name, value):
@@ -552,9 +566,9 @@ class ColorSpec(DataSpec):
 
     """
 
-    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform) -> None:
+    def __init__(self, default, *, help: str | None = None) -> None:
         help = f"{help or ''}\n{self._default_help}"
-        super().__init__(key_type, Nullable(Color), default=default, help=help)
+        super().__init__(Nullable(Color), default=default, help=help)
 
     @classmethod
     def isconst(cls, val):
@@ -587,32 +601,6 @@ class ColorSpec(DataSpec):
         """
         return isinstance(val, tuple) and len(val) in (3, 4) and all(isinstance(v, (float, int)) for v in val)
 
-    def to_serializable(self, obj, name, val):
-        if val is None:
-            return dict(value=None)
-
-        # Check for hexadecimal or named color
-        if self.isconst(val):
-            return dict(value=val)
-
-        # Check for RGB or RGBA tuple
-        if isinstance(val, tuple):
-            return dict(value=colors.RGB(*val).to_css())
-
-        # Check for data source field name
-        if isinstance(val, colors.RGB):
-            return val.to_css()
-
-        # Check for data source field name or rgb(a) string
-        if isinstance(val, str):
-            if val.startswith(("rgb(", "rgba(")):
-                return val
-
-            return dict(field=val)
-
-        # Must be dict, return new dict
-        return dict(val)
-
     def prepare_value(self, cls, name, value):
         # Some explanation is in order. We want to accept tuples like
         # (12.0, 100.0, 52.0) i.e. that have "float" byte values. The
@@ -626,107 +614,13 @@ class ColorSpec(DataSpec):
             value = tuple(int(v) if i < 3 else v for i, v in enumerate(value))
         return super().prepare_value(cls, name, value)
 
-# DataSpec helpers ------------------------------------------------------------
-
-class Expr(TypedDict, total=False):
-    expr: Expression
-    transform: Transform | None
-
-class Field(TypedDict, total=False):
-    field: str
-    transform: Transform | None
-
-class Value(TypedDict, total=False):
-    value: Unknown
-    transform: Transform | None
-
-def expr(expression: Expression, transform: Transform | None = None) -> Expr:
-    """ Convenience function to explicitly return an "expr" specification for
-    a Bokeh |DataSpec| property.
-
-    Args:
-        expression (Expression) : a computed expression for a
-            ``DataSpec`` property.
-
-        transform (Transform, optional) : a transform to apply (default: None)
-
-    Returns:
-        dict : ``{ "expr": expression }``
-
-    .. note::
-        This function is included for completeness. String values for
-        property specifications are by default interpreted as field names.
-
-    """
-    if transform:
-        return Expr(expr=expression, transform=transform)
-    return Expr(expr=expression)
-
-def field(name: str, transform: Transform | None = None) -> Field:
-    """ Convenience function to explicitly return a "field" specification for
-    a Bokeh |DataSpec| property.
-
-    Args:
-        name (str) : name of a data source field to reference for a
-            ``DataSpec`` property.
-
-        transform (Transform, optional) : a transform to apply (default: None)
-
-    Returns:
-        dict : ``{ "field": name }``
-
-    .. note::
-        This function is included for completeness. String values for
-        property specifications are by default interpreted as field names.
-
-    """
-    if transform:
-        return Field(field=name, transform=transform)
-    return Field(field=name)
-
-def value(val: Unknown, transform: Transform | None = None) -> Value:
-    """ Convenience function to explicitly return a "value" specification for
-    a Bokeh |DataSpec| property.
-
-    Args:
-        val (any) : a fixed value to specify for a ``DataSpec`` property.
-
-        transform (Transform, optional) : a transform to apply (default: None)
-
-    Returns:
-        dict : ``{ "value": name }``
-
-    .. note::
-        String values for property specifications are by default interpreted
-        as field names. This function is especially useful when you want to
-        specify a fixed value with text properties.
-
-    Example:
-
-        .. code-block:: python
-
-            # The following will take text values to render from a data source
-            # column "text_column", but use a fixed value "16px" for font size
-            p.text("x", "y", text="text_column",
-                   text_font_size=value("16px"), source=source)
-
-    """
-    if transform:
-        return Value(value=val, transform=transform)
-    return Value(value=val)
-
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
 
-def DataDistanceSpec(*args, **kw):
-    deprecated((2, 4, 0), "DataDistanceSpec()", "SizeSpec()")
-    return SizeSpec(*args, **kw)
-
-def ScreenDistanceSpec(*args, **kw):
-    deprecated((2, 4, 0), "DataDistanceSpec()", "SizeSpec()")
-    return SizeSpec(*args, **kw)
-
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
 
 #-----------------------------------------------------------------------------
 # Code
