@@ -9,6 +9,7 @@
 The Python `descriptor protocol`_ allows fine-grained control over all
 attribute access on instances ("You control the dot"). Bokeh uses the
 descriptor protocol to provide easy-to-use, declarative, type-based
+
 class properties that can automatically validate and serialize their
 values, as well as help provide sophisticated documentation.
 
@@ -63,7 +64,7 @@ There are many other examples like this throughout Bokeh. In this way users
 may operate simply and naturally, and not be concerned with the low-level
 details around validation, serialization, and documentation.
 
-This module provides the class |PropertyDescriptor| and various subclasses
+This module provides the class ``PropertyDescriptor`` and various subclasses
 that can be used to attach Bokeh properties to Bokeh models.
 
 .. note::
@@ -90,12 +91,12 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from copy import copy
+from types import FunctionType
 from typing import (
     TYPE_CHECKING,
     Any,
-    Dict,
+    Callable,
     Generic,
-    Type,
     TypeVar,
 )
 
@@ -104,9 +105,10 @@ from .singletons import Undefined
 from .wrappers import PropertyValueColumnData, PropertyValueContainer
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeGuard
+
     from ...document.events import DocumentPatchedEvent
     from ..has_props import HasProps, Setter
-    from ..types import ID, Unknown
     from .bases import Property
 
 #-----------------------------------------------------------------------------
@@ -148,7 +150,7 @@ class AliasPropertyDescriptor(Generic[T]):
         self.property = property
         self.__doc__ = f"This is a compatibility alias for the ``{aliased_name}`` property"
 
-    def __get__(self, obj: HasProps | None, owner: Type[HasProps] | None) -> T:
+    def __get__(self, obj: HasProps | None, owner: type[HasProps] | None) -> T:
         if obj is not None:
             return getattr(obj, self.aliased_name)
         elif owner is not None:
@@ -164,6 +166,9 @@ class AliasPropertyDescriptor(Generic[T]):
     def readonly(self) -> bool:
         return self.property.readonly
 
+    def has_unstable_default(self, obj: HasProps) -> bool:
+        return obj.lookup(self.aliased_name).has_unstable_default(obj)
+
 class PropertyDescriptor(Generic[T]):
     """ A base class for Bokeh properties with simple get/set and serialization
     behavior.
@@ -194,7 +199,7 @@ class PropertyDescriptor(Generic[T]):
         """
         return f"{self.property}"
 
-    def __get__(self, obj: HasProps | None, owner: Type[HasProps] | None) -> T:
+    def __get__(self, obj: HasProps | None, owner: type[HasProps] | None) -> T:
         """ Implement the getter for the Python `descriptor protocol`_.
 
         For instance attribute access, we delegate to the |Property|. For
@@ -275,7 +280,7 @@ class PropertyDescriptor(Generic[T]):
             class_name = obj.__class__.__name__
             raise RuntimeError(f"Cannot set a property value {self.name!r} on a {class_name} instance before HasProps.__init__")
 
-        if self.property._readonly and obj._initialized:
+        if self.property.readonly and obj._initialized:
             # Allow to set a value during object initialization (e.g. value -> value_throttled)
             class_name = obj.__class__.__name__
             raise RuntimeError(f"{class_name}.{self.name} is a readonly property")
@@ -300,19 +305,22 @@ class PropertyDescriptor(Generic[T]):
         if self.name in obj._unstable_default_values:
             del obj._unstable_default_values[self.name]
 
-    def class_default(self, cls):
+    def class_default(self, cls: type[HasProps], *, no_eval: bool = False):
         """ Get the default value for a specific subtype of ``HasProps``,
         which may not be used for an individual instance.
 
         Args:
             cls (class) : The class to get the default value for.
 
+            no_eval (bool, optional) :
+                Whether to evaluate callables for defaults (default: False)
+
         Returns:
             object
 
 
         """
-        return self.property.themed_default(cls, self.name, None)
+        return self.property.themed_default(cls, self.name, None, no_eval=no_eval)
 
     def instance_default(self, obj: HasProps) -> T:
         """ Get the default value that will be used for a specific instance.
@@ -326,8 +334,8 @@ class PropertyDescriptor(Generic[T]):
         """
         return self.property.themed_default(obj.__class__, self.name, obj.themed_values())
 
-    def serializable_value(self, obj: HasProps) -> Any:
-        """ Produce the value as it should be serialized.
+    def get_value(self, obj: HasProps) -> Any:
+        """ Produce the value used for serialization.
 
         Sometimes it is desirable for the serialized value to differ from
         the ``__get__`` in order for the ``__get__`` value to appear simpler
@@ -337,14 +345,12 @@ class PropertyDescriptor(Generic[T]):
             obj (HasProps) : the object to get the serialized attribute for
 
         Returns:
-            JSON-like
+            Any
 
         """
-        value = self.__get__(obj, obj.__class__)
-        return self.property.serialize_value(value)
+        return self.__get__(obj, obj.__class__)
 
-    def set_from_json(self, obj: HasProps, json: Any, *,
-            models: Dict[ID, HasProps] | None = None, setter: Setter | None = None):
+    def set_from_json(self, obj: HasProps, value: Any, *, setter: Setter | None = None):
         """Sets the value of this property from a JSON value.
 
         Args:
@@ -373,11 +379,11 @@ class PropertyDescriptor(Generic[T]):
             None
 
         """
-        value = self.property.prepare_value(obj, self.name, self.property.from_json(json, models=models))
+        value = self.property.prepare_value(obj, self.name, value)
         old = self._get(obj)
         self._set(obj, old, value, setter=setter)
 
-    def trigger_if_changed(self, obj: HasProps, old: Unknown) -> None:
+    def trigger_if_changed(self, obj: HasProps, old: Any) -> None:
         """ Send a change event notification if the property is set to a
         value is not equal to ``old``.
 
@@ -428,6 +434,16 @@ class PropertyDescriptor(Generic[T]):
         """
         return self.property.serialized
 
+    def has_unstable_default(self, obj: HasProps) -> bool:
+        # _may_have_unstable_default() doesn't have access to overrides, so check manually
+        return self.property._may_have_unstable_default() or \
+            self.is_unstable(obj.__overridden_defaults__.get(self.name, None))
+
+    @classmethod
+    def is_unstable(cls, value: Any) -> TypeGuard[Callable[[], Any]]:
+        from .instance import InstanceDefault
+        return isinstance(value, (FunctionType, InstanceDefault))
+
     def _get(self, obj: HasProps) -> T:
         """ Internal implementation of instance attribute access for the
         ``PropertyDescriptor`` getter.
@@ -460,7 +476,7 @@ class PropertyDescriptor(Generic[T]):
         """ Internal implementation of instance attribute access for default
         values.
 
-        Handles bookeeping around |PropertyContainer| value, etc.
+        Handles bookeeping around ``PropertyContainer`` value, etc.
 
         """
         if self.name in obj._property_values:
@@ -477,14 +493,14 @@ class PropertyDescriptor(Generic[T]):
         if self.name in unstable_dict:
             return unstable_dict[self.name]
 
-        if self.property._may_have_unstable_default():
+        if self.has_unstable_default(obj):
             if isinstance(default, PropertyValueContainer):
                 default._register_owner(obj, self)
             unstable_dict[self.name] = default
 
         return default
 
-    def _set_value(self, obj: HasProps, value: Unknown) -> None:
+    def _set_value(self, obj: HasProps, value: Any) -> None:
         """ Actual descriptor value assignment. """
         if isinstance(value, PropertyValueContainer):
             value._register_owner(obj, self)
@@ -497,7 +513,7 @@ class PropertyDescriptor(Generic[T]):
 
         obj._property_values[self.name] = value
 
-    def _set(self, obj: HasProps, old: Unknown, value: Unknown, *,
+    def _set(self, obj: HasProps, old: Any, value: Any, *,
             hint: DocumentPatchedEvent | None = None, setter: Setter | None = None) -> None:
         """ Internal implementation helper to set property values.
 
@@ -560,9 +576,9 @@ class PropertyDescriptor(Generic[T]):
 
     # called when a container is mutated "behind our back" and
     # we detect it with our collection wrappers.
-    def _notify_mutated(self, obj: HasProps, old: Unknown, hint: DocumentPatchedEvent | None = None) -> None:
+    def _notify_mutated(self, obj: HasProps, old: Any, hint: DocumentPatchedEvent | None = None) -> None:
         """ A method to call when a container is mutated "behind our back"
-        and we detect it with our |PropertyContainer| wrappers.
+        and we detect it with our ``PropertyContainer`` wrappers.
 
         Args:
             obj (HasProps) :
@@ -595,7 +611,7 @@ class PropertyDescriptor(Generic[T]):
 
         self._set(obj, old, value, hint=hint)
 
-    def _trigger(self, obj: HasProps, old: Unknown, value: Unknown, *,
+    def _trigger(self, obj: HasProps, old: Any, value: Any, *,
             hint: DocumentPatchedEvent | None = None, setter: Setter | None = None) -> None:
         """ Unconditionally send a change event notification for the property.
 
@@ -690,7 +706,7 @@ class ColumnDataPropertyDescriptor(PropertyDescriptor):
             class_name = obj.__class__.__name__
             raise RuntimeError(f"Cannot set a property value {self.name!r} on a {class_name} instance before HasProps.__init__")
 
-        if self.property._readonly and obj._initialized:
+        if self.property.readonly and obj._initialized:
             # Allow to set a value during object initialization (e.g. value -> value_throttled)
             class_name = obj.__class__.__name__
             raise RuntimeError(f"{class_name}.{self.name} is a readonly property")
@@ -699,8 +715,7 @@ class ColumnDataPropertyDescriptor(PropertyDescriptor):
             raise ValueError(_CDS_SET_FROM_CDS_ERROR)
 
         from ...document.events import ColumnDataChangedEvent
-
-        hint = ColumnDataChangedEvent(obj.document, obj, setter=setter) if obj.document else None
+        hint = ColumnDataChangedEvent(obj.document, obj, "data", setter=setter) if obj.document else None
 
         value = self.property.prepare_value(obj, self.name, value)
         old = self._get(obj)
@@ -712,13 +727,13 @@ class DataSpecPropertyDescriptor(PropertyDescriptor):
 
     """
 
-    def serializable_value(self, obj):
+    def get_value(self, obj: HasProps) -> Any:
         """
 
         """
         return self.property.to_serializable(obj, self.name, getattr(obj, self.name))
 
-    def set_from_json(self, obj, json, *, models=None, setter=None):
+    def set_from_json(self, obj: HasProps, value: Any, *, setter: Setter | None = None):
         """ Sets the value of this property from a JSON value.
 
         This method first
@@ -745,24 +760,24 @@ class DataSpecPropertyDescriptor(PropertyDescriptor):
             None
 
         """
-        if isinstance(json, dict):
+        if isinstance(value, dict):
             # we want to try to keep the "format" of the data spec as string, dict, or number,
             # assuming the serialized dict is compatible with that.
             old = getattr(obj, self.name)
             if old is not None:
                 try:
-                    self.property._type.validate(old, False)
-                    if 'value' in json:
-                        json = json['value']
+                    self.property.value_type.validate(old, False)
+                    if 'value' in value:
+                        value = value['value']
                 except ValueError:
-                    if isinstance(old, str) and 'field' in json:
-                        json = json['field']
+                    if isinstance(old, str) and 'field' in value:
+                        value = value['field']
                 # leave it as a dict if 'old' was a dict
 
-        super().set_from_json(obj, json, models=models, setter=setter)
+        super().set_from_json(obj, value, setter=setter)
 
 class UnitsSpecPropertyDescriptor(DataSpecPropertyDescriptor):
-    """ A ``PropertyDescriptor`` for Bokeh |PropertyUnitsSpec| properties that
+    """ A ``PropertyDescriptor`` for Bokeh ``UnitsSpec`` properties that
     contribute associated ``_units`` properties automatically as a side effect.
 
     """
@@ -857,11 +872,11 @@ class UnitsSpecPropertyDescriptor(DataSpecPropertyDescriptor):
 
         """
         json = self._extract_units(obj, json)
-        super().set_from_json(obj, json, models=models, setter=setter)
+        super().set_from_json(obj, json, setter=setter)
 
     def _extract_units(self, obj, value):
         """ Internal helper for dealing with units associated units properties
-        when setting values on |PropertyUnitsSpec| properties.
+        when setting values on ``UnitsSpec`` properties.
 
         When ``value`` is a dict, this function may mutate the value of the
         associated units property.
