@@ -23,10 +23,18 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import Any, TypeVar
+
 # Bokeh imports
-from ...util.string import nice_join
+from ...util.strings import nice_join
 from ._sphinx import property_link, register_type_link, type_link
-from .bases import DeserializationError, ParameterizedProperty
+from .bases import (
+    Init,
+    ParameterizedProperty,
+    Property,
+    TypeOrInst,
+)
 from .singletons import Intrinsic
 
 #-----------------------------------------------------------------------------
@@ -37,11 +45,13 @@ __all__ = (
     'Either',
 )
 
+T = TypeVar("T")
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-class Either(ParameterizedProperty):
+class Either(ParameterizedProperty[Any]):
     """ Accept values according to a sequence of other property types.
 
         Example:
@@ -66,32 +76,13 @@ class Either(ParameterizedProperty):
 
     """
 
-    def __init__(self, tp1, tp2, *type_params, default=Intrinsic, help=None, serialized=None, readonly=False) -> None:
-        type_params = list(map(self._validate_type_param, (tp1, tp2) + type_params))
-        default = default if default is not Intrinsic else type_params[0]._raw_default()
-        super().__init__(default=default, help=help, serialized=serialized, readonly=readonly)
-        self._type_params = type_params
-        for tp in self._type_params:
+    def __init__(self, type_param0: TypeOrInst[Property[Any]], *type_params: TypeOrInst[Property[Any]],
+            default: Init[T] = Intrinsic, help: str | None = None) -> None:
+        super().__init__(type_param0, *type_params, default=default, help=help)
+        for tp in self.type_params:
             self.alternatives.extend(tp.alternatives)
 
-    def __str__(self) -> str:
-        class_name = self.__class__.__name__
-        item_types = ", ".join(str(x) for x in self.type_params)
-        return f"{class_name}({item_types})"
-
-    @property
-    def type_params(self):
-        return self._type_params
-
-    def from_json(self, json, *, models=None):
-        for tp in self.type_params:
-            try:
-                return tp.from_json(json, models=models)
-            except DeserializationError:
-                pass
-        raise DeserializationError(f"{self} couldn't deserialize {json}")
-
-    def transform(self, value):
+    def transform(self, value: Any) -> Any:
         for param in self.type_params:
             try:
                 return param.transform(value)
@@ -100,13 +91,13 @@ class Either(ParameterizedProperty):
 
         raise ValueError(f"Could not transform {value!r}")
 
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
         if any(param.is_valid(value) for param in self.type_params):
             return
 
-        msg = "" if not detail else f"expected an element of either {nice_join(self.type_params)}, got {value!r}"
+        msg = "" if not detail else f"expected an element of either {nice_join([ str(param) for param in self.type_params ])}, got {value!r}"
         raise ValueError(msg)
 
     def wrap(self, value):
@@ -114,9 +105,12 @@ class Either(ParameterizedProperty):
             value = tp.wrap(value)
         return value
 
-    # TODO (bev) implement this
-    # def _may_have_unstable_default(self):
-    #     return any(tp._may_have_unstable_default() for tp in self.type_params)
+    def replace(self, old: type[Property[Any]], new: Property[Any]) -> Property[Any]:
+        if self.__class__ == old:
+            return new
+        else:
+            params = [ type_param.replace(old, new) for type_param in self.type_params ]
+            return Either(*params)
 
 #-----------------------------------------------------------------------------
 # Dev API
@@ -131,6 +125,6 @@ class Either(ParameterizedProperty):
 #-----------------------------------------------------------------------------
 
 @register_type_link(Either)
-def _sphinx_type_link(obj):
+def _sphinx_type_link(obj: Either[Any]):
     subtypes = ", ".join(type_link(x) for x in obj.type_params)
     return f"{property_link(obj)}({subtypes})"
