@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -22,10 +22,12 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from copy import copy
+from typing import TypeVar
 
 # Bokeh imports
 from ..has_props import HasProps
 from .descriptor_factory import PropertyDescriptorFactory
+from .descriptors import PropertyDescriptor
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -35,18 +37,20 @@ __all__ = (
     'Include',
 )
 
+T = TypeVar("T")
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-class Include(PropertyDescriptorFactory):
+class Include(PropertyDescriptorFactory[T]):
     """ Include "mix-in" property collection in a Bokeh model.
 
     See :ref:`bokeh.core.property_mixins` for more details.
 
     """
 
-    def __init__(self, delegate, help="", prefix=None) -> None:
+    def __init__(self, delegate: type[HasProps], *, help: str = "", prefix: str | None = None) -> None:
         if not (isinstance(delegate, type) and issubclass(delegate, HasProps)):
             raise ValueError(f"expected a subclass of HasProps, got {delegate!r}")
 
@@ -54,14 +58,13 @@ class Include(PropertyDescriptorFactory):
         self.help = help
         self.prefix = prefix + "_" if prefix else ""
 
-    def make_descriptors(self, _base_name):
+    def make_descriptors(self, _base_name: str) -> list[PropertyDescriptor[T]]:
         descriptors = []
 
-        for prop_name in self.delegate.properties():
-            prop_descriptor = self.delegate.lookup(prop_name)
-            prop = copy(prop_descriptor.property)
-            prop.__doc__ = self.help.format(prop=prop_name.replace('_', ' '))
-            descriptors += prop.make_descriptors(self.prefix + prop_name)
+        for descriptor in self.delegate.descriptors():
+            prop = copy(descriptor.property)
+            prop.__doc__ = self.help.format(prop=descriptor.name.replace("_", " "))
+            descriptors += prop.make_descriptors(self.prefix + descriptor.name)
 
         return descriptors
 
