@@ -20,10 +20,19 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import Any, TypeVar, Union
+
 # Bokeh imports
-from .bases import ParameterizedProperty
+from ...util.deprecation import deprecated
+from .bases import (
+    Init,
+    Property,
+    SingleParameterizedProperty,
+    TypeOrInst,
+)
 from .primitive import Float, Int
-from .singletons import Undefined
+from .singletons import Intrinsic, Undefined
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -33,36 +42,83 @@ __all__ = (
     'Angle',
     'Byte',
     'Interval',
+    'NonNegative',
     'NonNegativeInt',
     'Percent',
+    'Positive',
     'PositiveInt',
     'Size',
 )
 
+T = TypeVar("T", bound=Union[int, float])
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-class NonNegativeInt(Int):
-    """ Accept non-negative integers. """
+class NonNegative(SingleParameterizedProperty[T]):
+    """ A property accepting a value of some other type while having undefined default. """
 
-    def validate(self, value, detail=True):
+    def __init__(self, type_param: TypeOrInst[Property[T]], *, default: Init[T] = Intrinsic, help: str | None = None) -> None:
+        super().__init__(type_param, default=default, help=help)
+
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
-        if value < 0:
+        if not (0 <= value):
+            raise ValueError(f"expected a non-negative number, got {value!r}")
+
+class Positive(SingleParameterizedProperty[T]):
+    """ A property accepting a value of some other type while having undefined default. """
+
+    def __init__(self, type_param: TypeOrInst[Property[T]], *, default: Init[T] = Intrinsic, help: str | None = None) -> None:
+        super().__init__(type_param, default=default, help=help)
+
+    def validate(self, value: Any, detail: bool = True) -> None:
+        super().validate(value, detail)
+
+        if not (0 < value):
+            raise ValueError(f"expected a positive number, got {value!r}")
+
+class NonNegativeInt(Int):
+    """
+    Accept non-negative integers.
+
+    .. deprecated:: 3.0.0
+
+        Use ``NonNegative(Int)`` instead.
+    """
+
+    def __init__(self, default: Init[int] = Intrinsic, *, help: str | None = None) -> None:
+        deprecated((3, 0, 0), "NonNegativeInt", "NonNegative(Int)")
+        super().__init__(default=default, help=help)
+
+    def validate(self, value: Any, detail: bool = True) -> None:
+        super().validate(value, detail)
+
+        if not (0 <= value):
             raise ValueError(f"expected non-negative integer, got {value!r}")
 
 class PositiveInt(Int):
-    """ Accept positive integers. """
+    """
+    Accept positive integers.
 
-    def validate(self, value, detail=True):
+    .. deprecated:: 3.0.0
+
+        Use ``Positive(Int)`` instead.
+    """
+
+    def __init__(self, default: Init[int] = Intrinsic, *, help: str | None = None) -> None:
+        deprecated((3, 0, 0), "Positive", "Positive(Int)")
+        super().__init__(default=default, help=help)
+
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
-        if value <= 0:
+        if not (0 < value):
             raise ValueError(f"expected positive integer, got {value!r}")
 
-
-class Interval(ParameterizedProperty):
+class Interval(SingleParameterizedProperty[T]):
     """ Accept numeric values that are contained within a given interval.
 
     Args:
@@ -100,31 +156,26 @@ class Interval(ParameterizedProperty):
             >>> m.prop = "foo" # ValueError !!
 
     """
-    def __init__(self, interval_type, start, end, default=Undefined, help=None) -> None:
-        self.interval_type = self._validate_type_param(interval_type)
-        # Make up a property name for validation purposes
-        self.interval_type.validate(start)
-        self.interval_type.validate(end)
+    def __init__(self, type_param: TypeOrInst[Property[T]], start: T, end: T, *,
+            default: Init[T] = Undefined, help: str | None = None) -> None:
+        super().__init__(type_param, default=default, help=help)
+        self.type_param.validate(start)
+        self.type_param.validate(end)
         self.start = start
         self.end = end
-        super().__init__(default=default, help=help)
 
     def __str__(self) -> str:
         class_name = self.__class__.__name__
-        return f"{class_name}({self.interval_type}, {self.start!r}, {self.end!r})"
+        return f"{class_name}({self.type_param}, {self.start!r}, {self.end!r})"
 
-    @property
-    def type_params(self):
-        return [self.interval_type]
-
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
-        if not (self.interval_type.is_valid(value) and value >= self.start and value <= self.end):
-            msg = "" if not detail else f"expected a value of type {self.interval_type} in range [{self.start}, {self.end}], got {value!r}"
+        if not (self.type_param.is_valid(value) and value >= self.start and value <= self.end):
+            msg = "" if not detail else f"expected a value of type {self.type_param} in range [{self.start}, {self.end}], got {value!r}"
             raise ValueError(msg)
 
-class Byte(Interval):
+class Byte(Interval[int]):
     """ Accept integral byte values (0-255).
 
     Example:
@@ -144,7 +195,8 @@ class Byte(Interval):
             >>> m.prop = 10.3 # ValueError !!
 
     """
-    def __init__(self, default=0, help=None) -> None:
+
+    def __init__(self, default: Init[int] = 0, help: str | None = None) -> None:
         super().__init__(Int, 0, 255, default=default, help=help)
 
 class Size(Float):
@@ -159,14 +211,6 @@ class Size(Float):
             used by the :ref:`bokeh.sphinxext.bokeh_prop` extension when
             generating Spinx documentation. (default: None)
 
-        serialized (bool, optional) :
-            Whether attributes created from this property should be included
-            in serialization (default: True)
-
-        readonly (bool, optional) :
-            Whether attributes created from this property are read-only.
-            (default: False)
-
     Example:
 
         .. code-block:: python
@@ -186,7 +230,7 @@ class Size(Float):
             >>> m.prop = "foo" # ValueError !!
 
     """
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
         if value < 0:
@@ -208,14 +252,6 @@ class Percent(Float):
             used by the :ref:`bokeh.sphinxext.bokeh_prop` extension when
             generating Spinx documentation. (default: None)
 
-        serialized (bool, optional) :
-            Whether attributes created from this property should be included
-            in serialization (default: True)
-
-        readonly (bool, optional) :
-            Whether attributes created from this property are read-only.
-            (default: False)
-
     Example:
 
         .. code-block:: python
@@ -237,7 +273,8 @@ class Percent(Float):
             >>> m.prop = 5   # ValueError !!
 
     """
-    def validate(self, value, detail=True):
+
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
         if 0.0 <= value <= 1.0:
@@ -261,16 +298,7 @@ class Angle(Float):
             used by the :ref:`bokeh.sphinxext.bokeh_prop` extension when
             generating Spinx documentation. (default: None)
 
-        serialized (bool, optional) :
-            Whether attributes created from this property should be included
-            in serialization (default: True)
-
-        readonly (bool, optional) :
-            Whether attributes created from this property are read-only.
-            (default: False)
-
     """
-    pass
 
 #-----------------------------------------------------------------------------
 # Dev API
