@@ -23,10 +23,11 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-import base64
 import re
+from typing import Any
 
 # Bokeh imports
+from .bases import Init
 from .primitive import String
 from .singletons import Undefined
 
@@ -37,7 +38,6 @@ from .singletons import Undefined
 __all__ = (
     'Regex',
     'MathString',
-    'Base64String',
 )
 
 #-----------------------------------------------------------------------------
@@ -56,14 +56,6 @@ class Regex(String):
             used by the :ref:`bokeh.sphinxext.bokeh_prop` extension when
             generating Spinx documentation. (default: None)
 
-        serialized (bool, optional) :
-            Whether attributes created from this property should be included
-            in serialization (default: True)
-
-        readonly (bool, optional) :
-            Whether attributes created from this property are read-only.
-            (default: False)
-
     Example:
 
         .. code-block:: python
@@ -81,7 +73,8 @@ class Regex(String):
             >>> m.prop = [1, 2, 3]  # ValueError !!
 
     """
-    def __init__(self, regex, default=Undefined, help=None) -> None:
+
+    def __init__(self, regex: str, *, default: Init[str] = Undefined, help: str | None = None) -> None:
         self.regex = re.compile(regex)
         super().__init__(default=default, help=help)
 
@@ -89,7 +82,7 @@ class Regex(String):
         class_name = self.__class__.__name__
         return f"{class_name}({self.regex.pattern!r})"
 
-    def validate(self, value, detail=True):
+    def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
 
         if self.regex.match(value):
@@ -98,22 +91,6 @@ class Regex(String):
         msg = "" if not detail else f"expected a string matching {self.regex.pattern!r} pattern, got {value!r}"
         raise ValueError(msg)
 
-class Base64String(String):
-
-    def serialize_value(self, value):
-        """ Encode a ascii string using Base64.
-
-        Args:
-            value : a string to encode
-
-        Returns:
-            string
-
-        """
-        if isinstance(value, str):
-            value = base64.b64encode(value.encode("utf-8")).decode("utf-8")
-        return value
-
 class MathString(String):
     """ A string with math TeX/LaTeX delimiters.
 
