@@ -24,11 +24,19 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import TYPE_CHECKING
+
 # Bokeh imports
+from .bases import Init
 from .either import Either
 from .instance import Instance
+from .singletons import Intrinsic
 from .string import MathString
 
+if TYPE_CHECKING:
+    from ...models.text import BaseText
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -41,7 +49,13 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-TextLike = Either(MathString, Instance("bokeh.models.text.BaseText"))
+class TextLike(Either):
+    """ Accept a string that may be interpreted into text models or the models themselves.
+
+    """
+
+    def __init__(self, default: Init[str | BaseText] = Intrinsic, help: str | None = None) -> None:
+        super().__init__(MathString(), Instance("bokeh.models.text.BaseText"), default=default, help=help)
 
 #-----------------------------------------------------------------------------
 # Dev API
