@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -16,6 +16,8 @@ of properties, use the mix-in classes in this module:
 
 * |HatchProps| --- properties for hatching pattern, color, alpha, etc.
 
+* |ImageProps| --- properties for global alpha on images
+
 * |LineProps| --- properties for line color, dashing, width, etc.
 
 * |TextProps| --- properties for text color, font, etc.
@@ -40,22 +42,29 @@ specific to each property.
 
 .. |FillProps| replace:: :class:`~bokeh.core.property_mixins.FillProps`
 .. |HatchProps| replace:: :class:`~bokeh.core.property_mixins.HatchProps`
+.. |ImageProps| replace:: :class:`~bokeh.core.property_mixins.ImageProps`
 .. |LineProps| replace:: :class:`~bokeh.core.property_mixins.LineProps`
 .. |TextProps| replace:: :class:`~bokeh.core.property_mixins.TextProps`
 
 .. |miter_join| image:: /_images/miter_join.png
     :height: 15
+    :alt: Two line segments joined with a miter (sharp-angle) join style.
 .. |round_join| image:: /_images/round_join.png
     :height: 15
+    :alt: Two line segments joined with a round join style.
 .. |bevel_join| image:: /_images/bevel_join.png
     :height: 15
+    :alt: Two line segments joined with a bevel (truncated) join style.
 
 .. |butt_cap| image:: /_images/butt_cap.png
     :height: 12
+    :alt: A  line segment with no end cap extending beyond the explicit end coordinate.
 .. |round_cap| image:: /_images/round_cap.png
     :height: 12
+    :alt: A  line segment with a rounded end cap extending beyond the explicit end coordinate.
 .. |square_cap| image:: /_images/square_cap.png
     :height: 12
+    :alt: A  line segment with a squared end cap extending beyond the explicit end coordinate.
 
 '''
 #-----------------------------------------------------------------------------
@@ -117,10 +126,12 @@ from .properties import (
 __all__ = (
     'FillProps',
     'HatchProps',
+    'ImageProps',
     'LineProps',
     'TextProps',
     'ScalarFillProps',
     'ScalarHatchProps',
+    'ScalarImageProps',
     'ScalarLineProps',
     'ScalarTextProps',
 )
@@ -253,7 +264,7 @@ class ScalarFillProps(HasProps):
     '''
 
     fill_color = Nullable(Color, default="gray", help=_color_help  % "fill paths")
-    fill_alpha = Alpha(help=_alpha_help)
+    fill_alpha = Alpha(help=_alpha_help % "fill paths")
 
 class HatchProps(HasProps):
     ''' Properties relevant to rendering fill regions.
@@ -283,6 +294,25 @@ class ScalarHatchProps(HasProps):
     hatch_weight = Size(default=1.0, help=_hatch_weight_help)
     hatch_extra = Dict(String, Instance("bokeh.models.textures.Texture"))
 
+class ImageProps(HasProps):
+    ''' Properties relevant to rendering images.
+
+    Mirrors the BokehJS ``properties.ImageVector`` class.
+
+    '''
+
+    global_alpha = AlphaSpec(help=_alpha_help % "images")
+
+class ScalarImageProps(HasProps):
+    ''' Properties relevant to rendering images.
+
+    Mirrors the BokehJS ``properties.Image`` class.
+
+    '''
+
+    global_alpha = Alpha(help=_alpha_help % "images")
+
+
 class LineProps(HasProps):
     ''' Properties relevant to rendering path operations.
 
@@ -327,6 +357,7 @@ class TextProps(HasProps):
     '''
 
     text_color = ColorSpec(default="#444444", help=_color_help % "fill text")
+    text_outline_color = ColorSpec(default=None, help=_color_help % "outline text")
     text_alpha = AlphaSpec(help=_alpha_help % "fill text")
     text_font = StringSpec(default=value("helvetica"), help=_text_font_help)
     text_font_size = FontSizeSpec(default=value("16px"))
@@ -347,6 +378,7 @@ class ScalarTextProps(HasProps):
     '''
 
     text_color = Nullable(Color, default="#444444", help=_color_help % "fill text")
+    text_outline_color = Nullable(Color, default=None, help=_color_help % "outline text")
     text_alpha = Alpha(help=_alpha_help % "fill text")
     text_font = String(default="helvetica", help=_text_font_help)
     text_font_size = FontSize("16px") # XXX not great XXX why?
