@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -23,6 +23,7 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
@@ -34,6 +35,9 @@ from typing import (
 # Bokeh imports
 from ..model import Model
 
+if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -53,7 +57,7 @@ __all__ = (
     'is_single_string_selector',
 )
 
-SelectorType = Dict[Union[str, Type["_Operator"]], Any]
+SelectorType: TypeAlias = Dict[Union[str, Type["_Operator"]], Any]
 
 #-----------------------------------------------------------------------------
 # General API
@@ -347,7 +351,7 @@ class NEQ(_Operator):
 #-----------------------------------------------------------------------------
 
 # realizations of the abstract predicate operators
-_operators: Dict[Type["_Operator"], Callable[[Any, Any], Any]] = {
+_operators: dict[Type["_Operator"], Callable[[Any, Any], Any]] = {
    IN:  lambda x, y: x in y,
    GT:  lambda x, y: x > y,
    LT:  lambda x, y: x < y,
