@@ -22,10 +22,9 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
-    List,
-    Type,
     Union,
     cast,
 )
@@ -34,6 +33,9 @@ from typing import (
 from .check import ValidationIssue, Validator, ValidatorType
 from .issue import Error, Issue, Warning
 
+if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -47,10 +49,10 @@ __all__ = (
 # Private API
 #-----------------------------------------------------------------------------
 
-ValidationFunction = Callable[..., Union[str, None]]
-ValidationDecorator = Callable[[ValidationFunction], Validator]
+ValidationFunction: TypeAlias = Callable[..., Union[str, None]]
+ValidationDecorator: TypeAlias = Callable[[ValidationFunction], Validator]
 
-def _validator(code_or_name: Union[int, str, Issue], validator_type: ValidatorType) -> ValidationDecorator:
+def _validator(code_or_name: int | str | Issue, validator_type: ValidatorType) -> ValidationDecorator:
     """ Internal shared implementation to handle both error and warning
     validation checks.
 
@@ -62,11 +64,11 @@ def _validator(code_or_name: Union[int, str, Issue], validator_type: ValidatorTy
         validation decorator
 
     """
-    issues: Type[Error] | Type[Warning] = \
+    issues: type[Error] | type[Warning] = \
         Error if validator_type == "error" else Warning
 
     def decorator(func: ValidationFunction) -> Validator:
-        def _wrapper(*args: Any, **kwargs: Any) -> List[ValidationIssue]:
+        def _wrapper(*args: Any, **kwargs: Any) -> list[ValidationIssue]:
             extra = func(*args, **kwargs)
             if extra is None:
                 return []
@@ -102,7 +104,7 @@ def _validator(code_or_name: Union[int, str, Issue], validator_type: ValidatorTy
 # Dev API
 #-----------------------------------------------------------------------------
 
-def error(code_or_name: Union[int, str, Issue]) -> ValidationDecorator:
+def error(code_or_name: int | str | Issue) -> ValidationDecorator:
     """ Decorator to mark a validator method for a Bokeh error condition
 
     Args:
@@ -142,7 +144,7 @@ def error(code_or_name: Union[int, str, Issue]) -> ValidationDecorator:
     """
     return _validator(code_or_name, "error")
 
-def warning(code_or_name: Union[int, str, Issue]) -> ValidationDecorator:
+def warning(code_or_name: int | str | Issue) -> ValidationDecorator:
     """ Decorator to mark a validator method for a Bokeh error condition
 
     Args:
