@@ -26,21 +26,16 @@ from typing import (
     Any,
     Dict,
     List,
+    Literal,
+    TypedDict,
     Union,
 )
 
-# External imports
-from typing_extensions import Literal, TypedDict
-
-## Bokeh imports
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from ..core.has_props import ModelDef
-    from ..core.types import (
-        ID,
-        Ref,
-        ReferenceJson,
-        Unknown,
-    )
+    from ..core.serialization import ModelRep, Ref
     from ..models.sources import DataDict
 
 #-----------------------------------------------------------------------------
@@ -57,21 +52,20 @@ __all__ = ()
 # Dev API
 #-----------------------------------------------------------------------------
 
-Patch = Any # TODO
+Patch: TypeAlias = Any # TODO
 
-Patches = Dict[str, List[Patch]]
+Patches: TypeAlias = Dict[str, List[Patch]]
 
 class ModelChanged(TypedDict):
     kind: Literal["ModelChanged"]
     model: Ref
     attr: str
-    new: Unknown
-    hint: DocumentPatched | None  # type: ignore[misc] # https://github.com/python/mypy/issues/731
+    new: Any
 
 class MessageSent(TypedDict):
     kind: Literal["MessageSent"]
     msg_type: str
-    msg_data: Unknown | None
+    msg_data: Any | None
 
 class TitleChanged(TypedDict):
     kind: Literal["TitleChanged"]
@@ -87,22 +81,25 @@ class RootRemoved(TypedDict):
 
 class ColumnDataChanged(TypedDict):
     kind: Literal["ColumnDataChanged"]
-    column_source: Ref
-    cols: List[str] | None
-    new: Unknown
+    model: Ref
+    attr: str
+    data: DataDict
+    cols: list[str] | None
 
 class ColumnsStreamed(TypedDict):
     kind: Literal["ColumnsStreamed"]
-    column_source: Ref
+    model: Ref
+    attr: str
     data: DataDict
     rollover: int | None
 
 class ColumnsPatched(TypedDict):
     kind: Literal["ColumnsPatched"]
-    column_source: Ref
+    model: Ref
+    attr: str
     patches: Patches
 
-DocumentPatched = Union[ # type: ignore[misc] # https://github.com/python/mypy/issues/731
+DocumentPatched: TypeAlias = Union[
     MessageSent,
     ModelChanged,
     ColumnDataChanged,
@@ -113,21 +110,16 @@ DocumentPatched = Union[ # type: ignore[misc] # https://github.com/python/mypy/i
     RootRemoved,
 ]
 
-DocumentChanged = DocumentPatched  # type: ignore[misc] # https://github.com/python/mypy/issues/731
-
-class RootsJson(TypedDict):
-    root_ids: List[ID]
-    references: List[ReferenceJson]
+DocumentChanged = DocumentPatched
 
 class DocJson(TypedDict):
     version: str | None
     title: str | None
-    defs: List[ModelDef] | None
-    roots: RootsJson
+    defs: list[ModelDef] | None
+    roots: list[ModelRep]
 
 class PatchJson(TypedDict):
-    events: List[DocumentChanged]
-    references: List[ReferenceJson]
+    events: list[DocumentChanged]
 
 #-----------------------------------------------------------------------------
 # Private API
