@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -21,18 +21,18 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+import asyncio
 from functools import wraps
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
+    Literal,
+    Protocol,
     TypeVar,
     cast,
 )
 
-# External imports
-from typing_extensions import Literal, Protocol
-
 ## Bokeh imports
 if TYPE_CHECKING:
     from ..server.callbacks import NextTickCallback
@@ -82,10 +82,20 @@ def without_document_lock(func: F) -> NoLockCallback[F]:
     Attempts to otherwise access or change the Document will result in an
     exception being raised.
 
+    ``func`` can be a synchronous function, an async function, or a function
+    decorated with ``asyncio.coroutine``. The returned function will be an
+    async function if ``func`` is any of the latter two.
+
     '''
-    @wraps(func)
-    def _wrapper(*args: Any, **kw: Any) -> None:
-        func(*args, **kw)
+    if asyncio.iscoroutinefunction(func):
+        @wraps(func)
+        async def _wrapper(*args: Any, **kw: Any) -> None:
+            await func(*args, **kw)
+    else:
+        @wraps(func)
+        def _wrapper(*args: Any, **kw: Any) -> None:
+            func(*args, **kw)
+
     wrapper = cast(NoLockCallback[F], _wrapper)
     wrapper.nolock = True
     return wrapper
@@ -98,7 +108,7 @@ UNSAFE_DOC_ATTR_USAGE_MSG = (
 )
 
 
-class UnlockedDocumentProxy:
+class UnlockedDocumentProxy: # TODO(mypy): this needs to implement Document interface
     ''' Wrap a Document object so that only methods that can safely be used
     from unlocked callbacks or threads are exposed. Attempts to otherwise
     access or change the Document results in an exception.
