@@ -35,18 +35,10 @@ from os.path import (
 from typing import (
     TYPE_CHECKING,
     Callable,
-    Dict,
     Iterator,
-    List,
     Sequence,
-    Set,
-    Tuple,
-    Type,
+    TypedDict,
 )
-from warnings import warn
-
-# External imports
-from typing_extensions import TypedDict
 
 # Bokeh imports
 from ..core.templates import CSS_RESOURCES, JS_RESOURCES
@@ -55,6 +47,7 @@ from ..model import Model
 from ..resources import BaseResources, Resources
 from ..settings import settings
 from ..util.compiler import bundle_models
+from ..util.warnings import warn
 from .util import contains_tex_string, is_tex_string
 
 if TYPE_CHECKING:
@@ -104,14 +97,14 @@ class Style(Artifact):
 
 class Bundle:
 
-    js_files: List[str]
-    js_raw: List[str]
-    css_files: List[str]
-    css_raw: List[str]
+    js_files: list[str]
+    js_raw: list[str]
+    css_files: list[str]
+    css_raw: list[str]
     hashes: Hashes
 
-    def __init__(self, js_files: List[str] = [], js_raw: List[str] = [],
-            css_files: List[str] = [], css_raw: List[str] = [], hashes: Hashes = {}):
+    def __init__(self, js_files: list[str] = [], js_raw: list[str] = [],
+            css_files: list[str] = [], css_raw: list[str] = [], hashes: Hashes = {}):
         self.js_files = js_files[:]
         self.js_raw = js_raw[:]
         self.css_files = css_files[:]
@@ -135,11 +128,11 @@ class Bundle:
             return "\n".join(self.js_raw)
 
     @property
-    def js_urls(self) -> List[str]:
+    def js_urls(self) -> list[str]:
         return self.js_files
 
     @property
-    def css_urls(self) -> List[str]:
+    def css_urls(self) -> list[str]:
         return self.css_files
 
     def add(self, artifact: Artifact) -> None:
@@ -153,7 +146,7 @@ class Bundle:
             self.css_raw.append(artifact.content)
 
 def bundle_for_objs_and_resources(objs: Sequence[Model | Document] | None,
-        resources: BaseResources | Tuple[BaseResources | None, BaseResources | None] | None) -> Bundle:
+        resources: BaseResources | tuple[BaseResources | None, BaseResources | None] | None) -> Bundle:
     ''' Generate rendered CSS and JS resources suitable for the given
     collection of Bokeh objects
 
@@ -192,10 +185,10 @@ def bundle_for_objs_and_resources(objs: Sequence[Model | Document] | None,
     use_gl      = _use_gl(objs)      if objs else True
     use_mathjax = _use_mathjax(objs) if objs else True
 
-    js_files: List[str] = []
-    js_raw: List[str] = []
-    css_files: List[str] = []
-    css_raw: List[str] = []
+    js_files: list[str] = []
+    js_raw: list[str] = []
+    css_files: list[str] = []
+    css_raw: list[str] = []
 
     if js_resources:
         js_resources = deepcopy(js_resources)
@@ -243,8 +236,8 @@ def bundle_for_objs_and_resources(objs: Sequence[Model | Document] | None,
 # Private API
 #-----------------------------------------------------------------------------
 
-def _query_extensions(objs: Sequence[Model | Document], query: Callable[[Type[Model]], bool]) -> bool:
-    names: Set[str] = set()
+def _query_extensions(objs: Sequence[Model | Document], query: Callable[[type[Model]], bool]) -> bool:
+    names: set[str] = set()
 
     for obj in _all_objs(objs):
         if hasattr(obj, "__implementation__"):
@@ -265,7 +258,7 @@ def _query_extensions(objs: Sequence[Model | Document], query: Callable[[Type[Mo
 
 _default_cdn_host = "https://unpkg.com"
 
-@dataclass
+@dataclass(frozen=True)
 class ExtensionEmbed:
     artifact_path: str
     server_url: str
@@ -277,11 +270,11 @@ class Pkg(TypedDict, total=False):
     module: str
     main: str
 
-extension_dirs: Dict[str, str] = {} # name -> path
+extension_dirs: dict[str, str] = {} # name -> path
 
-def _bundle_extensions(objs: Sequence[Model | Document], resources: Resources) -> List[ExtensionEmbed]:
-    names: Set[str] = set()
-    bundles: List[ExtensionEmbed] = []
+def _bundle_extensions(objs: Sequence[Model | Document], resources: Resources) -> list[ExtensionEmbed]:
+    names: set[str] = set()
+    bundles: list[ExtensionEmbed] = []
 
     extensions = [".min.js", ".js"] if resources.minified else [".js"]
 
@@ -354,8 +347,8 @@ def _bundle_extensions(objs: Sequence[Model | Document], resources: Resources) -
 
     return bundles
 
-def _all_objs(objs: Sequence[Model | Document]) -> Set[Model]:
-    all_objs: Set[Model] = set()
+def _all_objs(objs: Sequence[Model | Document]) -> set[Model]:
+    all_objs: set[Model] = set()
 
     for obj in objs:
         if isinstance(obj, Document):
@@ -420,8 +413,18 @@ def _model_requires_mathjax(model: Model) -> bool:
     Returns:
         bool: True if MathJax required, False if not
     """
+    from ..models.annotations import TextAnnotation
     from ..models.axes import Axis
     from ..models.widgets.markups import Div, Paragraph
+    from ..models.widgets.sliders import AbstractSlider
+
+    if isinstance(model, TextAnnotation):
+        if isinstance(model.text, str) and is_tex_string(model.text):
+            return True
+
+    if isinstance(model, AbstractSlider):
+        if isinstance(model.title, str) and is_tex_string(model.title):
+            return True
 
     if isinstance(model, Axis):
         if isinstance(model.axis_label, str) and is_tex_string(model.axis_label):
