@@ -21,7 +21,7 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from typing import Tuple, Union
+from typing import TYPE_CHECKING, Union
 
 # Bokeh imports
 from ..core.json_encoder import serialize_json
@@ -32,6 +32,9 @@ from ..themes import Theme
 from .elements import div_for_render_item
 from .util import FromCurdoc, OutputDocumentFor, standalone_docs_json_and_render_items
 
+if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -48,9 +51,9 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-ThemeSource = Union[Theme, FromCurdoc, None]
+ThemeSource: TypeAlias = Union[Theme, FromCurdoc, None]
 
-def notebook_content(model: Model, notebook_comms_target: str | None = None, theme: ThemeSource = FromCurdoc) -> Tuple[str, str, Document]:
+def notebook_content(model: Model, notebook_comms_target: str | None = None, theme: ThemeSource = FromCurdoc) -> tuple[str, str, Document]:
     ''' Return script and div that will display a Bokeh plot in a Jupyter
     Notebook.
 
