@@ -25,19 +25,15 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Dict,
-    List,
+    Literal,
     Sequence,
-    Tuple,
     Type,
+    TypedDict,
     Union,
     cast,
     overload,
 )
 
-# External imports
-from jinja2 import Template
-from typing_extensions import Literal, TypedDict
-
 # Bokeh imports
 from .. import __version__
 from ..core.templates import (
@@ -63,6 +59,9 @@ from .util import (
 from .wrappers import wrap_in_onload
 
 if TYPE_CHECKING:
+    from jinja2 import Template
+    from typing_extensions import TypeAlias
+
     from ..core.types import ID
     from ..document.document import DocJson
 
@@ -77,16 +76,16 @@ __all__ = (
     'json_item',
 )
 
-ModelLike = Union[Model, Document]
-ModelLikeCollection = Union[Sequence[ModelLike], Dict[str, ModelLike]]
+ModelLike: TypeAlias = Union[Model, Document]
+ModelLikeCollection: TypeAlias = Union[Sequence[ModelLike], Dict[str, ModelLike]]
 
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-ThemeLike = Union[None, Theme, Type[FromCurdoc]]
+ThemeLike: TypeAlias = Union[None, Theme, Type[FromCurdoc]]
 
-def autoload_static(model: Union[Model, Document], resources: Resources, script_path: str) -> Tuple[str, str]:
+def autoload_static(model: Model | Document, resources: Resources, script_path: str) -> tuple[str, str]:
     ''' Return JavaScript code and a script tag that can be used to embed
     Bokeh Plots.
 
@@ -138,35 +137,36 @@ def autoload_static(model: Union[Model, Document], resources: Resources, script_
 
 @overload
 def components(models: Model, wrap_script: bool = ..., # type: ignore[misc] # XXX: mypy bug
-    wrap_plot_info: Literal[True] = ..., theme: ThemeLike = ...) -> Tuple[str, str]: ...
+    wrap_plot_info: Literal[True] = ..., theme: ThemeLike = ...) -> tuple[str, str]: ...
 @overload
 def components(models: Model, wrap_script: bool = ..., wrap_plot_info: Literal[False] = ...,
-    theme: ThemeLike = ...) -> Tuple[str, RenderRoot]: ...
+    theme: ThemeLike = ...) -> tuple[str, RenderRoot]: ...
 
 @overload
 def components(models: Sequence[Model], wrap_script: bool = ..., # type: ignore[misc] # XXX: mypy bug
-    wrap_plot_info: Literal[True] = ..., theme: ThemeLike = ...) -> Tuple[str, Sequence[str]]: ...
+    wrap_plot_info: Literal[True] = ..., theme: ThemeLike = ...) -> tuple[str, Sequence[str]]: ...
 @overload
 def components(models: Sequence[Model], wrap_script: bool = ..., wrap_plot_info: Literal[False] = ...,
-    theme: ThemeLike = ...) -> Tuple[str, Sequence[RenderRoot]]: ...
+    theme: ThemeLike = ...) -> tuple[str, Sequence[RenderRoot]]: ...
 
 @overload
-def components(models: Dict[str, Model], wrap_script: bool = ..., # type: ignore[misc] # XXX: mypy bug
-    wrap_plot_info: Literal[True] = ..., theme: ThemeLike = ...) -> Tuple[str, Dict[str, str]]: ...
+def components(models: dict[str, Model], wrap_script: bool = ..., # type: ignore[misc] # XXX: mypy bug
+    wrap_plot_info: Literal[True] = ..., theme: ThemeLike = ...) -> tuple[str, dict[str, str]]: ...
 @overload
-def components(models: Dict[str, Model], wrap_script: bool = ..., wrap_plot_info: Literal[False] = ...,
-    theme: ThemeLike = ...) -> Tuple[str, Dict[str, RenderRoot]]: ...
+def components(models: dict[str, Model], wrap_script: bool = ..., wrap_plot_info: Literal[False] = ...,
+    theme: ThemeLike = ...) -> tuple[str, dict[str, RenderRoot]]: ...
 
-def components(models: Model | Sequence[Model] | Dict[str, Model], wrap_script: bool = True,
-               wrap_plot_info: bool = True, theme: ThemeLike = None) -> Tuple[str, Any]:
+def components(models: Model | Sequence[Model] | dict[str, Model], wrap_script: bool = True,
+               wrap_plot_info: bool = True, theme: ThemeLike = None) -> tuple[str, Any]:
     ''' Return HTML components to embed a Bokeh plot. The data for the plot is
     stored directly in the returned HTML.
 
     An example can be found in examples/embed/embed_multiple.py
 
     The returned components assume that BokehJS resources are **already loaded**.
-    The html template in which they will be embedded needs to include the following
-    scripts tags:
+    The HTML document or template in which they will be embedded needs to
+    include scripts tags, either from a local URL or Bokeh's CDN (replacing
+    ``x.y.z`` with the version of Bokeh you are using):
 
     .. code-block:: html
 
@@ -176,10 +176,21 @@ def components(models: Model | Sequence[Model] | Dict[str, Model], wrap_script:
         <script src="https://cdn.bokeh.org/bokeh/release/bokeh-gl-x.y.z.min.js"></script>
         <script src="https://cdn.bokeh.org/bokeh/release/bokeh-mathjax-x.y.z.min.js"></script>
 
-    The ``"-widgets"``, ``"-tables"``, and ``"-mathjax"`` files are only necessary
-    if your document includes :ref:`Bokeh widgets <userguide_interaction_widgets>`,
-    :ref:`data tables <userguide_interaction_widgets_examples_datatable>`, or
-    :ref:`math text <userguide_styling_math>`, respectively.
+    Only the Bokeh core library ``bokeh-x.y.z.min.js`` is always required. The
+    other scripts are optional and only need to be included if you want to use
+    corresponding features:
+
+    * The ``"bokeh-widgets"`` files are only necessary if you are using any of the
+      :ref:`Bokeh widgets <ug_interaction_widgets>`.
+    * The ``"bokeh-tables"`` files are only necessary if you are using Bokeh's
+      :ref:`data tables <ug_interaction_widgets_examples_datatable>`.
+    * The ``"bokeh-api"`` files are required to use the
+      :ref:`BokehJS API <ug_advanced_bokehjs>` and must be loaded *after* the
+      core BokehJS library.
+    * The ``"bokeh-gl"`` files are required to enable
+      :ref:`WebGL support <ug_output_webgl>`.
+    * the ``"bokeh-mathjax"`` files are required to enable
+      :ref:`MathJax support <ug_styling_mathtext>`.
 
     Args:
         models (Model|list|dict|tuple) :
@@ -247,7 +258,7 @@ def components(models: Model | Sequence[Model] | Dict[str, Model], wrap_script:
 
     # now convert dict to list, saving keys in the same order
     model_keys = None
-    dict_type: Type[Dict[Any, Any]] = dict
+    dict_type: type[dict[Any, Any]] = dict
     if isinstance(models, dict):
         dict_type = models.__class__
         model_keys = models.keys()
@@ -265,9 +276,9 @@ def components(models: Model | Sequence[Model] | Dict[str, Model], wrap_script:
     def div_for_root(root: RenderRoot) -> str:
         return ROOT_DIV.render(root=root, macros=MACROS)
 
-    results: List[str] | List[RenderRoot]
+    results: list[str] | list[RenderRoot]
     if wrap_plot_info:
-        results = list(div_for_root(root) for root in render_item.roots)
+        results = [div_for_root(root) for root in render_item.roots]
     else:
         results = list(render_item.roots)
 
@@ -283,10 +294,10 @@ def components(models: Model | Sequence[Model] | Dict[str, Model], wrap_script:
     return script, result
 
 def file_html(models: Model | Document | Sequence[Model],
-              resources: Resources | Tuple[JSResources | None, CSSResources | None] | None,
+              resources: Resources | tuple[JSResources | None, CSSResources | None] | None,
               title: str | None = None,
               template: Template | str = FILE,
-              template_variables: Dict[str, Any] = {},
+              template_variables: dict[str, Any] = {},
               theme: ThemeLike = None,
               suppress_callback_warning: bool = False,
               _always_new: bool = False) -> str:
@@ -415,10 +426,9 @@ def json_item(model: Model, target: ID | None = None, theme: ThemeLike = None) -
     '''
     with OutputDocumentFor([model], apply_theme=theme) as doc:
         doc.title = ""
-        docs_json = standalone_docs_json([model])
+        [doc_json] = standalone_docs_json([model]).values()
 
-    doc_json = list(docs_json.values())[0]
-    root_id = doc_json['roots']['root_ids'][0]
+    root_id = doc_json["roots"][0]["id"]
 
     return StandaloneEmbedJson(
         target_id = target,
@@ -435,7 +445,7 @@ def json_item(model: Model, target: ID | None = None, theme: ThemeLike = None) -
 # Private API
 #-----------------------------------------------------------------------------
 
-def _check_models_or_docs(models: Union[ModelLike, ModelLikeCollection]) -> ModelLikeCollection:
+def _check_models_or_docs(models: ModelLike | ModelLikeCollection) -> ModelLikeCollection:
     '''
 
     '''
@@ -461,7 +471,7 @@ def _check_models_or_docs(models: Union[ModelLike, ModelLikeCollection]) -> Mode
 
     return models
 
-def _title_from_models(models: Sequence[Union[Model, Document]], title: str | None) -> str:
+def _title_from_models(models: Sequence[Model | Document], title: str | None) -> str:
     # use override title
     if title is not None:
         return title
