@@ -23,16 +23,14 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import io
 import os
-import warnings
 from contextlib import contextmanager
-from os.path import abspath, splitext
+from os.path import abspath, expanduser, splitext
 from tempfile import mkstemp
 from typing import (
     TYPE_CHECKING,
     Any,
     Iterator,
     List,
-    Tuple,
     cast,
 )
 
@@ -47,11 +45,14 @@ from ..core.types import PathLike
 from ..document import Document
 from ..embed import file_html
 from ..resources import INLINE, Resources
+from ..themes import Theme
+from ..util.warnings import warn
+from .state import State, curstate
 from .util import default_filename
 
 if TYPE_CHECKING:
-    from ..models.layouts import LayoutDOM
     from ..models.plots import Plot
+    from ..models.ui import UIElement
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -70,15 +71,15 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-def export_png(obj: LayoutDOM | Document, *, filename: PathLike | None = None, width: int | None = None,
-        height: int | None = None, webdriver: WebDriver | None = None, timeout: int = 5) -> str:
-    ''' Export the ``LayoutDOM`` object or document as a PNG.
+def export_png(obj: UIElement | Document, *, filename: PathLike | None = None, width: int | None = None,
+        height: int | None = None, webdriver: WebDriver | None = None, timeout: int = 5, state: State | None = None) -> str:
+    ''' Export the ``UIElement`` object or document as a PNG.
 
     If the filename is not given, it is derived from the script name (e.g.
     ``/foo/myplot.py`` will create ``/foo/myplot.png``)
 
     Args:
-        obj (LayoutDOM or Document) : a Layout (Row/Column), Plot or Widget
+        obj (UIElement or Document) : a Layout (Row/Column), Plot or Widget
             object or Document to export.
 
         filename (PathLike, e.g. str, Path, optional) : filename to save document under (default: None)
@@ -96,6 +97,10 @@ def export_png(obj: LayoutDOM | Document, *, filename: PathLike | None = None, w
         timeout (int) : the maximum amount of time (in seconds) to wait for
             Bokeh to initialize (default: 5) (Added in 1.1.1).
 
+        state (State, optional) :
+            A :class:`State` object. If None, then the current default
+            implicit state is used. (default: None).
+
     Returns:
         filename (str) : the filename where the static file is saved.
 
@@ -108,7 +113,7 @@ def export_png(obj: LayoutDOM | Document, *, filename: PathLike | None = None, w
         aspect ratios. It is recommended to use the default ``fixed`` sizing mode.
 
     '''
-    image = get_screenshot_as_png(obj, width=width, height=height, driver=webdriver, timeout=timeout)
+    image = get_screenshot_as_png(obj, width=width, height=height, driver=webdriver, timeout=timeout, state=state)
 
     if filename is None:
         filename = default_filename("png")
@@ -119,17 +124,17 @@ def export_png(obj: LayoutDOM | Document, *, filename: PathLike | None = None, w
     filename = os.fspath(filename) # XXX: Image.save() doesn't fully support PathLike
     image.save(filename)
 
-    return abspath(filename)
+    return abspath(expanduser(filename))
 
-def export_svg(obj: LayoutDOM | Document, *, filename: PathLike | None = None, width: int | None = None,
-        height: int | None = None, webdriver: WebDriver | None = None, timeout: int = 5) -> List[str]:
+def export_svg(obj: UIElement | Document, *, filename: PathLike | None = None, width: int | None = None,
+        height: int | None = None, webdriver: WebDriver | None = None, timeout: int = 5, state: State | None = None) -> list[str]:
     ''' Export a layout as SVG file or a document as a set of SVG files.
 
     If the filename is not given, it is derived from the script name
     (e.g. ``/foo/myplot.py`` will create ``/foo/myplot.svg``)
 
     Args:
-        obj (LayoutDOM object) : a Layout (Row/Column), Plot or Widget object to display
+        obj (UIElement object) : a Layout (Row/Column), Plot or Widget object to display
 
         filename (PathLike, e.g. str, Path, optional) : filename to save document under (default: None)
             If None, infer from the filename.
@@ -146,6 +151,10 @@ def export_svg(obj: LayoutDOM | Document, *, filename: PathLike | None = None, w
         timeout (int) : the maximum amount of time (in seconds) to wait for
             Bokeh to initialize (default: 5)
 
+        state (State, optional) :
+            A :class:`State` object. If None, then the current default
+            implicit state is used. (default: None).
+
     Returns:
         filenames (list(str)) : the list of filenames where the SVGs files are saved.
 
@@ -154,11 +163,11 @@ def export_svg(obj: LayoutDOM | Document, *, filename: PathLike | None = None, w
         aspect ratios. It is recommended to use the default ``fixed`` sizing mode.
 
     '''
-    svgs = get_svg(obj, width=width, height=height, driver=webdriver, timeout=timeout)
+    svgs = get_svg(obj, width=width, height=height, driver=webdriver, timeout=timeout, state=state)
     return _write_collection(svgs, filename, "svg")
 
-def export_svgs(obj: LayoutDOM | Document, *, filename: str | None = None, width: int | None = None,
-        height: int | None = None, webdriver: WebDriver | None = None, timeout: int = 5) -> List[str]:
+def export_svgs(obj: UIElement | Document, *, filename: str | None = None, width: int | None = None,
+        height: int | None = None, webdriver: WebDriver | None = None, timeout: int = 5, state: State | None = None) -> list[str]:
     ''' Export the SVG-enabled plots within a layout. Each plot will result
     in a distinct SVG file.
 
@@ -166,7 +175,7 @@ def export_svgs(obj: LayoutDOM | Document, *, filename: str | None = None, width
     (e.g. ``/foo/myplot.py`` will create ``/foo/myplot.svg``)
 
     Args:
-        obj (LayoutDOM object) : a Layout (Row/Column), Plot or Widget object to display
+        obj (UIElement object) : a Layout (Row/Column), Plot or Widget object to display
 
         filename (str, optional) : filename to save document under (default: None)
             If None, infer from the filename.
@@ -183,6 +192,10 @@ def export_svgs(obj: LayoutDOM | Document, *, filename: str | None = None, width
         timeout (int) : the maximum amount of time (in seconds) to wait for
             Bokeh to initialize (default: 5) (Added in 1.1.1).
 
+        state (State, optional) :
+            A :class:`State` object. If None, then the current default
+            implicit state is used. (default: None).
+
     Returns:
         filenames (list(str)) : the list of filenames where the SVGs files are saved.
 
@@ -191,7 +204,7 @@ def export_svgs(obj: LayoutDOM | Document, *, filename: str | None = None, width
         aspect ratios. It is recommended to use the default ``fixed`` sizing mode.
 
     '''
-    svgs = get_svgs(obj, width=width, height=height, driver=webdriver, timeout=timeout)
+    svgs = get_svgs(obj, width=width, height=height, driver=webdriver, timeout=timeout, state=state)
 
     if len(svgs) == 0:
         log.warning("No SVG Plots were found.")
@@ -203,12 +216,12 @@ def export_svgs(obj: LayoutDOM | Document, *, filename: str | None = None, width
 # Dev API
 #-----------------------------------------------------------------------------
 
-def get_screenshot_as_png(obj: LayoutDOM | Document, *, driver: WebDriver | None = None, timeout: int = 5,
-        resources: Resources = INLINE, width: int | None = None, height: int | None = None) -> Image.Image:
-    ''' Get a screenshot of a ``LayoutDOM`` object.
+def get_screenshot_as_png(obj: UIElement | Document, *, driver: WebDriver | None = None, timeout: int = 5,
+        resources: Resources = INLINE, width: int | None = None, height: int | None = None, state: State | None = None) -> Image.Image:
+    ''' Get a screenshot of a ``UIElement`` object.
 
     Args:
-        obj (LayoutDOM or Document) : a Layout (Row/Column), Plot or Widget
+        obj (UIElement or Document) : a Layout (Row/Column), Plot or Widget
             object or Document to export.
 
         driver (selenium.webdriver) : a selenium webdriver instance to use
@@ -218,6 +231,10 @@ def get_screenshot_as_png(obj: LayoutDOM | Document, *, driver: WebDriver | None
             It will be used as a timeout for loading Bokeh, then when waiting for
             the layout to be rendered.
 
+        state (State, optional) :
+            A :class:`State` object. If None, then the current default
+            implicit state is used. (default: None).
+
     Returns:
         image (PIL.Image.Image) : a pillow image loaded from PNG.
 
@@ -229,7 +246,8 @@ def get_screenshot_as_png(obj: LayoutDOM | Document, *, driver: WebDriver | None
     from .webdriver import webdriver_control
 
     with _tmp_html() as tmp:
-        html = get_layout_html(obj, resources=resources, width=width, height=height)
+        theme = (state or curstate()).document.theme
+        html = get_layout_html(obj, resources=resources, width=width, height=height, theme=theme)
         with open(tmp.path, mode="w", encoding="utf-8") as file:
             file.write(html)
 
@@ -245,12 +263,13 @@ def get_screenshot_as_png(obj: LayoutDOM | Document, *, driver: WebDriver | None
                  .crop((0, 0, width*dpr, height*dpr))
                  .resize((width, height)))
 
-def get_svg(obj: LayoutDOM | Document, *, driver: WebDriver | None = None, timeout: int = 5,
-        resources: Resources = INLINE, width: int | None = None, height: int | None = None) -> List[str]:
+def get_svg(obj: UIElement | Document, *, driver: WebDriver | None = None, timeout: int = 5,
+        resources: Resources = INLINE, width: int | None = None, height: int | None = None, state: State | None = None) -> list[str]:
     from .webdriver import webdriver_control
 
     with _tmp_html() as tmp:
-        html = get_layout_html(obj, resources=resources, width=width, height=height)
+        theme = (state or curstate()).document.theme
+        html = get_layout_html(obj, resources=resources, width=width, height=height, theme=theme)
         with open(tmp.path, mode="w", encoding="utf-8") as file:
             file.write(html)
 
@@ -261,12 +280,13 @@ def get_svg(obj: LayoutDOM | Document, *, driver: WebDriver | None = None, timeo
 
     return svgs
 
-def get_svgs(obj: LayoutDOM | Document, *, driver: WebDriver | None = None, timeout: int = 5,
-        resources: Resources = INLINE, width: int | None = None, height: int | None = None) -> List[str]:
+def get_svgs(obj: UIElement | Document, *, driver: WebDriver | None = None, timeout: int = 5,
+        resources: Resources = INLINE, width: int | None = None, height: int | None = None, state: State | None = None) -> list[str]:
     from .webdriver import webdriver_control
 
     with _tmp_html() as tmp:
-        html = get_layout_html(obj, resources=resources, width=width, height=height)
+        theme = (state or curstate()).document.theme
+        html = get_layout_html(obj, resources=resources, width=width, height=height, theme=theme)
         with open(tmp.path, mode="w", encoding="utf-8") as file:
             file.write(html)
 
@@ -277,8 +297,8 @@ def get_svgs(obj: LayoutDOM | Document, *, driver: WebDriver | None = None, time
 
     return svgs
 
-def get_layout_html(obj: LayoutDOM | Document, *, resources: Resources = INLINE,
-        width: int | None = None, height: int | None = None) -> str:
+def get_layout_html(obj: UIElement | Document, *, resources: Resources = INLINE,
+        width: int | None = None, height: int | None = None, theme: Theme | None = None) -> str:
     '''
 
     '''
@@ -299,13 +319,13 @@ def get_layout_html(obj: LayoutDOM | Document, *, resources: Resources = INLINE,
     """
 
     def html() -> str:
-        return file_html(obj, resources, title="", template=template, suppress_callback_warning=True, _always_new=True)
+        return file_html(obj, resources, title="", template=template, theme=theme, suppress_callback_warning=True, _always_new=True)
 
     if width is not None or height is not None:
         # Defer this import, it is expensive
         from ..models.plots import Plot
         if not isinstance(obj, Plot):
-            warnings.warn("Export method called with width or height argument on a non-Plot model. The size values will be ignored.")
+            warn("Export method called with width or height argument on a non-Plot model. The size values will be ignored.")
         else:
             with _resized(obj, width, height):
                 return html()
@@ -363,12 +383,12 @@ def _resized(obj: Plot, width: int | None, height: int | None) -> Iterator[None]
     obj.width = old_width
     obj.height = old_height
 
-def _write_collection(items: List[str], filename: PathLike | None, ext: str) -> List[str]:
+def _write_collection(items: list[str], filename: PathLike | None, ext: str) -> list[str]:
     if filename is None:
         filename = default_filename(ext)
     filename = os.fspath(filename)
 
-    filenames: List[str] = []
+    filenames: list[str] = []
 
     def _indexed(name: str, i: int) -> str:
         basename, ext = splitext(name)
@@ -396,19 +416,19 @@ def _log_console(driver: WebDriver) -> None:
         for message in messages:
             log.warning(message)
 
-def _maximize_viewport(web_driver: WebDriver) -> Tuple[int, int, int]:
+def _maximize_viewport(web_driver: WebDriver) -> tuple[int, int, int]:
     calculate_viewport_size = """\
-        const root = document.getElementsByClassName("bk-root")[0]
-        const {width, height} = root.children[0].getBoundingClientRect()
-        return [width, height, window.devicePixelRatio]
+        const root_view = Object.values(Bokeh.index)[0]
+        const {width, height} = root_view.el.getBoundingClientRect()
+        return [Math.round(width), Math.round(height), window.devicePixelRatio]
     """
-    viewport_size: Tuple[int, int, int] = web_driver.execute_script(calculate_viewport_size)
+    viewport_size: tuple[int, int, int] = web_driver.execute_script(calculate_viewport_size)
     calculate_window_size = """\
         const [width, height, dpr] = arguments
         return [
             // XXX: outer{Width,Height} can be 0 in headless mode under certain window managers
-            Math.max(0, window.outerWidth - window.innerWidth) + width*dpr,
-            Math.max(0, window.outerHeight - window.innerHeight) + height*dpr,
+            Math.round(Math.max(0, window.outerWidth - window.innerWidth) + width*dpr),
+            Math.round(Math.max(0, window.outerHeight - window.innerHeight) + height*dpr),
         ]
     """
     [width, height] = web_driver.execute_script(calculate_window_size, *viewport_size)
@@ -416,6 +436,7 @@ def _maximize_viewport(web_driver: WebDriver) -> Tuple[int, int, int]:
     web_driver.set_window_size(width + eps, height + eps)
     return viewport_size
 
+# TODO: consider UIElement like Pane
 _SVGS_SCRIPT = """
 const {LayoutDOMView} = Bokeh.require("models/layouts/layout_dom")
 const {PlotView} = Bokeh.require("models/plots/plot")
