@@ -26,29 +26,35 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
     List,
+    Literal,
+    Protocol,
+    TypedDict,
     cast,
     overload,
 )
 from uuid import uuid4
-from warnings import warn
-
-# External imports
-from typing_extensions import Literal, Protocol, TypedDict
 
+## External imports
 if TYPE_CHECKING:
     from ipykernel.comm import Comm
 
 # Bokeh imports
-from ..core.types import ID, JSON
+from ..core.types import ID
 from ..util.serialization import make_id
+from ..util.warnings import warn
 from .state import curstate
 
 if TYPE_CHECKING:
     from ..application.application import Application
     from ..document.document import Document
-    from ..document.events import DocumentPatchedEvent, ModelChangedEvent
+    from ..document.events import (
+        ColumnDataChangedEvent,
+        ColumnsPatchedEvent,
+        ColumnsStreamedEvent,
+        DocumentPatchedEvent,
+        ModelChangedEvent,
+    )
     from ..embed.bundle import Bundle
     from ..model import Model
     from ..resources import Resources
@@ -80,16 +86,6 @@ __all__ = (
     'show_doc',
 )
 
-#-----------------------------------------------------------------------------
-# Private API
-#-----------------------------------------------------------------------------
-
-# XXX: move this to the bottom when Python 3.7 is dropped
-
-_HOOKS: Dict[str, Hooks] = {}
-
-_NOTEBOOK_LOADED: Resources | None = None
-
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -100,7 +96,7 @@ class CommsHandle:
     '''
 
     '''
-    _json: JSON = {}
+    _json: Any = {}
     _cellno: int | None
     _doc: Document
 
@@ -148,6 +144,18 @@ class CommsHandle:
         if event.model.id in self.doc.models:
             self.doc.callbacks.trigger_on_change(event)
 
+    def _column_data_changed(self, event: ColumnDataChangedEvent) -> None:
+        if event.model.id in self.doc.models:
+            self.doc.callbacks.trigger_on_change(event)
+
+    def _columns_streamed(self, event: ColumnsStreamedEvent) -> None:
+        if event.model.id in self.doc.models:
+            self.doc.callbacks.trigger_on_change(event)
+
+    def _columns_patched(self, event: ColumnsPatchedEvent) -> None:
+        if event.model.id in self.doc.models:
+            self.doc.callbacks.trigger_on_change(event)
+
 class Load(Protocol):
     def __call__(self, resources: Resources, verbose: bool, hide_banner: bool, load_timeout: int) -> None: ...
 
@@ -256,7 +264,7 @@ def push_notebook(*, document: Document | None = None, state: State | None = Non
     previously rendered Jupyter output cell.
 
     Several example notebooks can be found in the GitHub repository in
-    the :bokeh-tree:`examples/howto/notebook_comms` directory.
+    the :bokeh-tree:`examples/output/jupyter/push_notebook` directory.
 
     Args:
 
@@ -322,12 +330,14 @@ def push_notebook(*, document: Document | None = None, state: State | None = Non
     handle.doc.callbacks._held_events = []
     msg = BokehProtocol().create("PATCH-DOC", cast(List["DocumentPatchedEvent"], events)) # XXX: either fix types or filter events
 
-    handle.comms.send(msg.header_json)  # type: ignore[no-untyped-call]
-    handle.comms.send(msg.metadata_json)  # type: ignore[no-untyped-call]
-    handle.comms.send(msg.content_json)  # type: ignore[no-untyped-call]
-    for header, payload in msg.buffers:
-        handle.comms.send(json.dumps(header))   # type: ignore[no-untyped-call]
-        handle.comms.send(buffers=[payload])   # type: ignore[no-untyped-call]
+    handle.comms.send(msg.header_json)
+    handle.comms.send(msg.metadata_json)
+    handle.comms.send(msg.content_json)
+    for buffer in msg.buffers:
+        header = json.dumps(buffer.ref)
+        payload = buffer.to_bytes()
+        handle.comms.send(header)
+        handle.comms.send(buffers=[payload])
 
 def run_notebook_hook(notebook_type: NotebookType, action: Literal["load", "doc", "app"], *args: Any, **kwargs: Any) -> Any:
     ''' Run an installed notebook hook with supplied arguments.
@@ -392,7 +402,7 @@ def get_comms(target_name: str) -> Comm:
     '''
     # NOTE: must defer all IPython imports inside functions
     from ipykernel.comm import Comm
-    return Comm(target_name=target_name, data={})  # type: ignore[no-untyped-call]
+    return Comm(target_name=target_name, data={})
 
 def install_jupyter_hooks() -> None:
     '''
@@ -439,8 +449,8 @@ def load_notebook(resources: Resources | None = None, verbose: bool = False,
 
     if not hide_banner:
         if resources.mode == 'inline':
-            js_info: str | List[str] = 'inline'
-            css_info: str | List[str] = 'inline'
+            js_info: str | list[str] = 'inline'
+            css_info: str | list[str] = 'inline'
         else:
             js_info = resources.js_files[0] if len(resources.js_files) == 1 else resources.js_files
             css_info = resources.css_files[0] if len(resources.css_files) == 1 else resources.css_files
@@ -477,14 +487,13 @@ def load_notebook(resources: Resources | None = None, verbose: bool = False,
         LOAD_MIME_TYPE : jl_js,
     })
 
-def publish_display_data(data: Dict[str, Any], metadata: Dict[Any, Any] | None = None,
-        source: str | None = None, *, transient: Dict[str, Any] | None = None, **kwargs: Any) -> None:
+def publish_display_data(data: dict[str, Any], metadata: dict[Any, Any] | None = None, *, transient: dict[str, Any] | None = None, **kwargs: Any) -> None:
     '''
 
     '''
     # This import MUST be deferred or it will introduce a hard dependency on IPython
     from IPython.display import publish_display_data
-    publish_display_data(data, metadata, source, transient=transient, **kwargs)
+    publish_display_data(data, metadata, transient=transient, **kwargs)
 
 def show_app(app: Application, state: State, notebook_url: str | Callable[[int | None], str], port: int = 0, **kw: Any) -> None:
     ''' Embed a Bokeh server application in a Jupyter Notebook output cell.
@@ -591,6 +600,10 @@ def show_doc(obj: Model, state: State, notebook_handle: CommsHandle | None = Non
 # Private API
 #-----------------------------------------------------------------------------
 
+_HOOKS: dict[str, Hooks] = {}
+
+_NOTEBOOK_LOADED: Resources | None = None
+
 def _loading_js(bundle: Bundle, element_id: ID | None, load_timeout: int = 5000, register_mime: bool = True) -> str:
     '''
 
@@ -613,14 +626,16 @@ def _origin_url(url: str) -> str:
         url = url.split("//")[1]
     return url
 
-def _server_url(url: str, port: int) -> str:
+def _server_url(url: str, port: int | None) -> str:
     '''
 
     '''
+    port_ = f":{port}" if port is not None else ""
+
     if url.startswith("http"):
-        return '%s:%d%s' % (url.rsplit(':', 1)[0], port, "/")
+        return f"{url.rsplit(':', 1)[0]}{port_}{'/'}"
     else:
-        return 'http://%s:%d%s' % (url.split(':')[0], port, "/")
+        return f"http://{url.split(':')[0]}{port_}{'/'}"
 
 #-----------------------------------------------------------------------------
 # Code
