@@ -22,39 +22,45 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import math
+from collections import defaultdict
 from typing import (
     TYPE_CHECKING,
     Any,
-    Dict,
+    Callable,
     Iterable,
     Iterator,
     List,
     Sequence,
-    Tuple,
+    Type,
     TypeVar,
-    cast,
+    Union,
     overload,
 )
 
 # Bokeh imports
 from .core.enums import Location, LocationType, SizingModeType
 from .models import (
-    Box,
     Column,
+    CopyTool,
+    ExamineTool,
+    FlexBox,
+    FullscreenTool,
     GridBox,
+    GridPlot,
     LayoutDOM,
     Plot,
-    ProxyToolbar,
     Row,
+    SaveTool,
     Spacer,
-    ToolbarBox,
-    WidgetBox,
+    Tool,
+    Toolbar,
+    ToolProxy,
+    UIElement,
 )
 from .util.dataclasses import dataclass
-from .util.deprecation import deprecated
 
 if TYPE_CHECKING:
-    from .models import Toolbar, Widget
+    from typing_extensions import TypeAlias
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -64,11 +70,9 @@ __all__ = (
     'column',
     'grid',
     'gridplot',
-    'GridSpec',
     'layout',
     'row',
     'Spacer',
-    'widgetbox',
 )
 
 #-----------------------------------------------------------------------------
@@ -76,11 +80,11 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 @overload
-def row(children: List[LayoutDOM], *, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Row: ...
+def row(children: list[UIElement], *, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Row: ...
 @overload
-def row(*children: LayoutDOM, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Row: ...
+def row(*children: UIElement, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Row: ...
 
-def row(*children: LayoutDOM | List[LayoutDOM], sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Row:
+def row(*children: UIElement | list[UIElement], sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Row:
     """ Create a row of Bokeh Layout objects. Forces all objects to
     have the same sizing_mode, which is required for complex layouts to work.
 
@@ -90,7 +94,6 @@ def row(*children: LayoutDOM | List[LayoutDOM], sizing_mode: SizingModeType | No
             :class:`~bokeh.models.Widget`,
             :class:`~bokeh.models.Row`,
             :class:`~bokeh.models.Column`,
-            :class:`~bokeh.models.ToolbarBox`,
             :class:`~bokeh.models.Spacer`.
 
         sizing_mode (``"fixed"``, ``"stretch_both"``, ``"scale_width"``, ``"scale_height"``, ``"scale_both"`` ): How
@@ -112,11 +115,11 @@ def row(*children: LayoutDOM | List[LayoutDOM], sizing_mode: SizingModeType | No
     return Row(children=_children, sizing_mode=sizing_mode, **kwargs)
 
 @overload
-def column(children: List[LayoutDOM], *, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Column: ...
+def column(children: list[UIElement], *, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Column: ...
 @overload
-def column(*children: LayoutDOM, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Column: ...
+def column(*children: UIElement, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Column: ...
 
-def column(*children: LayoutDOM | List[LayoutDOM], sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Column:
+def column(*children: UIElement | list[UIElement], sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Column:
     """ Create a column of Bokeh Layout objects. Forces all objects to
     have the same sizing_mode, which is required for complex layouts to work.
 
@@ -126,7 +129,6 @@ def column(*children: LayoutDOM | List[LayoutDOM], sizing_mode: SizingModeType |
             :class:`~bokeh.models.Widget`,
             :class:`~bokeh.models.Row`,
             :class:`~bokeh.models.Column`,
-            :class:`~bokeh.models.ToolbarBox`,
             :class:`~bokeh.models.Spacer`.
 
         sizing_mode (``"fixed"``, ``"stretch_both"``, ``"scale_width"``, ``"scale_height"``, ``"scale_both"`` ): How
@@ -148,33 +150,7 @@ def column(*children: LayoutDOM | List[LayoutDOM], sizing_mode: SizingModeType |
     return Column(children=_children, sizing_mode=sizing_mode, **kwargs)
 
 
-def widgetbox(*args: Widget, children: List[Widget] | None = None, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> WidgetBox:
-    """ Create a column of bokeh widgets with predefined styling.
-
-    Args:
-        children (list of :class:`~bokeh.models.Widget`): A list of widgets.
-
-        sizing_mode (``"fixed"``, ``"stretch_both"``, ``"scale_width"``, ``"scale_height"``, ``"scale_both"`` ): How
-            will the items in the layout resize to fill the available space.
-            Default is ``"fixed"``. For more information on the different
-            modes see :attr:`~bokeh.models.LayoutDOM.sizing_mode`
-            description on :class:`~bokeh.models.LayoutDOM`.
-
-    Returns:
-        WidgetBox: A column layout of widget instances all with the same ``sizing_mode``.
-
-    Examples:
-
-        >>> widgetbox([button, select])
-        >>> widgetbox(children=[slider], sizing_mode='scale_width')
-    """
-    _children = _parse_children_arg(*args, children=children)
-    _handle_child_sizing(_children, sizing_mode, widget="widget box")
-
-    return WidgetBox(children=_children, sizing_mode=sizing_mode, **kwargs)
-
-
-def layout(*args: LayoutDOM, children: List[LayoutDOM] | None = None, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Column:
+def layout(*args: UIElement, children: list[UIElement] | None = None, sizing_mode: SizingModeType | None = None, **kwargs: Any) -> Column:
     """ Create a grid-based arrangement of Bokeh Layout objects.
 
     Args:
@@ -183,7 +159,6 @@ def layout(*args: LayoutDOM, children: List[LayoutDOM] | None = None, sizing_mod
             :class:`~bokeh.models.Widget`,
             :class:`~bokeh.models.Row`,
             :class:`~bokeh.models.Column`,
-            :class:`~bokeh.models.ToolbarBox`,
             :class:`~bokeh.models.Spacer`.
 
         sizing_mode (``"fixed"``, ``"stretch_both"``, ``"scale_width"``, ``"scale_height"``, ``"scale_both"`` ): How
@@ -212,16 +187,14 @@ def layout(*args: LayoutDOM, children: List[LayoutDOM] | None = None, sizing_mod
     return _create_grid(_children, sizing_mode, **kwargs)
 
 def gridplot(
-        children: List[List[LayoutDOM | None]] | GridSpec, *,
+        children: list[list[UIElement | None]], *,
         sizing_mode: SizingModeType | None = None,
         toolbar_location: LocationType | None = "above",
         ncols: int | None = None,
         width: int | None = None,
         height: int | None = None,
-        plot_width: int | None = None,
-        plot_height: int | None = None,
         toolbar_options: Any = None, # TODO
-        merge_tools: bool = True) -> LayoutDOM:
+        merge_tools: bool = True) -> GridPlot:
     ''' Create a grid of plots rendered on separate canvases.
 
     The ``gridplot`` function builds a single toolbar for all the plots in the
@@ -232,7 +205,7 @@ def gridplot(
         children (list of lists of |Plot|): An array of plots to display in a
             grid, given as a list of lists of Plot objects. To leave a position
             in the grid empty, pass None for that position in the children list.
-            OR list of |Plot| if called with ncols. OR an instance of GridSpec.
+            OR list of |Plot| if called with ncols.
 
         sizing_mode (``"fixed"``, ``"stretch_both"``, ``"scale_width"``, ``"scale_height"``, ``"scale_both"`` ): How
             will the items in the layout resize to fill the available space.
@@ -254,16 +227,14 @@ def gridplot(
 
         toolbar_options (dict, optional) : A dictionary of options that will be
             used to construct the grid's toolbar (an instance of
-            :class:`~bokeh.models.ToolbarBox`). If none is supplied,
-            ToolbarBox's defaults will be used.
+            :class:`~bokeh.models.Toolbar`). If none is supplied,
+            Toolbar's defaults will be used.
 
         merge_tools (``True``, ``False``): Combine tools from all child plots into
             a single toolbar.
 
     Returns:
-        Row or Column: A row or column containing the grid toolbar and the grid
-            of plots (depending on whether the toolbar is left/right or
-            above/below. The grid is always a Column of Rows of plots.
+        GridPlot:
 
     Examples:
 
@@ -277,9 +248,6 @@ def gridplot(
             )
 
     '''
-    if plot_width is not None or plot_height is not None:
-        deprecated((2, 4, 0), "plot_width and plot_height", "width or height")
-
     if toolbar_options is None:
         toolbar_options = {}
 
@@ -298,8 +266,8 @@ def gridplot(
         children = []
 
     # Make the grid
-    toolbars: List[Toolbar] = []
-    items: List[Tuple[LayoutDOM, int, int]] = []
+    tools: list[Tool | ToolProxy] = []
+    items: list[tuple[UIElement, int, int]] = []
 
     for y, row in enumerate(children):
         for x, item in enumerate(row):
@@ -308,15 +276,9 @@ def gridplot(
             elif isinstance(item, LayoutDOM):
                 if merge_tools:
                     for plot in item.select(dict(type=Plot)):
-                        toolbars.append(plot.toolbar)
+                        tools.extend(plot.toolbar.tools)
                         plot.toolbar_location = None
 
-                if isinstance(item, Plot):
-                    if plot_width is not None:
-                        item.width = plot_width
-                    if plot_height is not None:
-                        item.height = plot_height
-
                 if width is not None:
                     item.width = width
                 if height is not None:
@@ -326,37 +288,31 @@ def gridplot(
                     item.sizing_mode = sizing_mode
 
                 items.append((item, y, x))
+            elif isinstance(item, UIElement):
+                continue
             else:
-                raise ValueError("Only LayoutDOM items can be inserted into a grid")
+                raise ValueError("Only UIElement and LayoutDOM items can be inserted into a grid")
 
-    if not merge_tools or not toolbar_location:
-        return GridBox(children=items, sizing_mode=sizing_mode)
+    def merge(cls: type[Tool], group: list[Tool]):
+        if issubclass(cls, (SaveTool, CopyTool, ExamineTool, FullscreenTool)):
+            return cls()
+        else:
+            return None
 
-    grid = GridBox(children=items)
-    tools = sum([ toolbar.tools for toolbar in toolbars ], [])
-    proxy = ProxyToolbar(toolbars=toolbars, tools=tools, **toolbar_options)
-    toolbar = ToolbarBox(toolbar=proxy, toolbar_location=toolbar_location)
-
-    if toolbar_location == 'above':
-        return Column(children=[toolbar, grid], sizing_mode=sizing_mode)
-    elif toolbar_location == 'below':
-        return Column(children=[grid, toolbar], sizing_mode=sizing_mode)
-    elif toolbar_location == 'left':
-        return Row(children=[toolbar, grid], sizing_mode=sizing_mode)
-    elif toolbar_location == 'right':
-        return Row(children=[grid, toolbar], sizing_mode=sizing_mode)
+    toolbar = Toolbar(tools=tools if not merge_tools else group_tools(tools, merge=merge), **toolbar_options)
+    return GridPlot(children=items, toolbar=toolbar, toolbar_location=toolbar_location, sizing_mode=sizing_mode)
 
 # XXX https://github.com/python/mypy/issues/731
 @overload
-def grid(children: List[LayoutDOM | List[LayoutDOM | List[Any]]], *, sizing_mode: SizingModeType | None = ...) -> GridBox: ...
+def grid(children: list[UIElement | list[UIElement | list[Any]]], *, sizing_mode: SizingModeType | None = ...) -> GridBox: ...
 @overload
 def grid(children: Row | Column, *, sizing_mode: SizingModeType | None = ...) -> GridBox: ...
 @overload
-def grid(children: List[LayoutDOM | None], *, sizing_mode: SizingModeType | None = ..., nrows: int) -> GridBox: ...
+def grid(children: list[UIElement | None], *, sizing_mode: SizingModeType | None = ..., nrows: int) -> GridBox: ...
 @overload
-def grid(children: List[LayoutDOM | None], *, sizing_mode: SizingModeType | None = ..., ncols: int) -> GridBox: ...
+def grid(children: list[UIElement | None], *, sizing_mode: SizingModeType | None = ..., ncols: int) -> GridBox: ...
 @overload
-def grid(children: List[LayoutDOM | None], *, sizing_mode: SizingModeType | None = ..., nrows: int, ncols: int) -> GridBox: ...
+def grid(children: list[UIElement | None], *, sizing_mode: SizingModeType | None = ..., nrows: int, ncols: int) -> GridBox: ...
 @overload
 def grid(children: str, *, sizing_mode: SizingModeType | None = ...) -> GridBox: ...
 
@@ -411,10 +367,10 @@ def grid(children: Any = [], sizing_mode: SizingModeType | None = None, nrows: i
     """
     @dataclass
     class row:
-        children: List[row | col]
+        children: list[row | col]
     @dataclass
     class col:
-        children: List[row | col]
+        children: list[row | col]
 
     @dataclass
     class Item:
@@ -428,7 +384,7 @@ def grid(children: Any = [], sizing_mode: SizingModeType | None = None, nrows: i
     class Grid:
         nrows: int
         ncols: int
-        items: List[Item]
+        items: list[Item]
 
     def flatten(layout) -> GridBox:
         def gcd(a: int, b: int) -> int:
@@ -454,7 +410,7 @@ def grid(children: Any = [], sizing_mode: SizingModeType | None = None, nrows: i
                 nrows = lcm(*[ child.nrows for child in children ])
                 ncols = sum(child.ncols for child in children)
 
-                items: List[Item] = []
+                items: list[Item] = []
                 offset = 0
                 for child in children:
                     factor = nrows//child.nrows
@@ -504,7 +460,7 @@ def grid(children: Any = [], sizing_mode: SizingModeType | None = None, nrows: i
                 ncols = math.ceil(N/nrows)
             layout = col([ row(children[i:i+ncols]) for i in range(0, N, ncols) ])
         else:
-            def traverse(children: List[LayoutDOM], level: int = 0):
+            def traverse(children: list[LayoutDOM], level: int = 0):
                 if isinstance(children, list):
                     container = col if level % 2 == 0 else row
                     return container([ traverse(child, level+1) for child in children ])
@@ -517,7 +473,7 @@ def grid(children: Any = [], sizing_mode: SizingModeType | None = None, nrows: i
             return _has_auto_sizing(child) and child.spacing == 0
 
         def traverse(item: LayoutDOM, top_level: bool = False):
-            if isinstance(item, Box) and (top_level or is_usable(item)):
+            if isinstance(item, FlexBox) and (top_level or is_usable(item)):
                 container = col if isinstance(item, Column) else row
                 return container(list(map(traverse, item.children)))
             else:
@@ -545,78 +501,52 @@ def grid(children: Any = [], sizing_mode: SizingModeType | None = None, nrows: i
 # Dev API
 #-----------------------------------------------------------------------------
 
-class GridSpec:
-    """ Simplifies grid layout specification. """
+T = TypeVar("T", bound=Tool)
+MergeFn: TypeAlias = Callable[[Type[T], List[T]], Union[Tool, ToolProxy, None]]
 
-    nrows: int
-    ncols: int
-    _arrangement: Dict[Tuple[int, int], LayoutDOM | None]
+def group_tools(tools: list[Tool | ToolProxy], *, merge: MergeFn[Tool] | None = None) -> list[Tool | ToolProxy]:
+    """ Group common tools into tool proxies. """
+    @dataclass
+    class ToolEntry:
+        tool: Tool
+        props: Any
 
-    def __init__(self, nrows: int, ncols: int) -> None:
-        self.nrows = nrows
-        self.ncols = ncols
-        self._arrangement = {}
+    by_type: defaultdict[type[Tool], list[ToolEntry]] = defaultdict(list)
+    computed: list[Tool | ToolProxy] = []
 
-        from .util.deprecation import deprecated
-        deprecated("'GridSpec' is deprecated and will be removed in Bokeh 3.0")
-
-    def __setitem__(self, key: Tuple[int | slice, int | slice], obj: LayoutDOM | List[LayoutDOM] | List[List[LayoutDOM]]) -> None:
-        k1, k2 = key
-
-        row1: int
-        row2: int | None
-        if isinstance(k1, slice):
-            row1, row2, _ = k1.indices(self.nrows)
+    for tool in tools:
+        if isinstance(tool, ToolProxy):
+            computed.append(tool)
         else:
-            if k1 < 0:
-                k1 += self.nrows
-            if k1 >= self.nrows or k1 < 0:
-                raise IndexError("index out of range")
-            row1, row2 = k1, None
+            props = tool.properties_with_values()
+            if "overlay" in props:
+                del props["overlay"]
+            by_type[tool.__class__].append(ToolEntry(tool, props))
 
-        col1: int
-        col2: int | None
-        if isinstance(k2, slice):
-            col1, col2, _ = k2.indices(self.ncols)
-        else:
-            if k2 < 0:
-                k2 += self.ncols
-            if k2 >= self.ncols or k2 < 0:
-                raise IndexError("index out of range")
-            col1, col2 = k2, None
+    for cls, entries in by_type.items():
+        if merge is not None:
+            merged = merge(cls, [entry.tool for entry in entries ])
+            if merged is not None:
+                computed.append(merged)
+                continue
 
-        # gs[row, col]             = obj
-        # gs[row1:row2, col]       = [...]
-        # gs[row, col1:col2]       = [...]
-        # gs[row1:row2, col1:col2] = [[...], ...]
+        while entries:
+            head, *tail = entries
+            group: list[Tool] = [head.tool]
+            for item in list(tail):
+                if item.props == head.props:
+                    group.append(item.tool)
+                    entries.remove(item)
+            entries.remove(head)
 
-        T = TypeVar("T")
-        def get(obj: List[T] | None, i: int) -> T | None:
-            return obj[i] if obj is not None and 0 <= i < len(obj) else None
+            if len(group) == 1:
+                computed.append(group[0])
+            elif merge is not None and (tool := merge(cls, group)) is not None:
+                computed.append(tool)
+            else:
+                computed.append(ToolProxy(tools=group))
 
-        if row2 is None and col2 is None:
-            assert isinstance(obj, LayoutDOM)
-            self._arrangement[row1, col1] = obj
-        elif row2 is None:
-            assert col2 is not None
-            _obj = cast(List[LayoutDOM], obj)
-            for col in range(col1, col2):
-                self._arrangement[row1, col] = get(_obj, col - col1)
-        elif col2 is None:
-            assert row2 is not None
-            _obj = cast(List[LayoutDOM], obj)
-            for row in range(row1, row2):
-                self._arrangement[row, col1] = get(_obj, row - row1)
-        else:
-            _obj =  cast(List[List[LayoutDOM]], obj)
-            for row, col in zip(range(row1, row2), range(col1, col2)):
-                self._arrangement[row, col] = get(get(_obj, row - row1), col - col1)
-
-    def __iter__(self) -> Iterator[List[LayoutDOM | None]]:
-        array: List[List[LayoutDOM | None]] = [ [None]*self.ncols for _ in range(0, self.nrows) ]
-        for (row, col), obj in self._arrangement.items():
-            array[row][col] = obj
-        return iter(array)
+    return computed
 
 #-----------------------------------------------------------------------------
 # Private API
@@ -626,7 +556,7 @@ def _has_auto_sizing(item: LayoutDOM) -> bool:
     return item.sizing_mode is None and item.width_policy == "auto" and item.height_policy == "auto"
 
 L = TypeVar("L", bound=LayoutDOM)
-def _parse_children_arg(*args: L | List[L] | GridSpec, children: List[L] | None = None) -> List[L] | GridSpec:
+def _parse_children_arg(*args: L | list[L], children: list[L] | None = None) -> list[L]:
     # Set-up Children from args or kwargs
     if len(args) > 0 and children is not None:
         raise ValueError("'children' keyword cannot be used with positional arguments")
@@ -634,24 +564,25 @@ def _parse_children_arg(*args: L | List[L] | GridSpec, children: List[L] | None
     if not children:
         if len(args) == 1:
             [arg] = args
-            if isinstance(arg, (GridSpec, list)):
+            if isinstance(arg, list):
                 return arg
 
         return list(args)
 
     return children
 
-def _handle_child_sizing(children: List[LayoutDOM], sizing_mode: SizingModeType | None, *, widget: str) -> None:
+def _handle_child_sizing(children: list[UIElement], sizing_mode: SizingModeType | None, *, widget: str) -> None:
     for item in children:
+        if isinstance(item, UIElement):
+            continue
         if not isinstance(item, LayoutDOM):
             raise ValueError(f"Only LayoutDOM items can be inserted into a {widget}. Tried to insert: {item} of type {type(item)}")
         if sizing_mode is not None and _has_auto_sizing(item):
             item.sizing_mode = sizing_mode
 
-
-def _create_grid(iterable: Iterable[LayoutDOM | List[LayoutDOM]], sizing_mode: SizingModeType | None, layer: int = 0, **kwargs) -> Row | Column:
+def _create_grid(iterable: Iterable[UIElement | list[UIElement]], sizing_mode: SizingModeType | None, layer: int = 0, **kwargs) -> Row | Column:
     """Recursively create grid from input lists."""
-    return_list: List[LayoutDOM] = []
+    return_list: list[UIElement] = []
     for item in iterable:
         if isinstance(item, list):
             return_list.append(_create_grid(item, sizing_mode, layer + 1))
@@ -659,6 +590,8 @@ def _create_grid(iterable: Iterable[LayoutDOM | List[LayoutDOM]], sizing_mode: S
             if sizing_mode is not None and _has_auto_sizing(item):
                 item.sizing_mode = sizing_mode
             return_list.append(item)
+        elif isinstance(item, UIElement):
+            return_list.append(item)
         else:
             raise ValueError(
                 """Only LayoutDOM items can be inserted into a layout.
@@ -669,9 +602,9 @@ def _create_grid(iterable: Iterable[LayoutDOM | List[LayoutDOM]], sizing_mode: S
     else:
         return row(children=return_list, sizing_mode=sizing_mode, **kwargs)
 
-T = TypeVar("T")
+I = TypeVar("I")
 
-def _chunks(l: Sequence[T], ncols: int) -> Iterator[Sequence[T]]:
+def _chunks(l: Sequence[I], ncols: int) -> Iterator[Sequence[I]]:
     """Yield successive n-sized chunks from list, l."""
     assert isinstance(ncols, int), "ncols must be an integer"
     for i in range(0, len(l), ncols):
