@@ -22,11 +22,11 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from operator import itemgetter
-from typing import TYPE_CHECKING, Any, Type
+from typing import TYPE_CHECKING, Any
 
 # Bokeh imports
 from ..util.serialization import make_id
-from ..util.string import append_docstring
+from ..util.strings import append_docstring
 
 if TYPE_CHECKING:
     from .model import Model
@@ -85,7 +85,7 @@ def html_repr(obj: Model):
 
     return html
 
-def process_example(cls: Type[Any]) -> None:
+def process_example(cls: type[Any]) -> None:
     ''' A decorator to mark abstract base classes derived from |HasProps|.
 
     '''
@@ -114,13 +114,12 @@ _HTML_REPR = """
 </script>
 """
 
-# The "../../" is needed for bokeh-plot to construct the correct path to examples
 _EXAMPLE_TEMPLATE = '''
 
     Example
     -------
 
-    .. bokeh-plot:: ../../{path}
+    .. bokeh-plot:: __REPO__/{path}
         :source-position: below
 
 '''
