@@ -21,45 +21,26 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from inspect import isclass
-from json import loads
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    ClassVar,
-    Dict,
-    Iterable,
-    List,
-    Set,
-    Type,
-)
+from inspect import Parameter, Signature, isclass
+from typing import TYPE_CHECKING, Any, Iterable
 
 # Bokeh imports
 from ..core import properties as p
-from ..core.has_props import HasProps, abstract
-from ..core.json_encoder import serialize_json
-from ..core.types import (
-    ID,
-    Ref,
-    ReferenceJson,
-    Unknown,
-)
+from ..core.has_props import HasProps, _default_resolver, abstract
+from ..core.property._sphinx import type_link
+from ..core.property.validation import without_property_validation
+from ..core.serialization import ObjectRefRep, Ref, Serializer
+from ..core.types import ID
 from ..events import Event
 from ..themes import default as default_theme
 from ..util.callback_manager import EventCallbackManager, PropertyCallbackManager
 from ..util.serialization import make_id
 from .docs import html_repr, process_example
-from .util import (
-    HasDocumentRef,
-    collect_models,
-    qualified_model,
-    visit_value_and_its_immediate_references,
-)
+from .util import HasDocumentRef, collect_models, visit_value_and_its_immediate_references
 
 if TYPE_CHECKING:
     from ..core.has_props import Setter
     from ..core.query import SelectorType
-    from ..core.types import JSON
     from ..document import Document
     from ..document.events import DocumentPatchedEvent
     from ..models.callbacks import Callback as JSEventCallback
@@ -83,47 +64,48 @@ __all__ = (
 
 @abstract
 class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackManager):
-    ''' Base class for all objects stored in Bokeh  |Document| instances.
+    ''' Base class for all objects stored in Bokeh |Document| instances.
 
     '''
 
-    model_class_reverse_map: ClassVar[Dict[str, Type[Model]]] = {}
+
+    # a canonical order for positional args that can be
+    # used for any functions derived from this class
+    _args = ()
+
+    _extra_kws = {}
 
     @classmethod
     def __init_subclass__(cls):
         super().__init_subclass__()
 
-        # use an explicitly provided view model name if there is one
-        if "__view_model__" not in cls.__dict__:
-            cls.__view_model__ = cls.__name__
-        if "__view_module__" not in cls.__dict__:
-            cls.__view_module__ = cls.__module__
-
-        qualified = qualified_model(cls)
-
-        cls.__qualified_model__ = qualified
-
-        # update the mapping of view model names to classes, checking for any duplicates
-        previous = cls.model_class_reverse_map.get(qualified, None)
-        if previous is not None and not hasattr(cls, "__implementation__"):
-            raise Warning(f"Duplicate qualified model declaration of '{qualified}'. Previous definition: {previous}")
-        cls.model_class_reverse_map[qualified] = cls
-
-        process_example(cls)
+        if cls.__module__.startswith("bokeh.models"):
+            assert "__init__" in cls.__dict__, str(cls)
+            parameters = [x[0] for x in  cls.parameters()]
+            cls.__init__.__signature__ = Signature(parameters=parameters)
+            process_example(cls)
 
     _id: ID
 
-    def __new__(cls, *args, **kwargs) -> Model:
-        obj =  super().__new__(cls)
-        obj._id = kwargs.pop("id", make_id())
+    def __new__(cls, *args: Any, id: ID | None = None, **kwargs: Any) -> Model:
+        obj = super().__new__(cls)
+
+        # Setting 'id' implies deferred initialization, which means properties
+        # will be initialized in a separate step by a deserializer, etc.
+        if id is not None:
+            if args or kwargs:
+                raise ValueError("'id' cannot be used together with property initializers")
+            obj._id = id
+        else:
+            obj._id = make_id()
+
         return obj
 
-    def __init__(self, **kwargs: Any) -> None:
-
-        # "id" is popped from **kw in __new__, so in an ideal world I don't
-        # think it should be here too. But Python has subtle behavior here, so
-        # it is necessary
-        kwargs.pop("id", None)
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        if args:
+            raise ValueError("positional arguments are not allowed")
+        if "id" in kwargs:
+            raise ValueError("initializing 'id' is not allowed")
 
         super().__init__(**kwargs)
         default_theme.apply_to_model(self)
@@ -165,7 +147,7 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
 
     """)
 
-    tags: List[Any] = p.List(p.AnyRef, help="""
+    tags: list[Any] = p.List(p.AnyRef, help="""
     An optional list of arbitrary, user-supplied values to attach to this
     model.
 
@@ -189,8 +171,7 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
 
     """)
 
-    js_event_callbacks = p.Dict(p.String, p.List(p.Instance("bokeh.models.callbacks.CustomJS")),
-    help="""
+    js_event_callbacks = p.Dict(p.String, p.List(p.Instance("bokeh.models.callbacks.Callback")), help="""
     A mapping of event names to lists of ``CustomJS`` callbacks.
 
     Typically, rather then modifying this property directly, callbacks should be
@@ -202,13 +183,7 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
         plot.js_on_event('tap', callback)
     """)
 
-    subscribed_events = p.List(p.String, help="""
-    List of events that are subscribed to by Python callbacks. This is
-    the set of events that will be communicated from BokehJS back to
-    Python for this model.
-    """)
-
-    js_property_callbacks = p.Dict(p.String, p.List(p.Instance("bokeh.models.callbacks.CustomJS")), help="""
+    js_property_callbacks = p.Dict(p.String, p.List(p.Instance("bokeh.models.callbacks.Callback")), help="""
     A mapping of attribute names to lists of ``CustomJS`` callbacks, to be set up on
     BokehJS side when the document is created.
 
@@ -222,6 +197,12 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
 
     """)
 
+    subscribed_events = p.Set(p.String, help="""
+    Collection of events that are subscribed to by Python callbacks. This is
+    the set of events that will be communicated from BokehJS back to Python
+    for this model.
+    """)
+
     syncable: bool = p.Bool(default=True, help="""
     Indicates whether this model should be synchronized back to a Bokeh server when
     updated in a web browser. Setting to ``False`` may be useful to reduce network
@@ -241,38 +222,85 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
     def ref(self) -> Ref:
         return Ref(id=self._id)
 
-    @property
-    def struct(self) -> ReferenceJson:
-        ''' A Bokeh protocol "structure" of this model, i.e. a dict of the form:
-
-        .. code-block:: python
-
-            {
-                'type' : << view model name >>
-                'id'   : << unique model id >>
-            }
-
-        Additionally there may be a `subtype` field if this model is a subtype.
-
-        '''
-        this = ReferenceJson(
-            id=self.id,
-            type=self.__qualified_model__,
-            attributes={},
-        )
-
-        if "__subtype__" in self.__class__.__dict__:
-            # XXX: remove __subtype__ and this garbage at 2.0
-            parts = this["type"].split(".")
-            parts[-1] = self.__view_model__
-            this["type"] = ".".join(parts)
-            this["subtype"] = self.__subtype__
-
-        return this
-
     # Public methods ----------------------------------------------------------
 
-    def js_on_event(self, event: str | Type[Event], *callbacks: JSEventCallback) -> None:
+    @classmethod
+    @without_property_validation
+    def parameters(cls: type[Model]) -> list[Parameter]:
+        ''' Generate Python ``Parameter`` values suitable for functions that are
+        derived from the glyph.
+
+        Returns:
+            list(Parameter)
+
+        '''
+        arg_params = []
+        no_more_defaults = False
+
+        for arg in reversed(cls._args):
+            descriptor = cls.lookup(arg)
+            default = descriptor.class_default(cls, no_eval=True)
+            if default is None:
+                no_more_defaults = True
+
+            # simplify field(x) defaults to just present the column name
+            if isinstance(default, dict) and set(default) == {"field"}:
+                default = default["field"]
+
+            # make sure built-ins don't hold on to references to actual Models
+            if cls.__module__.startswith("bokeh.models"):
+                assert not isinstance(default, Model)
+
+            param = Parameter(
+                name=arg,
+                kind=Parameter.POSITIONAL_OR_KEYWORD,
+                # For positional arg properties, default=None means no default.
+                default=Parameter.empty if no_more_defaults else default
+            )
+            if default:
+                del default
+            typ = type_link(descriptor.property)
+            arg_params.insert(0, (param, typ, descriptor.__doc__))
+
+        # these are not really useful, and should also really be private, just skip them
+        omissions = {'js_event_callbacks', 'js_property_callbacks', 'subscribed_events'}
+
+        kwarg_params = []
+
+        kws = set(cls.properties()) - set(cls._args) - omissions
+        for kw in kws:
+            descriptor = cls.lookup(kw)
+            default = descriptor.class_default(cls, no_eval=True)
+
+            # simplify field(x) defaults to just present the column name
+            if isinstance(default, dict) and set(default) == {"field"}:
+                default = default["field"]
+
+            # make sure built-ins don't hold on to references to actual Models
+            if cls.__module__.startswith("bokeh.models"):
+                assert not isinstance(default, Model)
+
+            param = Parameter(
+                name=kw,
+                kind=Parameter.KEYWORD_ONLY,
+                default=default
+            )
+            del default
+            typ = type_link(descriptor.property)
+            kwarg_params.append((param, typ, descriptor.__doc__))
+
+        for kw, (typ, doc) in cls._extra_kws.items():
+            param = Parameter(
+                name=kw,
+                kind=Parameter.KEYWORD_ONLY,
+            )
+            kwarg_params.append((param, typ, doc))
+
+        kwarg_params.sort(key=lambda x: x[0].name)
+
+        return arg_params + kwarg_params
+
+    def js_on_event(self, event: str | type[Event], *callbacks: JSEventCallback) -> None:
         if isinstance(event, str):
             pass
         elif isinstance(event, type) and issubclass(event, Event):
@@ -291,8 +319,9 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
     def js_link(self, attr: str, other: Model, other_attr: str, attr_selector: int | str | None = None) -> None:
         ''' Link two Bokeh model properties using JavaScript.
 
-        This is a convenience method that simplifies adding a CustomJS callback
-        to update one Bokeh model property whenever another changes value.
+        This is a convenience method that simplifies adding a
+        :class:`~bokeh.models.CustomJS` callback to update one Bokeh model
+        property whenever another changes value.
 
         Args:
 
@@ -370,7 +399,8 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
         self.js_on_change(attr, cb)
 
     def js_on_change(self, event: str, *callbacks: JSEventCallback) -> None:
-        ''' Attach a ``CustomJS`` callback to an arbitrary BokehJS model event.
+        ''' Attach a :class:`~bokeh.models.CustomJS` callback to an arbitrary
+        BokehJS model event.
 
         On the BokehJS side, change events for model properties have the
         form ``"change:property_name"``. As a convenience, if the event name
@@ -434,7 +464,7 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
         descriptor = self.lookup(attr)
         super().on_change(descriptor.name, *callbacks)
 
-    def references(self) -> Set[Model]:
+    def references(self) -> set[Model]:
         ''' Returns all ``Models`` that this object has references to.
 
         '''
@@ -471,7 +501,7 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
             return None
         return result[0]
 
-    def set_select(self, selector: Type[Model] | SelectorType, updates: Dict[str, Unknown]) -> None:
+    def set_select(self, selector: type[Model] | SelectorType, updates: dict[str, Any]) -> None:
         ''' Update objects that match a given selector with the specified
         attribute/value updates.
 
@@ -489,58 +519,23 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
             for key, val in updates.items():
                 setattr(obj, key, val)
 
-    def to_json(self, include_defaults: bool) -> JSON:
-        ''' Returns a dictionary of the attributes of this object,
-        containing only "JSON types" (string, number, boolean,
-        none, dict, list).
+    def to_serializable(self, serializer: Serializer) -> ObjectRefRep:
+        serializer.add_ref(self, self.ref)
 
-        References to other objects are serialized as "refs" (just
-        the object ID and type info), so the deserializer will
-        need to separately have the full attributes of those
-        other objects.
+        super_rep = super().to_serializable(serializer)
+        rep = ObjectRefRep(
+            type="object",
+            name=super_rep["name"],
+            id=self.id,
+        )
 
-        There's no corresponding ``from_json()`` because to
-        deserialize an object is normally done in the context of a
-        Document (since the Document can resolve references).
+        attributes = super_rep.get("attributes")
+        if attributes is not None:
+            rep["attributes"] = attributes
 
-        For most purposes it's best to serialize and deserialize
-        entire documents.
+        return rep
 
-        Args:
-            include_defaults (bool) : whether to include attributes
-                that haven't been changed from the default
-
-        '''
-        return loads(self.to_json_string(include_defaults=include_defaults))
-
-    def to_json_string(self, include_defaults: bool) -> str:
-        ''' Returns a JSON string encoding the attributes of this object.
-
-        References to other objects are serialized as references
-        (just the object ID and type info), so the deserializer
-        will need to separately have the full attributes of those
-        other objects.
-
-        There's no corresponding ``from_json_string()`` because to
-        deserialize an object is normally done in the context of a
-        Document (since the Document can resolve references).
-
-        For most purposes it's best to serialize and deserialize
-        entire documents.
-
-        Args:
-            include_defaults (bool) : whether to include attributes
-                that haven't been changed from the default
-
-        '''
-        json_like = self._to_json_like(include_defaults=include_defaults)
-        json_like['id'] = self.id
-        # serialize_json "fixes" the JSON from _to_json_like by converting
-        # all types into plain JSON types # (it converts Model into refs,
-        # for example).
-        return serialize_json(json_like)
-
-    def trigger(self, attr: str, old: Unknown, new: Unknown,
+    def trigger(self, attr: str, old: Any, new: Any,
             hint: DocumentPatchedEvent | None = None, setter: Setter | None = None) -> None:
         '''
 
@@ -585,12 +580,7 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
 
     @classmethod
     def _clear_extensions(cls) -> None:
-        cls.model_class_reverse_map = {
-            k: v for k, v in cls.model_class_reverse_map.items()
-            if getattr(v, "__implementation__", None) is None
-                and getattr(v, "__javascript__", None) is None
-                and getattr(v, "__css__", None) is None
-        }
+        _default_resolver.clear_extensions()
 
     def _detach_document(self) -> None:
         ''' Detach a model from a Bokeh |Document|.
@@ -602,55 +592,9 @@ class Model(HasProps, HasDocumentRef, PropertyCallbackManager, EventCallbackMana
         self.document = None
         default_theme.apply_to_model(self)
 
-    def _to_json_like(self, include_defaults: bool):
-        ''' Returns a dictionary of the attributes of this object, in
-        a layout corresponding to what BokehJS expects at unmarshalling time.
-
-        This method does not convert "Bokeh types" into "plain JSON types,"
-        for example each child Model will still be a Model, rather
-        than turning into a reference, numpy isn't handled, etc.
-        That's what "json like" means.
-
-        This method should be considered "private" or "protected",
-        for use internal to Bokeh; use ``to_json()`` instead because
-        it gives you only plain JSON-compatible types.
-
-        Args:
-            include_defaults (bool) : whether to include attributes
-                that haven't been changed from the default.
-
-        '''
-        all_attrs = self.properties_with_values(include_defaults=include_defaults)
-
-        # If __subtype__ is defined, then this model may introduce properties
-        # that don't exist on __view_model__ in bokehjs. Don't serialize such
-        # properties.
-        subtype = getattr(self.__class__, "__subtype__", None)
-        if subtype is not None and subtype != self.__class__.__view_model__:
-            attrs = {}
-            for attr, value in all_attrs.items():
-                if attr in self.__class__.__dict__:
-                    continue
-                else:
-                    attrs[attr] = value
-        else:
-            attrs = all_attrs
-
-        for (k, v) in attrs.items():
-            # we can't serialize Infinity, we send it as None and
-            # the other side has to fix it up. This transformation
-            # can't be in our json_encoder because the json
-            # module checks for inf before it calls the custom
-            # encoder.
-            if isinstance(v, float) and v == float('inf'):
-                attrs[k] = None
-
-        return attrs
-
     def _repr_html_(self) -> str:
         return html_repr(self)
 
-
     def _sphinx_height_hint(self) -> int|None:
         return None
 
