@@ -21,19 +21,13 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from typing import (
-    TYPE_CHECKING,
-    Callable,
-    List,
-    Set,
-    Type,
-)
+from typing import TYPE_CHECKING, Any, Callable
 
 # Bokeh imports
-from ..core.has_props import HasProps, abstract
+from ..core.has_props import HasProps, Qualified
 
 if TYPE_CHECKING:
-    from ..core.types import ID, Unknown
+    from ..core.types import ID
     from ..document import Document
     from .model import Model
 
@@ -43,7 +37,7 @@ if TYPE_CHECKING:
 
 __all__ = (
     'HasDocumentRef',
-    'Qualified',
+    'Qualified', # XXX: for backwards compatibility
     'collect_filtered_models',
     'collect_models',
     'get_class',
@@ -82,22 +76,7 @@ class HasDocumentRef:
     def document(self, doc: Document) -> None:
         self._document = doc
 
-@abstract
-class Qualified(HasProps):
-    pass
-
-def qualified_model(cls: Type[Model]) -> str:
-    module = cls.__view_module__
-    model = cls.__dict__.get("__subtype__", cls.__view_model__)
-    impl = cls.__dict__.get("__implementation__", None)
-
-    if not issubclass(cls, Qualified):
-        head = module.split(".")[0]
-        if head == "bokeh" or head == "__main__" or impl is not None:
-            return model
-    return f"{module}.{model}"
-
-def collect_filtered_models(discard: Callable[[Model], bool] | None, *input_values: Unknown) -> List[Model]:
+def collect_filtered_models(discard: Callable[[Model], bool] | None, *input_values: Any) -> list[Model]:
     ''' Collect a duplicate-free list of all other Bokeh models referred to by
     this model, or by any of its references, etc, unless filtered-out by the
     provided callable.
@@ -120,9 +99,9 @@ def collect_filtered_models(discard: Callable[[Model], bool] | None, *input_valu
 
     '''
 
-    ids: Set[ID] = set()
-    collected: List[Model] = []
-    queued: List[Model] = []
+    ids: set[ID] = set()
+    collected: list[Model] = []
+    queued: list[Model] = []
 
     def queue_one(obj: Model) -> None:
         if obj.id not in ids and not (callable(discard) and discard(obj)):
@@ -140,7 +119,7 @@ def collect_filtered_models(discard: Callable[[Model], bool] | None, *input_valu
 
     return collected
 
-def collect_models(*input_values: Unknown) -> List[Model]:
+def collect_models(*input_values: Any) -> list[Model]:
     ''' Collect a duplicate-free list of all other Bokeh models referred to by
     this model, or by any of its references, etc.
 
@@ -158,7 +137,7 @@ def collect_models(*input_values: Unknown) -> List[Model]:
     '''
     return collect_filtered_models(None, *input_values)
 
-def get_class(view_model_name: str) -> Type[Model]:
+def get_class(view_model_name: str) -> type[Model]:
     ''' Look up a Bokeh model class, given its view model name.
 
     Args:
@@ -181,11 +160,11 @@ def get_class(view_model_name: str) -> Type[Model]:
 
     '''
 
-    # in order to look up from the model catalog that Model maintains, it
-    # has to be creates first. These imports ensure that all built-in Bokeh
+    # In order to look up from the model catalog that Model maintains, it
+    # has to be created first. These imports ensure that all built-in Bokeh
     # models are represented in the catalog.
-    from .. import models; models
-    from ..plotting import Figure, GMap; Figure, GMap
+    from .. import models  # noqa: F401
+    from .. import plotting  # noqa: F401
     from .model import Model
 
     known_models = Model.model_class_reverse_map
@@ -194,7 +173,7 @@ def get_class(view_model_name: str) -> Type[Model]:
     else:
         raise KeyError(f"View model name '{view_model_name}' not found")
 
-def visit_immediate_value_references(value: Unknown, visitor: Callable[[Model], None]) -> None:
+def visit_immediate_value_references(value: Any, visitor: Callable[[Model], None]) -> None:
     ''' Visit all references to another Model without recursing into any
     of the child Model; may visit the same Model more than once if
     it's referenced more than once. Does not visit the passed-in value.
@@ -208,7 +187,7 @@ def visit_immediate_value_references(value: Unknown, visitor: Callable[[Model],
         visit_value_and_its_immediate_references(value, visitor)
 
 
-def visit_value_and_its_immediate_references(obj: Unknown, visitor: Callable[[Model], None]) -> None:
+def visit_value_and_its_immediate_references(obj: Any, visitor: Callable[[Model], None]) -> None:
     ''' Visit Models, HasProps, and Python containers.
 
     Recurses down HasProps references and Python containers (does not recurse
