@@ -4,7 +4,6 @@
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-
 ''' Client-side interactivity.
 
 '''
@@ -21,14 +20,24 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import Any as any
+
 # Bokeh imports
-from ..core.has_props import abstract
+from ..core.has_props import HasProps, abstract
 from ..core.properties import (
+    Any,
     AnyRef,
     Bool,
     Dict,
+    Instance,
+    Required,
     String,
 )
+from ..core.property.bases import Init
+from ..core.property.singletons import Intrinsic
+from ..core.validation import error
+from ..core.validation.errors import INVALID_PROPERTY_VALUE, NOT_A_PROPERTY_OF
 from ..model import Model
 
 #-----------------------------------------------------------------------------
@@ -39,6 +48,7 @@ __all__ = (
     'Callback',
     'OpenURL',
     'CustomJS',
+    'SetValue',
 )
 
 #-----------------------------------------------------------------------------
@@ -51,23 +61,31 @@ class Callback(Model):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 
 class OpenURL(Callback):
     ''' Open a URL in a new or current tab or window.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     url = String("http://", help="""
     The URL to direct the web browser to. This can be a template string,
     which will be formatted with data from the data source.
     """)
+
     same_tab = Bool(False, help="""
     Open URL in a new (`False`, default) or current (`True`) tab or window.
     For `same_tab=False`, whether tab or window will be opened is browser
     dependent.
     """)
 
-
 class CustomJS(Callback):
     ''' Execute a JavaScript function.
 
@@ -79,6 +97,10 @@ class CustomJS(Callback):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     args = Dict(String, AnyRef, help="""
     A mapping of names to Python objects. In particular those can be bokeh's models.
     These objects are made available to the callback's code snippet as the values of
@@ -94,6 +116,44 @@ class CustomJS(Callback):
     (i.e. mouse coordinates and hovered glyph indices for the ``HoverTool``).
     """)
 
+class SetValue(Callback):
+    """ Allows to update a property of an object. """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, obj: Init[HasProps] = Intrinsic, attr: Init[str] = Intrinsic, value: Init[any] = Intrinsic, **kwargs) -> None:
+        super().__init__(obj=obj, attr=attr, value=value, **kwargs)
+
+    obj: HasProps = Required(Instance(HasProps), help="""
+    Object to set the value on.
+    """)
+
+    attr: str = Required(String, help="""
+    The property to modify.
+    """)
+
+    value = Required(Any, help="""
+    The value to set.
+    """)
+
+    @error(NOT_A_PROPERTY_OF)
+    def _check_if_an_attribute_is_a_property_of_a_model(self):
+        if self.obj.lookup(self.attr, raises=False):
+            return None
+        else:
+            return f"{self.attr} is not a property of {self.obj}"
+
+    @error(INVALID_PROPERTY_VALUE)
+    def _check_if_provided_a_valid_value(self):
+        descriptor = self.obj.lookup(self.attr)
+
+        if descriptor.property.is_valid(self.value):
+            return None
+        else:
+            return f"{self.value!r} is not a valid value for {self.obj}.{self.attr}"
+
+# TODO: class Show(Callback): target = Required(Either(Instance(DOMNode), Instance(UIElement)))
+# TODO: class Hide(Callback): ...
+
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
