@@ -18,32 +18,40 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Bokeh imports
-from ..core.properties import Instance
-from ..model import Model
-from .ranges import DataRange1d, Range
-from .scales import LinearScale, Scale
+from ..core.enums import OutputBackend
+from ..core.properties import Bool, Enum
+from .ui import UIElement
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
 __all__ = (
-    "CoordinateMapping",
+    "Canvas",
 )
 
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-class CoordinateMapping(Model):
-    """ A mapping between two coordinate systems. """
+class Canvas(UIElement):
+    """ """
 
-    x_source = Instance(Range, default=lambda: DataRange1d())
-    y_source = Instance(Range, default=lambda: DataRange1d())
-    x_scale = Instance(Scale, default=lambda: LinearScale())
-    y_scale = Instance(Scale, default=lambda: LinearScale())
-    x_target = Instance(Range)
-    y_target = Instance(Range)
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    hidpi = Bool(default=True, help="""
+    Whether to use HiDPI mode when available.
+    """)
+
+    output_backend = Enum(OutputBackend, default="canvas", help="""
+    Specify the output backend for the plot area. Default is HTML5 Canvas.
+
+    .. note::
+        When set to ``webgl``, glyphs without a WebGL rendering implementation
+        will fall back to rendering onto 2D canvas.
+    """)
 
 #-----------------------------------------------------------------------------
 # Dev API
