@@ -22,7 +22,13 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Bokeh imports
-from ..core.enums import LatLon, NumeralLanguage, RoundingFunction
+from ..core.enums import (
+    ContextWhich,
+    LatLon,
+    LocationType,
+    NumeralLanguage,
+    RoundingFunction,
+)
 from ..core.has_props import abstract
 from ..core.properties import (
     AnyRef,
@@ -40,7 +46,9 @@ from ..core.properties import (
 from ..core.validation import error
 from ..core.validation.errors import MISSING_MERCATOR_DIMENSION
 from ..model import Model
-from ..util.string import format_docstring
+from ..util.deprecation import deprecated
+from ..util.strings import format_docstring
+from ..util.warnings import warn
 from .tickers import Ticker
 
 #-----------------------------------------------------------------------------
@@ -48,15 +56,17 @@ from .tickers import Ticker
 #-----------------------------------------------------------------------------
 
 __all__ = (
-    'TickFormatter',
-    'BasicTickFormatter',
-    'MercatorTickFormatter',
-    'NumeralTickFormatter',
-    'PrintfTickFormatter',
-    'LogTickFormatter',
-    'CategoricalTickFormatter',
-    'FuncTickFormatter',
-    'DatetimeTickFormatter',
+    "RELATIVE_DATETIME_CONTEXT",
+    "BasicTickFormatter",
+    "CategoricalTickFormatter",
+    "CustomJSTickFormatter",
+    "DatetimeTickFormatter",
+    "FuncTickFormatter",
+    "LogTickFormatter",
+    "MercatorTickFormatter",
+    "NumeralTickFormatter",
+    "PrintfTickFormatter",
+    "TickFormatter",
 )
 
 #-----------------------------------------------------------------------------
@@ -79,13 +89,21 @@ class TickFormatter(Model):
     ''' A base class for all tick formatter types.
 
     '''
-    pass
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 class BasicTickFormatter(TickFormatter):
     ''' Display tick values from continuous ranges as "basic numbers",
     using scientific notation when appropriate by default.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     precision = Either(Auto, Int, help="""
     How many digits of precision to display in tick labels.
     """)
@@ -120,6 +138,10 @@ class MercatorTickFormatter(BasicTickFormatter):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     dimension = Nullable(Enum(LatLon), help="""
     Specify whether to format ticks for Latitude or Longitude.
 
@@ -145,6 +167,10 @@ class MercatorTickFormatter(BasicTickFormatter):
 class NumeralTickFormatter(TickFormatter):
     ''' Tick formatter based on a human-readable format string. '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     format = String("0,0", help="""
     The number format, as defined in the following tables:
 
@@ -232,6 +258,10 @@ class NumeralTickFormatter(TickFormatter):
 class PrintfTickFormatter(TickFormatter):
     ''' Tick formatter based on a printf-style format string. '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     format = String("%s", help="""
     The number format, as defined as follows: the placeholder in the format
     string is marked by % and is followed by one or more of these elements,
@@ -283,6 +313,11 @@ class LogTickFormatter(TickFormatter):
     Most often useful in conjunction with a ``LogTicker``.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     ticker = Nullable(Instance(Ticker), help="""
     The corresponding ``LogTicker``, used to determine the correct
     base to use. If unset, the formatter will use base 10 as a default.
@@ -299,9 +334,12 @@ class CategoricalTickFormatter(TickFormatter):
     values.
 
     '''
-    pass
 
-class FuncTickFormatter(TickFormatter):
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+class CustomJSTickFormatter(TickFormatter):
     ''' Display tick values that are formatted by a user-defined function.
 
     .. warning::
@@ -312,6 +350,10 @@ class FuncTickFormatter(TickFormatter):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     args = Dict(String, AnyRef, help="""
     A mapping of names to Python objects. In particular those can be bokeh's models.
     These objects are made available to the formatter's code snippet as the values of
@@ -342,6 +384,19 @@ class FuncTickFormatter(TickFormatter):
             '''
     """)
 
+def FuncTickFormatter(*args, **kw):
+    from bokeh.util.deprecation import deprecated
+    deprecated((3, 0, 0), "FuncTickFormatter", "CustomJSTickFormatter")
+    return CustomJSTickFormatter(*args, **kw)
+
+def _deprecated_datetime_list_format(fmt: list[str]) -> str:
+    deprecated("Passing lists of formats for DatetimeTickFormatter scales was deprecated in Bokeh 3.0. Configure a single string format for each scale")
+    if len(fmt) == 0:
+        raise ValueError("Datetime format list must contain one element")
+    if len(fmt) > 1:
+        warn(f"DatetimeFormatter scales now only accept a single format. Using the first provided: {fmt[0]!r}")
+    return fmt[0]
+
 class DatetimeTickFormatter(TickFormatter):
     ''' A ``TickFormatter`` for displaying datetime values nicely across a
     range of scales.
@@ -387,8 +442,8 @@ class DatetimeTickFormatter(TickFormatter):
         The day of the month as a decimal number (range 01 to 31).
 
     %D
-        Equivalent to %m/%d/%y.  (Americans should note that in many
-        other countries %d/%m/%y is rather common. This means that in
+        Equivalent to **%m/%d/%y**.  (Americans should note that in many
+        other countries **%d/%m/%y** is rather common. This means that in
         international context this format is ambiguous and should not
         be used.)
 
@@ -402,7 +457,7 @@ class DatetimeTickFormatter(TickFormatter):
         available to `timezone`_.
 
     %F
-        Equivalent to %Y-%m-%d (the ISO 8601 date format).
+        Equivalent to **%Y-%m-%d** (the ISO 8601 date format).
 
     %G
         The ISO 8601 week-based year with century as a decimal number.
@@ -412,10 +467,10 @@ class DatetimeTickFormatter(TickFormatter):
         is used instead.
 
     %g
-        Like %G, but without century, that is, with a 2-digit year (00-99).
+        Like **%G**, but without century, that is, with a 2-digit year (00-99).
 
     %h
-        Equivalent to %b.
+        Equivalent to **%b**.
 
     %H
         The hour as a decimal number using a 24-hour clock (range 00
@@ -430,11 +485,11 @@ class DatetimeTickFormatter(TickFormatter):
 
     %k
         The hour (24-hour clock) as a decimal number (range 0 to 23).
-        Single digits are preceded by a blank.  (See also %H.)
+        Single digits are preceded by a blank. See also **%H**.
 
     %l
         The hour (12-hour clock) as a decimal number (range 1 to 12).
-        Single digits are preceded by a blank.  (See also %I.)  (TZ)
+        Single digits are preceded by a blank. See also **%I**.
 
     %m
         The month as a decimal number (range 01 to 12).
@@ -464,11 +519,11 @@ class DatetimeTickFormatter(TickFormatter):
 
     %r
         The time in a.m. or p.m. notation.  In the POSIX locale this
-        is equivalent to %I:%M:%S %p.
+        is equivalent to **%I:%M:%S %p**.
 
     %R
-        The time in 24-hour notation (%H:%M). For a version including
-        the seconds, see %T below.
+        The time in 24-hour notation (**%H:%M**). For a version including
+        the seconds, see **%T** below.
 
     %s
         The number of seconds since the Epoch, 1970-01-01 00:00:00
@@ -483,7 +538,7 @@ class DatetimeTickFormatter(TickFormatter):
         characters.
 
     %T
-        The time in 24-hour notation (%H:%M:%S).
+        The time in 24-hour notation (**%H:%M:%S**).
 
     %u
         The day of the week as a decimal, range 1 to 7, Monday being 1.
@@ -492,7 +547,7 @@ class DatetimeTickFormatter(TickFormatter):
     %U
         The week number of the current year as a decimal number, range
         00 to 53, starting with the first Sunday as the first day of
-        week 01.  See also %V and %W.
+        week 01.  See also **%V** and **%W**.
 
     %V
         The ISO 8601 week number (see NOTES) of the current year as a
@@ -546,45 +601,77 @@ class DatetimeTickFormatter(TickFormatter):
     .. _github issue: https://github.com/bokeh/bokeh/issues
 
     '''
-    microseconds = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``microseconds``"),
-                        default=['%fus']).accepts(String, lambda fmt: [fmt])
 
-    milliseconds = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``milliseconds``"),
-                        default=['%3Nms', '%S.%3Ns']).accepts(String, lambda fmt: [fmt])
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
-    seconds      = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``seconds``"),
-                        default=['%Ss']).accepts(String, lambda fmt: [fmt])
+    microseconds = String(help=_DATETIME_TICK_FORMATTER_HELP("``microseconds``"),
+                          default="%fus").accepts(List(String), _deprecated_datetime_list_format)
 
-    minsec       = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``minsec`` (for combined minutes and seconds)"),
-                        default=[':%M:%S']).accepts(String, lambda fmt: [fmt])
+    milliseconds = String(help=_DATETIME_TICK_FORMATTER_HELP("``milliseconds``"),
+                          default="%3Nms").accepts(List(String), _deprecated_datetime_list_format)
 
-    minutes      = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``minutes``"),
-                        default=[':%M', '%Mm']).accepts(String, lambda fmt: [fmt])
+    seconds = String(help=_DATETIME_TICK_FORMATTER_HELP("``seconds``"),
+                     default="%Ss").accepts(List(String), _deprecated_datetime_list_format)
 
-    hourmin      = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``hourmin`` (for combined hours and minutes)"),
-                        default=['%H:%M']).accepts(String, lambda fmt: [fmt])
+    minsec = String(help=_DATETIME_TICK_FORMATTER_HELP("``minsec`` (for combined minutes and seconds)"),
+                    default=":%M:%S").accepts(List(String), _deprecated_datetime_list_format)
 
-    hours        = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``hours``"),
-                        default=['%Hh', '%H:%M']).accepts(String, lambda fmt: [fmt])
+    minutes = String(help=_DATETIME_TICK_FORMATTER_HELP("``minutes``"),
+                     default=":%M").accepts(List(String), _deprecated_datetime_list_format)
 
-    days         = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``days``"),
-                        default=['%m/%d', '%a%d']).accepts(String, lambda fmt: [fmt])
+    hourmin = String(help=_DATETIME_TICK_FORMATTER_HELP("``hourmin`` (for combined hours and minutes)"),
+                     default="%H:%M").accepts(List(String), _deprecated_datetime_list_format)
 
-    months       = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``months``"),
-                        default=['%m/%Y', '%b %Y']).accepts(String, lambda fmt: [fmt])
+    hours = String(help=_DATETIME_TICK_FORMATTER_HELP("``hours``"),
+                   default="%Hh").accepts(List(String), _deprecated_datetime_list_format)
 
-    years        = List(String,
-                        help=_DATETIME_TICK_FORMATTER_HELP("``years``"),
-                        default=['%Y']).accepts(String, lambda fmt: [fmt])
+    days = String(help=_DATETIME_TICK_FORMATTER_HELP("``days``"),
+                  default="%m/%d").accepts(List(String), _deprecated_datetime_list_format)
+
+    months = String(help=_DATETIME_TICK_FORMATTER_HELP("``months``"),
+                    default="%m/%Y").accepts(List(String), _deprecated_datetime_list_format)
+
+    years = String(help=_DATETIME_TICK_FORMATTER_HELP("``years``"),
+                   default="%Y").accepts(List(String), _deprecated_datetime_list_format)
+
+    strip_leading_zeros = Bool(default=True, help="Whether to strip any leading zeros in the formatted ticks")
+
+    context = Nullable(Either(String, Instance("bokeh.models.formatters.DatetimeTickFormatter")), default=None, help="""
+    A format for adding context to the tick or ticks specified by ``context_which``.
+    Valid values are:
+
+    * None, no context is added
+    * A standard :class:`~bokeh.models.DatetimeTickFormatter` format string, the single format is
+      used across all scales
+    * Another :class:`~bokeh.models.DatetimeTickFormatter` instance, to have scale-dependent
+      context
+    """)
+
+    context_which = Enum(ContextWhich, default="start", help="""
+    Which tick or ticks to add a formatted context string to. Valid values are:
+    `"start"`, `"end"`, `"center"`, and  `"all"`.
+    """)
+
+    context_location = Enum(LocationType, default="below", help="""
+    Relative to the tick label text baseline, where the context should be
+    rendered. Valid values are: `"below"`, `"above"`, `"left"`, and `"right"`.
+    """)
+
+def RELATIVE_DATETIME_CONTEXT() -> DatetimeTickFormatter:
+    return DatetimeTickFormatter(
+        microseconds = "%T",
+        milliseconds = "%T",
+        seconds = "%H:%M",
+        minsec = "%Hh",
+        minutes = "%Hh",
+        hourmin = "%F",
+        hours = "%F",
+        days = "%Y",
+        months = "",
+        years = "",
+    )
 
 #-----------------------------------------------------------------------------
 # Dev API
@@ -595,10 +682,10 @@ class DatetimeTickFormatter(TickFormatter):
 #-----------------------------------------------------------------------------
 
 # This is to automate documentation of DatetimeTickFormatter formats and their defaults
-_df = DatetimeTickFormatter()
-_df_fields = ['microseconds', 'milliseconds', 'seconds', 'minsec', 'minutes', 'hourmin', 'hours', 'days', 'months', 'years']
-_df_defaults = _df.properties_with_values()
-_df_defaults_string = "\n\n        ".join("%s = %s" % (name, _df_defaults[name]) for name in _df_fields)
+_dttf = DatetimeTickFormatter()
+_dttf_fields = ('microseconds', 'milliseconds', 'seconds', 'minsec', 'minutes', 'hourmin', 'hours', 'days', 'months', 'years')
+_dttf_defaults = _dttf.properties_with_values()
+_dttf_defaults_string = "\n\n        ".join(f"{name} = {_dttf_defaults[name]!r}" for name in _dttf_fields)
 
-DatetimeTickFormatter.__doc__ = format_docstring(DatetimeTickFormatter.__doc__, defaults=_df_defaults_string)
-del _df, _df_fields, _df_defaults, _df_defaults_string
+DatetimeTickFormatter.__doc__ = format_docstring(DatetimeTickFormatter.__doc__, defaults=_dttf_defaults_string)
+del _dttf, _dttf_fields, _dttf_defaults, _dttf_defaults_string
