@@ -12,7 +12,7 @@ with data columns from ``ColumnDataSources``.
 The full list of glyphs is below:
 
 .. toctree::
-   :maxdepth: 2
+   :maxdepth: 1
    :glob:
 
    glyphs/*
@@ -38,9 +38,14 @@ log = logging.getLogger(__name__)
 
 # Bokeh imports
 from ..core.enums import (
+    Align,
     Anchor,
     Direction,
+    HAlign,
+    ImageOrigin,
+    Palette,
     StepMode,
+    VAlign,
     enumeration,
 )
 from ..core.has_props import abstract
@@ -48,31 +53,35 @@ from ..core.properties import (
     AngleSpec,
     Bool,
     DistanceSpec,
+    Either,
     Enum,
     Float,
     Include,
     Instance,
+    InstanceDefault,
     Int,
     MarkerSpec,
     NullDistanceSpec,
     NumberSpec,
     Override,
+    Percent,
     Size,
     SizeSpec,
     String,
     StringSpec,
+    Tuple,
+    field,
 )
-from ..core.property.dataspec import field
 from ..core.property_mixins import (
     FillProps,
     HatchProps,
+    ImageProps,
     LineProps,
     ScalarFillProps,
     ScalarHatchProps,
     ScalarLineProps,
     TextProps,
 )
-from ..util.deprecation import deprecated
 from .glyph import (
     ConnectedXYGlyph,
     FillGlyph,
@@ -93,6 +102,8 @@ __all__ = (
     'Annulus',
     'Arc',
     'Bezier',
+    'Block',
+    'Circle',
     'ConnectedXYGlyph',
     'Ellipse',
     'Glyph',
@@ -106,13 +117,13 @@ __all__ = (
     'Marker',
     'MultiLine',
     'MultiPolygons',
-    'Oval',
     'Patch',
     'Patches',
     'Quad',
     'Quadratic',
     'Ray',
     'Rect',
+    'Scatter',
     'Segment',
     'Step',
     'Text',
@@ -140,6 +151,10 @@ class Marker(XYGlyph, LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     _args = ('x', 'y', 'size', 'angle')
 
     x = NumberSpec(default=field("x"), help="""
@@ -182,6 +197,10 @@ class AnnularWedge(XYGlyph, LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/AnnularWedge.py"
 
     _args = ('x', 'y', 'inner_radius', 'outer_radius', 'start_angle', 'end_angle', 'direction')
@@ -231,6 +250,10 @@ class Annulus(XYGlyph, LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Annulus.py"
 
     _args = ('x', 'y', 'inner_radius', 'outer_radius')
@@ -268,6 +291,10 @@ class Arc(XYGlyph, LineGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Arc.py"
 
     _args = ('x', 'y', 'radius', 'start_angle', 'end_angle', 'direction')
@@ -309,6 +336,10 @@ class Bezier(LineGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Bezier.py"
 
     _args = ("x0", "y0", "x1", "y1", "cx0", "cy0", "cx1", "cy1")
@@ -349,9 +380,54 @@ class Bezier(LineGlyph):
     The {prop} values for the Bezier curves.
     """)
 
+class Block(LineGlyph, FillGlyph, HatchGlyph):
+    ''' Render rectangular regions, given a corner coordinate, width, and height.
+
+    '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    __example__ = "examples/reference/models/Block.py"
+
+    _args = ('x', 'y', 'width', 'height')
+
+    x = NumberSpec(default=field("x"), help="""
+    The x-coordinates of the centers of the blocks.
+    """)
+
+    y = NumberSpec(default=field("y"), help="""
+    The y-coordinates of the centers of the blocks.
+    """)
+
+    width = NumberSpec(default=1, help="""
+    The widths of the blocks.
+    """)
+
+    height = NumberSpec(default=1, help="""
+    The heights of the blocks.
+    """)
+
+    line_props = Include(LineProps, help="""
+    The {prop} values for the blocks.
+    """)
+
+    fill_props = Include(FillProps, help="""
+    The {prop} values for the blocks.
+    """)
+
+    hatch_props = Include(HatchProps, help="""
+    The {prop} values for the blocks.
+    """)
+
 class Circle(Marker):
     ''' Render circle markers. '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Circle.py"
 
     _args = ('x', 'y')
@@ -391,6 +467,10 @@ class Ellipse(XYGlyph, LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Ellipse.py"
 
     _args = ('x', 'y', 'width', 'height', 'angle')
@@ -433,6 +513,10 @@ class HArea(LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/HArea.py"
 
     _args = ('x1', 'x2', 'y')
@@ -463,6 +547,10 @@ class HBar(LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/HBar.py"
 
     _args = ('y', 'height', 'right', 'left')
@@ -500,6 +588,10 @@ class HexTile(LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/HexTile.py"
 
     _args = ('q', 'r')
@@ -515,7 +607,11 @@ class HexTile(LineGlyph, FillGlyph, HatchGlyph):
     the plot to True.
     """)
 
-    aspect_scale = Float(1.0, help="""
+    aspect_scale = Float(default=1.0, help="""
+    Match a plot's aspect ratio scaling.
+
+    Use this parameter to match the aspect ratio scaling of a plot when using
+    :class:`~bokeh.models.Plot.aspect_scale` with a value other than ``1.0``.
 
     """)
 
@@ -531,8 +627,11 @@ class HexTile(LineGlyph, FillGlyph, HatchGlyph):
     A scale factor for individual tiles.
     """)
 
-    orientation = String("pointytop", help="""
+    orientation = String(default="pointytop", help="""
+    The orientation of the hex tiles.
 
+    Use ``"pointytop"`` to orient the tile so that a pointed corner is at the top. Use
+    ``"flattop"`` to orient the tile so that a flat side is at the top.
     """)
 
     line_props = Include(LineProps, help="""
@@ -549,7 +648,61 @@ class HexTile(LineGlyph, FillGlyph, HatchGlyph):
     The {prop} values for the hex tiles.
     """)
 
-class Image(XYGlyph):
+@abstract
+class ImageBase(XYGlyph):
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    x = NumberSpec(default=field("x"), help="""
+    The x-coordinates to locate the image anchors.
+    """)
+
+    y = NumberSpec(default=field("y"), help="""
+    The y-coordinates to locate the image anchors.
+    """)
+
+    dw = DistanceSpec(default=field("dw"), help="""
+    The widths of the plot regions that the images will occupy.
+
+    .. note::
+        This is not the number of pixels that an image is wide.
+        That number is fixed by the image itself.
+    """)
+
+    dh = DistanceSpec(default=field("dh"), help="""
+    The height of the plot region that the image will occupy.
+
+    .. note::
+        This is not the number of pixels that an image is tall.
+        That number is fixed by the image itself.
+    """)
+
+    image_props = Include(ImageProps, help="""
+    The {prop} values for the images.
+    """)
+
+    dilate = Bool(False, help="""
+    Whether to always round fractional pixel locations in such a way
+    as to make the images bigger.
+
+    This setting may be useful if pixel rounding errors are causing
+    images to have a gap between them, when they should appear flush.
+    """)
+
+    origin = Enum(ImageOrigin, default="bottom_left", help="""
+    Defines the coordinate space of an image.
+    """)
+
+    anchor = Either(
+        Enum(Anchor),
+        Tuple(Either(Enum(Align), Enum(HAlign), Percent),
+              Either(Enum(Align), Enum(VAlign), Percent)), default="bottom_left", help="""
+    Position of the image should be anchored at the `x`, `y` coordinates.
+    """)
+
+class Image(ImageBase):
     ''' Render images given as scalar data together with a color mapper.
 
     In addition to the defined model properties, ``Image`` also can accept
@@ -564,7 +717,7 @@ class Image(XYGlyph):
 
     '''
 
-    def __init__(self, **kwargs) -> None:
+    def __init__(self, *args, **kwargs) -> None:
         if 'palette' in kwargs and 'color_mapper' in kwargs:
             raise ValueError("only one of 'palette' and 'color_mapper' may be specified")
         elif 'color_mapper' not in kwargs:
@@ -573,7 +726,7 @@ class Image(XYGlyph):
             mapper = LinearColorMapper(palette)
             kwargs['color_mapper'] = mapper
 
-        super().__init__(**kwargs)
+        super().__init__(*args, **kwargs)
 
     _args = ('image', 'x', 'y', 'dw', 'dh', 'dilate')
 
@@ -589,110 +742,41 @@ class Image(XYGlyph):
     The arrays of scalar data for the images to be colormapped.
     """)
 
-    x = NumberSpec(default=field("x"), help="""
-    The x-coordinates to locate the image anchors.
-    """)
-
-    y = NumberSpec(default=field("y"), help="""
-    The y-coordinates to locate the image anchors.
-    """)
-
-    dw = DistanceSpec(default=field("dw"), help="""
-    The widths of the plot regions that the images will occupy.
-
-    .. note::
-        This is not the number of pixels that an image is wide.
-        That number is fixed by the image itself.
-    """)
-
-    dh = DistanceSpec(default=field("dh"), help="""
-    The height of the plot region that the image will occupy.
-
-    .. note::
-        This is not the number of pixels that an image is tall.
-        That number is fixed by the image itself.
-    """)
-
-    global_alpha = NumberSpec(1.0, help="""
-    An overall opacity that each image is rendered with (in addition
-    to any alpha values applied explicitly in a color mapper).
-    """)
-
-    dilate = Bool(False, help="""
-    Whether to always round fractional pixel locations in such a way
-    as to make the images bigger.
-
-    This setting may be useful if pixel rounding errors are causing
-    images to have a gap between them, when they should appear flush.
-    """)
-
-    color_mapper = Instance(ColorMapper, lambda: LinearColorMapper(palette="Greys9"), help="""
+    color_mapper = Instance(ColorMapper, default=InstanceDefault(LinearColorMapper, palette="Greys9"), help="""
     A ``ColorMapper`` to use to map the scalar data from ``image``
     into RGBA values for display.
 
+    The name of a palette from ``bokeh.palettes`` may also be set, in which
+    case a ``LinearColorMapper`` configured with the named palette wil be used.
+
     .. note::
         The color mapping step happens on the client.
-    """)
+    """).accepts(Enum(Palette), lambda pal: LinearColorMapper(palette=pal))
 
-    # TODO: (bev) support anchor property for Image
-    # ref: https://github.com/bokeh/bokeh/issues/1763
-
-class ImageRGBA(XYGlyph):
+class ImageRGBA(ImageBase):
     ''' Render images given as RGBA data.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     _args = ('image', 'x', 'y', 'dw', 'dh', 'dilate')
 
     image = NumberSpec(default=field("image"), help="""
     The arrays of RGBA data for the images.
     """)
 
-    x = NumberSpec(default=field("x"), help="""
-    The x-coordinates to locate the image anchors.
-    """)
-
-    y = NumberSpec(default=field("y"), help="""
-    The y-coordinates to locate the image anchors.
-    """)
-
-    dw = DistanceSpec(default=field("dw"), help="""
-    The widths of the plot regions that the images will occupy.
-
-    .. note::
-        This is not the number of pixels that an image is wide.
-        That number is fixed by the image itself.
-    """)
-
-    dh = DistanceSpec(default=field("dh"), help="""
-    The height of the plot region that the image will occupy.
-
-    .. note::
-        This is not the number of pixels that an image is tall.
-        That number is fixed by the image itself.
-    """)
-
-    global_alpha = NumberSpec(1.0, help="""
-    An overall opacity that each image is rendered with (in addition
-    to any inherent alpha values in the image itself).
-    """)
-
-    dilate = Bool(False, help="""
-    Whether to always round fractional pixel locations in such a way
-    as to make the images bigger.
-
-    This setting may be useful if pixel rounding errors are causing
-    images to have a gap between them, when they should appear flush.
-    """)
-
-    # TODO: (bev) support anchor property for ImageRGBA
-    # ref: https://github.com/bokeh/bokeh/issues/1763
-
 class ImageURL(XYGlyph):
     ''' Render images loaded from given URLs.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/ImageURL.py"
 
     _args = ('url', 'x', 'y', 'w', 'h', 'angle', 'dilate')
@@ -744,9 +828,8 @@ class ImageURL(XYGlyph):
     images to have a gap between them, when they should appear flush.
     """)
 
-    anchor = Enum(Anchor, help="""
-    What position of the image should be anchored at the `x`, `y`
-    coordinates.
+    anchor = Enum(Anchor, default="top_left", help="""
+    Position of the image should be anchored at the `x`, `y` coordinates.
     """)
 
     retry_attempts = Int(0, help="""
@@ -772,6 +855,11 @@ class Line(ConnectedXYGlyph, LineGlyph):
         :bokeh-issue:`11498` for more information.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     _args = ('x', 'y')
 
     __example__ = "examples/reference/models/Line.py"
@@ -796,6 +884,10 @@ class MultiLine(LineGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/MultiLine.py"
 
     _args = ('xs', 'ys')
@@ -824,6 +916,10 @@ class MultiPolygons(LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/MultiPolygons.py"
 
     _args = ('xs', 'ys')
@@ -858,55 +954,6 @@ class MultiPolygons(LineGlyph, FillGlyph, HatchGlyph):
     The {prop} values for the multipolygons.
     """)
 
-class Oval(XYGlyph, LineGlyph, FillGlyph, HatchGlyph):
-    ''' Render ovals.
-
-    This glyph renders ovals using Bezier curves, which are similar,
-    but not identical to ellipses. In particular, widths equal to heights
-    will not render circles. Use the ``Ellipse`` glyph for that.
-
-    '''
-
-    def __init__(self, **kwargs) -> None:
-        deprecated("'Oval' is deprecated and will be removed in Bokeh 3.0, use the Ellipse glyph instead")
-        super().__init__(**kwargs)
-
-    __example__ = "examples/reference/models/Oval.py"
-
-    _args = ('x', 'y', 'width', 'height', 'angle')
-
-    x = NumberSpec(default=field("x"), help="""
-    The x-coordinates of the centers of the ovals.
-    """)
-
-    y = NumberSpec(default=field("y"), help="""
-    The y-coordinates of the centers of the ovals.
-    """)
-
-    width = DistanceSpec(default=field("width"), help="""
-    The overall widths of each oval.
-    """)
-
-    height = DistanceSpec(default=field("height"), help="""
-    The overall height of each oval.
-    """)
-
-    angle = AngleSpec(default=0.0, help="""
-    The angle the ovals are rotated from horizontal. [rad]
-    """)
-
-    line_props = Include(LineProps, help="""
-    The {prop} values for the ovals.
-    """)
-
-    fill_props = Include(FillProps, help="""
-    The {prop} values for the ovals.
-    """)
-
-    hatch_props = Include(HatchProps, help="""
-    The {prop} values for the ovals.
-    """)
-
 class Patch(ConnectedXYGlyph, LineGlyph, FillGlyph, HatchGlyph):
     ''' Render a single patch.
 
@@ -915,6 +962,10 @@ class Patch(ConnectedXYGlyph, LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Patch.py"
 
     _args = ('x', 'y')
@@ -960,6 +1011,10 @@ class Patches(LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Patches.py"
 
     _args = ('xs', 'ys')
@@ -999,6 +1054,10 @@ class Quad(LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Quad.py"
 
     _args = ('left', 'right', 'top', 'bottom')
@@ -1036,6 +1095,10 @@ class Quadratic(LineGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Quadratic.py"
 
     _args = ("x0", "y0", "x1", "y1", "cx", "cy")
@@ -1073,6 +1136,10 @@ class Ray(XYGlyph, LineGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Ray.py"
 
     _args = ('x', 'y', 'length', 'angle')
@@ -1103,6 +1170,10 @@ class Rect(XYGlyph, LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Rect.py"
 
     _args = ('x', 'y', 'width', 'height', 'angle', 'dilate')
@@ -1163,7 +1234,7 @@ class Scatter(Marker):
     Bokeh's built-in markers consist of a set of base markers, most of which can
     be combined with different kinds of additional visual features:
 
-    .. bokeh-plot:: docs/user_guide/examples/plotting_markertypes.py
+    .. bokeh-plot:: __REPO__/examples/basic/scatters/markertypes.py
         :source-position: none
 
     You can select marker types in two ways:
@@ -1202,6 +1273,10 @@ class Scatter(Marker):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Scatter.py"
 
     _args = ('x', 'y', 'size', 'angle', 'marker')
@@ -1216,6 +1291,10 @@ class Segment(LineGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Segment.py"
 
     _args = ('x0', 'y0', 'x1', 'y1')
@@ -1251,6 +1330,10 @@ class Step(XYGlyph, LineGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Step.py"
 
     _args = ('x', 'y')
@@ -1281,6 +1364,10 @@ class Text(XYGlyph, TextGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Text.py"
 
     _args = ('x', 'y', 'text', 'angle', 'x_offset', 'y_offset')
@@ -1325,6 +1412,10 @@ class VArea(FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/VArea.py"
 
     _args = ('x', 'y1', 'y2')
@@ -1354,6 +1445,10 @@ class VBar(LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/VBar.py"
 
     _args = ('x', 'width', 'top', 'bottom')
@@ -1391,6 +1486,10 @@ class Wedge(XYGlyph, LineGlyph, FillGlyph, HatchGlyph):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     __example__ = "examples/reference/models/Wedge.py"
 
     _args = ('x', 'y', 'radius', 'start_angle', 'end_angle', 'direction')
