@@ -55,6 +55,11 @@ class Grid(GuideRenderer):
     given by a supplied ``Ticker``.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     dimension = Int(0, help="""
     Which dimension the Axis Grid lines will intersect. The
     x-axis is dimension 0 (vertical Grid lines) and the y-axis
@@ -67,6 +72,11 @@ class Grid(GuideRenderer):
     will span the entire visible range.
     """)
 
+    cross_bounds = Either(Auto, Tuple(Float, Float), help="""
+    Bounds for the rendered grid lines in the orthogonal direction. By default,
+    a grid will span the entire visible range.
+    """)
+
     axis = Nullable(Instance(Axis), help="""
     An Axis to delegate ticking to. If the ticker property is None, then the
     Grid will use the ticker on the specified axis for computing where to draw
