@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -23,22 +25,20 @@ from ..core.enums import MapType
 from ..core.has_props import abstract
 from ..core.properties import (
     JSON,
-    Base64String,
     Bool,
+    Bytes,
     Enum,
     Float,
     Instance,
+    InstanceDefault,
     Int,
-    NonNullable,
+    Nullable,
     Override,
+    Required,
     String,
 )
 from ..core.validation import error, warning
-from ..core.validation.errors import (
-    INCOMPATIBLE_MAP_RANGE_TYPE,
-    MISSING_GOOGLE_API_KEY,
-    REQUIRED_RANGE,
-)
+from ..core.validation.errors import INCOMPATIBLE_MAP_RANGE_TYPE, MISSING_GOOGLE_API_KEY, REQUIRED_RANGE
 from ..core.validation.warnings import MISSING_RENDERERS
 from ..model import Model
 from ..models.ranges import Range1d
@@ -65,11 +65,15 @@ class MapOptions(Model):
 
     '''
 
-    lat = NonNullable(Float, help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    lat = Required(Float, help="""
     The latitude where the map should be centered.
     """)
 
-    lng = NonNullable(Float, help="""
+    lng = Required(Float, help="""
     The longitude where the map should be centered.
     """)
 
@@ -83,12 +87,12 @@ class MapPlot(Plot):
 
     '''
 
-    def __init__(self, *args, **kw):
+    def __init__(self, *args, **kwargs) -> None:
         from ..models.ranges import Range1d
         for r in ('x_range', 'y_range'):
-            if r in kw and not isinstance(kw.get(r), Range1d):
-                raise ValueError('Invalid value for %r, MapPlot ranges may only be Range1d, not data ranges' % r)
-        super().__init__(*args, **kw)
+            if r in kwargs and not isinstance(kwargs.get(r), Range1d):
+                raise ValueError(f"Invalid value for {r!r}, MapPlot ranges may only be Range1d, not data ranges")
+        super().__init__(*args, **kwargs)
 
     @error(INCOMPATIBLE_MAP_RANGE_TYPE)
     def _check_incompatible_map_range_type(self):
@@ -103,6 +107,10 @@ class GMapOptions(MapOptions):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     map_type = Enum(MapType, default="roadmap", help="""
     The `map type`_ to use for the ``GMapPlot``.
 
@@ -114,7 +122,7 @@ class GMapOptions(MapOptions):
     Whether the Google map should display its distance scale control.
     """)
 
-    styles = NonNullable(JSON, help="""
+    styles = Nullable(JSON, default=None, help="""
     A JSON array of `map styles`_ to use for the ``GMapPlot``. Many example styles can
     `be found here`_.
 
@@ -157,6 +165,10 @@ class GMapPlot(MapPlot):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     # TODO (bev) map plot might not have these
     @error(REQUIRED_RANGE)
     def _check_required_range(self):
@@ -177,12 +189,14 @@ class GMapPlot(MapPlot):
 
     border_fill_color = Override(default="#ffffff")
 
-    api_key = NonNullable(Base64String, help="""
+    background_fill_alpha = Override(default=0.0)
+
+    api_key = Required(Bytes, help="""
     Google Maps API requires an API key. See https://developers.google.com/maps/documentation/javascript/get-api-key
     for more information on how to obtain your own.
-    """)
+    """).accepts(String, lambda val: val.encode("utf-8"))
 
-    api_version = String(default="3.43", help="""
+    api_version = String(default="weekly", help="""
     The version of Google Maps API to use. See https://developers.google.com/maps/documentation/javascript/versions
     for more information.
 
@@ -191,9 +205,9 @@ class GMapPlot(MapPlot):
 
     """)
 
-    x_range = Override(default=lambda: Range1d())
+    x_range = Override(default=InstanceDefault(Range1d))
 
-    y_range = Override(default=lambda: Range1d())
+    y_range = Override(default=InstanceDefault(Range1d))
 
 #-----------------------------------------------------------------------------
 # Dev API
