@@ -21,16 +21,16 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-import warnings
+from contextlib import contextmanager
 from typing import (
-    TYPE_CHECKING,
     Any,
-    List as TList,
+    Generator,
+    Literal,
     overload,
 )
 
 # External imports
-from typing_extensions import Literal
+import xyzservices
 
 # Bokeh imports
 from ..core.enums import (
@@ -41,7 +41,6 @@ from ..core.enums import (
     ResetPolicy,
 )
 from ..core.properties import (
-    Alias,
     Bool,
     Dict,
     Either,
@@ -49,6 +48,7 @@ from ..core.properties import (
     Float,
     Include,
     Instance,
+    InstanceDefault,
     Int,
     List,
     Null,
@@ -56,29 +56,29 @@ from ..core.properties import (
     Override,
     Readonly,
     String,
+    Struct,
+    Tuple,
 )
+from ..core.property.struct import Optional
 from ..core.property_mixins import ScalarFillProps, ScalarLineProps
 from ..core.query import find
 from ..core.validation import error, warning
 from ..core.validation.errors import (
     BAD_EXTRA_RANGE_NAME,
     INCOMPATIBLE_SCALE_AND_RANGE,
+    REPEATED_LAYOUT_CHILD,
     REQUIRED_RANGE,
     REQUIRED_SCALE,
 )
-from ..core.validation.warnings import (
-    FIXED_HEIGHT_POLICY,
-    FIXED_SIZING_MODE,
-    FIXED_WIDTH_POLICY,
-    MISSING_RENDERERS,
-)
+from ..core.validation.warnings import MISSING_RENDERERS
 from ..model import Model
-from ..util.string import nice_join
+from ..util.strings import nice_join
+from ..util.warnings import warn
 from .annotations import Annotation, Legend, Title
 from .axes import Axis
 from .glyphs import Glyph
 from .grids import Grid
-from .layouts import LayoutDOM
+from .layouts import LayoutDOM, TracksSizing
 from .ranges import (
     DataRange1d,
     FactorRange,
@@ -93,19 +93,21 @@ from .scales import (
     Scale,
 )
 from .sources import ColumnarDataSource, ColumnDataSource, DataSource
+from .tiles import TileSource, WMTSTileSource
 from .tools import HoverTool, Tool, Toolbar
 
-if TYPE_CHECKING:
-    from .tiles import TileSource
-
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
 __all__ = (
+    'GridPlot',
     'Plot',
 )
 
+def LRTB(type: Any) -> Struct:
+    return Struct(left=type, right=type, top=type, bottom=type)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -115,6 +117,10 @@ class Plot(LayoutDOM):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     def select(self, *args, **kwargs):
         ''' Query this object and all of its references for objects that
         match the given selector.
@@ -269,11 +275,11 @@ class Plot(LayoutDOM):
         return _list_attr_splat(self.xgrid + self.ygrid)
 
     @property
-    def tools(self) -> TList[Tool]:
+    def tools(self) -> list[Tool]:
         return self.toolbar.tools
 
     @tools.setter
-    def tools(self, tools: TList[Tool]):
+    def tools(self, tools: list[Tool]):
         self.toolbar.tools = tools
 
     def add_layout(self, obj: Renderer, place: PlaceType = "center") -> None:
@@ -295,7 +301,7 @@ class Plot(LayoutDOM):
 
         getattr(self, place).append(obj)
 
-    def add_tools(self, *tools: Tool) -> None:
+    def add_tools(self, *tools: Tool | str) -> None:
         ''' Adds tools to the plot.
 
         Args:
@@ -306,10 +312,31 @@ class Plot(LayoutDOM):
 
         '''
         for tool in tools:
-            if not isinstance(tool, Tool):
-                raise ValueError("All arguments to add_tool must be Tool subclasses.")
+            if isinstance(tool, str):
+                tool_obj = Tool.from_string(tool)
+            elif isinstance(tool, Tool):
+                tool_obj = tool
+            else:
+                raise ValueError(f"expected a string or Tool instance, got {tool!r}")
 
-            self.toolbar.tools.append(tool)
+            self.toolbar.tools.append(tool_obj)
+
+    def remove_tools(self, *tools: Tool) -> None:
+        ''' Removes tools from the plot.
+
+        Args:
+            *tools (Tool) : the tools to remove from the Plot
+
+        Returns:
+            None
+
+        '''
+        for tool in tools:
+            if not isinstance(tool, Tool):
+                raise ValueError("All arguments to remove_tool must be Tool subclasses.")
+            elif tool not in self.toolbar.tools:
+                raise ValueError(f"Invalid tool {tool} specified. Available tools are {nice_join(self.toolbar.tools)}")
+            self.toolbar.tools.remove(tool)
 
     @overload
     def add_glyph(self, glyph: Glyph, **kwargs: Any) -> GlyphRenderer: ...
@@ -350,11 +377,15 @@ class Plot(LayoutDOM):
         self.renderers.append(g)
         return g
 
-    def add_tile(self, tile_source: TileSource, **kwargs: Any) -> TileRenderer:
+    def add_tile(self, tile_source: TileSource | xyzservices.TileProvider | str, retina: bool = False, **kwargs: Any) -> TileRenderer:
         ''' Adds new ``TileRenderer`` into ``Plot.renderers``
 
         Args:
-            tile_source (TileSource) : a tile source instance which contain tileset configuration
+            tile_source (TileSource, xyzservices.TileProvider, str) :
+                A tile source instance which contain tileset configuration
+
+            retina (bool) :
+                Whether to use retina version of tiles (if available)
 
         Keyword Arguments:
             Additional keyword arguments are passed on as-is to the tile renderer
@@ -363,13 +394,55 @@ class Plot(LayoutDOM):
             TileRenderer : TileRenderer
 
         '''
+        if not isinstance(tile_source, TileSource):
+
+            if isinstance(tile_source, xyzservices.TileProvider):
+                selected_provider = tile_source
+
+            # allow the same string input you can now pass to get_provider
+            elif isinstance(tile_source, str):
+                # Mapping of custom keys to those used in xyzservices
+                tile_source = tile_source.lower()
+
+                if tile_source == "esri_imagery":
+                    tile_source = "esri_worldimagery"
+                if tile_source == "osm":
+                    tile_source = "openstreetmap_mapnik"
+
+                if "retina" in tile_source:
+                    tile_source = tile_source.replace("retina", "")
+                    retina = True
+                selected_provider = xyzservices.providers.query_name(tile_source)
+
+            scale_factor = "@2x" if retina else None
+
+            tile_source = WMTSTileSource(
+                url=selected_provider.build_url(scale_factor=scale_factor),
+                attribution=selected_provider.html_attribution,
+                min_zoom=selected_provider.get("min_zoom", 0),
+                max_zoom=selected_provider.get("max_zoom", 30),
+            )
+
         tile_renderer = TileRenderer(tile_source=tile_source, **kwargs)
         self.renderers.append(tile_renderer)
         return tile_renderer
 
+    @contextmanager
+    def hold(self, *, render: bool) -> Generator[None, None, None]:
+        ''' Takes care of turning a property on and off within a scope.
+
+        Args:
+            render (bool) :
+                Turns the property hold_render on and off.
+        '''
+        if render:
+            self.hold_render = True
+            yield
+            self.hold_render = False
+
     @error(REQUIRED_RANGE)
     def _check_required_range(self) -> str | None:
-        missing: TList[str] = []
+        missing: list[str] = []
         if not self.x_range: missing.append('x_range')
         if not self.y_range: missing.append('y_range')
         if missing:
@@ -377,7 +450,7 @@ class Plot(LayoutDOM):
 
     @error(REQUIRED_SCALE)
     def _check_required_scale(self) -> str | None:
-        missing: TList[str] = []
+        missing: list[str] = []
         if not self.x_scale: missing.append('x_scale')
         if not self.y_scale: missing.append('y_scale')
         if missing:
@@ -385,7 +458,7 @@ class Plot(LayoutDOM):
 
     @error(INCOMPATIBLE_SCALE_AND_RANGE)
     def _check_compatible_scale_and_ranges(self) -> str | None:
-        incompatible: TList[str] = []
+        incompatible: list[str] = []
         x_ranges = list(self.extra_x_ranges.values())
         if self.x_range: x_ranges.append(self.x_range)
         y_ranges = list(self.extra_y_ranges.values())
@@ -438,11 +511,11 @@ class Plot(LayoutDOM):
         if msg:
             return msg
 
-    x_range = Instance(Range, default=lambda: DataRange1d(), help="""
+    x_range = Instance(Range, default=InstanceDefault(DataRange1d), help="""
     The (default) data range of the horizontal dimension of the plot.
     """)
 
-    y_range = Instance(Range, default=lambda: DataRange1d(), help="""
+    y_range = Instance(Range, default=InstanceDefault(DataRange1d), help="""
     The (default) data range of the vertical dimension of the plot.
     """)
 
@@ -457,12 +530,12 @@ class Plot(LayoutDOM):
         else:
             raise ValueError(f"Unknown mapper_type: {scale}")
 
-    x_scale = Instance(Scale, default=lambda: LinearScale(), help="""
+    x_scale = Instance(Scale, default=InstanceDefault(LinearScale), help="""
     What kind of scale to use to convert x-coordinates in data space
     into x-coordinates in screen space.
     """)
 
-    y_scale = Instance(Scale, default=lambda: LinearScale(), help="""
+    y_scale = Instance(Scale, default=InstanceDefault(LinearScale), help="""
     What kind of scale to use to convert y-coordinates in data space
     into y-coordinates in screen space.
     """)
@@ -499,7 +572,7 @@ class Plot(LayoutDOM):
     Whether to use HiDPI mode when available.
     """)
 
-    title = Either(Null, Instance(Title), default=lambda: Title(text=""), help="""
+    title = Either(Null, Instance(Title), default=InstanceDefault(Title, text=""), help="""
     A title for the plot. Can be a text string or a Title annotation.
     """).accepts(String, lambda text: Title(text=text))
 
@@ -515,15 +588,13 @@ class Plot(LayoutDOM):
     outline_line_color = Override(default="#e5e5e5")
 
     renderers = List(Instance(Renderer), help="""
-    A list of all renderers for this plot, including guides and annotations
-    in addition to glyphs.
+    A list of all glyph renderers for this plot.
 
-    This property can be manipulated by hand, but the ``add_glyph`` and
-    ``add_layout`` methods are recommended to help make sure all necessary
-    setup is performed.
+    This property can be manipulated by hand, but the ``add_glyph`` is
+    recommended to help make sure all necessary setup is performed.
     """)
 
-    toolbar = Instance(Toolbar, default=lambda: Toolbar(), help="""
+    toolbar = Instance(Toolbar, default=InstanceDefault(Toolbar), help="""
     The toolbar associated with this plot which holds all the tools. It is
     automatically created with the plot if necessary.
     """)
@@ -538,6 +609,11 @@ class Plot(LayoutDOM):
     the toolbar will be outside of the axes, titles etc.
     """)
 
+    toolbar_inner = Bool(default=False, help="""
+    Locate the toolbar inside the frame. Setting this property to ``True``
+    makes most sense with auto-hidden toolbars.
+    """)
+
     left = List(Instance(Renderer), help="""
     A list of renderers to occupy the area to the left of the plot.
     """)
@@ -562,14 +638,6 @@ class Plot(LayoutDOM):
 
     height: int | None = Override(default=600)
 
-    plot_width: int | None = Alias("width", help="""
-    The outer width of a plot, including any axes, titles, border padding, etc.
-    """)
-
-    plot_height: int | None = Alias("height", help="""
-    The outer height of a plot, including any axes, titles, border padding, etc.
-    """)
-
     frame_width = Nullable(Int, help="""
     The width of a plot frame or the inner width of a plot, excluding any
     axes, titles, border padding, etc.
@@ -580,6 +648,15 @@ class Plot(LayoutDOM):
     axes, titles, border padding, etc.
     """)
 
+    frame_align = Either(Bool, LRTB(Optional(Bool)), default=True, help="""
+    Allows to specify which frame edges to align in multiple-plot layouts.
+
+    The default is to align all edges, but users can opt-out from alignment
+    of each individual edge or all edges. Note also that other proproperties
+    may disable alignment of certain edges, especially when using fixed frame
+    size (``frame_width`` and ``frame_height`` properties).
+    """)
+
     inner_width = Readonly(Int, help="""
     This is the exact width of the plotting canvas, i.e. the width of
     the actual plot, without toolbars etc. Note this is computed in a
@@ -756,18 +833,80 @@ class Plot(LayoutDOM):
     suppress all of the actions except the Reset event.
     """)
 
-    # XXX: override LayoutDOM's definitions because of plot_{width,height}.
-    @error(FIXED_SIZING_MODE)
-    def _check_fixed_sizing_mode(self):
-        pass
+    hold_render = Bool(default=False, help="""
+    When set to True all requests to repaint the plot will be hold off.
 
-    @error(FIXED_WIDTH_POLICY)
-    def _check_fixed_width_policy(self):
-        pass
+    This is useful when periodically updating many glyphs. For example, let's
+    assume we have 10 lines on a plot, each with its own datasource. We stream
+    to all of them every second in a for loop like so:
 
-    @error(FIXED_HEIGHT_POLICY)
-    def _check_fixed_height_policy(self):
-        pass
+    .. code:: python
+
+        for line in lines:
+            line.stream(new_points())
+
+    The problem with this code is that every stream triggers a re-rendering of
+    the plot. Even tough repainting only on the last stream would produce almost
+    identical visual effect. Especially for lines with many points this becomes
+    computationally expensive and can freeze your browser. Using a convenience
+    method `hold`, we can control when rendering is initiated like so:
+
+    .. code:: python
+
+        with plot.hold(render=True):
+            for line in lines:
+                line.stream(new_points())
+
+    In this case we render newly appended points only after the last stream.
+    """)
+
+class GridPlot(LayoutDOM):
+    """
+
+    """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    toolbar = Instance(Toolbar, default=InstanceDefault(Toolbar), help="""
+    The toolbar associated with this grid plot, which holds all the tools.
+    It is automatically created with the plot if necessary.
+    """)
+
+    toolbar_location = Nullable(Enum(Location), default="above", help="""
+    Indicates where the layout the toolbar will be located. If set to None,
+    no toolbar will be attached to the grid plot.
+    """)
+
+    children = List(Either(
+        Tuple(Instance(LayoutDOM), Int, Int),
+        Tuple(Instance(LayoutDOM), Int, Int, Int, Int)), default=[], help="""
+    A list of subplots with their associated position in the grid, row and column
+    index and optional row and column spans (the defaul span is 1).
+    """)
+
+    rows = Nullable(TracksSizing, default="max-content", help="""
+    Describes how the grid should maintain its rows' heights.
+    """)
+
+    cols = Nullable(TracksSizing, default="max-content", help="""
+    Describes how the grid should maintain its columns' widths.
+    """)
+
+    spacing = Either(Int, Tuple(Int, Int), default=0, help="""
+    The gap between children (in pixels).
+
+    Either a number, if spacing is the same for both dimensions, or a pair
+    of numbers indicating spacing in the vertical and horizontal dimensions
+    respectively.
+    """)
+
+    @error(REPEATED_LAYOUT_CHILD)
+    def _check_repeated_layout_children(self):
+        children = [ child[0] for child in self.children ]
+        if len(children) != len(set(children)):
+            return str(self)
 
 #-----------------------------------------------------------------------------
 # Dev API
@@ -812,7 +951,7 @@ Before legend properties can be set, you must add a Legend explicitly, or call a
 class _legend_attr_splat(_list_attr_splat):
     def __setattr__(self, attr, value):
         if not len(self):
-            warnings.warn(_LEGEND_EMPTY_WARNING % attr)
+            warn(_LEGEND_EMPTY_WARNING % attr)
         return super().__setattr__(attr, value)
 
 def _select_helper(args, kwargs):
