@@ -42,7 +42,6 @@ from ..core.properties import (
     Null,
     Nullable,
     Readonly,
-    String,
     TimeDelta,
 )
 from ..core.validation import error
@@ -71,6 +70,11 @@ class Range(Model):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+
 class Range1d(Range):
     ''' A fixed, closed range [start, end] in a continuous scalar
     dimension.
@@ -83,6 +87,18 @@ class Range1d(Range):
 
     '''
 
+    def __init__(self, *args, **kwargs) -> None:
+        if args and ('start' in kwargs or 'end' in kwargs):
+            raise ValueError("'start' and 'end' keywords cannot be used with positional arguments")
+        if args and len(args) != 2:
+            raise ValueError('Only Range1d(start, end) acceptable when using positional arguments')
+
+        if args:
+            kwargs['start'] = args[0]
+            kwargs['end'] = args[1]
+
+        super().__init__(**kwargs)
+
     start = Either(Float, Datetime, TimeDelta, default=0, help="""
     The start of the range.
     """)
@@ -133,18 +149,6 @@ class Range1d(Range):
     maximum visible interval. Can be a ``TimeDelta``. Note that ``bounds`` can
     impose an implicit constraint on the maximum interval as well. """)
 
-    def __init__(self, *args, **kwargs) -> None:
-        if args and ('start' in kwargs or 'end' in kwargs):
-            raise ValueError("'start' and 'end' keywords cannot be used with positional arguments")
-        if args and len(args) != 2:
-            raise ValueError('Only Range1d(start, end) acceptable when using positional arguments')
-
-        if args:
-            kwargs['start'] = args[0]
-            kwargs['end'] = args[1]
-
-        super().__init__(**kwargs)
-
 
 @abstract
 class DataRange(Range):
@@ -152,15 +156,9 @@ class DataRange(Range):
 
     '''
 
-    names = List(String, help="""
-    A list of names to query for. If set, only renderers that
-    have a matching value for their ``name`` attribute will be used
-    for autoranging.
-
-    .. note:
-        This property is deprecated and will be removed in bokeh 3.0.
-
-    """)
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
     renderers = Either(List(Instance(Model)), Auto, help="""
     An explicit list of renderers to autorange against. If unset,
@@ -176,6 +174,11 @@ class DataRange1d(DataRange):
 
     '''
 
+    def __init__(self, *args, **kwargs) -> None:
+        if kwargs.get('follow') is not None:
+            kwargs['bounds'] = None
+        super().__init__(*args, **kwargs)
+
     range_padding = Either(Float, TimeDelta, default=0.1, help="""
     How much padding to add around the computed data bounds.
 
@@ -191,12 +194,12 @@ class DataRange1d(DataRange):
     as an absolute quantity. (default: ``"percent"``)
     """)
 
-    start = Either(Null, Float, Datetime, TimeDelta, help="""
+    start = Either(Float, Datetime, TimeDelta, default=float("nan"), help="""
     An explicitly supplied range start. If provided, will override
     automatically computed start value.
     """)
 
-    end = Either(Null, Float, Datetime, TimeDelta, help="""
+    end = Either(Float, Datetime, TimeDelta, default=float("nan"), help="""
     An explicitly supplied range end. If provided, will override
     automatically computed end value.
     """)
@@ -272,11 +275,6 @@ class DataRange1d(DataRange):
     bounds computations.
     """)
 
-    def __init__(self, *args, **kwargs) -> None:
-        if kwargs.get('follow') is not None:
-            kwargs['bounds'] = None
-        super().__init__(**kwargs)
-
 
 class FactorRange(Range):
     ''' A Range of values for a categorical dimension.
@@ -402,6 +400,7 @@ class FactorRange(Range):
     start = Readonly(Float, help="""
     The start of the range, in synthetic coordinates.
 
+    .. note::
         Synthetic coordinates are only computed in the browser, based on the
         factors and various padding properties. The value of ``start`` will only
         be available in situations where bidirectional communication is
