@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -18,25 +18,19 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-import warnings
 from typing import (
     TYPE_CHECKING,
     Any as TAny,
     Dict as TDict,
     List as TList,
     Sequence,
-    Set,
     Tuple,
     Union,
     overload,
 )
 
-## External imports
-if TYPE_CHECKING:
-    import pandas as pd
-else:
-    from ..util.dependencies import import_optional
-    pd = import_optional("pandas")
+# External imports
+import numpy as np
 
 # Bokeh imports
 from ..core.has_props import abstract
@@ -48,26 +42,28 @@ from ..core.properties import (
     Dict,
     Enum,
     Instance,
+    InstanceDefault,
     Int,
-    List,
-    NonNullable,
     Nullable,
-    PandasDataFrame,
-    PandasGroupBy,
+    Object,
     Readonly,
+    Required,
     Seq,
     String,
 )
 from ..model import Model
+from ..util.deprecation import deprecated
 from ..util.serialization import convert_datetime_array
-from ..util.warnings import BokehUserWarning
+from ..util.warnings import BokehUserWarning, warn
 from .callbacks import CustomJS
-from .filters import Filter
+from .filters import AllIndices, Filter, IntersectionFilter
 from .selections import Selection, SelectionPolicy, UnionRenderers
 
 if TYPE_CHECKING:
+    import pandas as pd
+    from typing_extensions import TypeAlias
+
     from ..core.has_props import Setter
-    from ..core.types import Unknown
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -82,7 +78,6 @@ __all__ = (
     'GeoJSONDataSource',
     'ServerSentDataSource',
     'WebDataSource',
-    'WebSource',
 )
 
 #-----------------------------------------------------------------------------
@@ -90,11 +85,13 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 if TYPE_CHECKING:
-    DataDict = TDict[str, Sequence[Unknown]]
+    import numpy.typing as npt
 
-    Index = Union[int, slice, Tuple[Union[int, slice], ...]]
+    DataDict: TypeAlias = TDict[str, Union[Sequence[TAny], npt.NDArray[TAny], pd.Series, pd.Index]]
 
-    Patches = TDict[str, TList[Tuple[Index, Unknown]]]
+    Index: TypeAlias = Union[int, slice, Tuple[Union[int, slice], ...]]
+
+    Patches: TypeAlias = TDict[str, TList[Tuple[Index, Any]]]
 
 @abstract
 class DataSource(Model):
@@ -102,7 +99,11 @@ class DataSource(Model):
 
     '''
 
-    selected = Readonly(Instance(Selection), default=lambda: Selection(), help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    selected = Readonly(Instance(Selection), default=InstanceDefault(Selection), help="""
     An instance of a ``Selection`` that indicates selected indices on this ``DataSource``.
     This is a read-only property. You may only change the attributes of this object
     to change the selection (e.g., ``selected.indices``).
@@ -115,7 +116,11 @@ class ColumnarDataSource(DataSource):
 
     '''
 
-    selection_policy = Instance(SelectionPolicy, default=lambda: UnionRenderers(), help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    selection_policy = Instance(SelectionPolicy, default=InstanceDefault(UnionRenderers), help="""
     An instance of a ``SelectionPolicy`` that determines how selections are set.
     """)
 
@@ -198,11 +203,11 @@ class ColumnDataSource(ColumnarDataSource):
     objects. In these cases, the behaviour is identical to passing the objects
     to the ``ColumnDataSource`` initializer.
     """).accepts(
-        PandasDataFrame, lambda x: ColumnDataSource._data_from_df(x)
+        Object("pandas.DataFrame"), lambda x: ColumnDataSource._data_from_df(x)
     ).accepts(
-        PandasGroupBy, lambda x: ColumnDataSource._data_from_groupby(x)
+        Object("pandas.core.groupby.GroupBy"), lambda x: ColumnDataSource._data_from_groupby(x)
     ).asserts(lambda _, data: len({len(x) for x in data.values()}) <= 1,
-                 lambda obj, name, data: warnings.warn(
+                 lambda obj, name, data: warn(
                     "ColumnDataSource's columns must be of the same length. " +
                     "Current lengths: %s" % ", ".join(sorted(str((k, len(v))) for k, v in data.items())), BokehUserWarning))
 
@@ -222,10 +227,11 @@ class ColumnDataSource(ColumnarDataSource):
         # TODO (bev) invalid to pass args and "data", check and raise exception
         raw_data: DataDict = kwargs.pop("data", {})
 
+        import pandas as pd
         if not isinstance(raw_data, dict):
-            if pd and isinstance(raw_data, pd.DataFrame):
+            if isinstance(raw_data, pd.DataFrame):
                 raw_data = self._data_from_df(raw_data)
-            elif pd and isinstance(raw_data, pd.core.groupby.GroupBy):
+            elif isinstance(raw_data, pd.core.groupby.GroupBy):
                 raw_data = self._data_from_groupby(raw_data)
             else:
                 raise ValueError(f"expected a dict or pandas.DataFrame, got {raw_data}")
@@ -233,7 +239,7 @@ class ColumnDataSource(ColumnarDataSource):
         self.data.update(raw_data)
 
     @property
-    def column_names(self) -> TList[str]:
+    def column_names(self) -> list[str]:
         ''' A list of the column names in this data source.
 
         '''
@@ -251,6 +257,8 @@ class ColumnDataSource(ColumnarDataSource):
             dict[str, np.array]
 
         '''
+        import pandas as pd
+
         _df = df.copy()
 
         # Flatten columns
@@ -365,11 +373,10 @@ class ColumnDataSource(ColumnarDataSource):
             DataFrame
 
         '''
-        if not pd:
-            raise RuntimeError('Pandas must be installed to convert to a Pandas Dataframe')
+        import pandas as pd
         return pd.DataFrame(self.data)
 
-    def add(self, data: Sequence[Unknown], name: str | None = None) -> str:
+    def add(self, data: Sequence[Any], name: str | None = None) -> str:
         ''' Appends a new column of data to the data source.
 
         Args:
@@ -405,7 +412,7 @@ class ColumnDataSource(ColumnarDataSource):
         try:
             del self.data[name]
         except (ValueError, KeyError):
-            warnings.warn("Unable to find column '%s' in data source" % name)
+            warn(f"Unable to find column '{name}' in data source")
 
     def stream(self, new_data: DataDict, rollover: int | None = None) -> None:
         ''' Efficiently update data source columns with new append-only data.
@@ -501,9 +508,11 @@ class ColumnDataSource(ColumnarDataSource):
             source.stream(new_data)
 
         '''
+        import pandas as pd
+
         needs_length_check = True
 
-        if pd and isinstance(new_data, (pd.Series, pd.DataFrame)):
+        if isinstance(new_data, (pd.Series, pd.DataFrame)):
             if isinstance(new_data, pd.Series):
                 new_data = new_data.to_frame().T
 
@@ -531,10 +540,9 @@ class ColumnDataSource(ColumnarDataSource):
             else:
                 raise ValueError("Must stream updates to all existing columns (extra: %s)" % ", ".join(sorted(extra)))
 
-        import numpy as np
         if needs_length_check:
-            lengths: Set[int] = set()
-            arr_types = (np.ndarray, pd.Series) if pd else np.ndarray
+            lengths: set[int] = set()
+            arr_types = (np.ndarray, pd.Series)
             for _, x in new_data.items():
                 if isinstance(x, arr_types):
                     if len(x.shape) != 1:
@@ -655,11 +663,9 @@ class ColumnDataSource(ColumnarDataSource):
 
             dict(foo=[11, 12, 30], bar=[101, 200, 301])
 
-        For a more comprehensive complete example, see :bokeh-tree:`examples/howto/patch_app.py`.
+        For a more comprehensive example, see :bokeh-tree:`examples/server/app/patch_app.py`.
 
         '''
-        import numpy as np
-
         extra = set(patches.keys()) - set(self.data.keys())
 
         if extra:
@@ -725,21 +731,58 @@ class CDSView(Model):
 
     '''
 
-    filters = List(Instance(Filter), default=[], help="""
-    List of filters that the view comprises.
+    def __init__(self, *args: TAny, **kwargs: TAny) -> None:
+        if "source" in kwargs:
+            del kwargs["source"]
+            deprecated("CDSView.source is no longer needed, and is now ignored. In a future release, passing source will result an error.")
+        super().__init__(*args, **kwargs)
+
+    filter = Instance(Filter, default=InstanceDefault(AllIndices), help="""
+    Defines the subset of indices to use from the data source this view applies to.
+
+    By default all indices are used (``AllIndices`` filter). This can be changed by
+    using specialized filters like ``IndexFilter``, ``BooleanFilter``, etc. Filters
+    can be composed using set operations to create non-trivial data masks. This can
+    be accomplished by directly using models like ``InversionFilter``, ``UnionFilter``,
+    etc., or by using set operators on filters, e.g.:
+
+    .. code-block:: python
+
+        # filters everything but indexes 10 and 11
+        cds_view.filter &= ~IndexFilter(indices=[10, 11])
     """)
 
-    source = Instance(ColumnarDataSource, help="""
-    The ``ColumnDataSource`` associated with this view. Used to determine
-    the length of the columns.
-    """)
+    @property
+    def filters(self) -> list[Filter]:
+        deprecated("CDSView.filters was deprecated in bokeh 3.0. Use CDSView.filter instead.")
+        filter = self.filter
+        if isinstance(filter, IntersectionFilter):
+            return filter.operands
+        elif isinstance(filter, AllIndices):
+            return []
+        else:
+            return [filter]
+
+    @filters.setter
+    def filters(self, filters: list[Filter]) -> None:
+        deprecated("CDSView.filters was deprecated in bokeh 3.0. Use CDSView.filter instead.")
+        if len(filters) == 0:
+            self.filter = AllIndices()
+        elif len(filters) == 1:
+            self.filter = filters[0]
+        else:
+            self.filter = IntersectionFilter(operands=filters)
 
 class GeoJSONDataSource(ColumnarDataSource):
     '''
 
     '''
 
-    geojson = NonNullable(JSON, help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    geojson = Required(JSON, help="""
     GeoJSON that contains features for plotting. Currently
     ``GeoJSONDataSource`` can only process a ``FeatureCollection`` or
     ``GeometryCollection``.
@@ -755,6 +798,10 @@ class WebDataSource(ColumnDataSource):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     adapter = Nullable(Instance(CustomJS), help="""
     A JavaScript callback to adapt raw JSON responses to Bokeh ``ColumnDataSource``
     format.
@@ -777,19 +824,20 @@ class WebDataSource(ColumnDataSource):
     replace existing data entirely.
     """)
 
-    data_url = NonNullable(String, help="""
+    data_url = Required(String, help="""
     A URL to to fetch data from.
     """)
 
-# TODO: deprecated, remove at bokeh 3.0
-WebSource = WebDataSource
-
 class ServerSentDataSource(WebDataSource):
     ''' A data source that can populate columns by receiving server sent
     events endpoints.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class AjaxDataSource(WebDataSource):
     ''' A data source that can populate columns by making Ajax calls to REST
     endpoints.
@@ -817,10 +865,14 @@ class AjaxDataSource(WebDataSource):
     This is necessary when used in conjunction with a ``FactorRange``, even
     if the columns in `data`` are empty.
 
-    A full example can be seen at :bokeh-tree:`examples/howto/ajax_source.py`
+    A full example can be seen at :bokeh-tree:`examples/basic/data/ajax_source.py`
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     polling_interval = Nullable(Int, help="""
     A polling interval (in milliseconds) for updating data source.
     """)
