@@ -23,7 +23,7 @@ log = logging.getLogger(__name__)
 # Bokeh imports
 from ..core.enums import TextureRepetition
 from ..core.has_props import abstract
-from ..core.properties import Enum, NonNullable, String
+from ..core.properties import Enum, Required, String
 from ..model import Model
 
 #-----------------------------------------------------------------------------
@@ -46,6 +46,10 @@ class Texture(Model):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     repetition = Enum(TextureRepetition, default="repeat", help="""
 
     """)
@@ -55,7 +59,11 @@ class CanvasTexture(Texture):
 
     '''
 
-    code = NonNullable(String, help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    code = Required(String, help="""
     A snippet of JavaScript code to execute in the browser.
 
     """)
@@ -65,7 +73,11 @@ class ImageURLTexture(Texture):
 
     '''
 
-    url = NonNullable(String, help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    url = Required(String, help="""
     A URL to a drawable resource like image, video, etc.
 
     """)
