@@ -31,9 +31,9 @@ from ..core.properties import (
     Float,
     Instance,
     Int,
-    NonNullable,
     Nullable,
     Override,
+    Required,
     Seq,
 )
 from ..core.validation import error
@@ -73,12 +73,20 @@ class Ticker(Model):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 @abstract
 class ContinuousTicker(Ticker):
     ''' A base class for non-categorical ticker types.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     num_minor_ticks = Int(5, help="""
     The number of minor tick positions to generate between
     adjacent major tick values.
@@ -102,6 +110,10 @@ class FixedTicker(ContinuousTicker):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     ticks = Seq(Float, default=[], help="""
     List of major tick locations.
     """)
@@ -121,11 +133,15 @@ class AdaptiveTicker(ContinuousTicker):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     base = Float(10.0, help="""
     The multiplier to use for scaling mantissas.
     """)
 
-    mantissas = Seq(Float, [1, 2, 5], help="""
+    mantissas = Seq(Float, default=[1, 2, 5], help="""
     The acceptable list numbers to generate multiples of.
     """)
 
@@ -149,6 +165,10 @@ class CompositeTicker(ContinuousTicker):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     tickers = Seq(Instance(Ticker), default=[], help="""
     A list of Ticker objects to combine at different scales in order
     to generate tick values. The supplied tickers should be in order.
@@ -164,7 +184,11 @@ class SingleIntervalTicker(ContinuousTicker):
 
     '''
 
-    interval = NonNullable(Float, help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    interval = Required(Float, help="""
     The interval between adjacent ticks.
     """)
 
@@ -172,6 +196,11 @@ class DaysTicker(SingleIntervalTicker):
     ''' Generate ticks spaced apart by specific, even multiples of days.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     days = Seq(Int, default=[], help="""
     The intervals of days to use.
     """)
@@ -182,6 +211,11 @@ class MonthsTicker(SingleIntervalTicker):
     ''' Generate ticks spaced apart by specific, even multiples of months.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     months = Seq(Int, default=[], help="""
     The intervals of months to use.
     """)
@@ -191,6 +225,10 @@ class YearsTicker(SingleIntervalTicker):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class BasicTicker(AdaptiveTicker):
     ''' Generate ticks on a linear scale.
 
@@ -199,10 +237,19 @@ class BasicTicker(AdaptiveTicker):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class LogTicker(AdaptiveTicker):
     ''' Generate ticks on a log scale.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     mantissas = Override(default=[1, 5])
 
 
@@ -211,6 +258,10 @@ class MercatorTicker(BasicTicker):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     dimension = Nullable(Enum(LatLon), help="""
     Specify whether to generate ticks for Latitude or Longitude.
 
@@ -238,6 +289,10 @@ class CategoricalTicker(Ticker):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 ONE_MILLI = 1.0
 ONE_SECOND = 1000.0
 ONE_MINUTE = 60.0 * ONE_SECOND
@@ -251,8 +306,13 @@ class DatetimeTicker(CompositeTicker):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     num_minor_ticks = Override(default=0)
 
+    # TODO: (bev) InstanceDefault for this, someday
     tickers = Override(default=lambda: [
         AdaptiveTicker(
             mantissas=[1, 2, 5],
@@ -289,7 +349,13 @@ class DatetimeTicker(CompositeTicker):
     ])
 
 class BinnedTicker(Ticker):
-    """Ticker that aligns ticks exactly at bin boundaries of a scanning color mapper. """
+    """ Ticker that aligns ticks exactly at bin boundaries of a scanning color mapper.
+
+    """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
     mapper = Instance(ScanningColorMapper, help="""
     A scanning color mapper (e.g. ``EqHistColorMapper``) to use.
