@@ -42,23 +42,22 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import difflib
 import typing as tp
-
-# External imports
-from typing_extensions import Literal
+from typing import Literal
 
 # Bokeh imports
 from ..core.enums import (
     Anchor,
     Dimension,
     Dimensions,
-    Location,
     SelectionMode,
+    ToolIcon,
     TooltipAttachment,
     TooltipFieldFormatter,
 )
 from ..core.has_props import abstract
 from ..core.properties import (
     Alpha,
+    AnyRef,
     Auto,
     Bool,
     Color,
@@ -70,13 +69,15 @@ from ..core.properties import (
     Float,
     Image,
     Instance,
+    InstanceDefault,
     Int,
     List,
-    NonNullable,
     Null,
     Nullable,
     Override,
     Percent,
+    Regex,
+    Required,
     Seq,
     String,
     Tuple,
@@ -93,9 +94,8 @@ from ..core.validation.errors import (
     NO_RANGE_TOOL_RANGES,
 )
 from ..model import Model
-from ..util.deprecation import deprecated
-from ..util.string import nice_join
-from .annotations import BoxAnnotation, PolyAnnotation
+from ..util.strings import nice_join
+from .annotations import BoxAnnotation, PolyAnnotation, Span
 from .callbacks import Callback
 from .dom import Template
 from .glyphs import (
@@ -106,31 +106,30 @@ from .glyphs import (
     Rect,
     XYGlyph,
 )
-from .layouts import LayoutDOM
 from .ranges import Range1d
 from .renderers import DataRenderer, GlyphRenderer
+from .ui import UIElement
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
 __all__ = (
-    'Action',
     'ActionTool',
     'BoxEditTool',
     'BoxSelectTool',
     'BoxZoomTool',
+    'CopyTool',
     'CrosshairTool',
     'CustomAction',
     'CustomJSHover',
     'Drag',
     'EditTool',
     'FreehandDrawTool',
+    'FullscreenTool',
     'HelpTool',
     'HoverTool',
-    'Inspection',
     'InspectTool',
-    'Gesture',
     'GestureTool',
     'LassoSelectTool',
     'LineEditTool',
@@ -139,18 +138,17 @@ __all__ = (
     'PolyDrawTool',
     'PolyEditTool',
     'PolySelectTool',
-    'ProxyToolbar',
     'RangeTool',
     'RedoTool',
     'ResetTool',
     'SaveTool',
     'Scroll',
+    'ExamineTool',
     'Tap',
     'TapTool',
     'Tool',
+    'ToolProxy',
     'Toolbar',
-    'ToolbarBase',
-    'ToolbarBox',
     'UndoTool',
     'WheelPanTool',
     'WheelZoomTool',
@@ -168,16 +166,29 @@ class Tool(Model):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    icon = Nullable(Either(Enum(ToolIcon), Regex(r"^\."), Image), help="""
+    An icon to display in the toolbar.
+
+    The icon can provided as well known tool icon name, a CSS class selector,
+    a data URI with an ``image/*`` MIME, a path to an image, a PIL ``Image``
+    object, or an RGB(A) NumPy array. If ``None``, then the intrinsic icon
+    will be used (may depend on tool's configuration).
+    """)
+
     description = Nullable(String, help="""
     A string describing the purpose of this tool. If not defined, an auto-generated
     description will be used. This description will be typically presented in the
     user interface as a tooltip.
     """)
 
-    _known_aliases: tp.ClassVar[tp.Dict[str, tp.Callable[[], "Tool"]]] = {}
+    _known_aliases: tp.ClassVar[dict[str, tp.Callable[[], Tool]]] = {}
 
     @classmethod
-    def from_string(cls, name: str) -> "Tool":
+    def from_string(cls, name: str) -> Tool:
         """ Takes a string and returns a corresponding `Tool` instance. """
         constructor = cls._known_aliases.get(name)
         if constructor is not None:
@@ -190,49 +201,78 @@ class Tool(Model):
             raise ValueError(f"unexpected tool name '{name}', {text} tools are {nice_join(matches)}")
 
     @classmethod
-    def register_alias(cls, name: str, constructor: tp.Callable[[], "Tool"]) -> None:
+    def register_alias(cls, name: str, constructor: tp.Callable[[], Tool]) -> None:
         cls._known_aliases[name] = constructor
 
+class ToolProxy(Model):
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    tools = List(Instance(Tool))
+    active = Bool(default=False)
+    disabled = Bool(default=False)
+
 @abstract
 class ActionTool(Tool):
     ''' A base class for tools that are buttons in the toolbar.
 
     '''
-    pass
 
-# TODO: deprecated, remove at bokeh 3.0
-Action = ActionTool
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+@abstract
+class PlotActionTool(ActionTool):
+    ''' A base class action tools acting on plots.
+
+    '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 @abstract
 class GestureTool(Tool):
     ''' A base class for tools that respond to drag events.
 
     '''
-    pass
 
-# TODO: deprecated, remove at bokeh 3.0
-Gesture = GestureTool
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 @abstract
 class Drag(GestureTool):
     ''' A base class for tools that respond to drag events.
 
     '''
-    pass
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 @abstract
 class Scroll(GestureTool):
     ''' A base class for tools that respond to scroll events.
 
     '''
-    pass
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 @abstract
 class Tap(GestureTool):
     ''' A base class for tools that respond to tap/click events.
 
     '''
-    pass
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 @abstract
 class SelectTool(GestureTool):
@@ -240,14 +280,9 @@ class SelectTool(GestureTool):
 
     '''
 
-    names = List(String, help="""
-    A list of names to query for. If set, only renderers that have a matching
-    value for their ``name`` attribute will be used.
-
-    .. note:
-        This property is deprecated and will be removed in bokeh 3.0.
-
-    """)
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
     renderers = Either(Auto, List(Instance(DataRenderer)), default="auto", help="""
     An explicit list of renderers to hit test against. If unset, defaults to
@@ -265,21 +300,26 @@ class InspectTool(GestureTool):
     ''' A base class for tools that perform "inspections", e.g. ``HoverTool``.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     toggleable = Bool(True, help="""
     Whether an on/off toggle button should appear in the toolbar for this
     inspection tool. If ``False``, the viewers of a plot will not be able to
     toggle the inspector on or off using the toolbar.
     """)
 
-# TODO: deprecated, remove at bokeh 3.0
-Inspection = InspectTool
-
-@abstract
-class ToolbarBase(Model):
-    ''' A base class for different toolbars.
+class Toolbar(UIElement):
+    ''' Collect tools to display for a single plot.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     logo = Nullable(Enum("normal", "grey"), default="normal", help="""
     What version of the Bokeh logo to display on the toolbar. If
     set to None, no logo will be displayed.
@@ -290,34 +330,29 @@ class ToolbarBase(Model):
     If True, hides toolbar when cursor is not in canvas.
     """)
 
-    tools = List(Instance(Tool), help="""
+    tools = List(Either(Instance(Tool), Instance(ToolProxy)), help="""
     A list of tools to add to the plot.
     """)
 
-class Toolbar(ToolbarBase):
-    ''' Collect tools to display for a single plot.
-
-    '''
-
-    active_drag: tp.Union[Literal["auto"], Drag, None] = Either(Null, Auto, Instance(Drag), default="auto", help="""
+    active_drag: Literal["auto"] | Drag | None = Either(Null, Auto, Instance(Drag), default="auto", help="""
     Specify a drag tool to be active when the plot is displayed.
     """)
 
-    active_inspect: tp.Union[Literal["auto"], InspectTool, tp.Sequence[InspectTool], None] = \
+    active_inspect: Literal["auto"] | InspectTool | tp.Sequence[InspectTool] | None = \
         Either(Null, Auto, Instance(InspectTool), Seq(Instance(InspectTool)), default="auto", help="""
     Specify an inspection tool or sequence of inspection tools to be active when
     the plot is displayed.
     """)
 
-    active_scroll: tp.Union[Literal["auto"], Scroll, None] = Either(Null, Auto, Instance(Scroll), default="auto", help="""
+    active_scroll: Literal["auto"] | Scroll | None = Either(Null, Auto, Instance(Scroll), default="auto", help="""
     Specify a scroll/pinch tool to be active when the plot is displayed.
     """)
 
-    active_tap: tp.Union[Literal["auto"], Tap, None] = Either(Null, Auto, Instance(Tap), default="auto", help="""
+    active_tap: Literal["auto"] | Tap | None = Either(Null, Auto, Instance(Tap), default="auto", help="""
     Specify a tap/click tool to be active when the plot is displayed.
     """)
 
-    active_multi: tp.Union[Literal["auto"], GestureTool, None] = Either(Null, Auto, Instance(GestureTool), default="auto", help="""
+    active_multi: Literal["auto"] | GestureTool | None = Either(Null, Auto, Instance(GestureTool), default="auto", help="""
     Specify an active multi-gesture tool, for instance an edit tool or a range
     tool.
 
@@ -327,27 +362,6 @@ class Toolbar(ToolbarBase):
     be deactivated (i.e. the multi-gesture tool will take precedence).
     """)
 
-class ProxyToolbar(ToolbarBase):
-    ''' A toolbar that allow to merge and proxy tools of toolbars in multiple
-    plots.
-
-    '''
-
-    toolbars = List(Instance(Toolbar), help="""
-    """)
-
-class ToolbarBox(LayoutDOM):
-    ''' A layoutable toolbar that can accept the tools of multiple plots, and
-    can merge the tools into a single button for convenience.
-
-    '''
-
-    toolbar = Instance(ToolbarBase, help="""
-    A toolbar associated with a plot which holds all its tools.
-    """)
-
-    toolbar_location = Enum(Location, default="right")
-
 class PanTool(Drag):
     ''' *toolbar icon*: |pan_icon|
 
@@ -360,9 +374,14 @@ class PanTool(Drag):
 
     .. |pan_icon| image:: /_images/icons/Pan.png
         :height: 24px
+        :alt: Icon of four arrows meeting in a plus shape representing the pan tool in the toolbar.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     dimensions = Enum(Dimensions, default="both", help="""
     Which dimensions the pan tool is constrained to act in. By default
     the pan tool will pan in any dimension, but can be configured to only
@@ -370,7 +389,7 @@ class PanTool(Drag):
     height of the plot.
     """)
 
-DEFAULT_RANGE_OVERLAY = lambda: BoxAnnotation(
+DEFAULT_RANGE_OVERLAY = InstanceDefault(BoxAnnotation,
     syncable=False,
     level="overlay",
     fill_color="lightgrey",
@@ -396,8 +415,13 @@ class RangeTool(Drag):
     .. |range_icon| image:: /_images/icons/Range.png
         :height: 24px
 
+
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     x_range = Nullable(Instance(Range1d), help="""
     A range synchronized to the x-dimension of the overlay. If None, the overlay
     will span the entire x-dimension.
@@ -445,9 +469,14 @@ class WheelPanTool(Scroll):
 
     .. |wheel_pan_icon| image:: /_images/icons/WheelPan.png
         :height: 24px
+        :alt: Icon of a mouse shape next to crossed arrows representing the wheel-pan tool in the toolbar.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     dimension = Enum(Dimension, default="width", help="""
     Which dimension the wheel pan tool is constrained to act in. By default the
     wheel pan tool will pan the plot along the x-axis.
@@ -466,9 +495,14 @@ class WheelZoomTool(Scroll):
 
     .. |wheel_zoom_icon| image:: /_images/icons/WheelZoom.png
         :height: 24px
+        :alt: Icon of a mouse shape next to an hourglass representing the wheel-zoom tool in the toolbar.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     dimensions = Enum(Dimensions, default="both", help="""
     Which dimensions the wheel zoom tool is constrained to act in. By default
     the wheel zoom tool will zoom in any dimension, but can be configured to
@@ -508,35 +542,16 @@ class CustomAction(ActionTool):
 
     '''
 
+    # explicit __init__ to support Init signatures
     def __init__(self, *args, **kwargs) -> None:
-        action_tooltip = kwargs.pop("action_tooltip", None)
-        if action_tooltip is not None:
-            deprecated((2, 3, 0), "CustomAction.action_tooltip", "CustomAction.description")
-            kwargs["description"] = action_tooltip
         super().__init__(*args, **kwargs)
 
-    @property
-    def action_tooltip(self):
-        deprecated((2, 3, 0), "CustomAction.action_tooltip", "CustomAction.description")
-        return self.description
-    @action_tooltip.setter
-    def action_tooltip(self, description):
-        deprecated((2, 3, 0), "CustomAction.action_tooltip", "CustomAction.description")
-        self.description = description
-
     description = Override(default="Perform a Custom Action")
 
     callback = Nullable(Instance(Callback), help="""
     A Bokeh callback to execute when the custom action icon is activated.
     """)
 
-    icon = Image(help="""
-    An icon to display in the toolbar.
-
-    The icon can provided as a string filename for an image, a PIL ``Image``
-    object, or an RGB(A) NumPy array.
-    """)
-
 class SaveTool(ActionTool):
     ''' *toolbar icon*: |save_icon|
 
@@ -549,10 +564,38 @@ class SaveTool(ActionTool):
 
     .. |save_icon| image:: /_images/icons/Save.png
         :height: 24px
+        :alt: Icon of a floppy disk representing the save tool in the toolbar.
 
     '''
 
-class ResetTool(ActionTool):
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    filename = Nullable(String, help="""
+    Optional string specifying the filename of the saved image (extension not
+    needed). If a filename is not provided or set to None, the user is prompted
+    for a filename at save time.
+    """)
+
+class CopyTool(ActionTool):
+    ''' *toolbar icon*: |copy_icon|
+
+    The copy tool is an action tool, that allows copying the rendererd contents of
+    a plot or a collection of plots to system's clipboard. This tools is browser
+    dependent and may not function in certain browsers, or require additional
+    permissions to be granted to the web page.
+
+    .. |copy_icon| image:: /_images/icons/Copy.png
+        :height: 24px
+
+    '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+class ResetTool(PlotActionTool):
     ''' *toolbar icon*: |reset_icon|
 
     The reset tool is an action. When activated in the toolbar, the tool resets
@@ -561,10 +604,13 @@ class ResetTool(ActionTool):
 
     .. |reset_icon| image:: /_images/icons/Reset.png
         :height: 24px
+        :alt: Icon of two arrows on a circular arc forming a circle representing the reset tool in the toolbar.
 
     '''
 
-    pass
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 class TapTool(Tap, SelectTool):
     ''' *toolbar icon*: |tap_icon|
@@ -572,11 +618,12 @@ class TapTool(Tap, SelectTool):
     The tap selection tool allows the user to select at single points by
     left-clicking a mouse, or tapping with a finger.
 
-    See :ref:`userguide_styling_selected_unselected_glyphs` for information
+    See :ref:`ug_styling_plots_selected_unselected_glyphs` for information
     on styling selected and unselected glyphs.
 
     .. |tap_icon| image:: /_images/icons/Tap.png
         :height: 24px
+        :alt:  Icon of two concentric circles with a + in the lower right representing the tap tool in the toolbar.
 
     .. note::
         Selections can be comprised of multiple regions, even those
@@ -586,6 +633,10 @@ class TapTool(Tap, SelectTool):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     behavior = Enum("select", "inspect", default="select", help="""
     This tool can be configured to either make selections or inspections
     on associated data sources. The difference is that selection changes
@@ -624,7 +675,7 @@ class TapTool(Tap, SelectTool):
     .. note::
         This callback does *not* execute on every tap, only when a glyph is
         "hit". If you would like to execute a callback on every mouse tap,
-        please see :ref:`userguide_interaction_jscallbacks_customjs_interactions`.
+        please see :ref:`ug_interaction_js_callbacks_customjs_js_on_event`.
 
     """)
 
@@ -642,9 +693,36 @@ class CrosshairTool(InspectTool):
 
     .. |crosshair_icon| image:: /_images/icons/Crosshair.png
         :height: 24px
+        :alt: Icon of circle with aiming reticle marks representing the crosshair tool in the toolbar.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    overlay = Either(
+        Auto,
+        Instance(Span),
+        Tuple(Instance(Span), Instance(Span)), default="auto", help="""
+    An annotation drawn to indicate the crosshair.
+
+    If ``"auto"``, this will create spans depending on the ``dimensions``
+    property, which based on its value, will result in either one span
+    (horizontal or vertical) or two spans (horizontal and vertical).
+
+    Alternatively the user can provide one ``Span`` instance, where the
+    dimension is indicated by the ``dimension`` property of the ``Span``.
+    Also two ``Span`` instances can be provided. Providing explicit
+    ``Span`` instances allows for constructing linked crosshair, when
+    those instances are shared between crosshair tools of different plots.
+
+    .. note::
+        This property is experimental and may change at any point. In
+        particular in future this will allow using other annotations
+        than ``Span`` and annotation groups.
+    """)
+
     dimensions = Enum(Dimensions, default="both", help="""
     Which dimensions the crosshair tool is to track. By default, both vertical
     and horizontal lines will be drawn. If only "width" is supplied, only a
@@ -664,13 +742,14 @@ class CrosshairTool(InspectTool):
     Stroke width in units of pixels.
     """)
 
-DEFAULT_BOX_OVERLAY = lambda: BoxAnnotation(
+DEFAULT_BOX_OVERLAY = InstanceDefault(BoxAnnotation,
     syncable=False,
     level="overlay",
-    top_units="screen",
-    left_units="screen",
-    bottom_units="screen",
-    right_units="screen",
+    visible=False,
+    top_units="canvas",
+    left_units="canvas",
+    bottom_units="canvas",
+    right_units="canvas",
     fill_color="lightgrey",
     fill_alpha=0.5,
     line_color="black",
@@ -688,6 +767,7 @@ class BoxZoomTool(Drag):
 
     .. |box_zoom_icon| image:: /_images/icons/BoxZoom.png
         :height: 24px
+        :alt: Icon of a dashed box with a magnifying glass in the upper right representing the box-zoom tool in the toolbar.
 
     .. note::
         ``BoxZoomTool`` is incompatible with ``GMapPlot`` due to the manner in
@@ -696,7 +776,11 @@ class BoxZoomTool(Drag):
 
     '''
 
-    dimensions = Enum(Dimensions, default="both", help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    dimensions = Either(Enum(Dimensions), Auto, default="both", help="""
     Which dimensions the zoom box is to be free in. By default, users may
     freely draw zoom boxes with any dimensions. If only "width" is supplied,
     the box will be constrained to span the entire vertical space of the plot,
@@ -724,7 +808,7 @@ class BoxZoomTool(Drag):
     (top-left or bottom-right depending on direction) or the center of the box.
     """)
 
-class ZoomInTool(ActionTool):
+class ZoomInTool(PlotActionTool):
     ''' *toolbar icon*: |zoom_in_icon|
 
     The zoom-in tool allows users to click a button to zoom in
@@ -732,8 +816,14 @@ class ZoomInTool(ActionTool):
 
     .. |zoom_in_icon| image:: /_images/icons/ZoomIn.png
         :height: 24px
+        :alt: Icon of a plus sign next to a magnifying glass representing the zoom-in tool in the toolbar.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     # TODO ZoomInTool dimensions should probably be constrained to be the same as ZoomOutTool
     dimensions = Enum(Dimensions, default="both", help="""
     Which dimensions the zoom-in tool is constrained to act in. By default the
@@ -746,7 +836,7 @@ class ZoomInTool(ActionTool):
     Percentage to zoom for each click of the zoom-in tool.
     """)
 
-class ZoomOutTool(ActionTool):
+class ZoomOutTool(PlotActionTool):
     ''' *toolbar icon*: |zoom_out_icon|
 
     The zoom-out tool allows users to click a button to zoom out
@@ -754,8 +844,14 @@ class ZoomOutTool(ActionTool):
 
     .. |zoom_out_icon| image:: /_images/icons/ZoomOut.png
         :height: 24px
+        :alt: Icon of a minus sign next to a magnifying glass representing the zoom-out tool in the toolbar.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     dimensions = Enum(Dimensions, default="both", help="""
     Which dimensions the zoom-out tool is constrained to act in. By default the
     zoom-out tool will zoom in any dimension, but can be configured to only
@@ -781,15 +877,20 @@ class BoxSelectTool(Drag, SelectTool):
     a rectangular region by dragging the mouse or a finger over the plot area.
     The end of the drag event indicates the selection region is ready.
 
-    See :ref:`userguide_styling_selected_unselected_glyphs` for information
+    See :ref:`ug_styling_plots_selected_unselected_glyphs` for information
     on styling selected and unselected glyphs.
 
 
     .. |box_select_icon| image:: /_images/icons/BoxSelect.png
         :height: 24px
+        :alt: Icon of a dashed box with a + in the lower right representing the box-selection tool in the toolbar.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     select_every_mousemove = Bool(False, help="""
     Whether a selection computation should happen on every mouse event, or only
     once, when the selection region is completed. Default: False
@@ -813,11 +914,12 @@ class BoxSelectTool(Drag, SelectTool):
     (top-left or bottom-right depending on direction) or the center of the box.
     """)
 
-DEFAULT_POLY_OVERLAY = lambda: PolyAnnotation(
+DEFAULT_POLY_OVERLAY = InstanceDefault(PolyAnnotation,
     syncable=False,
     level="overlay",
-    xs_units="screen",
-    ys_units="screen",
+    visible=False,
+    xs_units="canvas",
+    ys_units="canvas",
     fill_color="lightgrey",
     fill_alpha=0.5,
     line_color="black",
@@ -834,7 +936,7 @@ class LassoSelectTool(Drag, SelectTool):
     over the plot region. The end of the drag event indicates the selection
     region is ready.
 
-    See :ref:`userguide_styling_selected_unselected_glyphs` for information
+    See :ref:`ug_styling_plots_selected_unselected_glyphs` for information
     on styling selected and unselected glyphs.
 
     .. note::
@@ -845,9 +947,14 @@ class LassoSelectTool(Drag, SelectTool):
 
     .. |lasso_select_icon| image:: /_images/icons/LassoSelect.png
         :height: 24px
+        :alt:  Icon of a looped lasso shape representing the lasso-selection tool in the toolbar.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     select_every_mousemove = Bool(True, help="""
     Whether a selection computation should happen on every mouse event, or only
     once, when the selection region is completed.
@@ -866,7 +973,7 @@ class PolySelectTool(Tap, SelectTool):
     polygon, and a double click (or tap) indicates the selection
     region is ready.
 
-    See :ref:`userguide_styling_selected_unselected_glyphs` for information
+    See :ref:`ug_styling_plots_selected_unselected_glyphs` for information
     on styling selected and unselected glyphs.
 
     .. note::
@@ -877,9 +984,14 @@ class PolySelectTool(Tap, SelectTool):
 
     .. |poly_select_icon| image:: /_images/icons/PolygonSelect.png
         :height: 24px
+        :alt: Icon of a dashed trapezoid with an arrow pointing at the lower right representing the polygon-selection tool in the toolbar.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     overlay = Instance(PolyAnnotation, default=DEFAULT_POLY_OVERLAY, help="""
     A shaded annotation drawn to indicate the selection region.
     """)
@@ -936,7 +1048,11 @@ class CustomJSHover(Model):
 
     '''
 
-    args = Dict(String, Instance(Model), help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    args = Dict(String, AnyRef, help="""
     A mapping of names to Bokeh plot objects. These objects are made available
     to the callback code snippet as the values of named parameters to the
     callback.
@@ -1035,17 +1151,13 @@ class HoverTool(InspectTool):
 
     .. |hover_icon| image:: /_images/icons/Hover.png
         :height: 24px
+        :alt: Icon of a popup tooltip with abstract lines of text representing the hover tool in the toolbar.
 
     '''
 
-    names = List(String, help="""
-    A list of names to query for. If set, only renderers that have a matching
-    value for their ``name`` attribute will be used.
-
-    .. note:
-        This property is deprecated and will be removed in bokeh 3.0.
-
-    """)
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
     renderers = Either(Auto, List(Instance(DataRenderer)), default="auto", help="""
     An explicit list of renderers to hit test against. If unset, defaults to
@@ -1144,16 +1256,16 @@ class HoverTool(InspectTool):
     according to the "datetime" formatting scheme. The following schemes are
     available:
 
-    :``"numeral"``:
+    :"numeral":
         Provides a wide variety of formats for numbers, currency, bytes, times,
         and percentages. The full set of formats can be found in the
         |NumeralTickFormatter| reference documentation.
 
-    :``"datetime"``:
+    :"datetime":
         Provides formats for date and time values. The full set of formats is
         listed in the |DatetimeTickFormatter| reference documentation.
 
-    :``"printf"``:
+    :"printf":
         Provides formats similar to C-style "printf" type specifiers. See the
         |PrintfTickFormatter| reference documentation for complete details.
 
@@ -1184,10 +1296,19 @@ class HoverTool(InspectTool):
 
     line_policy = Enum("prev", "next", "nearest", "interp", "none",
                        default="nearest", help="""
-    When showing tooltips for lines, designates whether the tooltip position
-    should be the "previous" or "next" points on the line, the "nearest" point
-    to the current mouse position, or "interpolate" along the line to the
-    current mouse position.
+    Specifies where the tooltip will be positioned when hovering over line
+    glyphs:
+
+    :"prev": between the nearest two adjacent line points, positions the
+        tooltip at the point with the lower ("previous") index
+    :"next": between the nearest two adjacent line points, positions the
+        tooltip at the point with the higher ("next") index
+    :"nearest": between the nearest two adjacent line points, positions the
+        tooltip on the point that is nearest to the mouse cursor location
+    :"interp": positions the tooltip at an interpolated point on the segment
+        joining the two nearest adjacent line points.
+    :"none": positions the tooltip directly under the mouse cursor location
+
     """)
 
     anchor = Enum(Anchor, default="center", help="""
@@ -1216,71 +1337,71 @@ class HelpTool(ActionTool):
 
     '''
 
+    # explicit __init__ to support Init signatures
     def __init__(self, *args, **kwargs) -> None:
-        help_tooltip = kwargs.pop("help_tooltip", None)
-        if help_tooltip is not None:
-            deprecated((2, 3, 0), "HelpTool.help_tooltip", "HelpTool.description")
-            kwargs["description"] = help_tooltip
         super().__init__(*args, **kwargs)
 
-    @property
-    def help_tooltip(self):
-        deprecated((2, 3, 0), "HelpTool.help_tooltip", "HelpTool.description")
-        return self.description
-    @help_tooltip.setter
-    def help_tooltip(self, description):
-        deprecated((2, 3, 0), "HelpTool.help_tooltip", "HelpTool.description")
-        self.description = description
-
     description = Override(default=DEFAULT_HELP_TIP)
 
     redirect = String(default=DEFAULT_HELP_URL, help="""
     Site to be redirected through upon click.
     """)
 
-class UndoTool(ActionTool):
+class ExamineTool(ActionTool):
+    ''' A tool that allows to inspect and configure a model. '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+class FullscreenTool(ActionTool):
+    ''' A tool that allows to enlarge a UI element to fullscreen. '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+class UndoTool(PlotActionTool):
     ''' *toolbar icon*: |undo_icon|
 
     Undo tool allows to restore previous state of the plot.
 
     .. |undo_icon| image:: /_images/icons/Undo.png
         :height: 24px
+        :alt: Icon of an arrow on a circular arc pointing to the left representing the undo tool in the toolbar.
 
     '''
 
-class RedoTool(ActionTool):
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+class RedoTool(PlotActionTool):
     ''' *toolbar icon*: |redo_icon|
 
     Redo tool reverses the last action performed by undo tool.
 
     .. |redo_icon| image:: /_images/icons/Redo.png
         :height: 24px
+        :alt: Icon of an arrow on a circular arc pointing to the right representing the redo tool in the toolbar.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 @abstract
 class EditTool(GestureTool):
     ''' A base class for all interactive draw tool types.
 
     '''
 
+    # explicit __init__ to support Init signatures
     def __init__(self, *args, **kwargs) -> None:
-        custom_tooltip = kwargs.pop("custom_tooltip", None)
-        if custom_tooltip is not None:
-            deprecated((2, 3, 0), "EditTool.custom_tooltip", "EditTool.description")
-            kwargs["description"] = custom_tooltip
         super().__init__(*args, **kwargs)
 
-    @property
-    def custom_tooltip(self):
-        deprecated((2, 3, 0), "EditTool.custom_tooltip", "EditTool.description")
-        return self.description
-    @custom_tooltip.setter
-    def custom_tooltip(self, description):
-        deprecated((2, 3, 0), "EditTool.custom_tooltip", "EditTool.description")
-        self.description = description
-
-    empty_value = NonNullable(Either(Bool, Int, Float, Date, Datetime, Color, String), help="""
+    empty_value = Required(Either(Bool, Int, Float, Date, Datetime, Color, String), help="""
     Defines the value to insert on non-coordinate columns when a new
     glyph is inserted into the ``ColumnDataSource`` columns, e.g. when a
     circle glyph defines 'x', 'y' and 'color' columns, adding a new
@@ -1288,22 +1409,24 @@ class EditTool(GestureTool):
     the color column will be filled with the defined empty value.
     """)
 
-    custom_icon = Nullable(Image, help="""
-    An icon to display in the toolbar.
-
-    The icon can provided as a string filename for an image, a PIL ``Image``
-    object, or an RGB(A) NumPy array.
-    """)
-
     renderers = List(Instance(GlyphRenderer), help="""
-    An explicit list of renderers corresponding to scatter glyphs that may
-    be edited.
+    An explicit list of renderers corresponding to glyphs that may be edited.
+
+    .. note::
+        The kind of renderer will typically depend on the specific type of the
+        edit tool subclass. For instance, ``PointDrawTool`` expects renderers
+        for ``Scatter`` glyphs, while ``BoxEditTool`` expects renderers for
+        ``Rect`` glyphs, etc.
     """)
 
 @abstract
 class PolyTool(EditTool):
     ''' A base class for polygon draw/edit tools. '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     vertex_renderer = Nullable(Instance(GlyphRenderer), help="""
     The renderer used to render the vertices of a selected line or polygon.
     """)
@@ -1353,8 +1476,13 @@ class BoxEditTool(EditTool, Drag, Tap):
 
     .. |box_edit_icon| image:: /_images/icons/BoxEdit.png
         :height: 24px
+        :alt: Icon of a solid line box with a plus sign in the lower right representing the box-edit tool in the toolbar.
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     dimensions = Enum(Dimensions, default="both", help="""
     Which dimensions the box drawing is to be free in. By default, users may
     freely draw boxes with any dimensions. If only "width" is set, the box will
@@ -1413,8 +1541,14 @@ class PointDrawTool(EditTool, Drag, Tap):
 
     .. |point_draw_icon| image:: /_images/icons/PointDraw.png
         :height: 24px
+        :alt: Icon of three points with an arrow pointing to one representing the point-edit tool in the toolbar.
+
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     add = Bool(default=True, help="""
     Enables adding of new points on tap events.
     """)
@@ -1471,8 +1605,14 @@ class PolyDrawTool(PolyTool, Drag, Tap):
 
     .. |poly_draw_icon| image:: /_images/icons/PolyDraw.png
         :height: 24px
+        :alt: Icon of a solid line trapezoid with an arrow pointing at the lower right representing the polygon-draw tool in the toolbar.
+
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     drag = Bool(default=True, help="""
     Enables dragging of existing patches and multi-lines on pan events.
     """)
@@ -1514,8 +1654,14 @@ class FreehandDrawTool(EditTool, Drag, Tap):
 
     .. |freehand_draw_icon| image:: /_images/icons/FreehandDraw.png
         :height: 24px
+        :alt: Icon of a pen drawing a wavy line representing the freehand-draw tool in the toolbar.
+
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     num_objects = Int(default=0, help="""
     Defines a limit on the number of patches or multi-lines that can be drawn.
     By default there is no limit on the number of objects, but if enabled the
@@ -1562,8 +1708,14 @@ class PolyEditTool(PolyTool, Drag, Tap):
 
     .. |poly_edit_icon| image:: /_images/icons/PolyEdit.png
         :height: 24px
+        :alt: Icon of two lines meeting in a vertex with an arrow pointing at it representing the polygon-edit tool in the toolbar.
+
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     @error(INCOMPATIBLE_POLY_EDIT_RENDERER)
     def _check_compatible_renderers(self):
         incompatible_renderers = []
@@ -1598,7 +1750,13 @@ class LineEditTool(EditTool, Drag, Tap):
 
     .. |line_edit_icon| image:: /_images/icons/LineEdit.png
         :height: 24px
-     '''
+        :alt: Icon of a line with a point on it with an arrow pointing at it representing the line-edit tool in the toolbar.
+
+    '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
     intersection_renderer = Instance(GlyphRenderer, help="""
     The renderer used to render the intersections of a selected line
@@ -1667,11 +1825,15 @@ Tool.register_alias("lasso_select", lambda: LassoSelectTool())
 Tool.register_alias("box_zoom", lambda: BoxZoomTool(dimensions="both"))
 Tool.register_alias("xbox_zoom", lambda: BoxZoomTool(dimensions="width"))
 Tool.register_alias("ybox_zoom", lambda: BoxZoomTool(dimensions="height"))
+Tool.register_alias("auto_box_zoom", lambda: BoxZoomTool(dimensions="auto"))
 Tool.register_alias("save", lambda: SaveTool())
+Tool.register_alias("copy", lambda: CopyTool())
 Tool.register_alias("undo", lambda: UndoTool())
 Tool.register_alias("redo", lambda: RedoTool())
 Tool.register_alias("reset", lambda: ResetTool())
 Tool.register_alias("help", lambda: HelpTool())
+Tool.register_alias("examine", lambda: ExamineTool())
+Tool.register_alias("fullscreen", lambda: FullscreenTool())
 Tool.register_alias("box_edit", lambda: BoxEditTool())
 Tool.register_alias("line_edit", lambda: LineEditTool())
 Tool.register_alias("point_draw", lambda: PointDrawTool())
