@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -31,8 +31,8 @@ from ..core.properties import (
     Enum,
     Float,
     Instance,
-    NonNullable,
     Nullable,
+    Required,
     Seq,
     String,
 )
@@ -75,7 +75,10 @@ class Transform(Model):
         }
 
     '''
-    pass
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 
 class CustomJSTransform(Transform):
@@ -89,6 +92,10 @@ class CustomJSTransform(Transform):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     args = Dict(String, AnyRef, help="""
     A mapping of names to Python objects. In particular those can be bokeh's models.
     These objects are made available to the transform' code snippet as the values of
@@ -139,6 +146,10 @@ class Dodge(Transform):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     value = Float(default=0, help="""
     The amount to dodge the input data.
     """)
@@ -154,6 +165,10 @@ class Jitter(Transform):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     mean = Float(default=0, help="""
     The central value for the random sample
     """)
@@ -203,11 +218,16 @@ class Interpolator(Transform):
     interpolation.
 
     '''
-    x = NonNullable(Either(String, Seq(Float)), help="""
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    x = Required(Either(String, Seq(Float)), help="""
     Independent coordinate denoting the location of a point.
     """)
 
-    y = NonNullable(Either(String, Seq(Float)), help="""
+    y = Required(Either(String, Seq(Float)), help="""
     Dependant coordinate denoting the value of a point at a location.
     """)
 
@@ -220,17 +240,16 @@ class Interpolator(Transform):
     If this is set to False, it will return the most value of the closest point.
     """)
 
-    # Define an initialization routine to do some cross checking of input values
-    def __init__(self, **kwargs) -> None:
-        super().__init__(**kwargs)
-
 
 class LinearInterpolator(Interpolator):
     ''' Compute a linear interpolation between the control points provided through
     the ``x``, ``y``, and ``data`` parameters.
 
     '''
-    pass
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 
 class StepInterpolator(Interpolator):
@@ -239,6 +258,10 @@ class StepInterpolator(Interpolator):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     mode = Enum(StepMode, default="after", help="""
     Adjust the behavior of the returned value in relation to the control points.  The parameter can assume one of three values:
 
