@@ -35,15 +35,17 @@ from ...core.properties import (
     Int,
     Interval,
     List,
+    NonNegative,
     Null,
     Nullable,
     Override,
-    PositiveInt,
+    Positive,
     Readonly,
     String,
     Tuple,
 )
 from ..formatters import TickFormatter
+from ..ui import Tooltip
 from .widget import Widget
 
 #-----------------------------------------------------------------------------
@@ -52,6 +54,7 @@ from .widget import Widget
 
 __all__ = (
     'AutocompleteInput',
+    'Checkbox',
     'ColorPicker',
     'DatePicker',
     'FileInput',
@@ -62,6 +65,7 @@ __all__ = (
     'PasswordInput',
     'Select',
     'Spinner',
+    'Switch',
     'TextInput',
     'TextAreaInput'
 )
@@ -77,10 +81,18 @@ class InputWidget(Widget):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     title = String(default="", help="""
     Widget's label.
     """)
 
+    description = Nullable(Either(String, Instance(Tooltip)), default=None, help="""
+    Either a plain text or a tooltip with a rich HTML description of the function of this widget.
+    """)
+
     @classmethod
     def coerce_value(cls, val):
         prop_obj = cls.lookup('value')
@@ -97,11 +109,15 @@ class InputWidget(Widget):
 # General API
 #-----------------------------------------------------------------------------
 
-class FileInput(Widget):
+class FileInput(InputWidget):
     ''' Present a file-chooser dialog to users and return the contents of the
     selected files.
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     value = Readonly(Either(String, List(String)), default="", help='''
     The base64-enconded contents of the file or files that were loaded.
 
@@ -142,7 +158,7 @@ class FileInput(Widget):
         access to that information for security reasons.
     ''')
 
-    accept = String(default="", help="""
+    accept = Either(String, List(String), default="", help="""
     Comma-separated list of standard HTML file input filters that restrict what
     files the user can pick from. Values can be:
 
@@ -183,6 +199,10 @@ class NumericInput(InputWidget):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     value = Either(Null, Float, Int, help="""
     Initial or entered value.
 
@@ -218,11 +238,11 @@ class Spinner(NumericInput):
 
     '''
 
-    def __init__(self, **kwargs) -> None:
+    def __init__(self, *args, **kwargs) -> None:
         if "value" in kwargs and "value_throttled" not in kwargs:
             kwargs["value_throttled"] = kwargs["value"]
 
-        super().__init__(**kwargs)
+        super().__init__(*args, **kwargs)
 
     value_throttled = Readonly(Either(Null, Float, Int), help="""
     value reported at the end of interactions
@@ -244,11 +264,47 @@ class Spinner(NumericInput):
     mouse wheel is used to change the input
     """)
 
+@abstract
+class ToggleInput(Widget):
+    """ Base class for toggleable (boolean) input widgets. """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    active = Bool(default=False, help="""
+    The state of the widget.
+    """)
+
+class Checkbox(ToggleInput):
+    """ A checkbox widget. """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    label = String(default="", help="""
+    The label next to the checkbox.
+    """)
+
+class Switch(ToggleInput):
+    """ A checkbox-like widget. """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    width = Override(default=32)
+
 class TextLikeInput(InputWidget):
     ''' Base class for text-like input widgets.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     value = String(default="", help="""
     Initial or entered text value.
 
@@ -275,12 +331,29 @@ class TextInput(TextLikeInput):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    prefix = Nullable(String, help="""
+    An optional string prefix to display before the input. This is useful to
+    indicate e.g. a variable the entered value will be assigned to.
+    """)
+
+    suffix = Nullable(String, help="""
+    An optional string suffix to display after the input. This is useful to
+    indicate e.g. the units of measurement of the entered value.
+    """)
 
 class TextAreaInput(TextLikeInput):
     ''' Multi-line input widget.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     cols = Int(default=20, help="""
     Specifies the width of the text area (in average character width). Default: 20
     """)
@@ -303,22 +376,36 @@ class PasswordInput(TextInput):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 
 class AutocompleteInput(TextInput):
     ''' Single-line input widget with auto-completion.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     completions = List(String, help="""
     A list of completion strings. This will be used to guide the
     user upon typing the beginning of a desired value.
     """)
 
-    min_characters = PositiveInt(default=2, help="""
+    max_completions = Nullable(Positive(Int), help="""
+    Optional maximum number of completions displayed.
+    """)
+
+    min_characters = NonNegative(Int, default=2, help="""
     The number of characters a user must type before completions are presented.
     """)
 
-    case_sensitive = Bool(default=True, help="""Enable or disable case sensitivity""")
+    case_sensitive = Bool(default=True, help="""
+    Enable or disable case sensitivity.
+    """)
 
     restrict = Bool(default=True, help="""
     Set to False in order to allow users to enter text that is not present in the list of completion strings.
@@ -329,6 +416,11 @@ class Select(InputWidget):
     ''' Single-select widget.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     options = Either(List(Either(String, Tuple(String, String))),
         Dict(String, List(Either(String, Tuple(String, String)))), help="""
     Available selection options. Options may be provided either as a list of
@@ -348,6 +440,10 @@ class MultiSelect(InputWidget):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     options = List(Either(String, Tuple(String, String)), help="""
     Available selection options. Options may be provided either as a list of
     possible string values, or as a list of tuples, each of the form
@@ -371,6 +467,10 @@ class MultiChoice(InputWidget):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     options = List(Either(String, Tuple(String, String)), help="""
     Available selection options. Options may be provided either as a list of
     possible string values, or as a list of tuples, each of the form
@@ -412,6 +512,10 @@ class DatePicker(InputWidget):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     value = Date(help="""
     The initial or picked date.
     """)
@@ -424,7 +528,7 @@ class DatePicker(InputWidget):
     Optional latest allowable date.
     """)
 
-    disabled_dates = List(Either(Date, Tuple(Date, Date)), default=[], help="""
+    disabled_dates = Nullable(List(Either(Date, Tuple(Date, Date))), default=None, help="""
     A list of dates of ``(start, end)`` date ranges to make unavailable for
     selection. All other dates will be avalable.
 
@@ -432,7 +536,7 @@ class DatePicker(InputWidget):
         Only one of ``disabled_dates`` and ``enabled_dates`` should be specified.
     """)
 
-    enabled_dates = List(Either(Date, Tuple(Date, Date)), default=[], help="""
+    enabled_dates = Nullable(List(Either(Date, Tuple(Date, Date))), default=None, help="""
     A list of dates of ``(start, end)`` date ranges to make available for
     selection. All other dates will be unavailable.
 
@@ -457,6 +561,10 @@ class ColorPicker(InputWidget):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     color = ColorHex(default='#000000', help="""
     The initial color of the picked color (named or hexadecimal)
     """)
