@@ -28,12 +28,7 @@ log = logging.getLogger(__name__)
 
 # Bokeh imports
 from ...core.has_props import abstract
-from ...core.properties import (
-    Any,
-    Bool,
-    Dict,
-    String,
-)
+from ...core.properties import Bool, String
 from .widget import Widget
 
 #-----------------------------------------------------------------------------
@@ -63,6 +58,10 @@ class Markup(Widget):
     .. _`TeX and LaTeX input`: https://docs.mathjax.org/en/latest/basic/mathematics.html#tex-and-latex-input
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     text = String(default="", help="""
     The text or HTML contents of the widget.
 
@@ -76,10 +75,6 @@ class Markup(Widget):
         or inline images' contents using data URIs.
     """)
 
-    style = Dict(String, Any, default={}, help="""
-    Raw CSS style declaration. Note this may be web browser dependent.
-    """)
-
     disable_math = Bool(False, help="""
     Whether the contents should not be processed as TeX/LaTeX input.
     """)
@@ -95,7 +90,11 @@ class Paragraph(Markup):
 
     '''
 
-    __example__ = "sphinx/source/docs/user_guide/examples/interaction_paragraph.py"
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    __example__ = "examples/interaction/widgets/paragraph.py"
 
 class Div(Markup):
     ''' A block (div) of text.
@@ -103,7 +102,11 @@ class Div(Markup):
     This Bokeh model corresponds to an HTML ``<div>`` element.
     '''
 
-    __example__ = "sphinx/source/docs/user_guide/examples/interaction_div.py"
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    __example__ = "examples/interaction/widgets/div.py"
 
     render_as_text = Bool(False, help="""
     Whether the contents should be rendered as raw text or as interpreted HTML.
@@ -117,7 +120,11 @@ class PreText(Paragraph):
 
     '''
 
-    __example__ = "sphinx/source/docs/user_guide/examples/interaction_pretext.py"
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    __example__ = "examples/interaction/widgets/pretext.py"
 
 #-----------------------------------------------------------------------------
 # Private API
