@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -37,11 +37,12 @@ from ...core.properties import (
     Enum,
     Float,
     Instance,
+    InstanceDefault,
     Int,
     List,
-    NonNullable,
     Nullable,
     Override,
+    Required,
     String,
 )
 from ...model import Model
@@ -91,17 +92,30 @@ class CellFormatter(Model):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 @abstract
 class CellEditor(Model):
     ''' Abstract base class for data table's cell editors.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 @abstract
 class RowAggregator(Model):
     ''' Abstract base class for data cube's row formatters.
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     field_ = String('', help="""
     Refers to the table column being aggregated
     """)
@@ -115,6 +129,10 @@ class StringFormatter(CellFormatter):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     font_style = Enum(FontStyle, default="normal", help="""
     An optional text font style, e.g. bold, italic.
     """)
@@ -127,14 +145,20 @@ class StringFormatter(CellFormatter):
     An optional text color.
     """)
 
+    nan_format = String("-", help="""
+    Formatting to apply to NaN and None values.
+    """)
+
+
+
 class ScientificFormatter(StringFormatter):
     ''' Display numeric values from continuous ranges as "basic numbers",
     using scientific notation when appropriate by default.
     '''
 
-    nan_format = Nullable(String, help="""
-    Formatting to apply to NaN and None values (falls back to scientific formatting if not set).
-    """)
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
     precision = Int(10, help="""
     How many digits of precision to display.
@@ -155,6 +179,10 @@ class NumberFormatter(StringFormatter):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     format = String("0,0", help="""
     The number format, as defined in the following tables:
 
@@ -235,10 +263,6 @@ class NumberFormatter(StringFormatter):
     The language to use for formatting language-specific features (e.g. thousands separator).
     """)
 
-    nan_format = Nullable(String, help="""
-    Formatting to apply to NaN and None values (falls back to Numbro formatting if not set).
-    """)
-
     rounding = Enum(RoundingFunction, help="""
     Rounding functions (round, floor, ceil) and their synonyms (nearest, rounddown, roundup).
     """)
@@ -248,6 +272,10 @@ class BooleanFormatter(CellFormatter):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     icon = Enum('check', 'check-circle', 'check-circle-o', 'check-square', 'check-square-o', help="""
     The icon visualizing the check mark.
     """)
@@ -257,6 +285,10 @@ class DateFormatter(StringFormatter):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     format = Either(Enum(DateFormat), String, default='ISO-8601', help="""
     The date format can be any standard  `strftime`_ format string, as well
     as any of the following predefined format names:
@@ -462,10 +494,6 @@ class DateFormatter(StringFormatter):
 
     """)
 
-    nan_format = Nullable(String, help="""
-    Formatting to apply to NaN and None values (falls back to regular date formatting if not set).
-    """)
-
 
 class HTMLTemplateFormatter(CellFormatter):
     ''' HTML formatter using a template.
@@ -495,6 +523,11 @@ class HTMLTemplateFormatter(CellFormatter):
         )
 
     '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     template = String('<%= value %>', help="""
     Template string to be used by Underscore's template method.
     """)
@@ -504,6 +537,10 @@ class StringEditor(CellEditor):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     completions = List(String, help="""
     An optional list of completion strings.
     """)
@@ -513,11 +550,19 @@ class TextEditor(CellEditor):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class SelectEditor(CellEditor):
     ''' Select cell editor.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     options = List(String, help="""
     The list of options to select from.
     """)
@@ -527,16 +572,28 @@ class PercentEditor(CellEditor):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class CheckboxEditor(CellEditor):
     ''' Boolean value cell editor.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class IntEditor(CellEditor):
     ''' Spinner-based integer cell editor.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     step = Int(1, help="""
     The major step value.
     """)
@@ -546,6 +603,10 @@ class NumberEditor(CellEditor):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     step = Float(0.01, help="""
     The major step value.
     """)
@@ -555,37 +616,65 @@ class TimeEditor(CellEditor):
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class DateEditor(CellEditor):
     ''' Calendar-based date cell editor.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class AvgAggregator(RowAggregator):
     ''' Simple average across multiple rows.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class MinAggregator(RowAggregator):
     ''' Smallest value across multiple rows.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class MaxAggregator(RowAggregator):
     ''' Largest value across multiple rows.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class SumAggregator(RowAggregator):
     ''' Simple sum across multiple rows.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
 class TableColumn(Model):
     ''' Table column widget.
 
     '''
 
-    field = NonNullable(String, help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    field = Required(String, help="""
     The name of the field mapping to a column in the data source.
     """)
 
@@ -599,12 +688,12 @@ class TableColumn(Model):
     in pixels of this column.
     """)
 
-    formatter = Instance(CellFormatter, lambda: StringFormatter(), help="""
+    formatter = Instance(CellFormatter, InstanceDefault(StringFormatter), help="""
     The cell formatter for this column. By default, a simple string
     formatter is used.
     """)
 
-    editor = Instance(CellEditor, lambda: StringEditor(), help="""
+    editor = Instance(CellEditor, InstanceDefault(StringEditor), help="""
     The cell editor for this column. By default, a simple string editor
     is used.
     """)
@@ -628,27 +717,30 @@ class TableWidget(Widget):
 
     '''
 
-    source = Instance(DataSource, default=lambda: ColumnDataSource(), help="""
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
+    source = Instance(DataSource, default=InstanceDefault(ColumnDataSource), help="""
     The source of data for the widget.
     """)
 
-    view = Instance(CDSView, help="""
+    view = Instance(CDSView, default=InstanceDefault(CDSView), help="""
     A view into the data source to use when rendering table rows. A default view
     of the entire data source is created if a view is not passed in during
     initialization.
     """)
 
-    def __init__(self, **kw) -> None:
-        super().__init__(**kw)
-        if "view" not in kw:
-            self.view = CDSView(source=self.source)
-
 class DataTable(TableWidget):
     ''' Two-dimensional grid for visualization and editing large amounts
     of data.
 
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     autosize_mode = Enum(AutosizeMode, default="force_fit", help="""
     Describes the column autosizing mode with one of the following options:
 
@@ -768,8 +860,13 @@ class DataTable(TableWidget):
 
 class GroupingInfo(Model):
     '''Describes how to calculate totals and sub-totals
+
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     getter = String('', help="""
     References the column which generates the unique keys of this sub-total (groupby).
     """)
@@ -784,8 +881,13 @@ class GroupingInfo(Model):
 
 class DataCube(DataTable):
     '''Specialized DataTable with collapsing groups, totals, and sub-totals.
+
     '''
 
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
+
     grouping = List(Instance(GroupingInfo), help="""
     Describe what aggregation operations used to define sub-totals and totals
     """)
