@@ -27,7 +27,6 @@ log = logging.getLogger(__name__)
 
 # Bokeh imports
 from ...core.has_props import abstract
-from ...core.properties import Int, Override
 from ..layouts import LayoutDOM
 
 #-----------------------------------------------------------------------------
@@ -52,13 +51,9 @@ class Widget(LayoutDOM):
 
     '''
 
-    default_size = Int(default=300, help="""
-    The default size (width or height) in the dominating dimension.
-
-    The dominating dimension is determined by widget orientation.
-    """)
-
-    margin = Override(default=(5, 5, 5, 5))
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args, **kwargs) -> None:
+        super().__init__(*args, **kwargs)
 
 #-----------------------------------------------------------------------------
 # Private API
