@@ -31,6 +31,7 @@ from ._renderer import create_renderer
 
 __all__ = (
     'glyph_method',
+    'marker_method',
 )
 
 #-----------------------------------------------------------------------------
@@ -42,7 +43,7 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 def marker_method():
-    from ..models.markers import Marker, Scatter
+    from ..models import Marker, Scatter
     glyphclass = Marker
     def decorator(func):
         parameters = glyphclass.parameters()
