@@ -19,6 +19,7 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from collections.abc import Sequence
+from typing import TYPE_CHECKING, Any, Literal
 
 # External imports
 import numpy as np
@@ -27,6 +28,7 @@ import numpy as np
 from ..core.properties import Datetime
 from ..core.property.singletons import Intrinsic
 from ..models import (
+    Axis,
     CategoricalAxis,
     CategoricalScale,
     ContinuousTicker,
@@ -41,15 +43,21 @@ from ..models import (
     MercatorAxis,
     Range,
     Range1d,
+    Scale,
 )
-from ..util.dependencies import import_optional
+
+if TYPE_CHECKING:
+    import pandas as pd
+    from pandas.core.groupby import GroupBy
+    from typing_extensions import TypeAlias
+
+    from ..models.plots import Plot
+    from ..models.text import BaseText
 
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
 
-pd = import_optional('pandas')
-
 __all__ = (
     'get_range',
     'get_scale',
@@ -64,14 +72,17 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-def get_range(range_input):
+def get_range(range_input: Range | tuple[float, float] | Sequence[str] | pd.Series[Any] | GroupBy | None) -> Range:
+    import pandas as pd
+    from pandas.core.groupby import GroupBy
+
     if range_input is None:
         return DataRange1d()
-    if pd and isinstance(range_input, pd.core.groupby.GroupBy):
+    if isinstance(range_input, GroupBy):
         return FactorRange(factors=sorted(list(range_input.groups.keys())))
     if isinstance(range_input, Range):
         return range_input
-    if pd and isinstance(range_input, pd.Series):
+    if isinstance(range_input, pd.Series):
         range_input = range_input.values
     if isinstance(range_input, (Sequence, np.ndarray)):
         if all(isinstance(x, str) for x in range_input):
@@ -86,9 +97,13 @@ def get_range(range_input):
                 return Range1d(start=start, end=end)
             except ValueError:  # @mattpap suggests ValidationError instead
                 pass
-    raise ValueError("Unrecognized range input: '%s'" % str(range_input))
+    raise ValueError(f"Unrecognized range input: '{range_input}'")
 
-def get_scale(range_input, axis_type):
+AxisType: TypeAlias = Literal["linear", "log", "datetime", "mercator", "auto"]
+AxisLocation: TypeAlias = Literal["above", "below", "left", "right"]
+Dim: TypeAlias = Literal[0, 1]
+
+def get_scale(range_input: Range, axis_type: AxisType | None) -> Scale:
     if isinstance(range_input, (DataRange1d, Range1d)) and axis_type in ["linear", "datetime", "mercator", "auto", None]:
         return LinearScale()
     elif isinstance(range_input, (DataRange1d, Range1d)) and axis_type == "log":
@@ -96,9 +111,10 @@ def get_scale(range_input, axis_type):
     elif isinstance(range_input, FactorRange):
         return CategoricalScale()
     else:
-        raise ValueError("Unable to determine proper scale for: '%s'" % str(range_input))
+        raise ValueError(f"Unable to determine proper scale for: '{range_input}'")
 
-def process_axis_and_grid(plot, axis_type, axis_location, minor_ticks, axis_label, rng, dim):
+def process_axis_and_grid(plot: Plot, axis_type: AxisType | None, axis_location: AxisLocation | None,
+        minor_ticks: int | Literal["auto"] | None, axis_label: str | BaseText | None, rng: Range, dim: Dim) -> None:
     axiscls, axiskw = _get_axis_class(axis_type, rng, dim)
 
     if axiscls:
@@ -120,7 +136,7 @@ def process_axis_and_grid(plot, axis_type, axis_location, minor_ticks, axis_labe
 # Private API
 #-----------------------------------------------------------------------------
 
-def _get_axis_class(axis_type, range_input, dim):
+def _get_axis_class(axis_type: AxisType | None, range_input: Range, dim: Dim) -> tuple[type[Axis] | None, Any]:
     if axis_type is None:
         return None, {}
     elif axis_type == "linear":
@@ -130,7 +146,7 @@ def _get_axis_class(axis_type, range_input, dim):
     elif axis_type == "datetime":
         return DatetimeAxis, {}
     elif axis_type == "mercator":
-        return MercatorAxis, {'dimension': 'lon' if dim == 0 else 'lat'}
+        return MercatorAxis, dict(dimension='lon' if dim == 0 else 'lat')
     elif axis_type == "auto":
         if isinstance(range_input, FactorRange):
             return CategoricalAxis, {}
@@ -147,9 +163,9 @@ def _get_axis_class(axis_type, range_input, dim):
                 pass
         return LinearAxis, {}
     else:
-        raise ValueError("Unrecognized axis_type: '%r'" % axis_type)
+        raise ValueError(f"Unrecognized axis_type: '{axis_type!r}'")
 
-def _get_num_minor_ticks(axis_class, num_minor_ticks):
+def _get_num_minor_ticks(axis_class: type[Axis], num_minor_ticks: int | Literal["auto"] | None) -> int:
     if isinstance(num_minor_ticks, int):
         if num_minor_ticks <= 1:
             raise ValueError("num_minor_ticks must be > 1")
