@@ -25,7 +25,7 @@ from ..models import glyphs
 from ._decorators import glyph_method, marker_method
 
 if TYPE_CHECKING:
-    from ..models.canvas import CoordinateMapping
+    from ..models.coordinates import CoordinateMapping
     from ..models.plots import Plot
     from ..models.renderers import GlyphRenderer
 
@@ -123,6 +123,22 @@ Examples:
 
         show(plot)
 
+"""
+
+    @glyph_method(glyphs.Block)
+    def block(self, *args: Any, **kwargs: Any) -> GlyphRenderer:
+        """
+Examples:
+
+    .. code-block:: python
+
+        from bokeh.plotting import figure, output_file, show
+
+        plot = figure(width=300, height=300)
+        plot.block(x=[1, 2, 3], y=[1,2,3], width=0.5, height=1, , color="#CAB2D6")
+
+        show(plot)
+
 """
 
     @marker_method()
@@ -488,23 +504,6 @@ Examples:
                         color=['red', 'green'])
         show(p)
 
-"""
-
-    @glyph_method(glyphs.Oval)
-    def oval(self, *args: Any, **kwargs: Any) -> GlyphRenderer:
-        """
-Examples:
-
-    .. code-block:: python
-
-        from bokeh.plotting import figure, output_file, show
-
-        plot = figure(width=300, height=300)
-        plot.oval(x=[1, 2, 3], y=[1, 2, 3], width=0.2, height=0.4,
-                  angle=-0.7, color="#1D91C0")
-
-        show(plot)
-
 """
 
     @glyph_method(glyphs.Patch)
