@@ -26,15 +26,10 @@ import json
 from typing import (
     TYPE_CHECKING,
     Any,
-    Dict,
-    List,
-    Type,
+    Literal,
     overload,
 )
 
-# External imports
-from typing_extensions import Literal
-
 # Bokeh imports
 from .exceptions import ProtocolError
 from .message import Message
@@ -52,7 +47,6 @@ if TYPE_CHECKING:
     from ..core.types import ID
     from ..document.document import Document
     from ..document.events import DocumentPatchedEvent
-    from .receiver import Fragment
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -74,7 +68,7 @@ MessageType = Literal[
     "SERVER-INFO-REQ",
 ]
 
-SPEC: Dict[MessageType, Type[Message[Any]]] = {
+SPEC: dict[MessageType, type[Message[Any]]] = {
     "ACK": ack,
     "ERROR": error,
     "OK": ok,
@@ -98,7 +92,7 @@ class Protocol:
     ''' Provide a message factory for the Bokeh Server message protocol.
 
     '''
-    _messages: Dict[MessageType, Type[Message[Any]]]
+    _messages: dict[MessageType, type[Message[Any]]]
 
     def __init__(self) -> None:
         self._messages = SPEC
@@ -113,7 +107,7 @@ class Protocol:
     @overload
     def create(self, msgtype: Literal["OK"], request_id: ID, **metadata: Any) -> ok: ...
     @overload
-    def create(self, msgtype: Literal["PATCH-DOC"], events: List[DocumentPatchedEvent], use_buffers: bool = ..., **metadata: Any) -> patch_doc: ...
+    def create(self, msgtype: Literal["PATCH-DOC"], events: list[DocumentPatchedEvent], **metadata: Any) -> patch_doc: ...
     @overload
     def create(self, msgtype: Literal["PULL-DOC-REPLY"], request_id: ID, document: Document, **metadata: Any) -> pull_doc_reply: ...
     @overload
