@@ -64,22 +64,23 @@ from typing import (
     ClassVar,
     Dict,
     Generic,
-    List,
     Tuple,
+    TypedDict,
     TypeVar,
 )
 
-# External imports
-from typing_extensions import TypedDict
-
 # Bokeh imports
 import bokeh.util.serialization as bkserial
 
 # Bokeh imports
+from ..core.json_encoder import serialize_json
+from ..core.serialization import Buffer, Serialized
 from ..core.types import ID
 from .exceptions import MessageError, ProtocolError
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from ..client.websocket import WebSocketClientConnectionWrapper
 
 #-----------------------------------------------------------------------------
@@ -106,15 +107,14 @@ class Header(_Header, total=False):
     reqid: ID
     num_buffers: int
 
-# class BufferHeader(TypedDict):
-#     id: ID
-BufferHeader = str
+class BufferHeader(TypedDict):
+    id: ID
 
 Content = TypeVar("Content")
 
-Metadata = Dict[str, Any]
+Metadata: TypeAlias = Dict[str, Any]
 
-BufferRef = Tuple[BufferHeader, bytes]
+BufferRef: TypeAlias = Tuple[BufferHeader, bytes]
 
 class Empty(TypedDict):
     pass
@@ -137,7 +137,7 @@ class Message(Generic[Content]):
     _metadata: Metadata
     _metadata_json: str | None
 
-    _buffers: List[BufferRef]
+    _buffers: list[Buffer]
 
     def __init__(self, header: Header, metadata: Metadata, content: Content) -> None:
         ''' Initialize a new message from header, metadata, and content
@@ -207,7 +207,7 @@ class Message(Generic[Content]):
 
         return msg
 
-    def add_buffer(self, buf_header: BufferHeader, buf_payload: bytes) -> None:
+    def add_buffer(self, buffer: Buffer) -> None:
         ''' Associate a buffer header and payload with this message.
 
         Args:
@@ -227,8 +227,7 @@ class Message(Generic[Content]):
             self._header['num_buffers'] = 1
 
         self._header_json = None
-
-        self._buffers.append((buf_header, buf_payload))
+        self._buffers.append(buffer)
 
     def assemble_buffer(self, buf_header: BufferHeader, buf_payload: bytes) -> None:
         ''' Add a buffer header and payload that we read from the socket.
@@ -249,7 +248,7 @@ class Message(Generic[Content]):
         num_buffers = self.header.get("num_buffers", 0)
         if num_buffers <= len(self._buffers):
             raise ProtocolError(f"too many buffers received expecting {num_buffers}")
-        self._buffers.append((buf_header, buf_payload))
+        self._buffers.append(Buffer(buf_header["id"], buf_payload))
 
     async def write_buffers(self, conn: WebSocketClientConnectionWrapper, locked: bool = True) -> int:
         ''' Write any buffer headers and payloads to the given connection.
@@ -268,10 +267,12 @@ class Message(Generic[Content]):
         if conn is None:
             raise ValueError("Cannot write_buffers to connection None")
         sent = 0
-        for header, payload in self._buffers:
+        for buffer in self._buffers:
+            header = json.dumps(buffer.ref)
+            payload = buffer.to_bytes()
             await conn.write_message(header, locked=locked)
             await conn.write_message(payload, binary=True, locked=locked)
-            sent += (len(header) + len(payload))
+            sent += len(header) + len(payload)
         return sent
 
     @classmethod
@@ -342,6 +343,10 @@ class Message(Generic[Content]):
                self.content is not None and \
                self.header.get('num_buffers', 0) == len(self._buffers)
 
+    @property
+    def payload(self) -> Serialized[Content]:
+        return Serialized(self.content, self.buffers)
+
     # header fragment properties
 
     @property
@@ -373,7 +378,7 @@ class Message(Generic[Content]):
     @property
     def content_json(self) -> str:
         if not self._content_json:
-            self._content_json = json.dumps(self.content)
+            self._content_json = serialize_json(self.payload)
         return self._content_json
 
     # metadata fragment properties
@@ -396,8 +401,8 @@ class Message(Generic[Content]):
     # buffer properties
 
     @property
-    def buffers(self) -> List[BufferRef]:
-        return self._buffers
+    def buffers(self) -> list[Buffer]:
+        return list(self._buffers)
 
 #-----------------------------------------------------------------------------
 # Private API
