@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -22,11 +22,11 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+import json
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    List,
     Union,
     cast,
 )
@@ -36,6 +36,8 @@ from .exceptions import ValidationError
 from .message import BufferHeader, Message
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from . import Protocol
 
 #-----------------------------------------------------------------------------
@@ -54,7 +56,7 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-Fragment = Union[str, bytes]
+Fragment: TypeAlias = Union[str, bytes]
 
 class Receiver:
     ''' Receive wire message fragments and assemble complete Bokeh server
@@ -101,8 +103,8 @@ class Receiver:
 
     '''
 
-    _current_consumer: Callable[[Receiver, Fragment], None]
-    _fragments: List[Fragment]
+    _current_consumer: Callable[[Fragment], None]
+    _fragments: list[Fragment]
     _message: Message[Any] | None
     _buf_header: BufferHeader | None
     _partial: Message[Any] | None
@@ -116,7 +118,7 @@ class Receiver:
                 fragments.
         '''
         self._protocol = protocol
-        self._current_consumer = self._HEADER  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
+        self._current_consumer = self._HEADER
         self._message = None
         self._partial = None
         self._buf_header = None
@@ -138,12 +140,12 @@ class Receiver:
         self._message = None
         self._partial = None
         self._fragments = [self._assume_text(fragment)]
-        self._current_consumer = self._METADATA  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
+        self._current_consumer = self._METADATA
 
     def _METADATA(self, fragment: Fragment) -> None:
         metadata = self._assume_text(fragment)
         self._fragments.append(metadata)
-        self._current_consumer = self._CONTENT  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
+        self._current_consumer = self._CONTENT
 
     def _CONTENT(self, fragment: Fragment) -> None:
         content = self._assume_text(fragment)
@@ -156,21 +158,27 @@ class Receiver:
         self._check_complete()
 
     def _BUFFER_HEADER(self, fragment: Fragment) -> None:
-        self._buf_header = self._assume_text(fragment)
-        self._current_consumer = self._BUFFER_PAYLOAD  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
+        header = json.loads(self._assume_text(fragment))
+        if set(header) != { "id" }:
+            raise ValidationError(f"Malformed buffer header {header!r}")
+        self._buf_header = header
+        self._current_consumer = self._BUFFER_PAYLOAD
 
     def _BUFFER_PAYLOAD(self, fragment: Fragment) -> None:
         payload = self._assume_binary(fragment)
-        cast(Message[Any], self._partial).assemble_buffer(cast(BufferHeader, self._buf_header), payload)
+        if self._buf_header is None:
+            raise ValidationError("Consuming a buffer payload, but current buffer header is None")
+        header = BufferHeader(id=self._buf_header["id"])
+        cast(Message[Any], self._partial).assemble_buffer(header, payload)
 
         self._check_complete()
 
     def _check_complete(self) -> None:
         if self._partial and self._partial.complete:
             self._message = self._partial
-            self._current_consumer = self._HEADER  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
+            self._current_consumer = self._HEADER
         else:
-            self._current_consumer = self._BUFFER_HEADER  # type: ignore[assignment] # https://github.com/python/mypy/issues/2427
+            self._current_consumer = self._BUFFER_HEADER
 
     def _assume_text(self, fragment: Fragment) -> str:
         if not isinstance(fragment, str):
