@@ -50,6 +50,8 @@ log = logging.getLogger(__name__)
 # Import
 #-----------------------------------------------------------------------------
 
+# isort: skip_file
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -62,7 +64,7 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-from ..util.sampledata import download; download
+from ..util.sampledata import download
 
 #-----------------------------------------------------------------------------
 # Dev API
