@@ -6,7 +6,9 @@
 #-----------------------------------------------------------------------------
 ''' Browser market share by version from November 2013.
 
-Data sourced from http://gs.statcounter.com/#browser_version-ww-monthly-201311-201311-bar
+License: `CC BY-SA 3.0`_
+
+Sourced from http://gs.statcounter.com/#browser_version-ww-monthly-201311-201311-bar
 
 Icon images sourced from https://github.com/alrra/browser-logos
 
@@ -19,6 +21,7 @@ This module contains one pandas Dataframe: ``browsers_nov_2013``.
 The module also contains a dictionary ``icons`` with base64-encoded PNGs of the
 logos for Chrome, Firefox, Safari, Opera, and IE.
 
+.. bokeh-sampledata-xref:: browsers
 '''
 
 #-----------------------------------------------------------------------------
@@ -35,6 +38,10 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from os.path import join
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from pandas import DataFrame
 
 # Bokeh imports
 from ..util.sampledata import package_csv, package_path
@@ -60,7 +67,7 @@ __all__ = (
 # Private API
 #-----------------------------------------------------------------------------
 
-def _read_data():
+def _read_data() -> tuple[DataFrame, dict[str, bytes]]:
     '''
 
     '''
