@@ -6,14 +6,16 @@
 #-----------------------------------------------------------------------------
 ''' A small subset of data from the `Open Movie Database`_.
 
-Data is licensed `CC BY-NC 4.0`_.
+License: `CC BY-NC 4.0`_
+
+Sourced from http://www.omdbapi.com
 
 This modules has an attribute ``movie_path``. This attribute contains the path
 to a SQLite database with the data.
 
-.. _CC BY-NC 4.0: https://creativecommons.org/licenses/by-nc/4.0/
-.. _Open Movie Database: http://www.omdbapi.com
+.. bokeh-sampledata-xref:: movies_data
 
+.. _Open Movie Database: http://www.omdbapi.com
 '''
 
 #-----------------------------------------------------------------------------
