@@ -6,12 +6,15 @@
 #-----------------------------------------------------------------------------
 ''' Route data (including altitude) for a bike race in Eastern Europe.
 
+Sourced from https://bikemaraton.com.pl
+
 This module contains one pandas Dataframe: ``obiszow_mtb_xcm``.
 
 .. rubric:: ``obiszow_mtb_xcm``
 
 :bokeh-dataframe:`bokeh.sampledata.mtb.obiszow_mtb_xcm`
 
+.. bokeh-sampledata-xref:: mtb
 '''
 
 #-----------------------------------------------------------------------------
