@@ -1,14 +1,14 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 ''' Historical and projected population data by age, gender, and country.
 
-Sourced from: https://population.un.org/wpp/Download/Standard/Population/
+License: `CC BY 3.0 IGO`_
 
-Data is licenced `CC BY 3.0 IGO`_.
+Sourced from: https://population.un.org/wpp/Download/Standard/Population/
 
 This module contains one pandas Dataframe: ``data``.
 
@@ -16,8 +16,7 @@ This module contains one pandas Dataframe: ``data``.
 
 :bokeh-dataframe:`bokeh.sampledata.population.data`
 
-.. _CC BY 3.0 IGO: https://creativecommons.org/licenses/by/3.0/igo/
-
+.. bokeh-sampledata-xref:: population
 '''
 
 #-----------------------------------------------------------------------------
@@ -32,6 +31,12 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from pandas import DataFrame
+
 # Bokeh imports
 from ..util.sampledata import external_csv
 
@@ -55,7 +60,7 @@ __all__ = (
 # Private API
 #-----------------------------------------------------------------------------
 
-def _read_data():
+def _read_data() -> DataFrame:
     '''
 
     '''
