@@ -6,12 +6,15 @@
 #-----------------------------------------------------------------------------
 ''' Historical results for Olympic sprints by year.
 
+Sourced from public news sources.
+
 This module contains one pandas Dataframe: ``sprint``.
 
 .. rubric:: ``sprint``
 
 :bokeh-dataframe:`bokeh.sampledata.sprint.sprint`
 
+.. bokeh-sampledata-xref:: sprint
 '''
 
 #-----------------------------------------------------------------------------
