@@ -6,10 +6,9 @@
 #-----------------------------------------------------------------------------
 ''' Provide U.S. marriage and divorce statistics between 1867 and 2014
 
-Data from the CDC's National Center for Health Statistics (NHCS) database
-(http://www.cdc.gov/nchs/).
+License: `Public Domain`_
 
-Data organized by Randal S. Olson (http://www.randalolson.com)
+Sourced from http://www.cdc.gov/nchs/
 
 This module contains one pandas Dataframe: ``data``.
 
@@ -17,6 +16,7 @@ This module contains one pandas Dataframe: ``data``.
 
 :bokeh-dataframe:`bokeh.sampledata.us_marriages_divorces.data`
 
+.. bokeh-sampledata-xref:: us_marriages_divorces
 '''
 
 #-----------------------------------------------------------------------------
@@ -31,6 +31,12 @@ log = logging.getLogger(__name__)
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from pandas import DataFrame
+
 # Bokeh imports
 from ..util.sampledata import package_csv
 
@@ -54,7 +60,7 @@ __all__ = (
 # Private API
 #-----------------------------------------------------------------------------
 
-def _read_data():
+def _read_data() -> DataFrame:
     '''
 
     '''
