@@ -22,6 +22,7 @@ following structure:
         'lons': [-124.03622, ..., -124.15935]
     }
 
+.. bokeh-sampledata-xref:: us_states
 '''
 
 #-----------------------------------------------------------------------------
@@ -41,10 +42,14 @@ import codecs
 import csv
 import gzip
 import xml.etree.ElementTree as et
+from typing import TYPE_CHECKING, TypedDict
 
 # Bokeh imports
 from ..util.sampledata import package_path
 
+if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -65,38 +70,46 @@ __all__ = (
 # Private API
 #-----------------------------------------------------------------------------
 
-def _read_data():
+State: TypeAlias = str
+
+class StateData(TypedDict):
+    name: str
+    region: str
+    lats: list[float]
+    lons: list[float]
+
+nan = float('NaN')
+
+def _read_data() -> dict[State, StateData]:
     '''
 
     '''
-    nan = float('NaN')
-
-    data = {}
+    data: dict[State, StateData] = {}
 
     with gzip.open(package_path('US_Regions_State_Boundaries.csv.gz')) as f:
         decoded = codecs.iterdecode(f, "utf-8")
         next(decoded)
         reader = csv.reader(decoded, delimiter=",", quotechar='"')
         for row in reader:
-            region, name, code, geometry, dummy = row
+            region, name, code, geometry, _ = row
             xml = et.fromstring(geometry)
-            lats = []
-            lons = []
+            lats: list[float] = []
+            lons: list[float] = []
             for i, poly in enumerate(xml.findall('.//outerBoundaryIs/LinearRing/coordinates')):
                 if i > 0:
                     lats.append(nan)
                     lons.append(nan)
+                assert isinstance(poly.text, str)
                 coords = (c.split(',')[:2] for c in poly.text.split())
-                lat, lon = list(zip(*[(float(lat), float(lon)) for lon, lat in
-                    coords]))
+                lat, lon = list(zip(*[(float(lat), float(lon)) for lon, lat in coords]))
                 lats.extend(lat)
                 lons.extend(lon)
-            data[code] = {
-                'name'   : name,
-                'region' : region,
-                'lats'   : lats,
-                'lons'   : lons,
-            }
+            data[code] = StateData(
+                name   = name,
+                region = region,
+                lats   = lats,
+                lons   = lons,
+            )
 
     return data
 
