@@ -6,9 +6,9 @@
 #-----------------------------------------------------------------------------
 ''' Names and locations of world cities with at least 5000 inhabitants.
 
-Derived from ``cities5000.zip`` file downloaded from http://www.geonames.org/export/
+License: `CC BY 2.0`_
 
-Licensed under `CC-BY`_.
+Sourced from http://www.geonames.org/export/ (*cities5000.zip*)
 
 This module contains one pandas Dataframe: ``data``.
 
@@ -16,8 +16,7 @@ This module contains one pandas Dataframe: ``data``.
 
 :bokeh-dataframe:`bokeh.sampledata.world_cities.data`
 
-.. _CC-BY: https://creativecommons.org/licenses/by/2.0/
-
+.. bokeh-sampledata-xref:: world_cities
 '''
 
 #-----------------------------------------------------------------------------
