@@ -38,16 +38,10 @@ import signal
 import socket
 import sys
 from types import FrameType
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Dict,
-    List,
-    Mapping,
-)
+from typing import TYPE_CHECKING, Any, Mapping
 
 # External imports
-from tornado import version as tornado_version
+from tornado import netutil, version as tornado_version
 from tornado.httpserver import HTTPServer
 from tornado.ioloop import IOLoop
 
@@ -204,7 +198,7 @@ class BaseServer:
             self.start()
         # Install shutdown hooks
         atexit.register(self._atexit)
-        signal.signal(signal.SIGTERM, self._sigterm)   # type: ignore[arg-type]
+        signal.signal(signal.SIGTERM, self._sigterm)
         try:
             self._loop.start()
         except KeyboardInterrupt:
@@ -228,7 +222,7 @@ class BaseServer:
         '''
         return self._tornado.get_session(app_path, session_id)
 
-    def get_sessions(self, app_path: str | None = None) -> List[ServerSession]:
+    def get_sessions(self, app_path: str | None = None) -> list[ServerSession]:
         ''' Gets all currently active sessions for applications.
 
         Args:
@@ -243,7 +237,7 @@ class BaseServer:
         '''
         if app_path is not None:
             return self._tornado.get_sessions(app_path)
-        all_sessions: List[ServerSession] = []
+        all_sessions: list[ServerSession] = []
         for path in self._tornado.app_paths:
             all_sessions += self._tornado.get_sessions(path)
         return all_sessions
@@ -295,13 +289,13 @@ class BaseServer:
         if not self._stopped:
             self.stop(wait=False)
 
-    def _sigterm(self, signum: signal.Signals, frame: FrameType) -> None:
+    def _sigterm(self, signum: int, frame: FrameType | None) -> None:
         print(f"Received signal {signum}, shutting down")
         # Tell self._loop.start() to return.
         self._loop.add_callback_from_signal(self._loop.stop)
 
     @property
-    def port(self) -> int:
+    def port(self) -> int | None:
         ''' The configured port number that the server listens on for HTTP requests
         '''
         sock = next(
@@ -357,7 +351,7 @@ class Server(BaseServer):
     '''
 
     def __init__(self, applications: Mapping[str, Application | ModifyDoc] | Application | ModifyDoc,
-            io_loop: IOLoop | None = None, http_server_kwargs: Dict[str, Any] | None = None, **kwargs: Any) -> None:
+            io_loop: IOLoop | None = None, http_server_kwargs: dict[str, Any] | None = None, **kwargs: Any) -> None:
         ''' Create a ``Server`` instance.
 
         Args:
@@ -412,6 +406,9 @@ class Server(BaseServer):
         if opts.num_procs > 1 and sys.platform == "win32":
             raise RuntimeError("num_procs > 1 not supported on Windows")
 
+        if opts.unix_socket and sys.platform == "win32":
+            raise RuntimeError("Unix sockets are not supported on windows.")
+
         if http_server_kwargs is None:
             http_server_kwargs = {}
         http_server_kwargs.setdefault('xheaders', opts.use_xheaders)
@@ -423,10 +420,16 @@ class Server(BaseServer):
             context.load_cert_chain(certfile=opts.ssl_certfile, keyfile=opts.ssl_keyfile, password=opts.ssl_password)
             http_server_kwargs['ssl_options'] = context
 
-        sockets, self._port = bind_sockets(opts.address, opts.port)
-        self._address = opts.address
+        if opts.unix_socket:
+            sockets = [netutil.bind_unix_socket(opts.unix_socket)]
+            self._unix_socket = opts.unix_socket
+            self._address, self._port = None, None
+            extra_websocket_origins = []
+        else:
+            sockets, self._port = bind_sockets(opts.address, opts.port)
+            self._address = opts.address
 
-        extra_websocket_origins = create_hosts_allowlist(opts.allow_websocket_origin, self.port)
+            extra_websocket_origins = create_hosts_allowlist(opts.allow_websocket_origin, self.port)
         try:
             tornado_app = BokehTornado(applications,
                                        extra_websocket_origins=extra_websocket_origins,
@@ -457,13 +460,19 @@ class Server(BaseServer):
         super().__init__(io_loop, tornado_app, http_server)
 
     @property
-    def port(self) -> int:
+    def port(self) -> int | None:
         ''' The configured port number that the server listens on for HTTP
         requests.
-
         '''
         return self._port
 
+    @property
+    def unix_socket(self) -> str:
+        ''' The configured unix socket that the server listens to.
+
+        '''
+        return self._unix_socket
+
     @property
     def address(self) -> str | None:
         ''' The configured address that the server listens on for HTTP
@@ -501,6 +510,12 @@ class _ServerOpts(Options):
     The port number the server should listen on for HTTP requests.
     """)  # type: ignore[assignment]
 
+    unix_socket : str | None = Nullable(String, help="""
+    The unix socket the server should bind to. Other network args
+    such as port, address, ssl options etc are incompatible with unix sockets.
+    Unix socket support is not available on windows.
+    """)  # type: ignore[assignment]
+
     prefix: str = String(default="", help="""
     A URL prefix to use for all Bokeh server paths.
     """)  # type: ignore[assignment]
@@ -509,7 +524,7 @@ class _ServerOpts(Options):
     A path to a Jinja2 template to use for the index "/"
     """)  # type: ignore[assignment]
 
-    allow_websocket_origin: List[str] | None = Nullable(p.List(String), help="""
+    allow_websocket_origin: list[str] | None = Nullable(p.List(String), help="""
     A list of hosts that can connect to the websocket.
 
     This is typically required when embedding a Bokeh server app in an external
