@@ -25,7 +25,7 @@ from typing import TYPE_CHECKING
 
 # External imports
 from tornado.httputil import HTTPServerRequest
-from tornado.web import HTTPError, RequestHandler, authenticated
+from tornado.web import HTTPError, authenticated
 
 # Bokeh imports
 from bokeh.util.token import (
@@ -36,7 +36,7 @@ from bokeh.util.token import (
 )
 
 # Bokeh imports
-from .auth_mixin import AuthMixin
+from .auth_request_handler import AuthRequestHandler
 
 if TYPE_CHECKING:
     from ...core.types import ID
@@ -60,7 +60,7 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-class SessionHandler(AuthMixin, RequestHandler):
+class SessionHandler(AuthRequestHandler):
     ''' Implements a custom Tornado handler for document display page
 
     '''
@@ -127,7 +127,8 @@ class SessionHandler(AuthMixin, RequestHandler):
                 # Do not include Cookie header since cookies can be restored from cookies dict
                 del headers['Cookie']
 
-            payload = {'headers': headers, 'cookies': cookies}
+            arguments = {} if self.request.arguments is None else self.request.arguments
+            payload = {'headers': headers, 'cookies': cookies, 'arguments': arguments}
             payload.update(self.application_context.application.process_request(self.request))
             token = generate_jwt_token(session_id,
                                        secret_key=app.secret_key,
@@ -145,12 +146,6 @@ class SessionHandler(AuthMixin, RequestHandler):
 
         return session
 
-    # NOTE: The methods below exist on both AuthMixin and RequestHandler. This
-    # makes it explicit which of the versions is intended to be called.
-    get_login_url = AuthMixin.get_login_url
-    get_current_user = AuthMixin.get_current_user
-    prepare = AuthMixin.prepare
-
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
