@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -123,9 +123,8 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
     Generic,
-    List,
+    Literal,
     Sequence,
     Type,
     TypeVar,
@@ -135,12 +134,13 @@ from typing import (
 
 # External imports
 import yaml
-from typing_extensions import Literal
 
 # Bokeh imports
-from .util.paths import bokehjsdir
+from .util.paths import bokehjsdir, serverdir
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from .core.types import PathLike
     from .resources import ResourcesMode
 
@@ -165,7 +165,7 @@ def convert_str(value: str) -> str:
     '''
     return value
 
-def convert_bool(value: Union[bool, str]) -> bool:
+def convert_bool(value: bool | str) -> bool:
     ''' Convert a string to True or False.
 
     If a boolean is passed in, it is returned as-is. Otherwise the function
@@ -196,7 +196,7 @@ def convert_bool(value: Union[bool, str]) -> bool:
 
     raise ValueError(f"Cannot convert {value} to boolean value")
 
-def convert_str_seq(value: Union[List[str], str]) -> List[str]:
+def convert_str_seq(value: list[str] | str) -> list[str]:
     ''' Convert a string to a list of strings.
 
     If a list or tuple is passed in, it is returned as-is.
@@ -218,9 +218,9 @@ def convert_str_seq(value: Union[List[str], str]) -> List[str]:
         raise ValueError(f"Cannot convert {value} to list value")
 
 
-LogLevel = Literal["trace", "debug", "info", "warn", "error", "fatal"]
+LogLevel: TypeAlias = Literal["trace", "debug", "info", "warn", "error", "fatal"]
 
-PyLogLevel = Union[int, None]
+PyLogLevel: TypeAlias = Union[int, None]
 
 _log_levels = {
     "CRITICAL" : logging.CRITICAL,
@@ -232,7 +232,7 @@ _log_levels = {
     "NONE"     : None,
 }
 
-def convert_logging(value: Union[str, int]) -> PyLogLevel:
+def convert_logging(value: str | int) -> PyLogLevel:
     '''Convert a string to a Python logging level
 
     If a log level is passed in, it is returned as-is. Otherwise the function
@@ -269,7 +269,7 @@ def convert_logging(value: Union[str, int]) -> PyLogLevel:
 
 ValidationLevel = Literal["none", "errors", "all"]
 
-def convert_validation(value: Union[str, ValidationLevel]) -> ValidationLevel:
+def convert_validation(value: str | ValidationLevel) -> ValidationLevel:
     '''Convert a string to a validation level
 
     If a validation level is passed in, it is returned as-is.
@@ -293,11 +293,42 @@ def convert_validation(value: Union[str, ValidationLevel]) -> ValidationLevel:
 
     raise ValueError(f"Cannot convert {value!r} to validation level, valid values are: {VALID_LEVELS!r}")
 
+def convert_ico_path(value: str) -> str:
+    '''Convert a string to an .ico path
+
+    Args:
+        value (str):
+            A string value to convert to a .ico path
+
+    Returns:
+        string
+
+    Raises:
+        ValueError
+
+    '''
+    lowered = value.lower()
+
+    if lowered == "none":
+        return "none"
+
+    if lowered == "default":
+        return join(serverdir(), "views", "bokeh.ico")
+
+    # undocumented
+    if lowered == "default-dev":
+        return join(serverdir(), "views", "bokeh-dev.ico")
+
+    if not value.endswith(".ico"):
+        raise ValueError(f"Cannot convert {value!r} to valid .ico path")
+
+    return value
+
 class _Unset: pass
 
 T = TypeVar("T")
 
-Unset = Union[T, Type[_Unset]]
+Unset: TypeAlias = Union[T, Type[_Unset]]
 
 def is_dev() -> bool:
     return convert_bool(os.environ.get("BOKEH_DEV", False))
@@ -332,10 +363,10 @@ class PrioritizedSetting(Generic[T]):
     '''
 
     _parent: Settings | None
-    _user_value: Unset[Union[str, T]]
+    _user_value: Unset[str | T]
 
     def __init__(self, name: str, env_var: str | None = None, default: Unset[T] = _Unset,
-            dev_default: Unset[T] = _Unset, convert: Callable[[Union[T, str]], T] | None = None, help: str = "") -> None:
+            dev_default: Unset[T] = _Unset, convert: Callable[[T | str], T] | None = None, help: str = "") -> None:
         self._convert = convert if convert else convert_str
         self._default = default
         self._dev_default = dev_default
@@ -345,7 +376,7 @@ class PrioritizedSetting(Generic[T]):
         self._parent = None
         self._user_value = _Unset
 
-    def __call__(self, value: Union[T, str] | None = None, default: Unset[T] = _Unset) -> T:
+    def __call__(self, value: T | str | None = None, default: Unset[T] = _Unset) -> T:
         '''Return the setting value according to the standard precedence.
 
         Args:
@@ -402,13 +433,13 @@ class PrioritizedSetting(Generic[T]):
 
         raise RuntimeError(f"No configured value found for setting {self._name!r}")
 
-    def __get__(self, instance: Any, owner: Type[Any]) -> PrioritizedSetting[T]:
+    def __get__(self, instance: Any, owner: type[Any]) -> PrioritizedSetting[T]:
         return self
 
-    def __set__(self, instance: Any, value: Union[str, T]) -> None:
+    def __set__(self, instance: Any, value: str | T) -> None:
         self.set_value(value)
 
-    def set_value(self, value: Union[str, T]) -> None:
+    def set_value(self, value: str | T) -> None:
         ''' Specify a value for this setting programmatically.
 
         A value set this way takes precedence over all other methods except
@@ -465,6 +496,8 @@ class PrioritizedSetting(Generic[T]):
             return "List[String]"
         if self._convert is convert_validation:
             return "Validation Level"
+        if self._convert is convert_ico_path:
+            return "Ico Path"
         raise RuntimeError("unreachable")
 
 _config_user_locations: Sequence[str] = (
@@ -476,9 +509,9 @@ class Settings:
 
     '''
 
-    _config_override: Dict[str, Any]
-    _config_user: Dict[str, Any]
-    _config_system: Dict[str, Any]
+    _config_override: dict[str, Any]
+    _config_user: dict[str, Any]
+    _config_system: dict[str, Any]
 
     def __init__(self) -> None:
         self._config_override = {}
@@ -490,22 +523,22 @@ class Settings:
                 x._parent = self
 
     @property
-    def config_system(self) -> Dict[str, Any]:
+    def config_system(self) -> dict[str, Any]:
         return dict(self._config_system)
 
     @property
-    def config_user(self) -> Dict[str, Any]:
+    def config_user(self) -> dict[str, Any]:
         return dict(self._config_user)
 
     @property
-    def config_override(self) -> Dict[str, Any]:
+    def config_override(self) -> dict[str, Any]:
         return dict(self._config_override)
 
     @property
     def dev(self) -> bool:
         return is_dev()
 
-    allowed_ws_origin: PrioritizedSetting[List[str]] = PrioritizedSetting("allowed_ws_origin", "BOKEH_ALLOW_WS_ORIGIN", default=[], convert=convert_str_seq, help="""
+    allowed_ws_origin: PrioritizedSetting[list[str]] = PrioritizedSetting("allowed_ws_origin", "BOKEH_ALLOW_WS_ORIGIN", default=[], convert=convert_str_seq, help="""
     A comma-separated list of allowed websocket origins for Bokeh server applications.
     """)
 
@@ -538,11 +571,6 @@ class Settings:
     long, random sequence of bytes
     """)
 
-    docs_alert: PrioritizedSetting[str | None] = PrioritizedSetting("docs_alert", "BOKEH_DOCS_ALERT", default=None, help="""
-    Text for an alert banner to display when building the docs locally (for
-    testing the alert banner capability).
-    """)
-
     docs_cdn: PrioritizedSetting[str | None] = PrioritizedSetting("docs_cdn", "BOKEH_DOCS_CDN", default=None, help="""
     The version of BokehJS that should be use for loading CDN resources when
     building the docs.
@@ -573,6 +601,19 @@ class Settings:
     fixes or changes.
     """)
 
+    ico_path: PrioritizedSetting[str] = PrioritizedSetting("ico_path", "BOKEH_ICO_PATH",
+        default="default", dev_default="default-dev", convert=convert_ico_path, help="""
+    Configure the file path to a .ico file for the Bokeh server to use as a
+    favicon.ico file.
+
+    The value should be the full path to a .ico file, or one the following
+    special values:
+
+    - ``default`` to use the default project .ico file
+    - ``none`` to turn off favicon.ico support entirely
+
+    """)
+
     ignore_filename: PrioritizedSetting[bool] = PrioritizedSetting("ignore_filename", "BOKEH_IGNORE_FILENAME", default=False, convert=convert_bool, help="""
     Whether to ignore the current script filename when saving Bokeh content.
     """)
@@ -595,10 +636,6 @@ class Settings:
     Whether Bokeh should use minified BokehJS resources.
     """)
 
-    legacy: PrioritizedSetting[bool] = PrioritizedSetting("legacy", "BOKEH_LEGACY", convert=convert_bool, default=False, dev_default=False, help="""
-    Whether Bokeh should use legacy (IE and phantomjs compatible) BokehJS resources.
-    """)
-
     nodejs_path: PrioritizedSetting[str | None] = PrioritizedSetting("nodejs_path", "BOKEH_NODEJS_PATH", default=None, help="""
     Path to the Node executable.
 
@@ -618,7 +655,8 @@ class Settings:
     Whether JSON strings should be pretty-printed.
     """)
 
-    py_log_level: PrioritizedSetting[PyLogLevel] = PrioritizedSetting("py_log_level", "BOKEH_PY_LOG_LEVEL", default="none", dev_default="debug", convert=convert_logging, help="""
+    py_log_level: PrioritizedSetting[PyLogLevel] = PrioritizedSetting("py_log_level", "BOKEH_PY_LOG_LEVEL",
+        default="none", dev_default="debug", convert=convert_logging, help="""
     The log level for Python Bokeh code.
 
     Valid values are, in order of increasing severity:
@@ -674,7 +712,8 @@ class Settings:
     A password to decrypt the SSL keyfile, if necessary.
     """)
 
-    validation_level: PrioritizedSetting[ValidationLevel] = PrioritizedSetting("validation_level", "BOKEH_VALIDATION_LEVEL", default="none", convert=convert_validation, help="""
+    validation_level: PrioritizedSetting[ValidationLevel] = PrioritizedSetting("validation_level", "BOKEH_VALIDATION_LEVEL",
+        default="none", convert=convert_validation, help="""
     Whether validation checks should log or raise exceptions on errors and warnings.
 
     Valid values are:
@@ -704,22 +743,22 @@ class Settings:
         '''
         return bokehjsdir(self.dev)
 
-    def css_files(self) -> List[str]:
+    def css_files(self) -> list[str]:
         ''' The CSS files in the BokehJS directory.
 
         '''
-        css_files: List[str] = []
+        css_files: list[str] = []
         for root, _, files in os.walk(self.bokehjsdir()):
             for fname in files:
                 if fname.endswith(".css"):
                     css_files.append(join(root, fname))
         return css_files
 
-    def js_files(self) -> List[str]:
+    def js_files(self) -> list[str]:
         ''' The JS files in the BokehJS directory.
 
         '''
-        js_files: List[str] = []
+        js_files: list[str] = []
         for root, _, files in os.walk(self.bokehjsdir()):
             for fname in files:
                 if fname.endswith(".js"):
@@ -749,7 +788,7 @@ class Settings:
                 self._secret_key_bytes = key.encode("utf-8")
         return self._secret_key_bytes
 
-    def _try_load_config(self, locations: Sequence[str]) -> Dict[str, Any]:
+    def _try_load_config(self, locations: Sequence[str]) -> dict[str, Any]:
         for location in locations:
             try:
                 return yaml.load(open(location), Loader=yaml.SafeLoader)
@@ -769,10 +808,10 @@ settings = Settings()
 
 _secret_key = settings.secret_key()
 if _secret_key is not None and len(_secret_key) < 32:
-    import warnings
-    warnings.warn("BOKEH_SECRET_KEY is recommended to have at least 32 bytes of entropy chosen with a cryptographically-random algorithm")
+    from .util.warnings import warn
+    warn("BOKEH_SECRET_KEY is recommended to have at least 32 bytes of entropy chosen with a cryptographically-random algorithm")
 del _secret_key
 
 if settings.sign_sessions() and settings.secret_key() is None:
-    import warnings
-    warnings.warn("BOKEH_SECRET_KEY must be set if BOKEH_SIGN_SESSIONS is set to True")
+    from .util.warnings import warn
+    warn("BOKEH_SECRET_KEY must be set if BOKEH_SIGN_SESSIONS is set to True")
