@@ -1,5 +1,5 @@
 # -----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -89,6 +89,12 @@ class EnumDocumenter(ModuleLevelDocumenter):
     def can_document_member(cls, member, membername, isattr, parent):
         return isinstance(member, Enumeration)
 
+    # Override the Sphinx default `Documenter.get_object_members()`
+    # which is deprecated, and will soon be removed.
+    # Ref: https://github.com/bokeh/bokeh/issues/12462
+    def get_object_members(self, want_all):
+        return False, []
+
 
 class PropDocumenter(AttributeDocumenter):
     directivetype = "bokeh-prop"
