@@ -10,8 +10,8 @@ The ``bokeh-example-metadata`` directive can be used by supplying:
 
     .. bokeh-example-metadata::
         :sampledata: `sampledata_iris`
-        :apis: `~bokeh.plotting.Figure.vbar`, :func:`~bokeh.transform.factor_cmap`
-        :refs: `userguide_categorical_bars`
+        :apis: `~bokeh.plotting.figure.vbar`, :func:`~bokeh.transform.factor_cmap`
+        :refs: `ug_basic_bars`
         :words: bar, vbar, legend, factor_cmap, palette
 
 
@@ -106,7 +106,7 @@ def _sampledata(mods: str | None) -> str | None:
     if mods is None:
         return
 
-    # options lines might need a flake8 noqa comment for line length, etc
+    # options lines might need a ruff noqa comment for line length, etc
     mods = mods.split("#")[0].strip()
 
     mods = (mod.strip() for mod in mods.split(","))
@@ -117,7 +117,7 @@ def _apis(apis: str | None) -> str | None:
     if apis is None:
         return
 
-    # options lines might need a flake8 noqa comment for line length, etc
+    # options lines might need a ruff noqa comment for line length, etc
     apis = apis.split("#")[0].strip()
 
     results = []
@@ -128,8 +128,8 @@ def _apis(apis: str | None) -> str | None:
         # handle a few special cases more carefully
         if api.startswith("bokeh.models"):
             results.append(f":class:`bokeh.models.{last} <{api}>`")
-        elif "Figure." in api:
-            results.append(f":meth:`Figure.{last} <{api}>`")
+        elif "figure." in api:
+            results.append(f":meth:`figure.{last} <{api}>`")
         elif "GMap." in api:
             results.append(f":meth:`GMap.{last} <{api}>`")
 
