@@ -57,12 +57,14 @@ log = logging.getLogger(__name__)
 import importlib
 import json
 import warnings
+from os import getenv
 
 # External imports
 from docutils.parsers.rst.directives import unchanged
 from sphinx.errors import SphinxError
 
 # Bokeh imports
+from bokeh.core.serialization import AnyRep, Serializer
 from bokeh.model import Model
 from bokeh.util.warnings import BokehDeprecationWarning
 
@@ -93,13 +95,15 @@ class BokehModelDirective(BokehDirective):
 
     has_content = True
     required_arguments = 1
-    optional_arguments = 1
+    optional_arguments = 0
+    final_argument_whitespace = True
     option_spec = {
         "module": unchanged,
         "canonical": unchanged,
     }
 
     def run(self):
+
         sig = " ".join(self.arguments)
 
         m = py_sig_re.match(sig)
@@ -107,6 +111,9 @@ class BokehModelDirective(BokehDirective):
             raise SphinxError(f"Unable to parse signature for bokeh-model: {sig!r}")
         name_prefix, model_name, arglist, retann = m.groups()
 
+        if getenv("BOKEH_SPHINX_QUICK") == "1":
+            return self.parse(f"{model_name}\n{'-'*len(model_name)}\n", "<bokeh-model>")
+
         module_name = self.options["module"]
 
         try:
@@ -128,11 +135,14 @@ class BokehModelDirective(BokehDirective):
             warnings.filterwarnings("ignore", category=BokehDeprecationWarning)
             model_obj = model()
 
-        model_json = json.dumps(model_obj.to_json(include_defaults=True), sort_keys=True, indent=2, separators=(",", ": "))
+        model_json = json.dumps(to_json_rep(model_obj), sort_keys=True, indent=2, separators=(",", ": "))
+
+        # we only want to document things as coming from top-level `bokeh.models`
+        adjusted_module_name = "bokeh.models" if module_name.startswith("bokeh.models") else module_name
 
         rst_text = MODEL_DETAIL.render(
             name=model_name,
-            module_name="bokeh.models" if module_name.startswith("bokeh.models") else module_name,
+            module_name=adjusted_module_name,
             model_json=model_json,
         )
 
@@ -149,6 +159,14 @@ def setup(app):
 # Private API
 # -----------------------------------------------------------------------------
 
+def to_json_rep(obj: Model) -> dict[str, AnyRep]:
+    serializer = Serializer()
+
+    properties = obj.properties_with_values(include_defaults=True)
+    attributes = {key: serializer.encode(val) for key, val in properties.items()}
+
+    return dict(id=obj.id, **attributes)
+
 # -----------------------------------------------------------------------------
 # Code
 # -----------------------------------------------------------------------------
