@@ -1,5 +1,5 @@
 # -----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -47,8 +47,7 @@ process-docstring (bool):
     separate from the source.
 
 source-position (enum('above', 'below', 'none')):
-    Where to locate the the block of formatted source
-    code (if anywhere).
+    Where to locate the block of formatted source code (if anywhere).
 
 linenos (bool):
     Whether to display line numbers along with the source.
@@ -112,7 +111,7 @@ from bokeh.util.warnings import BokehDeprecationWarning
 from . import PARALLEL_SAFE
 from .bokeh_directive import BokehDirective
 from .example_handler import ExampleHandler
-from .util import get_sphinx_resources
+from .util import _REPO_TOP, get_sphinx_resources
 
 # -----------------------------------------------------------------------------
 # Globals and constants
@@ -147,7 +146,6 @@ class autoload_script(nodes.General, nodes.Element):
 
     html = visit_html.__func__, None
 
-
 class BokehPlotDirective(BokehDirective):
 
     has_content = True
@@ -160,6 +158,9 @@ class BokehPlotDirective(BokehDirective):
     }
 
     def run(self):
+        if getenv("BOKEH_SPHINX_QUICK") == "1":
+            return []
+
         source, path = self.process_args_or_content()
 
         dashed_docname = self.env.docname.replace("/", "-")
@@ -176,6 +177,8 @@ class BokehPlotDirective(BokehDirective):
         target_id = f"{dashed_docname}.{basename(js_path)}"
         target = [nodes.target("", "", ids=[target_id])]
 
+        self.process_sampledata(source)
+
         process_docstring = self.options.get("process-docstring", False)
         intro = self.parse(docstring, '<bokeh-content>') if docstring and process_docstring else []
 
@@ -215,13 +218,15 @@ class BokehPlotDirective(BokehDirective):
 
         path = self.arguments[0]
         log.debug(f"[bokeh-plot] handling external content in {self.env.docname!r}: {path}")
-        if not path.startswith("/"):
+        if path.startswith("__REPO__/"):
+            path = join(_REPO_TOP, path.replace("__REPO__/", ""))
+        elif not path.startswith("/"):
             path = join(self.env.app.srcdir, path)
         try:
             with open(path) as f:
                 return f.read(), path
         except Exception as e:
-            raise SphinxError(f"bokeh-plot:: error reading source for {self.env.docname!r}: {e!r}")
+            raise SphinxError(f"bokeh-plot:: error reading {path!r} for {self.env.docname!r}: {e!r}")
 
     def process_source(self, source, path, js_filename):
         Model._clear_extensions()
@@ -238,6 +243,36 @@ class BokehPlotDirective(BokehDirective):
 
         return (script_tag, js_path, source, docstring, height_hint)
 
+    def process_sampledata(self, source):
+
+        if not hasattr(self.env, 'solved_sampledata'):
+            self.env.solved_sampledata = []
+
+        file, lineno =  self.get_source_info()
+        # collect links to all standalone examples
+
+        if '/docs/examples/' in file and file not in self.env.solved_sampledata:
+            self.env.solved_sampledata.append(file)
+            if not hasattr(self.env, 'all_sampledata_xrefs'):
+                self.env.all_sampledata_xrefs = []
+            if not hasattr(self.env, 'all_gallery_overview'):
+                self.env.all_gallery_overview = []
+
+            self.env.all_gallery_overview.append({
+                'docname': self.env.docname,
+            })
+
+            regex = r"(:|bokeh\.)sampledata(:|\.| import )\s*(\w+(\,\s*\w+)*)"
+            matches = re.findall(regex, source)
+            if matches:
+                keywords = set()
+                for m in matches:
+                    keywords.update(m[2].replace(" ","").split(','))
+                for keyword in keywords:
+                    self.env.all_sampledata_xrefs.append({
+                        'docname': self.env.docname,
+                        'keyword': keyword,
+                    })
 # -----------------------------------------------------------------------------
 # Dev API
 # -----------------------------------------------------------------------------
@@ -245,7 +280,7 @@ class BokehPlotDirective(BokehDirective):
 
 def builder_inited(app):
     app.env.bokeh_plot_auxdir = join(app.env.doctreedir, "bokeh_plot")
-    ensuredir(app.env.bokeh_plot_auxdir)  # sphinx/build/doctrees/bokeh_plot
+    ensuredir(app.env.bokeh_plot_auxdir)  # docs/bokeh/build/doctrees/bokeh_plot
 
     if not hasattr(app.env, "bokeh_plot_files"):
         app.env.bokeh_plot_files = set()
