@@ -14,6 +14,10 @@
 # -----------------------------------------------------------------------------
 from __future__ import annotations
 
+# Standard library imports
+import os
+from pathlib import PurePath
+
 import logging  # isort:skip
 
 log = logging.getLogger(__name__)
@@ -40,6 +44,8 @@ __all__ = ("get_sphinx_resources",)
 # Dev API
 # -----------------------------------------------------------------------------
 
+# TODO (bv) this needs to be a configuration
+_REPO_TOP = PurePath(os.getcwd()).parents[1]
 
 def get_sphinx_resources(include_bokehjs_api=False):
     docs_cdn = settings.docs_cdn()
@@ -65,7 +71,6 @@ def get_sphinx_resources(include_bokehjs_api=False):
         resources.js_components.append("bokeh-api")
     return resources
 
-
 # -----------------------------------------------------------------------------
 # Private API
 # -----------------------------------------------------------------------------
