@@ -7,7 +7,7 @@
 
 json = {
     "attrs": {
-        "Figure" : {
+        "Plot": {
             "background_fill_color": "#2C001E",
             "border_fill_color": "#15191C",
             "outline_line_color": "#E0E0E0",
@@ -54,7 +54,7 @@ json = {
             "background_fill_color": "#2C001E"
         },
 
-        "ColorBar": {
+        "BaseColorBar": {
             "title_text_color": "#E0E0E0",
             "title_text_font": "Helvetica",
             "title_text_font_size": "1.025em",
