@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -95,17 +95,17 @@ functionality.
 .. _bokeh.util.token:
 
 ``bokeh.util.token``
--------------------------
+--------------------
 
 .. automodule:: bokeh.util.token
   :members:
 
-.. _bokeh.util.string:
+.. _bokeh.util.strings:
 
-``bokeh.util.string``
----------------------
+``bokeh.util.strings``
+----------------------
 
-.. automodule:: bokeh.util.string
+.. automodule:: bokeh.util.strings
   :members:
 
 .. _bokeh.util.tornado:
