@@ -22,25 +22,24 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+from collections import defaultdict
 from inspect import signature
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
-    List,
     Sequence,
-    Type,
     Union,
     cast,
 )
 
 # Bokeh imports
-from ..core.types import Unknown
 from ..events import Event, ModelEvent
 from ..util.functions import get_param_info
 
 if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
     from ..core.has_props import Setter
     from ..core.types import ID
     from ..document.document import Document
@@ -62,11 +61,11 @@ __all__ = (
 # TODO (bev) the situation with no-argument Button callbacks is a mess. We
 # should migrate to all callbacks receving the event as the param, even if that
 # means auto-magically wrapping user-supplied callbacks for awhile.
-EventCallbackWithEvent = Callable[[Event], None]
-EventCallbackWithoutEvent = Callable[[], None]
-EventCallback = Union[EventCallbackWithEvent, EventCallbackWithoutEvent]
+EventCallbackWithEvent: TypeAlias = Callable[[Event], None]
+EventCallbackWithoutEvent: TypeAlias = Callable[[], None]
+EventCallback: TypeAlias = Union[EventCallbackWithEvent, EventCallbackWithoutEvent]
 
-PropertyCallback = Callable[[str, Unknown, Unknown], None]
+PropertyCallback: TypeAlias = Callable[[str, Any, Any], None]
 
 class EventCallbackManager:
     ''' A mixin class to provide an interface for registering and
@@ -76,14 +75,14 @@ class EventCallbackManager:
 
     document: Document | None
     id: ID
-    subscribed_events: List[str]
-    _event_callbacks: Dict[str, List[EventCallback]]
+    subscribed_events: set[str]
+    _event_callbacks: dict[str, list[EventCallback]]
 
     def __init__(self, *args: Any, **kw: Any) -> None:
         super().__init__(*args, **kw)
-        self._event_callbacks = {}
+        self._event_callbacks = defaultdict(list)
 
-    def on_event(self, event: str | Type[Event], *callbacks: EventCallback) -> None:
+    def on_event(self, event: str | type[Event], *callbacks: EventCallback) -> None:
         ''' Run callbacks when the specified event occurs on this Model
 
         Not all Events are supported for all Models.
@@ -96,19 +95,14 @@ class EventCallbackManager:
         for callback in callbacks:
             if _nargs(callback) != 0:
                 _check_callback(callback, ('event',), what='Event callback')
+            self._event_callbacks[event].append(callback)
 
-        if event not in self._event_callbacks:
-            self._event_callbacks[event] = [cb for cb in callbacks]
-        else:
-            self._event_callbacks[event].extend(callbacks)
-
-        if event not in self.subscribed_events:
-            self.subscribed_events.append(event)
+        self.subscribed_events.add(event)
 
     def _trigger_event(self, event: ModelEvent) -> None:
         def invoke() -> None:
-            for callback in self._event_callbacks.get(event.event_name,[]):
-                if event._model_id is not None and self.id == event._model_id:
+            for callback in self._event_callbacks.get(event.event_name, []):
+                if event.model is not None and self.id == event.model.id:
                     if _nargs(callback) == 0:
                         cast(EventCallbackWithoutEvent, callback)()
                     else:
@@ -135,7 +129,7 @@ class PropertyCallbackManager:
     '''
 
     document: Document | None
-    _callbacks: Dict[str, List[PropertyCallback]]
+    _callbacks: dict[str, list[PropertyCallback]]
 
     def __init__(self, *args: Any, **kw: Any) -> None:
         super().__init__(*args, **kw)
@@ -171,7 +165,7 @@ class PropertyCallbackManager:
         for callback in callbacks:
             _callbacks.remove(callback)
 
-    def trigger(self, attr: str, old: Unknown, new: Unknown,
+    def trigger(self, attr: str, old: Any, new: Any,
             hint: DocumentPatchedEvent | None = None, setter: Setter | None = None) -> None:
         ''' Trigger callbacks for ``attr`` on this object.
 
