@@ -35,16 +35,16 @@ from os.path import (
 )
 from subprocess import PIPE, Popen
 from typing import (
+    Any,
+    Callable,
     Dict,
-    List,
     Sequence,
-    Type,
 )
 
 # Bokeh imports
-from ..model import Model
+from ..core.has_props import HasProps
 from ..settings import settings
-from .string import snakify
+from .strings import snakify
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -74,24 +74,24 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-class AttrDict(dict):
+class AttrDict(Dict[str, Any]):
     ''' Provide a dict subclass that supports access by named attributes.
 
     '''
-    def __getattr__(self, key):
+    def __getattr__(self, key: str) -> Any:
         return self[key]
 
 class CompilationError(RuntimeError):
     ''' A ``RuntimeError`` subclass for reporting JS compilation errors.
 
     '''
-    def __init__(self, error) -> None:
+    def __init__(self, error: dict[str, str] | str) -> None:
         super().__init__()
         if isinstance(error, dict):
             self.line = error.get("line")
             self.column = error.get("column")
             self.message = error.get("message")
-            self.text = error.get("text")
+            self.text = error.get("text", "")
             self.annotated = error.get("annotated")
         else:
             self.text = error
@@ -108,7 +108,7 @@ def nodejs_version() -> str | None:
 def npmjs_version() -> str | None:
     return _version(_run_npmjs)
 
-def nodejs_compile(code, lang="javascript", file=None):
+def nodejs_compile(code: str, lang: str = "javascript", file: str | None = None) -> AttrDict:
     compilejs_script = join(bokehjs_dir, "js", "compiler.js")
     output = _run_nodejs([compilejs_script], dict(code=code, lang=lang, file=file, bokehjs_dir=bokehjs_dir))
     lines = output.split("\n")
@@ -120,14 +120,17 @@ def nodejs_compile(code, lang="javascript", file=None):
     obj = json.loads("\n".join(lines[i:]))
     if isinstance(obj, dict):
         return AttrDict(obj)
-    else:
-        return dict(error=obj)
+    raise CompilationError(obj)
 
 class Implementation:
     ''' Base class for representing Bokeh custom model implementations.
 
     '''
-    file = None
+    file: str | None = None
+    code: str
+    @property
+    def lang(self) -> str:
+        raise NotImplementedError()
 
 class Inline(Implementation):
     ''' Base class for representing Bokeh custom model implementations that may
@@ -141,7 +144,7 @@ class Inline(Implementation):
             A file path to a file containing the source text (default: None)
 
     '''
-    def __init__(self, code, file=None) -> None:
+    def __init__(self, code: str, file: str|None = None) -> None:
         self.code = code
         self.file = file
 
@@ -157,7 +160,7 @@ class TypeScript(Inline):
 
     '''
     @property
-    def lang(self):
+    def lang(self) -> str:
         return "typescript"
 
 class JavaScript(Inline):
@@ -172,7 +175,7 @@ class JavaScript(Inline):
 
     '''
     @property
-    def lang(self):
+    def lang(self) -> str:
         return "javascript"
 
 class Less(Inline):
@@ -180,7 +183,7 @@ class Less(Inline):
 
     '''
     @property
-    def lang(self):
+    def lang(self) -> str:
         return "less"
 
 class FromFile(Implementation):
@@ -197,13 +200,15 @@ class FromFile(Implementation):
         self.file = path
 
     @property
-    def lang(self) -> str | None:
-        if self.file.endswith(".ts"):
-            return "typescript"
-        if self.file.endswith(".js"):
-            return "javascript"
-        if self.file.endswith((".css", ".less")):
-            return "less"
+    def lang(self) -> str:
+        if self.file is not None:
+            if self.file.endswith(".ts"):
+                return "typescript"
+            if self.file.endswith(".js"):
+                return "javascript"
+            if self.file.endswith((".css", ".less")):
+                return "less"
+        raise ValueError(f"unknown file type {self.file}")
 
 #: recognized extensions that can be compiled
 exts = (".ts", ".js", ".css", ".less")
@@ -212,7 +217,7 @@ class CustomModel:
     ''' Represent a custom (user-defined) Bokeh model.
 
     '''
-    def __init__(self, cls: Type[Model]) -> None:
+    def __init__(self, cls: type[HasProps]) -> None:
         self.cls = cls
 
     @property
@@ -228,8 +233,8 @@ class CustomModel:
     def file(self) -> str | None:
         module = sys.modules[self.cls.__module__]
 
-        if hasattr(module, "__file__"):
-            return abspath(module.__file__)
+        if hasattr(module, "__file__") and (file := module.__file__) is not None:
+            return abspath(file)
         else:
             return None
 
@@ -246,7 +251,7 @@ class CustomModel:
 
     @property
     def implementation(self) -> Implementation:
-        impl = self.cls.__implementation__
+        impl = getattr(self.cls, "__implementation__")
 
         if isinstance(impl, str):
             if "\n" not in impl and impl.endswith(exts):
@@ -261,25 +266,25 @@ class CustomModel:
         return impl
 
     @property
-    def dependencies(self):
+    def dependencies(self) -> dict[str, str]:
         return getattr(self.cls, "__dependencies__", {})
 
     @property
-    def module(self):
-        return "custom/%s" % snakify(self.full_name)
+    def module(self) -> str:
+        return f"custom/{snakify(self.full_name)}"
 
-def get_cache_hook():
+def get_cache_hook() -> Callable[[CustomModel, Implementation], AttrDict | None]:
     '''Returns the current cache hook used to look up the compiled
        code given the CustomModel and Implementation'''
     return _CACHING_IMPLEMENTATION
 
-def set_cache_hook(hook):
+def set_cache_hook(hook: Callable[[CustomModel, Implementation], AttrDict | None]) -> None:
     '''Sets a compiled model cache hook used to look up the compiled
        code given the CustomModel and Implementation'''
     global _CACHING_IMPLEMENTATION
     _CACHING_IMPLEMENTATION = hook
 
-def calc_cache_key(custom_models: Dict[str, CustomModel]) -> str:
+def calc_cache_key(custom_models: dict[str, CustomModel]) -> str:
     ''' Generate a key to cache a custom extension implementation with.
 
     There is no metadata other than the Model classes, so this is the only
@@ -293,9 +298,9 @@ def calc_cache_key(custom_models: Dict[str, CustomModel]) -> str:
     encoded_names = ",".join(sorted(model_names)).encode('utf-8')
     return hashlib.sha256(encoded_names).hexdigest()
 
-_bundle_cache: Dict[str, str] = {}
+_bundle_cache: dict[str, str] = {}
 
-def bundle_models(models: Sequence[Type[Model]] | None) -> str | None:
+def bundle_models(models: Sequence[type[HasProps]] | None) -> str | None:
     """Create a bundle of selected `models`. """
     custom_models = _get_custom_models(models)
     if custom_models is None:
@@ -383,11 +388,9 @@ _export_template = \
 _module_template = \
 """"%(module)s": function(require, module, exports) {\n%(source)s\n}"""
 
-def _detect_nodejs():
-    if settings.nodejs_path() is not None:
-        nodejs_paths = [settings.nodejs_path()]
-    else:
-        nodejs_paths = ["nodejs", "node"]
+def _detect_nodejs() -> str:
+    nodejs_path = settings.nodejs_path()
+    nodejs_paths = [nodejs_path] if nodejs_path is not None else ["nodejs", "node"]
 
     for nodejs_path in nodejs_paths:
         try:
@@ -408,8 +411,8 @@ def _detect_nodejs():
                 return nodejs_path
 
     # if we've reached here, no valid version was found
-    version = ".".join(map(str, nodejs_min_version))
-    raise RuntimeError('node.js v%s or higher is needed to allow compilation of custom models ' % version +
+    version_repr = ".".join(str(x) for x in nodejs_min_version)
+    raise RuntimeError(f'node.js v{version_repr} or higher is needed to allow compilation of custom models ' +
                        '("conda install nodejs" or follow https://nodejs.org/en/download/)')
 
 _nodejs = None
@@ -421,7 +424,7 @@ def _nodejs_path() -> str:
         _nodejs = _detect_nodejs()
     return _nodejs
 
-def _npmjs_path():
+def _npmjs_path() -> str:
     global _npmjs
     if _npmjs is None:
         _npmjs = join(dirname(_nodejs_path()), "npm")
@@ -429,10 +432,10 @@ def _npmjs_path():
             _npmjs += '.cmd'
     return _npmjs
 
-def _crlf_cr_2_lf(s):
+def _crlf_cr_2_lf(s: str) -> str:
     return re.sub(r"\\r\\n|\\r|\\n", r"\\n", s)
 
-def _run(app, argv, input=None):
+def _run(app: str, argv: list[str], input: dict[str, Any] | None = None) -> str:
     proc = Popen([app] + argv, stdout=PIPE, stderr=PIPE, stdin=PIPE)
     (stdout, errout) = proc.communicate(input=None if input is None else json.dumps(input).encode())
 
@@ -441,31 +444,31 @@ def _run(app, argv, input=None):
     else:
         return _crlf_cr_2_lf(stdout.decode('utf-8'))
 
-def _run_nodejs(argv, input=None):
+def _run_nodejs(argv: list[str], input: dict[str, Any] | None = None) -> str:
     return _run(_nodejs_path(), argv, input)
 
-def _run_npmjs(argv, input=None):
+def _run_npmjs(argv: list[str], input: dict[str, Any] | None = None) -> str:
     return _run(_npmjs_path(), argv, input)
 
-def _version(run_app) -> str | None:
+def _version(run_app: Callable[[list[str], dict[str, Any] | None], str]) -> str | None:
     try:
-        version = run_app(["--version"])
+        version = run_app(["--version"], None)  # explicit None to make mypy happy
     except RuntimeError:
         return None
     else:
         return version.strip()
 
-def _model_cache_no_op(model, implementation):
+def _model_cache_no_op(model: CustomModel, implementation: Implementation) -> AttrDict | None:
     """Return cached compiled implementation"""
     return None
 
 _CACHING_IMPLEMENTATION = _model_cache_no_op
 
-def _get_custom_models(models: Sequence[Type[Model]] | None) -> Dict[str, CustomModel] | None:
+def _get_custom_models(models: Sequence[type[HasProps]] | None) -> dict[str, CustomModel] | None:
     """Returns CustomModels for models with a custom `__implementation__`"""
-    custom_models: Dict[str, CustomModel] = dict()
+    custom_models: dict[str, CustomModel] = dict()
 
-    for cls in models or Model.model_class_reverse_map.values():
+    for cls in models or HasProps.model_class_reverse_map.values():
         impl = getattr(cls, "__implementation__", None)
 
         if impl is not None:
@@ -474,12 +477,12 @@ def _get_custom_models(models: Sequence[Type[Model]] | None) -> Dict[str, Custom
 
     return custom_models if custom_models else None
 
-def _compile_models(custom_models: Dict[str, CustomModel]):
+def _compile_models(custom_models: dict[str, CustomModel]) -> dict[str, AttrDict]:
     """Returns the compiled implementation of supplied `models`. """
     ordered_models = sorted(custom_models.values(), key=lambda model: model.full_name)
     custom_impls = {}
 
-    dependencies: List[str] = []
+    dependencies: list[tuple[str, str]] = []
     for model in ordered_models:
         dependencies.extend(list(model.dependencies.items()))
 
@@ -492,21 +495,20 @@ def _compile_models(custom_models: Dict[str, CustomModel]):
         compiled = _CACHING_IMPLEMENTATION(model, impl)
         if compiled is None:
             compiled = nodejs_compile(impl.code, lang=impl.lang, file=impl.file)
-
-        if "error" in compiled:
-            raise CompilationError(compiled.error)
+            if "error" in compiled:
+                raise CompilationError(compiled.error)
 
         custom_impls[model.full_name] = compiled
 
     return custom_impls
 
-def _bundle_models(custom_models) -> str:
+def _bundle_models(custom_models: dict[str, CustomModel]) -> str:
     """ Create a JavaScript bundle with selected `models`. """
     exports = []
     modules = []
 
     with open(join(bokehjs_dir, "js", "bokeh.json"), encoding="utf-8") as f:
-        bokeh = json.loads(f.read())
+        bokeh = json.load(f)
 
     known_modules = set()
     for artifact in bokeh["artifacts"]:
@@ -518,11 +520,11 @@ def _bundle_models(custom_models) -> str:
 
     extra_modules = {}
 
-    def resolve_modules(to_resolve, root):
+    def resolve_modules(to_resolve: set[str], root: str) -> dict[str, str]:
         resolved = {}
         for module in to_resolve:
             if module.startswith(("./", "../")):
-                def mkpath(module, ext=""):
+                def mkpath(module: str, ext: str = "") -> str:
                     return abspath(join(root, *module.split("/")) + ext)
 
                 if module.endswith(exts):
@@ -540,9 +542,6 @@ def _bundle_models(custom_models) -> str:
                 impl = FromFile(path)
                 compiled = nodejs_compile(impl.code, lang=impl.lang, file=impl.file)
 
-                if "error" in compiled:
-                    raise CompilationError(compiled.error)
-
                 if impl.lang == "less":
                     code = _style_template % dict(css=json.dumps(compiled.code))
                     deps = []
@@ -565,7 +564,7 @@ def _bundle_models(custom_models) -> str:
 
         return resolved
 
-    def resolve_deps(deps, root):
+    def resolve_deps(deps : list[str], root: str) -> dict[str, str]:
         custom_modules = {model.module for model in custom_models.values()}
         missing = set(deps) - known_modules - custom_modules
         return resolve_modules(missing, root)
@@ -581,18 +580,19 @@ def _bundle_models(custom_models) -> str:
     exports = sorted(exports, key=lambda spec: spec[1])
     modules = sorted(modules, key=lambda spec: spec[0])
 
+    bare_modules = []
     for i, (module, code, deps) in enumerate(modules):
         for name, ref in deps.items():
             code = code.replace("""require("%s")""" % name, """require("%s")""" % ref)
             code = code.replace("""require('%s')""" % name, """require('%s')""" % ref)
-        modules[i] = (module, code)
+        bare_modules.append((module, code))
 
     sep = ",\n"
 
-    exports = sep.join(_export_template % dict(name=name, module=module) for (name, module) in exports)
-    modules = sep.join(_module_template % dict(module=module, source=code) for (module, code) in modules)
+    rendered_exports = sep.join(_export_template % dict(name=name, module=module) for (name, module) in exports)
+    rendered_modules = sep.join(_module_template % dict(module=module, source=code) for (module, code) in bare_modules)
 
-    content = _plugin_template % dict(prelude=_plugin_prelude, exports=exports, modules=modules)
+    content = _plugin_template % dict(prelude=_plugin_prelude, exports=rendered_exports, modules=rendered_modules)
     return _plugin_umd % dict(content=content)
 
 #-----------------------------------------------------------------------------
