@@ -18,11 +18,13 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-import warnings  # lgtm [py/import-and-import-from]
-from typing import Tuple, overload
+from typing import TYPE_CHECKING, Tuple, overload
 
 # Bokeh imports
-from .warnings import BokehDeprecationWarning
+from .warnings import BokehDeprecationWarning, warn
+
+if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -30,18 +32,14 @@ from .warnings import BokehDeprecationWarning
 
 __all__ = (
     'deprecated',
-    'warn',
 )
 
-Version = Tuple[int, int, int]
+Version: TypeAlias = Tuple[int, int, int]
 
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-def warn(message: str, stacklevel: int = 2) -> None:
-    warnings.warn(message, BokehDeprecationWarning, stacklevel=stacklevel)
-
 @overload
 def deprecated(since_or_msg: Version, old: str, new: str, extra: str | None = None) -> None:
     ...
@@ -72,7 +70,7 @@ def deprecated(since_or_msg: Version | str,
 
         message = since_or_msg
 
-    warn(message)
+    warn(message, BokehDeprecationWarning)
 
 #-----------------------------------------------------------------------------
 # Dev API
