@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,37 +11,30 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import annotations
+from __future__ import annotations # isort:skip
 
-import logging # isort:skip
-log = logging.getLogger(__name__)
+# NOTE: skip logging imports so that this module may be run as a script
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# NOTE: since downloading sampledata is not a common occurrnce, non-stdlib
-# imports are generally deferrered in this module
-
 # Standard library imports
 import hashlib
 import json
-from os import mkdir, remove
-from os.path import (
-    abspath,
-    dirname,
-    exists,
-    expanduser,
-    isdir,
-    isfile,
-    join,
-    splitext,
-)
+from os.path import splitext
+from pathlib import Path
 from sys import stdout
-from typing import Any, TextIO, cast
+from typing import TYPE_CHECKING, Any, TextIO
 from urllib.parse import urljoin
 from urllib.request import urlopen
 
+# NOTE: since downloading sampledata is not a common occurrence, non-stdlib
+# imports are generally deferrered in this module
+
+if TYPE_CHECKING:
+    import pandas as pd
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -50,8 +43,6 @@ __all__ = (
     'download',
 )
 
-DataFrame = Any
-
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -61,11 +52,11 @@ def download(progress: bool = True) -> None:
 
     '''
     data_dir = external_data_dir(create=True)
-    print("Using data directory: %s" % data_dir)
+    print(f"Using data directory: {data_dir}")
 
     # HTTP requests are cheaper for us, and there is nothing private to protect
     s3 = 'http://sampledata.bokeh.org'
-    files = json.load(open(join(dirname(__file__), "sampledata.json")))
+    files = json.load(open(Path(__file__).parent / "sampledata.json"))
 
     for filename, md5 in files:
         real_name, ext = splitext(filename)
@@ -74,10 +65,10 @@ def download(progress: bool = True) -> None:
                 real_name += ".csv"
         else:
             real_name += ext
-        real_path = join(data_dir, real_name)
+        real_path = data_dir /real_name
 
-        if exists(real_path):
-            local_md5 = hashlib.md5(open(real_path,'rb').read()).hexdigest()
+        if real_path.exists():
+            local_md5 = hashlib.md5(open(real_path, 'rb').read()).hexdigest()
             if local_md5 == md5:
                 print(f"Skipping {filename!r} (checksum match)")
                 continue
@@ -90,15 +81,14 @@ def download(progress: bool = True) -> None:
 # Dev API
 #-----------------------------------------------------------------------------
 
-def external_csv(module: str, name: str, **kw: Any) -> DataFrame:
+def external_csv(module: str, name: str, **kw: Any) -> pd.DataFrame:
     '''
 
     '''
-    from .dependencies import import_required
-    pd = import_required('pandas', '%s sample data requires Pandas (http://pandas.pydata.org) to be installed' % module)
-    return cast(Any, pd).read_csv(external_path(name), **kw)
+    import pandas as pd
+    return pd.read_csv(external_path(name), **kw)
 
-def external_data_dir(create: bool = False) -> str:
+def external_data_dir(create: bool = False) -> Path:
     '''
 
     '''
@@ -108,84 +98,79 @@ def external_data_dir(create: bool = False) -> str:
         raise RuntimeError("'yaml' and 'pyyaml' are required to use bokeh.sampledata functions")
 
     bokeh_dir = _bokeh_dir(create=create)
-    data_dir = join(bokeh_dir, "data")
+    data_dir = bokeh_dir / "data"
 
     try:
-        config = yaml.safe_load(open(join(bokeh_dir, 'config')))
-        data_dir = expanduser(config['sampledata_dir'])
+        config = yaml.safe_load(open(bokeh_dir / 'config'))
+        data_dir = Path.expanduser(config['sampledata_dir'])
     except (OSError, TypeError):
         pass
 
-    if not exists(data_dir):
+    if not data_dir.exists():
         if not create:
             raise RuntimeError('bokeh sample data directory does not exist, please execute bokeh.sampledata.download()')
-        print("Creating %s directory" % data_dir)
+        print(f"Creating {data_dir} directory")
         try:
-            mkdir(data_dir)
+            data_dir.mkdir()
         except OSError:
-            raise RuntimeError("could not create bokeh data directory at %s" % data_dir)
+            raise RuntimeError(f"could not create bokeh data directory at {data_dir}")
     else:
-        if not isdir(data_dir):
-            raise RuntimeError("%s exists but is not a directory" % data_dir)
+        if not data_dir.is_dir():
+            raise RuntimeError(f"{data_dir} exists but is not a directory")
 
     return data_dir
 
-def external_path(filename: str) -> str:
+def external_path(filename: str | Path) -> Path:
     data_dir = external_data_dir()
-    fn = join(data_dir, filename)
-    if not exists(fn) and isfile(fn):
-        raise RuntimeError('Could not locate external data file %s. Please execute bokeh.sampledata.download()' % fn)
+    fn = data_dir / filename
+    if not fn.exists() or not fn.is_file():
+        raise RuntimeError(f"Could not locate external data file {fn}. Please execute bokeh.sampledata.download()")
     return fn
 
-def package_csv(module: str, name: str, **kw: Any) -> DataFrame:
+def package_csv(module: str, name: str, **kw: Any) -> pd.DataFrame:
     '''
 
     '''
-    from .dependencies import import_required
-    pd = import_required('pandas', '%s sample data requires Pandas (http://pandas.pydata.org) to be installed' % module)
-    return cast(Any, pd).read_csv(package_path(name), **kw)
+    import pandas as pd
+    return pd.read_csv(package_path(name), **kw)
 
-
-def package_dir() -> str:
+def package_dir() -> Path:
     '''
 
     '''
-    return abspath(join(dirname(__file__), "..", "sampledata", "_data"))
+    return Path(__file__).parents[1].joinpath("sampledata", "_data").resolve()
 
-def package_path(filename: str) -> str:
+def package_path(filename: str | Path) -> Path:
     '''
 
     '''
-    return join(package_dir(), filename)
+    return package_dir() / filename
 
-def open_csv(filename: str) -> TextIO:
+def open_csv(filename: str | Path) -> TextIO:
     '''
 
     '''
-    return open(filename, 'r', newline='', encoding='utf8')
+    return open(filename, newline='', encoding='utf8')
 
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
 
-def _bokeh_dir(create: bool = False) -> str:
-    '''
-
-    '''
-    bokeh_dir = join(expanduser("~"), ".bokeh")
-    if not exists(bokeh_dir):
+def _bokeh_dir(create: bool = False) -> Path:
+    bokeh_dir = Path("~").expanduser() / ".bokeh"
+    if not bokeh_dir.exists():
         if not create: return bokeh_dir
-        print("Creating %s directory" % bokeh_dir)
+        print(f"Creating {bokeh_dir} directory")
         try:
-            mkdir(bokeh_dir)
+            bokeh_dir.mkdir()
         except OSError:
-            raise RuntimeError("could not create bokeh config directory at %s" % bokeh_dir)
+            raise RuntimeError(f"could not create bokeh config directory at {bokeh_dir}")
     else:
-        if not isdir(bokeh_dir):
-            raise RuntimeError("%s exists but is not a directory" % bokeh_dir)
+        if not bokeh_dir.is_dir():
+            raise RuntimeError(f"{bokeh_dir} exists but is not a directory")
     return bokeh_dir
 
-def _download_file(base_url: str, filename: str, data_dir: str, progress: bool = True) -> None:
+def _download_file(base_url: str, filename: str, data_dir: Path, progress: bool = True) -> None:
     '''
 
     '''
@@ -195,13 +180,13 @@ def _download_file(base_url: str, filename: str, data_dir: str, progress: bool =
     from zipfile import ZipFile
 
     file_url = urljoin(base_url, filename)
-    file_path = join(data_dir, filename)
+    file_path = data_dir / filename
 
     url = urlopen(file_url)
 
     with open(file_path, 'wb') as file:
         file_size = int(url.headers["Content-Length"])
-        print("Downloading: %s (%d bytes)" % (filename, file_size))
+        print(f"Downloading: {filename} ({file_size} bytes)")
 
         fetch_size = 0
         block_size = 16384
@@ -215,7 +200,7 @@ def _download_file(base_url: str, filename: str, data_dir: str, progress: bool =
             file.write(data)
 
             if progress:
-                status = "\r%10d [%6.2f%%]" % (fetch_size, fetch_size*100.0/file_size)
+                status = f"\r{fetch_size:< 10d} [{fetch_size*100.0/file_size:6.2f}%%]"
                 stdout.write(status)
                 stdout.flush()
 
@@ -228,13 +213,18 @@ def _download_file(base_url: str, filename: str, data_dir: str, progress: bool =
         if not splitext(real_name)[1]:
             real_name += ".csv"
 
-        print("Unpacking: %s" % real_name)
+        print(f"Unpacking: {real_name}")
 
         with ZipFile(file_path, 'r') as zip_file:
             zip_file.extract(real_name, data_dir)
 
-        remove(file_path)
+        file_path.unlink()
 
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
+
+# This is necessary so that we can run the sampledata download code in the
+# release build, before an actual package exists.
+if __name__ == "__main__":
+    download(progress=False)
