@@ -26,6 +26,7 @@ import threading
 from collections import defaultdict
 from traceback import format_exception
 from typing import (
+    TYPE_CHECKING,
     Any,
     Awaitable,
     Callable,
@@ -36,12 +37,16 @@ from typing import (
 )
 
 # External imports
+import tornado
 from tornado import gen
-from tornado.ioloop import IOLoop
 
 # Bokeh imports
 from ..core.types import ID
 
+if TYPE_CHECKING:
+    from tornado.ioloop import IOLoop
+    from typing_extensions import TypeAlias
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -58,7 +63,11 @@ __all__ = ()
 
 # See https://github.com/bokeh/bokeh/issues/9507
 def fixup_windows_event_loop_policy() -> None:
-    if sys.platform == 'win32' and sys.version_info[:3] >= (3, 8, 0):
+    if (
+        sys.platform == 'win32'
+        and sys.version_info[:3] >= (3, 8, 0)
+        and tornado.version_info < (6, 1)
+    ):
         import asyncio
         if type(asyncio.get_event_loop_policy()) is asyncio.WindowsProactorEventLoopPolicy:
             # WindowsProactorEventLoopPolicy is not compatible with tornado 6
@@ -69,17 +78,17 @@ def fixup_windows_event_loop_policy() -> None:
 # Private API
 #-----------------------------------------------------------------------------
 
-CallbackSync = Callable[[], None]
-CallbackAsync = Callable[[], Awaitable[None]]
-Callback = Union[CallbackSync, CallbackAsync]
+CallbackSync: TypeAlias = Callable[[], None]
+CallbackAsync: TypeAlias = Callable[[], Awaitable[None]]
+Callback: TypeAlias = Union[CallbackSync, CallbackAsync]
 
-InvokeResult = Union[Awaitable[None], Awaitable[List[Any]], Awaitable[Dict[Any, Any]]]
+InvokeResult: TypeAlias = Union[Awaitable[None], Awaitable[List[Any]], Awaitable[Dict[Any, Any]]]
 
-Remover = Callable[[], None]
+Remover: TypeAlias = Callable[[], None]
 
-Removers = Dict[ID, Remover]
+Removers: TypeAlias = Dict[ID, Remover]
 
-RemoversByCallable = Dict[Callback, Set[ID]]
+RemoversByCallable: TypeAlias = Dict[Callback, Set[ID]]
 
 class _AsyncPeriodic:
     ''' Like ioloop.PeriodicCallback except the 'func' can be async and return
